@@ -1,29 +1,34 @@
 package lxd
 
 import (
+	"context"
 	"fmt"
 	"sync"
+
+	"github.com/lxc/lxd/shared/api"
 )
 
-// The EventListener struct is used to interact with a LXD event stream
+// The EventListener struct is used to interact with a LXD event stream.
 type EventListener struct {
-	r            *ProtocolLXD
-	chActive     chan bool
-	disconnected bool
-	err          error
+	r         *ProtocolLXD
+	ctx       context.Context
+	ctxCancel context.CancelFunc
+	err       error
 
+	// projectName stores which project this event listener is associated with (empty for all projects).
+	projectName string
 	targets     []*EventTarget
 	targetsLock sync.Mutex
 }
 
-// The EventTarget struct is returned to the caller of AddHandler and used in RemoveHandler
+// The EventTarget struct is returned to the caller of AddHandler and used in RemoveHandler.
 type EventTarget struct {
-	function func(interface{})
+	function func(api.Event)
 	types    []string
 }
 
-// AddHandler adds a function to be called whenever an event is received
-func (e *EventListener) AddHandler(types []string, function func(interface{})) (*EventTarget, error) {
+// AddHandler adds a function to be called whenever an event is received.
+func (e *EventListener) AddHandler(types []string, function func(api.Event)) (*EventTarget, error) {
 	if function == nil {
 		return nil, fmt.Errorf("A valid function must be provided")
 	}
@@ -44,7 +49,7 @@ func (e *EventListener) AddHandler(types []string, function func(interface{})) (
 	return &target, nil
 }
 
-// RemoveHandler removes a function to be called whenever an event is received
+// RemoveHandler removes a function to be called whenever an event is received.
 func (e *EventListener) RemoveHandler(target *EventTarget) error {
 	if target == nil {
 		return fmt.Errorf("A valid event target must be provided")
@@ -67,44 +72,38 @@ func (e *EventListener) RemoveHandler(target *EventTarget) error {
 	return fmt.Errorf("Couldn't find this function and event types combination")
 }
 
-// Disconnect must be used once done listening for events
+// Disconnect must be used once done listening for events.
 func (e *EventListener) Disconnect() {
-	if e.disconnected {
-		return
-	}
-
 	// Handle locking
 	e.r.eventListenersLock.Lock()
 	defer e.r.eventListenersLock.Unlock()
 
+	if e.ctx.Err() != nil {
+		return
+	}
+
 	// Locate and remove it from the global list
-	for i, listener := range e.r.eventListeners {
+	for i, listener := range e.r.eventListeners[e.projectName] {
 		if listener == e {
-			copy(e.r.eventListeners[i:], e.r.eventListeners[i+1:])
-			e.r.eventListeners[len(e.r.eventListeners)-1] = nil
-			e.r.eventListeners = e.r.eventListeners[:len(e.r.eventListeners)-1]
+			copy(e.r.eventListeners[e.projectName][i:], e.r.eventListeners[e.projectName][i+1:])
+			e.r.eventListeners[e.projectName][len(e.r.eventListeners[e.projectName])-1] = nil
+			e.r.eventListeners[e.projectName] = e.r.eventListeners[e.projectName][:len(e.r.eventListeners[e.projectName])-1]
 			break
 		}
 	}
 
 	// Turn off the handler
 	e.err = nil
-	e.disconnected = true
-	close(e.chActive)
+	e.ctxCancel()
 }
 
-// Wait hangs until the server disconnects the connection or Disconnect() is called
+// Wait blocks until the server disconnects the connection or Disconnect() is called.
 func (e *EventListener) Wait() error {
-	<-e.chActive
+	<-e.ctx.Done()
 	return e.err
 }
 
 // IsActive returns true if this listener is still connected, false otherwise.
 func (e *EventListener) IsActive() bool {
-	select {
-	case <-e.chActive:
-		return false // If the chActive channel is closed we got disconnected
-	default:
-		return true
-	}
+	return e.ctx.Err() == nil
 }
