@@ -1,6 +1,7 @@
 package lxd
 
 import (
+	"context"
 	"io"
 	"net"
 	"net/http"
@@ -22,6 +23,7 @@ type Operation interface {
 	RemoveHandler(target *EventTarget) (err error)
 	Refresh() (err error)
 	Wait() (err error)
+	WaitContext(ctx context.Context) error
 }
 
 // The RemoteOperation type represents an Operation that may be using multiple servers.
@@ -334,6 +336,20 @@ type InstanceServer interface {
 	UpdateStoragePool(name string, pool api.StoragePoolPut, ETag string) (err error)
 	DeleteStoragePool(name string) (err error)
 
+	// Storage bucket functions ("storage_buckets" API extension)
+	GetStoragePoolBucketNames(poolName string) ([]string, error)
+	GetStoragePoolBuckets(poolName string) ([]api.StorageBucket, error)
+	GetStoragePoolBucket(poolName string, bucketName string) (bucket *api.StorageBucket, ETag string, err error)
+	CreateStoragePoolBucket(poolName string, bucket api.StorageBucketsPost) (*api.StorageBucketKey, error)
+	UpdateStoragePoolBucket(poolName string, bucketName string, bucket api.StorageBucketPut, ETag string) (err error)
+	DeleteStoragePoolBucket(poolName string, bucketName string) (err error)
+	GetStoragePoolBucketKeyNames(poolName string, bucketName string) ([]string, error)
+	GetStoragePoolBucketKeys(poolName string, bucketName string) ([]api.StorageBucketKey, error)
+	GetStoragePoolBucketKey(poolName string, bucketName string, keyName string) (key *api.StorageBucketKey, ETag string, err error)
+	CreateStoragePoolBucketKey(poolName string, bucketName string, key api.StorageBucketKeysPost) (newKey *api.StorageBucketKey, err error)
+	UpdateStoragePoolBucketKey(poolName string, bucketName string, keyName string, key api.StorageBucketKeyPut, ETag string) (err error)
+	DeleteStoragePoolBucketKey(poolName string, bucketName string, keyName string) (err error)
+
 	// Storage volume functions ("storage" API extension)
 	GetStoragePoolVolumeNames(pool string) (names []string, err error)
 	GetStoragePoolVolumeNamesAllProjects(pool string) (names []string, err error)
