@@ -3,36 +3,29 @@ package lxd
 import (
 	"fmt"
 	"net/url"
-	"strings"
 
 	"github.com/lxc/lxd/shared/api"
 )
 
-// GetNetworkNames returns a list of network names
+// GetNetworkNames returns a list of network names.
 func (r *ProtocolLXD) GetNetworkNames() ([]string, error) {
 	if !r.HasExtension("network") {
 		return nil, fmt.Errorf("The server is missing the required \"network\" API extension")
 	}
 
+	// Fetch the raw values.
 	urls := []string{}
-
-	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/networks", nil, "", &urls)
+	baseURL := "/networks"
+	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
 
-	// Parse it
-	names := []string{}
-	for _, url := range urls {
-		fields := strings.Split(url, "/networks/")
-		names = append(names, fields[len(fields)-1])
-	}
-
-	return names, nil
+	// Parse it.
+	return urlsToResourceNames(baseURL, urls...)
 }
 
-// GetNetworks returns a list of Network struct
+// GetNetworks returns a list of Network struct.
 func (r *ProtocolLXD) GetNetworks() ([]api.Network, error) {
 	if !r.HasExtension("network") {
 		return nil, fmt.Errorf("The server is missing the required \"network\" API extension")
@@ -49,7 +42,7 @@ func (r *ProtocolLXD) GetNetworks() ([]api.Network, error) {
 	return networks, nil
 }
 
-// GetNetwork returns a Network entry for the provided name
+// GetNetwork returns a Network entry for the provided name.
 func (r *ProtocolLXD) GetNetwork(name string) (*api.Network, string, error) {
 	if !r.HasExtension("network") {
 		return nil, "", fmt.Errorf("The server is missing the required \"network\" API extension")
@@ -58,7 +51,7 @@ func (r *ProtocolLXD) GetNetwork(name string) (*api.Network, string, error) {
 	network := api.Network{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), nil, "", &network)
+	etag, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s", url.PathEscape(name)), nil, "", &network)
 	if err != nil {
 		return nil, "", err
 	}
@@ -66,7 +59,7 @@ func (r *ProtocolLXD) GetNetwork(name string) (*api.Network, string, error) {
 	return &network, etag, nil
 }
 
-// GetNetworkLeases returns a list of Network struct
+// GetNetworkLeases returns a list of Network struct.
 func (r *ProtocolLXD) GetNetworkLeases(name string) ([]api.NetworkLease, error) {
 	if !r.HasExtension("network_leases") {
 		return nil, fmt.Errorf("The server is missing the required \"network_leases\" API extension")
@@ -75,7 +68,7 @@ func (r *ProtocolLXD) GetNetworkLeases(name string) ([]api.NetworkLease, error)
 	leases := []api.NetworkLease{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s/leases", url.QueryEscape(name)), nil, "", &leases)
+	_, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s/leases", url.PathEscape(name)), nil, "", &leases)
 	if err != nil {
 		return nil, err
 	}
@@ -83,7 +76,7 @@ func (r *ProtocolLXD) GetNetworkLeases(name string) ([]api.NetworkLease, error)
 	return leases, nil
 }
 
-// GetNetworkState returns metrics and information on the running network
+// GetNetworkState returns metrics and information on the running network.
 func (r *ProtocolLXD) GetNetworkState(name string) (*api.NetworkState, error) {
 	if !r.HasExtension("network_state") {
 		return nil, fmt.Errorf("The server is missing the required \"network_state\" API extension")
@@ -92,7 +85,7 @@ func (r *ProtocolLXD) GetNetworkState(name string) (*api.NetworkState, error) {
 	state := api.NetworkState{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s/state", url.QueryEscape(name)), nil, "", &state)
+	_, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s/state", url.PathEscape(name)), nil, "", &state)
 	if err != nil {
 		return nil, err
 	}
@@ -100,7 +93,7 @@ func (r *ProtocolLXD) GetNetworkState(name string) (*api.NetworkState, error) {
 	return &state, nil
 }
 
-// CreateNetwork defines a new network using the provided Network struct
+// CreateNetwork defines a new network using the provided Network struct.
 func (r *ProtocolLXD) CreateNetwork(network api.NetworksPost) error {
 	if !r.HasExtension("network") {
 		return fmt.Errorf("The server is missing the required \"network\" API extension")
@@ -115,14 +108,14 @@ func (r *ProtocolLXD) CreateNetwork(network api.NetworksPost) error {
 	return nil
 }
 
-// UpdateNetwork updates the network to match the provided Network struct
+// UpdateNetwork updates the network to match the provided Network struct.
 func (r *ProtocolLXD) UpdateNetwork(name string, network api.NetworkPut, ETag string) error {
 	if !r.HasExtension("network") {
 		return fmt.Errorf("The server is missing the required \"network\" API extension")
 	}
 
 	// Send the request
-	_, _, err := r.query("PUT", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), network, ETag)
+	_, _, err := r.query("PUT", fmt.Sprintf("/networks/%s", url.PathEscape(name)), network, ETag)
 	if err != nil {
 		return err
 	}
@@ -130,14 +123,14 @@ func (r *ProtocolLXD) UpdateNetwork(name string, network api.NetworkPut, ETag st
 	return nil
 }
 
-// RenameNetwork renames an existing network entry
+// RenameNetwork renames an existing network entry.
 func (r *ProtocolLXD) RenameNetwork(name string, network api.NetworkPost) error {
 	if !r.HasExtension("network") {
 		return fmt.Errorf("The server is missing the required \"network\" API extension")
 	}
 
 	// Send the request
-	_, _, err := r.query("POST", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), network, "")
+	_, _, err := r.query("POST", fmt.Sprintf("/networks/%s", url.PathEscape(name)), network, "")
 	if err != nil {
 		return err
 	}
@@ -145,14 +138,14 @@ func (r *ProtocolLXD) RenameNetwork(name string, network api.NetworkPost) error
 	return nil
 }
 
-// DeleteNetwork deletes an existing network
+// DeleteNetwork deletes an existing network.
 func (r *ProtocolLXD) DeleteNetwork(name string) error {
 	if !r.HasExtension("network") {
 		return fmt.Errorf("The server is missing the required \"network\" API extension")
 	}
 
 	// Send the request
-	_, _, err := r.query("DELETE", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), nil, "")
+	_, _, err := r.query("DELETE", fmt.Sprintf("/networks/%s", url.PathEscape(name)), nil, "")
 	if err != nil {
 		return err
 	}
