@@ -3,38 +3,31 @@ package lxd
 import (
 	"fmt"
 	"net/url"
-	"strings"
 
 	"github.com/lxc/lxd/shared/api"
 )
 
 // Storage pool handling functions
 
-// GetStoragePoolNames returns the names of all storage pools
+// GetStoragePoolNames returns the names of all storage pools.
 func (r *ProtocolLXD) GetStoragePoolNames() ([]string, error) {
 	if !r.HasExtension("storage") {
 		return nil, fmt.Errorf("The server is missing the required \"storage\" API extension")
 	}
 
+	// Fetch the raw URL values.
 	urls := []string{}
-
-	// Fetch the raw value
-	_, err := r.queryStruct("GET", "/storage-pools", nil, "", &urls)
+	baseURL := "/storage-pools"
+	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
 
-	// Parse it
-	names := []string{}
-	for _, url := range urls {
-		fields := strings.Split(url, "/storage-pools/")
-		names = append(names, fields[len(fields)-1])
-	}
-
-	return names, nil
+	// Parse it.
+	return urlsToResourceNames(baseURL, urls...)
 }
 
-// GetStoragePools returns a list of StoragePool entries
+// GetStoragePools returns a list of StoragePool entries.
 func (r *ProtocolLXD) GetStoragePools() ([]api.StoragePool, error) {
 	if !r.HasExtension("storage") {
 		return nil, fmt.Errorf("The server is missing the required \"storage\" API extension")
@@ -51,7 +44,7 @@ func (r *ProtocolLXD) GetStoragePools() ([]api.StoragePool, error) {
 	return pools, nil
 }
 
-// GetStoragePool returns a StoragePool entry for the provided pool name
+// GetStoragePool returns a StoragePool entry for the provided pool name.
 func (r *ProtocolLXD) GetStoragePool(name string) (*api.StoragePool, string, error) {
 	if !r.HasExtension("storage") {
 		return nil, "", fmt.Errorf("The server is missing the required \"storage\" API extension")
@@ -60,7 +53,7 @@ func (r *ProtocolLXD) GetStoragePool(name string) (*api.StoragePool, string, err
 	pool := api.StoragePool{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name)), nil, "", &pool)
+	etag, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s", url.PathEscape(name)), nil, "", &pool)
 	if err != nil {
 		return nil, "", err
 	}
@@ -68,7 +61,7 @@ func (r *ProtocolLXD) GetStoragePool(name string) (*api.StoragePool, string, err
 	return &pool, etag, nil
 }
 
-// CreateStoragePool defines a new storage pool using the provided StoragePool struct
+// CreateStoragePool defines a new storage pool using the provided StoragePool struct.
 func (r *ProtocolLXD) CreateStoragePool(pool api.StoragePoolsPost) error {
 	if !r.HasExtension("storage") {
 		return fmt.Errorf("The server is missing the required \"storage\" API extension")
@@ -87,14 +80,14 @@ func (r *ProtocolLXD) CreateStoragePool(pool api.StoragePoolsPost) error {
 	return nil
 }
 
-// UpdateStoragePool updates the pool to match the provided StoragePool struct
+// UpdateStoragePool updates the pool to match the provided StoragePool struct.
 func (r *ProtocolLXD) UpdateStoragePool(name string, pool api.StoragePoolPut, ETag string) error {
 	if !r.HasExtension("storage") {
 		return fmt.Errorf("The server is missing the required \"storage\" API extension")
 	}
 
 	// Send the request
-	_, _, err := r.query("PUT", fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name)), pool, ETag)
+	_, _, err := r.query("PUT", fmt.Sprintf("/storage-pools/%s", url.PathEscape(name)), pool, ETag)
 	if err != nil {
 		return err
 	}
@@ -102,14 +95,14 @@ func (r *ProtocolLXD) UpdateStoragePool(name string, pool api.StoragePoolPut, ET
 	return nil
 }
 
-// DeleteStoragePool deletes a storage pool
+// DeleteStoragePool deletes a storage pool.
 func (r *ProtocolLXD) DeleteStoragePool(name string) error {
 	if !r.HasExtension("storage") {
 		return fmt.Errorf("The server is missing the required \"storage\" API extension")
 	}
 
 	// Send the request
-	_, _, err := r.query("DELETE", fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name)), nil, "")
+	_, _, err := r.query("DELETE", fmt.Sprintf("/storage-pools/%s", url.PathEscape(name)), nil, "")
 	if err != nil {
 		return err
 	}
@@ -117,7 +110,7 @@ func (r *ProtocolLXD) DeleteStoragePool(name string) error {
 	return nil
 }
 
-// GetStoragePoolResources gets the resources available to a given storage pool
+// GetStoragePoolResources gets the resources available to a given storage pool.
 func (r *ProtocolLXD) GetStoragePoolResources(name string) (*api.ResourcesStoragePool, error) {
 	if !r.HasExtension("resources") {
 		return nil, fmt.Errorf("The server is missing the required \"resources\" API extension")
@@ -126,7 +119,7 @@ func (r *ProtocolLXD) GetStoragePoolResources(name string) (*api.ResourcesStorag
 	res := api.ResourcesStoragePool{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/resources", url.QueryEscape(name)), nil, "", &res)
+	_, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s/resources", url.PathEscape(name)), nil, "", &res)
 	if err != nil {
 		return nil, err
 	}
