@@ -1,7 +1,9 @@
 package lxd
 
 import (
+	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"sync"
 
@@ -10,7 +12,7 @@ import (
 	"github.com/lxc/lxd/shared/api"
 )
 
-// The Operation type represents an ongoing LXD operation (asynchronous processing)
+// The Operation type represents an ongoing LXD operation (asynchronous processing).
 type operation struct {
 	api.Operation
 
@@ -22,7 +24,7 @@ type operation struct {
 	chActive chan bool
 }
 
-// AddHandler adds a function to be called whenever an event is received
+// AddHandler adds a function to be called whenever an event is received.
 func (op *operation) AddHandler(function func(api.Operation)) (*EventTarget, error) {
 	// Make sure we have a listener setup
 	err := op.setupListener()
@@ -40,34 +42,40 @@ func (op *operation) AddHandler(function func(api.Operation)) (*EventTarget, err
 	}
 
 	// Wrap the function to filter unwanted messages
-	wrapped := func(data interface{}) {
-		newOp := op.extractOperation(data)
-		if newOp == nil {
+	wrapped := func(event api.Event) {
+		op.handlerLock.Lock()
+
+		newOp := api.Operation{}
+		err := json.Unmarshal(event.Metadata, &newOp)
+		if err != nil || newOp.ID != op.ID {
+			op.handlerLock.Unlock()
 			return
 		}
 
-		function(*newOp)
+		op.handlerLock.Unlock()
+
+		function(newOp)
 	}
 
 	return op.listener.AddHandler([]string{"operation"}, wrapped)
 }
 
-// Cancel will request that LXD cancels the operation (if supported)
+// Cancel will request that LXD cancels the operation (if supported).
 func (op *operation) Cancel() error {
 	return op.r.DeleteOperation(op.ID)
 }
 
-// Get returns the API operation struct
+// Get returns the API operation struct.
 func (op *operation) Get() api.Operation {
 	return op.Operation
 }
 
-// GetWebsocket returns a raw websocket connection from the operation
+// GetWebsocket returns a raw websocket connection from the operation.
 func (op *operation) GetWebsocket(secret string) (*websocket.Conn, error) {
 	return op.r.GetOperationWebsocket(op.ID, secret)
 }
 
-// RemoveHandler removes a function to be called whenever an event is received
+// RemoveHandler removes a function to be called whenever an event is received.
 func (op *operation) RemoveHandler(target *EventTarget) error {
 	// Make sure we're not racing with ourselves
 	op.handlerLock.Lock()
@@ -81,7 +89,7 @@ func (op *operation) RemoveHandler(target *EventTarget) error {
 	return op.listener.RemoveHandler(target)
 }
 
-// Refresh pulls the current version of the operation and updates the struct
+// Refresh pulls the current version of the operation and updates the struct.
 func (op *operation) Refresh() error {
 	// Get the current version of the operation
 	newOp, _, err := op.r.GetOperation(op.ID)
@@ -95,28 +103,42 @@ func (op *operation) Refresh() error {
 	return nil
 }
 
-// Wait lets you wait until the operation reaches a final state
+// Wait lets you wait until the operation reaches a final state.
 func (op *operation) Wait() error {
+	return op.WaitContext(context.Background())
+}
+
+// WaitContext lets you wait until the operation reaches a final state with context.Context.
+func (op *operation) WaitContext(ctx context.Context) error {
+	op.handlerLock.Lock()
 	// Check if not done already
 	if op.StatusCode.IsFinal() {
 		if op.Err != "" {
-			return fmt.Errorf(op.Err)
+			op.handlerLock.Unlock()
+			return errors.New(op.Err)
 		}
 
+		op.handlerLock.Unlock()
 		return nil
 	}
 
+	op.handlerLock.Unlock()
+
 	// Make sure we have a listener setup
 	err := op.setupListener()
 	if err != nil {
 		return err
 	}
 
-	<-op.chActive
+	select {
+	case <-ctx.Done():
+		return ctx.Err()
+	case <-op.chActive:
+	}
 
 	// We're done, parse the result
 	if op.Err != "" {
-		return fmt.Errorf(op.Err)
+		return errors.New(op.Err)
 	}
 
 	return nil
@@ -131,6 +153,7 @@ func (op *operation) setupListener() error {
 	if op.handlerReady {
 		return nil
 	}
+
 	op.handlerReady = true
 
 	// Get a new listener
@@ -145,7 +168,7 @@ func (op *operation) setupListener() error {
 
 	// Setup the handler
 	chReady := make(chan bool)
-	_, err := op.listener.AddHandler([]string{"operation"}, func(data interface{}) {
+	_, err := op.listener.AddHandler([]string{"operation"}, func(event api.Event) {
 		<-chReady
 
 		// We don't want concurrency while processing events
@@ -158,13 +181,14 @@ func (op *operation) setupListener() error {
 		}
 
 		// Get an operation struct out of this data
-		newOp := op.extractOperation(data)
-		if newOp == nil {
+		newOp := api.Operation{}
+		err := json.Unmarshal(event.Metadata, &newOp)
+		if err != nil || newOp.ID != op.ID {
 			return
 		}
 
 		// Update the struct
-		op.Operation = *newOp
+		op.Operation = newOp
 
 		// And check if we're done
 		if op.StatusCode.IsFinal() {
@@ -196,16 +220,18 @@ func (op *operation) setupListener() error {
 			op.handlerLock.Unlock()
 			return
 		}
+
 		op.handlerLock.Unlock()
 
 		// Wait for the listener or operation to be done
 		select {
-		case <-listener.chActive:
+		case <-listener.ctx.Done():
 			op.handlerLock.Lock()
 			if op.listener != nil {
-				op.Err = fmt.Sprintf("%v", listener.err)
+				op.Err = listener.err.Error()
 				close(op.chActive)
 			}
+
 			op.handlerLock.Unlock()
 		case <-op.chActive:
 			return
@@ -231,7 +257,7 @@ func (op *operation) setupListener() error {
 		close(chReady)
 
 		if op.Err != "" {
-			return fmt.Errorf(op.Err)
+			return errors.New(op.Err)
 		}
 
 		return nil
@@ -243,49 +269,26 @@ func (op *operation) setupListener() error {
 	return nil
 }
 
-func (op *operation) extractOperation(data interface{}) *api.Operation {
-	// Extract the metadata
-	meta, ok := data.(map[string]interface{})["metadata"]
-	if !ok {
-		return nil
-	}
-
-	// And attempt to decode it as JSON operation data
-	encoded, err := json.Marshal(meta)
-	if err != nil {
-		return nil
-	}
-
-	newOp := api.Operation{}
-	err = json.Unmarshal(encoded, &newOp)
-	if err != nil {
-		return nil
-	}
-
-	// And now check that it's what we want
-	if newOp.ID != op.ID {
-		return nil
-	}
-
-	return &newOp
-}
-
-// The remoteOperation type represents an ongoing LXD operation between two servers
+// The remoteOperation type represents an ongoing LXD operation between two servers.
 type remoteOperation struct {
 	targetOp Operation
 
-	handlers []func(api.Operation)
+	handlers    []func(api.Operation)
+	handlerLock sync.Mutex
 
 	chDone chan bool
 	chPost chan bool
 	err    error
 }
 
-// AddHandler adds a function to be called whenever an event is received
+// AddHandler adds a function to be called whenever an event is received.
 func (op *remoteOperation) AddHandler(function func(api.Operation)) (*EventTarget, error) {
 	var err error
 	var target *EventTarget
 
+	op.handlerLock.Lock()
+	defer op.handlerLock.Unlock()
+
 	// Attach to the existing target operation
 	if op.targetOp != nil {
 		target, err = op.targetOp.AddHandler(function)
@@ -295,7 +298,7 @@ func (op *remoteOperation) AddHandler(function func(api.Operation)) (*EventTarge
 	} else {
 		// Generate a mock EventTarget
 		target = &EventTarget{
-			function: func(interface{}) { function(api.Operation{}) },
+			function: func(api.Event) { function(api.Operation{}) },
 			types:    []string{"operation"},
 		}
 	}
@@ -306,7 +309,7 @@ func (op *remoteOperation) AddHandler(function func(api.Operation)) (*EventTarge
 	return target, nil
 }
 
-// CancelTarget attempts to cancel the target operation
+// CancelTarget attempts to cancel the target operation.
 func (op *remoteOperation) CancelTarget() error {
 	if op.targetOp == nil {
 		return fmt.Errorf("No associated target operation")
@@ -315,7 +318,7 @@ func (op *remoteOperation) CancelTarget() error {
 	return op.targetOp.Cancel()
 }
 
-// GetTarget returns the target operation
+// GetTarget returns the target operation.
 func (op *remoteOperation) GetTarget() (*api.Operation, error) {
 	if op.targetOp == nil {
 		return nil, fmt.Errorf("No associated target operation")
@@ -325,7 +328,7 @@ func (op *remoteOperation) GetTarget() (*api.Operation, error) {
 	return &opAPI, nil
 }
 
-// Wait lets you wait until the operation reaches a final state
+// Wait lets you wait until the operation reaches a final state.
 func (op *remoteOperation) Wait() error {
 	<-op.chDone
 
