@@ -7,7 +7,7 @@ import (
 	"github.com/lxc/lxd/shared/simplestreams"
 )
 
-// ProtocolSimpleStreams implements a SimpleStreams API client
+// ProtocolSimpleStreams implements a SimpleStreams API client.
 type ProtocolSimpleStreams struct {
 	ssClient *simplestreams.SimpleStreams
 
@@ -17,7 +17,11 @@ type ProtocolSimpleStreams struct {
 	httpCertificate string
 }
 
-// GetConnectionInfo returns the basic connection information used to interact with the server
+// Disconnect is a no-op for simplestreams.
+func (r *ProtocolSimpleStreams) Disconnect() {
+}
+
+// GetConnectionInfo returns the basic connection information used to interact with the server.
 func (r *ProtocolSimpleStreams) GetConnectionInfo() (*ConnectionInfo, error) {
 	info := ConnectionInfo{}
 	info.Addresses = []string{r.httpHost}
@@ -36,3 +40,13 @@ func (r *ProtocolSimpleStreams) GetHTTPClient() (*http.Client, error) {
 
 	return r.http, nil
 }
+
+// DoHTTP performs a Request, using macaroon authentication if set.
+func (r *ProtocolSimpleStreams) DoHTTP(req *http.Request) (*http.Response, error) {
+	// Set the user agent
+	if r.httpUserAgent != "" {
+		req.Header.Set("User-Agent", r.httpUserAgent)
+	}
+
+	return r.http.Do(req)
+}
