@@ -2,12 +2,11 @@ package main
 
 import (
 	"fmt"
-	"os"
 	"sort"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
 
+	"github.com/lxc/lxd/lxc/utils"
 	cli "github.com/lxc/lxd/shared/cmd"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -18,7 +17,7 @@ type cmdAlias struct {
 
 func (c *cmdAlias) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("alias")
+	cmd.Use = usage("alias")
 	cmd.Short = i18n.G("Manage command aliases")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Manage command aliases`))
@@ -39,10 +38,13 @@ func (c *cmdAlias) Command() *cobra.Command {
 	aliasRemoveCmd := cmdAliasRemove{global: c.global, alias: c}
 	cmd.AddCommand(aliasRemoveCmd.Command())
 
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Add
+// Add.
 type cmdAliasAdd struct {
 	global *cmdGlobal
 	alias  *cmdAlias
@@ -50,7 +52,7 @@ type cmdAliasAdd struct {
 
 func (c *cmdAliasAdd) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("add <alias> <target>")
+	cmd.Use = usage("add", i18n.G("<alias> <target>"))
 	cmd.Short = i18n.G("Add new aliases")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Add new aliases`))
@@ -66,7 +68,7 @@ func (c *cmdAliasAdd) Command() *cobra.Command {
 func (c *cmdAliasAdd) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -85,19 +87,22 @@ func (c *cmdAliasAdd) Run(cmd *cobra.Command, args []string) error {
 	return conf.SaveConfig(c.global.confPath)
 }
 
-// List
+// List.
 type cmdAliasList struct {
 	global *cmdGlobal
 	alias  *cmdAlias
+
+	flagFormat string
 }
 
 func (c *cmdAliasList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list")
+	cmd.Use = usage("list")
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List aliases")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List aliases`))
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	cmd.RunE = c.Run
 
@@ -107,7 +112,7 @@ func (c *cmdAliasList) Command() *cobra.Command {
 func (c *cmdAliasList) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 0)
 	if exit {
 		return err
@@ -119,21 +124,17 @@ func (c *cmdAliasList) Run(cmd *cobra.Command, args []string) error {
 		data = append(data, []string{k, v})
 	}
 
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAutoWrapText(false)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetRowLine(true)
-	table.SetHeader([]string{
-		i18n.G("ALIAS"),
-		i18n.G("TARGET")})
-	sort.Sort(byName(data))
-	table.AppendBulk(data)
-	table.Render()
+	sort.Sort(utils.ByName(data))
 
-	return nil
+	header := []string{
+		i18n.G("ALIAS"),
+		i18n.G("TARGET"),
+	}
+
+	return utils.RenderTable(c.flagFormat, header, data, conf.Aliases)
 }
 
-// Rename
+// Rename.
 type cmdAliasRename struct {
 	global *cmdGlobal
 	alias  *cmdAlias
@@ -141,7 +142,7 @@ type cmdAliasRename struct {
 
 func (c *cmdAliasRename) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("rename <old alias> <new alias>")
+	cmd.Use = usage("rename", i18n.G("<old alias> <new alias>"))
 	cmd.Aliases = []string{"mv"}
 	cmd.Short = i18n.G("Rename aliases")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -158,7 +159,7 @@ func (c *cmdAliasRename) Command() *cobra.Command {
 func (c *cmdAliasRename) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -184,7 +185,7 @@ func (c *cmdAliasRename) Run(cmd *cobra.Command, args []string) error {
 	return conf.SaveConfig(c.global.confPath)
 }
 
-// Remove
+// Remove.
 type cmdAliasRemove struct {
 	global *cmdGlobal
 	alias  *cmdAlias
@@ -192,7 +193,7 @@ type cmdAliasRemove struct {
 
 func (c *cmdAliasRemove) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("remove <alias>")
+	cmd.Use = usage("remove", i18n.G("<alias>"))
 	cmd.Aliases = []string{"rm"}
 	cmd.Short = i18n.G("Remove aliases")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -209,7 +210,7 @@ func (c *cmdAliasRemove) Command() *cobra.Command {
 func (c *cmdAliasRemove) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
