@@ -2,10 +2,9 @@ package main
 
 import (
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 	"strings"
-	"syscall"
 
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
@@ -20,14 +19,16 @@ import (
 
 type cmdConfig struct {
 	global *cmdGlobal
+
+	flagTarget string
 }
 
 func (c *cmdConfig) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("config")
-	cmd.Short = i18n.G("Manage container and server configuration options")
+	cmd.Use = usage("config")
+	cmd.Short = i18n.G("Manage instance and server configuration options")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Manage container and server configuration options`))
+		`Manage instance and server configuration options`))
 
 	// Device
 	configDeviceCmd := cmdConfigDevice{global: c.global, config: c}
@@ -72,10 +73,13 @@ func (c *cmdConfig) Command() *cobra.Command {
 	configUnsetCmd := cmdConfigUnset{global: c.global, config: c, configSet: &configSetCmd}
 	cmd.AddCommand(configUnsetCmd.Command())
 
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Edit
+// Edit.
 type cmdConfigEdit struct {
 	global *cmdGlobal
 	config *cmdConfig
@@ -83,14 +87,15 @@ type cmdConfigEdit struct {
 
 func (c *cmdConfigEdit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("edit [<remote>:][<container>]")
-	cmd.Short = i18n.G("Edit container or server configurations as YAML")
+	cmd.Use = usage("edit", i18n.G("[<remote>:][<instance>[/<snapshot>]]"))
+	cmd.Short = i18n.G("Edit instance or server configurations as YAML")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Edit container or server configurations as YAML`))
+		`Edit instance or server configurations as YAML`))
 	cmd.Example = cli.FormatSection("", i18n.G(
-		`lxc config edit <container> < container.yaml
-    Update the container configuration from config.yaml.`))
+		`lxc config edit <instance> < instance.yaml
+    Update the instance configuration from config.yaml.`))
 
+	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
 
 	return cmd
@@ -98,11 +103,11 @@ func (c *cmdConfigEdit) Command() *cobra.Command {
 
 func (c *cmdConfigEdit) helpTemplate() string {
 	return i18n.G(
-		`### This is a yaml representation of the configuration.
+		`### This is a YAML representation of the configuration.
 ### Any line starting with a '# will be ignored.
 ###
 ### A sample configuration looks like:
-### name: container1
+### name: instance1
 ### profiles:
 ### - default
 ### config:
@@ -118,7 +123,7 @@ func (c *cmdConfigEdit) helpTemplate() string {
 }
 
 func (c *cmdConfigEdit) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 1)
 	if exit {
 		return err
@@ -137,39 +142,89 @@ func (c *cmdConfigEdit) Run(cmd *cobra.Command, args []string) error {
 
 	resource := resources[0]
 
+	fields := strings.SplitN(resource.name, "/", 2)
+	isSnapshot := len(fields) == 2
+
 	// Edit the config
 	if resource.name != "" {
+		// Quick checks.
+		if c.config.flagTarget != "" {
+			return fmt.Errorf(i18n.G("--target cannot be used with instances"))
+		}
+
 		// If stdin isn't a terminal, read text from it
-		if !termios.IsTerminal(int(syscall.Stdin)) {
-			contents, err := ioutil.ReadAll(os.Stdin)
+		if !termios.IsTerminal(getStdinFd()) {
+			contents, err := io.ReadAll(os.Stdin)
 			if err != nil {
 				return err
 			}
 
-			newdata := api.ContainerPut{}
-			err = yaml.Unmarshal(contents, &newdata)
-			if err != nil {
-				return err
-			}
+			var op lxd.Operation
 
-			op, err := resource.server.UpdateContainer(resource.name, newdata, "")
-			if err != nil {
-				return err
+			if isSnapshot {
+				newdata := api.InstanceSnapshotPut{}
+
+				err = yaml.Unmarshal(contents, &newdata)
+				if err != nil {
+					return err
+				}
+
+				op, err = resource.server.UpdateInstanceSnapshot(fields[0], fields[1], newdata, "")
+				if err != nil {
+					return err
+				}
+			} else {
+				newdata := api.InstancePut{}
+				err = yaml.Unmarshal(contents, &newdata)
+				if err != nil {
+					return err
+				}
+
+				op, err = resource.server.UpdateInstance(resource.name, newdata, "")
+				if err != nil {
+					return err
+				}
 			}
 
 			return op.Wait()
 		}
 
-		// Extract the current value
-		container, etag, err := resource.server.GetContainer(resource.name)
-		if err != nil {
-			return err
-		}
+		var data []byte
+		var etag string
 
-		brief := container.Writable()
-		data, err := yaml.Marshal(&brief)
-		if err != nil {
-			return err
+		// Extract the current value
+		if isSnapshot {
+			var inst *api.InstanceSnapshot
+
+			inst, etag, err = resource.server.GetInstanceSnapshot(fields[0], fields[1])
+			if err != nil {
+				return err
+			}
+
+			// Empty expanded config so it isn't shown in edit screen (relies on omitempty tag).
+			inst.ExpandedConfig = nil
+			inst.ExpandedDevices = nil
+
+			data, err = yaml.Marshal(&inst)
+			if err != nil {
+				return err
+			}
+		} else {
+			var inst *api.Instance
+
+			inst, etag, err = resource.server.GetInstance(resource.name)
+			if err != nil {
+				return err
+			}
+
+			// Empty expanded config so it isn't shown in edit screen (relies on omitempty tag).
+			inst.ExpandedConfig = nil
+			inst.ExpandedDevices = nil
+
+			data, err = yaml.Marshal(&inst)
+			if err != nil {
+				return err
+			}
 		}
 
 		// Spawn the editor
@@ -180,20 +235,33 @@ func (c *cmdConfigEdit) Run(cmd *cobra.Command, args []string) error {
 
 		for {
 			// Parse the text received from the editor
-			newdata := api.ContainerPut{}
-			err = yaml.Unmarshal(content, &newdata)
-			if err == nil {
-				var op lxd.Operation
-				op, err = resource.server.UpdateContainer(resource.name, newdata, etag)
+			if isSnapshot {
+				newdata := api.InstanceSnapshotPut{}
+				err = yaml.Unmarshal(content, &newdata)
 				if err == nil {
-					err = op.Wait()
+					var op lxd.Operation
+					op, err = resource.server.UpdateInstanceSnapshot(fields[0], fields[1],
+						newdata, etag)
+					if err == nil {
+						err = op.Wait()
+					}
+				}
+			} else {
+				newdata := api.InstancePut{}
+				err = yaml.Unmarshal(content, &newdata)
+				if err == nil {
+					var op lxd.Operation
+					op, err = resource.server.UpdateInstance(resource.name, newdata, etag)
+					if err == nil {
+						err = op.Wait()
+					}
 				}
 			}
 
 			// Respawn the editor
 			if err != nil {
 				fmt.Fprintf(os.Stderr, i18n.G("Config parsing error: %s")+"\n", err)
-				fmt.Println(i18n.G("Press enter to start the editor again"))
+				fmt.Println(i18n.G("Press enter to open the editor again or ctrl+c to abort change"))
 
 				_, err := os.Stdin.Read(make([]byte, 1))
 				if err != nil {
@@ -204,17 +272,28 @@ func (c *cmdConfigEdit) Run(cmd *cobra.Command, args []string) error {
 				if err != nil {
 					return err
 				}
+
 				continue
 			}
+
 			break
 		}
 
 		return nil
 	}
 
+	// Targeting
+	if c.config.flagTarget != "" {
+		if !resource.server.IsClustered() {
+			return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+		}
+
+		resource.server = resource.server.UseTarget(c.config.flagTarget)
+	}
+
 	// If stdin isn't a terminal, read text from it
-	if !termios.IsTerminal(int(syscall.Stdin)) {
-		contents, err := ioutil.ReadAll(os.Stdin)
+	if !termios.IsTerminal(getStdinFd()) {
+		contents, err := io.ReadAll(os.Stdin)
 		if err != nil {
 			return err
 		}
@@ -257,7 +336,7 @@ func (c *cmdConfigEdit) Run(cmd *cobra.Command, args []string) error {
 		// Respawn the editor
 		if err != nil {
 			fmt.Fprintf(os.Stderr, i18n.G("Config parsing error: %s")+"\n", err)
-			fmt.Println(i18n.G("Press enter to start the editor again"))
+			fmt.Println(i18n.G("Press enter to open the editor again or ctrl+c to abort change"))
 
 			_, err := os.Stdin.Read(make([]byte, 1))
 			if err != nil {
@@ -268,34 +347,40 @@ func (c *cmdConfigEdit) Run(cmd *cobra.Command, args []string) error {
 			if err != nil {
 				return err
 			}
+
 			continue
 		}
+
 		break
 	}
 
 	return nil
 }
 
-// Get
+// Get.
 type cmdConfigGet struct {
 	global *cmdGlobal
 	config *cmdConfig
+
+	flagExpanded bool
 }
 
 func (c *cmdConfigGet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("get [<remote>:][<container>] <key>")
-	cmd.Short = i18n.G("Get values for container or server configuration keys")
+	cmd.Use = usage("get", i18n.G("[<remote>:][<instance>] <key>"))
+	cmd.Short = i18n.G("Get values for instance or server configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Get values for container or server configuration keys`))
+		`Get values for instance or server configuration keys`))
 
+	cmd.Flags().BoolVarP(&c.flagExpanded, "expanded", "e", false, i18n.G("Access the expanded configuration"))
+	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
 func (c *cmdConfigGet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 2)
 	if exit {
 		return err
@@ -316,12 +401,36 @@ func (c *cmdConfigGet) Run(cmd *cobra.Command, args []string) error {
 
 	// Get the config key
 	if resource.name != "" {
-		resp, _, err := resource.server.GetContainer(resource.name)
+		// Quick checks.
+		if c.config.flagTarget != "" {
+			return fmt.Errorf(i18n.G("--target cannot be used with instances"))
+		}
+
+		resp, _, err := resource.server.GetInstance(resource.name)
 		if err != nil {
 			return err
 		}
-		fmt.Println(resp.Config[args[len(args)-1]])
+
+		if c.flagExpanded {
+			fmt.Println(resp.ExpandedConfig[args[len(args)-1]])
+		} else {
+			fmt.Println(resp.Config[args[len(args)-1]])
+		}
 	} else {
+		// Quick check.
+		if c.flagExpanded {
+			return fmt.Errorf(i18n.G("--expanded cannot be used with a server"))
+		}
+
+		// Targeting
+		if c.config.flagTarget != "" {
+			if !resource.server.IsClustered() {
+				return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+			}
+
+			resource.server = resource.server.UseTarget(c.config.flagTarget)
+		}
+
 		resp, _, err := resource.server.GetServer()
 		if err != nil {
 			return err
@@ -342,7 +451,7 @@ func (c *cmdConfigGet) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Set
+// Set.
 type cmdConfigSet struct {
 	global *cmdGlobal
 	config *cmdConfig
@@ -350,35 +459,71 @@ type cmdConfigSet struct {
 
 func (c *cmdConfigSet) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("set [<remote>:][<container>] <key> <value>")
-	cmd.Short = i18n.G("Set container or server configuration keys")
+	cmd.Use = usage("set", i18n.G("[<remote>:][<instance>] <key>=<value>..."))
+	cmd.Short = i18n.G("Set instance or server configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Set container or server configuration keys`))
-	cmd.Example = cli.FormatSection("", i18n.G(
-		`lxc config set [<remote>:]<container> limits.cpu 2
-    Will set a CPU limit of "2" for the container.
+		`Set instance or server configuration keys
 
-lxc config set core.https_address [::]:8443
+For backward compatibility, a single configuration key may still be set with:
+    lxc config set [<remote>:][<instance>] <key> <value>`))
+	cmd.Example = cli.FormatSection("", i18n.G(
+		`lxc config set [<remote>:]<instance> limits.cpu=2
+    Will set a CPU limit of "2" for the instance.
+
+lxc config set core.https_address=[::]:8443
     Will have LXD listen on IPv4 and IPv6 port 8443.
 
-lxc config set core.trust_password blah
+lxc config set core.trust_password=blah
     Will set the server's trust password to blah.`))
 
+	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
 func (c *cmdConfigSet) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
-	exit, err := c.global.CheckArgs(cmd, args, 2, 3)
+	// Quick checks.
+	exit, err := c.global.CheckArgs(cmd, args, 1, -1)
 	if exit {
 		return err
 	}
 
+	hasKeyValue := func(args []string) bool {
+		for _, arg := range args {
+			if strings.Contains(arg, "=") {
+				return true
+			}
+		}
+
+		return false
+	}
+
+	onlyKeyValue := func(args []string) bool {
+		for _, arg := range args {
+			if !strings.Contains(arg, "=") {
+				return false
+			}
+		}
+
+		return true
+	}
+
+	isConfig := func(value string) bool {
+		fields := strings.SplitN(value, ":", 2)
+		key := fields[len(fields)-1]
+		return strings.Contains(key, ".")
+	}
+
 	// Parse remote
 	remote := ""
-	if len(args) > 2 {
+	if onlyKeyValue(args) || isConfig(args[0]) {
+		// server set with: <key>=<value>...
+		remote = ""
+	} else if len(args) == 2 && !hasKeyValue(args) {
+		// server set with: <key> <value>
+		remote = ""
+	} else {
 		remote = args[0]
 	}
 
@@ -389,36 +534,37 @@ func (c *cmdConfigSet) Run(cmd *cobra.Command, args []string) error {
 
 	resource := resources[0]
 
-	// Set the config key
+	// Set the config keys
 	if resource.name != "" {
-		key := args[len(args)-2]
-		value := args[len(args)-1]
-
-		if !termios.IsTerminal(int(syscall.Stdin)) && value == "-" {
-			buf, err := ioutil.ReadAll(os.Stdin)
-			if err != nil {
-				return fmt.Errorf(i18n.G("Can't read from stdin: %s"), err)
-			}
-			value = string(buf[:])
+		// Quick checks.
+		if c.config.flagTarget != "" {
+			return fmt.Errorf(i18n.G("--target cannot be used with instances"))
 		}
 
-		container, etag, err := resource.server.GetContainer(resource.name)
+		keys, err := getConfig(args[1:]...)
 		if err != nil {
 			return err
 		}
 
-		if cmd.Name() == "unset" {
-			_, ok := container.Config[key]
-			if !ok {
-				return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set"), key)
-			}
-
-			delete(container.Config, key)
-		} else {
-			container.Config[key] = value
+		inst, etag, err := resource.server.GetInstance(resource.name)
+		if err != nil {
+			return err
 		}
 
-		op, err := resource.server.UpdateContainer(resource.name, container.Writable(), etag)
+		for k, v := range keys {
+			if cmd.Name() == "unset" {
+				_, ok := inst.Config[k]
+				if !ok {
+					return fmt.Errorf(i18n.G("Can't unset key '%s', it's not currently set"), k)
+				}
+
+				delete(inst.Config, k)
+			} else {
+				inst.Config[k] = v
+			}
+		}
+
+		op, err := resource.server.UpdateInstance(resource.name, inst.Writable(), etag)
 		if err != nil {
 			return err
 		}
@@ -426,18 +572,42 @@ func (c *cmdConfigSet) Run(cmd *cobra.Command, args []string) error {
 		return op.Wait()
 	}
 
+	// Targeting
+	if c.config.flagTarget != "" {
+		if !resource.server.IsClustered() {
+			return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+		}
+
+		resource.server = resource.server.UseTarget(c.config.flagTarget)
+	}
+
 	// Server keys
 	server, etag, err := resource.server.GetServer()
 	if err != nil {
 		return err
 	}
 
-	server.Config[args[len(args)-2]] = args[len(args)-1]
+	var keys map[string]string
+	if remote == "" {
+		keys, err = getConfig(args[0:]...)
+		if err != nil {
+			return err
+		}
+	} else {
+		keys, err = getConfig(args[1:]...)
+		if err != nil {
+			return err
+		}
+	}
+
+	for k, v := range keys {
+		server.Config[k] = v
+	}
 
 	return resource.server.UpdateServer(server.Writable(), etag)
 }
 
-// Show
+// Show.
 type cmdConfigShow struct {
 	global *cmdGlobal
 	config *cmdConfig
@@ -447,19 +617,20 @@ type cmdConfigShow struct {
 
 func (c *cmdConfigShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("show [<remote>:][<container>]")
-	cmd.Short = i18n.G("Show container or server configurations")
+	cmd.Use = usage("show", i18n.G("[<remote>:][<instance>[/<snapshot>]]"))
+	cmd.Short = i18n.G("Show instance or server configurations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Show container or server configurations`))
+		`Show instance or server configurations`))
 
-	cmd.Flags().BoolVar(&c.flagExpanded, "expanded", false, i18n.G("Show the expanded configuration"))
+	cmd.Flags().BoolVarP(&c.flagExpanded, "expanded", "e", false, i18n.G("Show the expanded configuration"))
+	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
 func (c *cmdConfigShow) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 1)
 	if exit {
 		return err
@@ -482,6 +653,20 @@ func (c *cmdConfigShow) Run(cmd *cobra.Command, args []string) error {
 	var data []byte
 
 	if resource.name == "" {
+		// Quick check.
+		if c.flagExpanded {
+			return fmt.Errorf(i18n.G("--expanded cannot be used with a server"))
+		}
+
+		// Targeting
+		if c.config.flagTarget != "" {
+			if !resource.server.IsClustered() {
+				return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+			}
+
+			resource.server = resource.server.UseTarget(c.config.flagTarget)
+		}
+
 		// Server config
 		server, _, err := resource.server.GetServer()
 		if err != nil {
@@ -494,44 +679,41 @@ func (c *cmdConfigShow) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 	} else {
-		// Container config
-		var brief api.ContainerPut
+		// Quick checks.
+		if c.config.flagTarget != "" {
+			return fmt.Errorf(i18n.G("--target cannot be used with instances"))
+		}
+
+		// Instance or snapshot config
+		var brief any
 
 		if shared.IsSnapshot(resource.name) {
 			// Snapshot
 			fields := strings.Split(resource.name, shared.SnapshotDelimiter)
 
-			snap, _, err := resource.server.GetContainerSnapshot(fields[0], fields[1])
+			snap, _, err := resource.server.GetInstanceSnapshot(fields[0], fields[1])
 			if err != nil {
 				return err
 			}
 
-			brief = api.ContainerPut{
-				Profiles:  snap.Profiles,
-				Config:    snap.Config,
-				Devices:   snap.Devices,
-				Ephemeral: snap.Ephemeral,
-			}
-
+			brief = snap
 			if c.flagExpanded {
-				brief = api.ContainerPut{
-					Profiles:  snap.Profiles,
-					Config:    snap.ExpandedConfig,
-					Devices:   snap.ExpandedDevices,
-					Ephemeral: snap.Ephemeral,
-				}
+				brief.(*api.InstanceSnapshot).Config = snap.ExpandedConfig
+				brief.(*api.InstanceSnapshot).Devices = snap.ExpandedDevices
 			}
 		} else {
-			// Container
-			container, _, err := resource.server.GetContainer(resource.name)
+			// Instance
+			inst, _, err := resource.server.GetInstance(resource.name)
 			if err != nil {
 				return err
 			}
 
-			brief = container.Writable()
+			writable := inst.Writable()
+			brief = &writable
+
 			if c.flagExpanded {
-				brief.Config = container.ExpandedConfig
-				brief.Devices = container.ExpandedDevices
+				brief.(*api.InstancePut).Config = inst.ExpandedConfig
+				brief.(*api.InstancePut).Devices = inst.ExpandedDevices
 			}
 		}
 
@@ -546,7 +728,7 @@ func (c *cmdConfigShow) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// Unset
+// Unset.
 type cmdConfigUnset struct {
 	global    *cmdGlobal
 	config    *cmdConfig
@@ -555,18 +737,19 @@ type cmdConfigUnset struct {
 
 func (c *cmdConfigUnset) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("unset [<remote>:][<container>] <key>")
-	cmd.Short = i18n.G("Unset container or server configuration keys")
+	cmd.Use = usage("unset", i18n.G("[<remote>:][<instance>] <key>"))
+	cmd.Short = i18n.G("Unset instance or server configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Unset container or server configuration keys`))
+		`Unset instance or server configuration keys`))
 
+	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
 func (c *cmdConfigUnset) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 2)
 	if exit {
 		return err
