@@ -2,7 +2,6 @@ package config
 
 import (
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 
@@ -15,7 +14,7 @@ import (
 // not exist, it returns a default configuration.
 func LoadConfig(path string) (*Config, error) {
 	// Open the config file
-	content, err := ioutil.ReadFile(path)
+	content, err := os.ReadFile(path)
 	if err != nil {
 		return nil, fmt.Errorf("Unable to read the configuration file: %w", err)
 	}
@@ -36,7 +35,7 @@ func LoadConfig(path string) (*Config, error) {
 
 	// Apply the global (system-wide) remotes
 	globalConf := NewConfig("", false)
-	content, err = ioutil.ReadFile(globalConf.GlobalConfigPath("config.yml"))
+	content, err = os.ReadFile(globalConf.GlobalConfigPath("config.yml"))
 	if err == nil {
 		err = yaml.Unmarshal(content, &globalConf)
 		if err != nil {
@@ -72,7 +71,7 @@ func LoadConfig(path string) (*Config, error) {
 	if len(envDefaultRemote) > 0 {
 		c.DefaultRemote = envDefaultRemote
 	} else if c.DefaultRemote == "" {
-		c.DefaultRemote = DefaultConfig.DefaultRemote
+		c.DefaultRemote = DefaultConfig().DefaultRemote
 	}
 
 	// NOTE: Remove this once we only see a small fraction of non-simplestreams users
@@ -105,9 +104,11 @@ func (c *Config) SaveConfig(path string) error {
 		}
 	}
 
+	defaultRemote := DefaultConfig().DefaultRemote
+
 	// Remove the static remotes
 	for k := range StaticRemotes {
-		if k == DefaultConfig.DefaultRemote {
+		if k == defaultRemote {
 			continue
 		}
 
