@@ -4,24 +4,33 @@ import (
 	"crypto/x509"
 	"encoding/pem"
 	"fmt"
-	"io/ioutil"
+	"net/url"
+	"os"
+	"runtime"
 	"strings"
 
+	"github.com/go-macaroon-bakery/macaroon-bakery/v3/httpbakery"
+	"github.com/go-macaroon-bakery/macaroon-bakery/v3/httpbakery/form"
+	"github.com/juju/persistent-cookiejar"
+	schemaform "gopkg.in/juju/environschema.v1/form"
+
 	"github.com/lxc/lxd/client"
 	"github.com/lxc/lxd/shared"
 )
 
-// Remote holds details for communication with a remote daemon
+// Remote holds details for communication with a remote daemon.
 type Remote struct {
 	Addr     string `yaml:"addr"`
-	Public   bool   `yaml:"public"`
-	Protocol string `yaml:"protocol,omitempty"`
 	AuthType string `yaml:"auth_type,omitempty"`
-	Static   bool   `yaml:"-"`
+	Domain   string `yaml:"domain,omitempty"`
 	Project  string `yaml:"project,omitempty"`
+	Protocol string `yaml:"protocol,omitempty"`
+	Public   bool   `yaml:"public"`
+	Global   bool   `yaml:"-"`
+	Static   bool   `yaml:"-"`
 }
 
-// ParseRemote splits remote and object
+// ParseRemote splits remote and object.
 func (c *Config) ParseRemote(raw string) (string, string, error) {
 	result := strings.SplitN(raw, ":", 2)
 	if len(result) == 1 {
@@ -31,7 +40,7 @@ func (c *Config) ParseRemote(raw string) (string, string, error) {
 	_, ok := c.Remotes[result[0]]
 	if !ok {
 		// Attempt to play nice with snapshots containing ":"
-		if shared.IsSnapshot(raw) && strings.Contains(result[0], "/") {
+		if shared.IsSnapshot(raw) && shared.IsSnapshot(result[0]) {
 			return c.DefaultRemote, raw, nil
 		}
 
@@ -41,15 +50,20 @@ func (c *Config) ParseRemote(raw string) (string, string, error) {
 	return result[0], result[1], nil
 }
 
-// GetContainerServer returns a ContainerServer struct for the remote
-func (c *Config) GetContainerServer(name string) (lxd.ContainerServer, error) {
+// GetInstanceServer returns a InstanceServer struct for the remote.
+func (c *Config) GetInstanceServer(name string) (lxd.InstanceServer, error) {
+	// Handle "local" on non-Linux
+	if name == "local" && runtime.GOOS != "linux" {
+		return nil, ErrNotLinux
+	}
+
 	// Get the remote
 	remote, ok := c.Remotes[name]
 	if !ok {
 		return nil, fmt.Errorf("The remote \"%s\" doesn't exist", name)
 	}
 
-	// Sanity checks
+	// Quick checks.
 	if remote.Public || remote.Protocol == "simplestreams" {
 		return nil, fmt.Errorf("The remote isn't a private LXD server")
 	}
@@ -99,8 +113,13 @@ func (c *Config) GetContainerServer(name string) (lxd.ContainerServer, error) {
 	return d, nil
 }
 
-// GetImageServer returns a ImageServer struct for the remote
+// GetImageServer returns a ImageServer struct for the remote.
 func (c *Config) GetImageServer(name string) (lxd.ImageServer, error) {
+	// Handle "local" on non-Linux
+	if name == "local" && runtime.GOOS != "linux" {
+		return nil, ErrNotLinux
+	}
+
 	// Get the remote
 	remote, ok := c.Remotes[name]
 	if !ok {
@@ -169,15 +188,61 @@ func (c *Config) GetImageServer(name string) (lxd.ImageServer, error) {
 }
 
 func (c *Config) getConnectionArgs(name string) (*lxd.ConnectionArgs, error) {
-	remote, _ := c.Remotes[name]
+	remote := c.Remotes[name]
 	args := lxd.ConnectionArgs{
-		UserAgent:      c.UserAgent,
-		AuthType:       remote.AuthType,
-		AuthInteractor: c.authInteractor,
+		UserAgent: c.UserAgent,
+		AuthType:  remote.AuthType,
 	}
 
-	if c.cookiejar != nil {
-		args.CookieJar = c.cookiejar
+	if args.AuthType == "candid" {
+		args.AuthInteractor = []httpbakery.Interactor{
+			form.Interactor{Filler: schemaform.IOFiller{}},
+			httpbakery.WebBrowserInteractor{
+				OpenWebBrowser: func(uri *url.URL) error {
+					if remote.Domain != "" {
+						query := uri.Query()
+						query.Set("domain", remote.Domain)
+						uri.RawQuery = query.Encode()
+					}
+
+					return httpbakery.OpenWebBrowser(uri)
+				},
+			},
+		}
+
+		if c.cookieJars == nil || c.cookieJars[name] == nil {
+			if !shared.PathExists(c.ConfigPath("jars")) {
+				err := os.MkdirAll(c.ConfigPath("jars"), 0700)
+				if err != nil {
+					return nil, err
+				}
+			}
+
+			if !shared.PathExists(c.CookiesPath(name)) {
+				if shared.PathExists(c.ConfigPath("cookies")) {
+					err := shared.FileCopy(c.ConfigPath("cookies"), c.CookiesPath(name))
+					if err != nil {
+						return nil, err
+					}
+				}
+			}
+
+			jar, err := cookiejar.New(
+				&cookiejar.Options{
+					Filename: c.CookiesPath(name),
+				})
+			if err != nil {
+				return nil, err
+			}
+
+			if c.cookieJars == nil {
+				c.cookieJars = map[string]*cookiejar.Jar{}
+			}
+
+			c.cookieJars[name] = jar
+		}
+
+		args.CookieJar = c.cookieJars[name]
 	}
 
 	// Stop here if no TLS involved
@@ -187,7 +252,7 @@ func (c *Config) getConnectionArgs(name string) (*lxd.ConnectionArgs, error) {
 
 	// Server certificate
 	if shared.PathExists(c.ServerCertPath(name)) {
-		content, err := ioutil.ReadFile(c.ServerCertPath(name))
+		content, err := os.ReadFile(c.ServerCertPath(name))
 		if err != nil {
 			return nil, err
 		}
@@ -202,7 +267,7 @@ func (c *Config) getConnectionArgs(name string) (*lxd.ConnectionArgs, error) {
 
 	// Client certificate
 	if shared.PathExists(c.ConfigPath("client.crt")) {
-		content, err := ioutil.ReadFile(c.ConfigPath("client.crt"))
+		content, err := os.ReadFile(c.ConfigPath("client.crt"))
 		if err != nil {
 			return nil, err
 		}
@@ -212,7 +277,7 @@ func (c *Config) getConnectionArgs(name string) (*lxd.ConnectionArgs, error) {
 
 	// Client CA
 	if shared.PathExists(c.ConfigPath("client.ca")) {
-		content, err := ioutil.ReadFile(c.ConfigPath("client.ca"))
+		content, err := os.ReadFile(c.ConfigPath("client.ca"))
 		if err != nil {
 			return nil, err
 		}
@@ -222,13 +287,16 @@ func (c *Config) getConnectionArgs(name string) (*lxd.ConnectionArgs, error) {
 
 	// Client key
 	if shared.PathExists(c.ConfigPath("client.key")) {
-		content, err := ioutil.ReadFile(c.ConfigPath("client.key"))
+		content, err := os.ReadFile(c.ConfigPath("client.key"))
 		if err != nil {
 			return nil, err
 		}
 
 		pemKey, _ := pem.Decode(content)
-		if x509.IsEncryptedPEMBlock(pemKey) {
+		// Golang has deprecated all methods relating to PEM encryption due to a vulnerability.
+		// However, the weakness does not make PEM unsafe for our purposes as it pertains to password protection on the
+		// key file (client.key is only readable to the user in any case), so we'll ignore deprecation.
+		if x509.IsEncryptedPEMBlock(pemKey) { //nolint:staticcheck
 			if c.PromptPassword == nil {
 				return nil, fmt.Errorf("Private key is password protected and no helper was configured")
 			}
@@ -238,7 +306,7 @@ func (c *Config) getConnectionArgs(name string) (*lxd.ConnectionArgs, error) {
 				return nil, err
 			}
 
-			derKey, err := x509.DecryptPEMBlock(pemKey, []byte(password))
+			derKey, err := x509.DecryptPEMBlock(pemKey, []byte(password)) //nolint:staticcheck
 			if err != nil {
 				return nil, err
 			}
