@@ -1,25 +1,21 @@
-// +build !windows
+//go:build !windows
 
 package main
 
 import (
-	"io"
 	"os"
+	"os/exec"
 	"os/signal"
-	"syscall"
 
 	"github.com/gorilla/websocket"
+	"golang.org/x/sys/unix"
 
 	"github.com/lxc/lxd/shared/logger"
 )
 
-func (c *cmdConsole) getStdout() io.WriteCloser {
-	return os.Stdout
-}
-
 func (c *cmdConsole) controlSocketHandler(control *websocket.Conn) {
 	ch := make(chan os.Signal, 10)
-	signal.Notify(ch, syscall.SIGWINCH)
+	signal.Notify(ch, unix.SIGWINCH)
 
 	for {
 		sig := <-ch
@@ -32,5 +28,10 @@ func (c *cmdConsole) controlSocketHandler(control *websocket.Conn) {
 	}
 
 	closeMsg := websocket.FormatCloseMessage(websocket.CloseNormalClosure, "")
-	control.WriteMessage(websocket.CloseMessage, closeMsg)
+	_ = control.WriteMessage(websocket.CloseMessage, closeMsg)
+}
+
+func (c *cmdConsole) findCommand(name string) string {
+	path, _ := exec.LookPath(name)
+	return path
 }
