@@ -2,14 +2,11 @@ package main
 
 import (
 	"bytes"
-	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"strconv"
 	"strings"
-	"syscall"
 
 	"github.com/gorilla/websocket"
 	"github.com/spf13/cobra"
@@ -30,21 +27,26 @@ type cmdExec struct {
 	flagForceInteractive    bool
 	flagForceNonInteractive bool
 	flagDisableStdin        bool
+	flagUser                uint32
+	flagGroup               uint32
+	flagCwd                 string
+
+	interactive bool
 }
 
 func (c *cmdExec) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("exec [<remote>:]<container> [flags] [--] <command line>")
-	cmd.Short = i18n.G("Execute commands in containers")
+	cmd.Use = usage("exec", i18n.G("[<remote>:]<instance> [flags] [--] <command line>"))
+	cmd.Short = i18n.G("Execute commands in instances")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Execute commands in containers
+		`Execute commands in instances
 
 The command is executed directly using exec, so there is no shell and
 shell patterns (variables, file redirects, ...) won't be understood.
 If you need a shell environment you need to execute the shell
 executable, passing the shell commands as arguments, for example:
 
-  lxc exec <container> -- sh -c "cd /tmp && pwd"
+  lxc exec <instance> -- sh -c "cd /tmp && pwd"
 
 Mode defaults to non-interactive, interactive mode is selected if both stdin AND stdout are terminals (stderr is ignored).`))
 
@@ -54,65 +56,34 @@ Mode defaults to non-interactive, interactive mode is selected if both stdin AND
 	cmd.Flags().BoolVarP(&c.flagForceInteractive, "force-interactive", "t", false, i18n.G("Force pseudo-terminal allocation"))
 	cmd.Flags().BoolVarP(&c.flagForceNonInteractive, "force-noninteractive", "T", false, i18n.G("Disable pseudo-terminal allocation"))
 	cmd.Flags().BoolVarP(&c.flagDisableStdin, "disable-stdin", "n", false, i18n.G("Disable stdin (reads from /dev/null)"))
+	cmd.Flags().Uint32Var(&c.flagUser, "user", 0, i18n.G("User ID to run the command as (default 0)")+"``")
+	cmd.Flags().Uint32Var(&c.flagGroup, "group", 0, i18n.G("Group ID to run the command as (default 0)")+"``")
+	cmd.Flags().StringVar(&c.flagCwd, "cwd", "", i18n.G("Directory to run the command in (default /root)")+"``")
 
 	return cmd
 }
 
 func (c *cmdExec) sendTermSize(control *websocket.Conn) error {
-	width, height, err := termios.GetSize(int(syscall.Stdout))
+	width, height, err := termios.GetSize(getStdoutFd())
 	if err != nil {
 		return err
 	}
 
 	logger.Debugf("Window size is now: %dx%d", width, height)
 
-	w, err := control.NextWriter(websocket.TextMessage)
-	if err != nil {
-		return err
-	}
-
-	msg := api.ContainerExecControl{}
+	msg := api.InstanceExecControl{}
 	msg.Command = "window-resize"
 	msg.Args = make(map[string]string)
 	msg.Args["width"] = strconv.Itoa(width)
 	msg.Args["height"] = strconv.Itoa(height)
 
-	buf, err := json.Marshal(msg)
-	if err != nil {
-		return err
-	}
-	_, err = w.Write(buf)
-
-	w.Close()
-	return err
-}
-
-func (c *cmdExec) forwardSignal(control *websocket.Conn, sig syscall.Signal) error {
-	logger.Debugf("Forwarding signal: %s", sig)
-
-	w, err := control.NextWriter(websocket.TextMessage)
-	if err != nil {
-		return err
-	}
-
-	msg := api.ContainerExecControl{}
-	msg.Command = "signal"
-	msg.Signal = int(sig)
-
-	buf, err := json.Marshal(msg)
-	if err != nil {
-		return err
-	}
-	_, err = w.Write(buf)
-
-	w.Close()
-	return err
+	return control.WriteJSON(msg)
 }
 
 func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
 		return err
@@ -132,7 +103,7 @@ func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	d, err := conf.GetContainerServer(remote)
+	d, err := conf.GetInstanceServer(remote)
 	if err != nil {
 		return err
 	}
@@ -150,40 +121,46 @@ func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 		if len(pieces) > 1 {
 			value = pieces[1]
 		}
+
 		env[pieces[0]] = value
 	}
 
 	// Configure the terminal
-	cfd := int(syscall.Stdin)
+	stdinFd := getStdinFd()
+	stdoutFd := getStdoutFd()
 
-	var interactive bool
+	stdinTerminal := termios.IsTerminal(stdinFd)
+	stdoutTerminal := termios.IsTerminal(stdoutFd)
+
+	// Determine interaction mode
 	if c.flagDisableStdin {
-		interactive = false
+		c.interactive = false
 	} else if c.flagMode == "interactive" || c.flagForceInteractive {
-		interactive = true
+		c.interactive = true
 	} else if c.flagMode == "non-interactive" || c.flagForceNonInteractive {
-		interactive = false
+		c.interactive = false
 	} else {
-		interactive = termios.IsTerminal(cfd) && termios.IsTerminal(int(syscall.Stdout))
+		c.interactive = stdinTerminal && stdoutTerminal
 	}
 
+	// Record terminal state
 	var oldttystate *termios.State
-	if interactive {
-		oldttystate, err = termios.MakeRaw(cfd)
+	if c.interactive && stdinTerminal {
+		oldttystate, err = termios.MakeRaw(stdinFd)
 		if err != nil {
 			return err
 		}
-		defer termios.Restore(cfd, oldttystate)
+
+		defer func() { _ = termios.Restore(stdinFd, oldttystate) }()
 	}
 
+	// Setup interactive console handler
 	handler := c.controlSocketHandler
-	if !interactive {
-		handler = nil
-	}
 
+	// Grab current terminal dimensions
 	var width, height int
-	if interactive {
-		width, height, err = termios.GetSize(int(syscall.Stdout))
+	if stdoutTerminal {
+		width, height, err = termios.GetSize(getStdoutFd())
 		if err != nil {
 			return err
 		}
@@ -192,22 +169,25 @@ func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 	var stdin io.ReadCloser
 	stdin = os.Stdin
 	if c.flagDisableStdin {
-		stdin = ioutil.NopCloser(bytes.NewReader(nil))
+		stdin = io.NopCloser(bytes.NewReader(nil))
 	}
 
-	stdout := c.getStdout()
+	stdout := getStdout()
 
 	// Prepare the command
-	req := api.ContainerExecPost{
+	req := api.InstanceExecPost{
 		Command:     args[1:],
 		WaitForWS:   true,
-		Interactive: interactive,
+		Interactive: c.interactive,
 		Environment: env,
 		Width:       width,
 		Height:      height,
+		User:        c.flagUser,
+		Group:       c.flagGroup,
+		Cwd:         c.flagCwd,
 	}
 
-	execArgs := lxd.ContainerExecArgs{
+	execArgs := lxd.InstanceExecArgs{
 		Stdin:    stdin,
 		Stdout:   stdout,
 		Stderr:   os.Stderr,
@@ -215,33 +195,28 @@ func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 		DataDone: make(chan bool),
 	}
 
-	// Run the command in the container
-	op, err := d.ExecContainer(name, req, &execArgs)
+	// Run the command in the instance
+	op, err := d.ExecInstance(name, req, &execArgs)
 	if err != nil {
 		return err
 	}
 
 	// Wait for the operation to complete
 	err = op.Wait()
+	opAPI := op.Get()
+	if opAPI.Metadata != nil {
+		exitStatusRaw, ok := opAPI.Metadata["return"].(float64)
+		if ok {
+			c.global.ret = int(exitStatusRaw)
+		}
+	}
+
 	if err != nil {
 		return err
 	}
-	opAPI := op.Get()
 
 	// Wait for any remaining I/O to be flushed
 	<-execArgs.DataDone
 
-	if oldttystate != nil {
-		/* A bit of a special case here: we want to exit with the same code as
-		 * the process inside the container, so we explicitly exit here
-		 * instead of returning an error.
-		 *
-		 * Additionally, since os.Exit() exits without running deferred
-		 * functions, we restore the terminal explicitly.
-		 */
-		termios.Restore(cfd, oldttystate)
-	}
-
-	os.Exit(int(opAPI.Metadata["return"].(float64)))
 	return nil
 }
