@@ -1,4 +1,4 @@
-// +build windows
+//go:build windows
 
 package main
 
@@ -6,11 +6,11 @@ import (
 	"io"
 	"os"
 	"os/signal"
-	"syscall"
 
 	"github.com/gorilla/websocket"
-	"github.com/mattn/go-colorable"
+	"golang.org/x/sys/windows"
 
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/logger"
 )
 
@@ -25,10 +25,6 @@ func (wwc *WrappedWriteCloser) Write(p []byte) (int, error) {
 	return wwc.wrapper.Write(p)
 }
 
-func (c *cmdExec) getStdout() io.WriteCloser {
-	return &WrappedWriteCloser{os.Stdout, colorable.NewColorableStdout()}
-}
-
 func (c *cmdExec) getTERM() (string, bool) {
 	return "dumb", true
 }
@@ -45,13 +41,24 @@ func (c *cmdExec) controlSocketHandler(control *websocket.Conn) {
 		switch sig {
 		case os.Interrupt:
 			logger.Debugf("Received '%s signal', forwarding to executing program.", sig)
-			err := c.forwardSignal(control, syscall.SIGINT)
+			err := c.forwardSignal(control, windows.SIGINT)
 			if err != nil {
-				logger.Debugf("Failed to forward signal '%s'.", syscall.SIGINT)
+				logger.Debugf("Failed to forward signal '%s'.", windows.SIGINT)
 				return
 			}
+
 		default:
 			break
 		}
 	}
 }
+
+func (c *cmdExec) forwardSignal(control *websocket.Conn, sig windows.Signal) error {
+	logger.Debugf("Forwarding signal: %s", sig)
+
+	msg := api.InstanceExecControl{}
+	msg.Command = "signal"
+	msg.Signal = int(sig)
+
+	return control.WriteJSON(msg)
+}
