@@ -2,7 +2,7 @@ package main
 
 import (
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 	"strings"
 
@@ -22,6 +22,7 @@ type cmdInit struct {
 	global *cmdGlobal
 
 	flagConfig     []string
+	flagDevice     []string
 	flagEphemeral  bool
 	flagNetwork    string
 	flagProfile    []string
@@ -35,7 +36,7 @@ type cmdInit struct {
 
 func (c *cmdInit) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = usage("init", i18n.G("[[<remote>:]<image>] [<remote>:][<name>] [< config"))
+	cmd.Use = usage("init", i18n.G("[<remote>:]<image> [<remote>:][<name>]"))
 	cmd.Short = i18n.G("Create instances from images")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Create instances from images`))
 	cmd.Example = cli.FormatSection("", i18n.G(`lxc init ubuntu:22.04 u1
@@ -47,6 +48,7 @@ lxc init ubuntu:22.04 u1 < config.yaml
 	cmd.RunE = c.Run
 	cmd.Flags().StringArrayVarP(&c.flagConfig, "config", "c", nil, i18n.G("Config key/value to apply to the new instance")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagProfile, "profile", "p", nil, i18n.G("Profile to apply to the new instance")+"``")
+	cmd.Flags().StringArrayVarP(&c.flagDevice, "device", "d", nil, i18n.G("New key/value to apply to a specific device")+"``")
 	cmd.Flags().BoolVarP(&c.flagEphemeral, "ephemeral", "e", false, i18n.G("Ephemeral instance"))
 	cmd.Flags().StringVarP(&c.flagNetwork, "network", "n", "", i18n.G("Network name")+"``")
 	cmd.Flags().StringVarP(&c.flagStorage, "storage", "s", "", i18n.G("Storage pool name")+"``")
@@ -88,7 +90,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 	// If stdin isn't a terminal, read text from it
 	if !termios.IsTerminal(getStdinFd()) {
-		contents, err := ioutil.ReadAll(os.Stdin)
+		contents, err := io.ReadAll(os.Stdin)
 		if err != nil {
 			return nil, "", err
 		}
@@ -146,7 +148,12 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 		d = d.UseTarget(c.flagTarget)
 	}
 
-	profiles = append(profiles, c.flagProfile...)
+	// Overwrite profiles.
+	if c.flagProfile != nil {
+		profiles = c.flagProfile
+	} else if c.flagNoProfiles {
+		profiles = []string{}
+	}
 
 	if !c.global.flagQuiet {
 		if name == "" {
@@ -240,7 +247,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	}
 
 	req.Config = configMap
-	req.Devices = devicesMap
+	req.Ephemeral = c.flagEphemeral
 
 	if !c.flagNoProfiles && len(profiles) == 0 {
 		if len(stdinData.Profiles) > 0 {
@@ -252,7 +259,71 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 		req.Profiles = profiles
 	}
 
-	req.Ephemeral = c.flagEphemeral
+	// Handle device overrides.
+	deviceOverrides, err := parseDeviceOverrides(c.flagDevice)
+	if err != nil {
+		return nil, "", err
+	}
+
+	// Check to see if any of the overridden devices are for devices that are not yet defined in the
+	// local devices (and thus maybe expected to be coming from profiles).
+	profileDevices := make(map[string]map[string]string)
+	needProfileExpansion := false
+	for deviceName := range deviceOverrides {
+		_, isLocalDevice := devicesMap[deviceName]
+		if !isLocalDevice {
+			needProfileExpansion = true
+			break
+		}
+	}
+
+	// If there are device overrides that are expected to be applied to profile devices then load the profiles
+	// that would be applied server-side.
+	if needProfileExpansion {
+		// If the list of profiles is empty then LXD would apply the default profile on the server side.
+		serverSideProfiles := req.Profiles
+		if len(serverSideProfiles) == 0 {
+			serverSideProfiles = []string{"default"}
+		}
+
+		// Get the effective expanded devices by overlaying each profile's devices in order.
+		for _, profileName := range serverSideProfiles {
+			profile, _, err := d.GetProfile(profileName)
+			if err != nil {
+				return nil, "", fmt.Errorf(i18n.G("Failed loading profile %q for device override: %w"), profileName, err)
+			}
+
+			for k, v := range profile.Devices {
+				profileDevices[k] = v
+			}
+		}
+	}
+
+	// Apply device overrides.
+	for deviceName := range deviceOverrides {
+		_, isLocalDevice := devicesMap[deviceName]
+		if isLocalDevice {
+			// Apply overrides to local device.
+			for k, v := range deviceOverrides[deviceName] {
+				devicesMap[deviceName][k] = v
+			}
+		} else {
+			// Check device exists in expanded profile devices.
+			profileDeviceConfig, found := profileDevices[deviceName]
+			if !found {
+				return nil, "", fmt.Errorf(i18n.G("Cannot override config for device %q: Device not found in profile devices"), deviceName)
+			}
+
+			for k, v := range deviceOverrides[deviceName] {
+				profileDeviceConfig[k] = v
+			}
+
+			// Add device to local devices.
+			devicesMap[deviceName] = profileDeviceConfig
+		}
+	}
+
+	req.Devices = devicesMap
 
 	var opInfo api.Operation
 	if !c.flagEmpty {
