@@ -8,8 +8,6 @@ import (
 	"path/filepath"
 
 	"github.com/spf13/cobra"
-	"gopkg.in/macaroon-bakery.v2/httpbakery"
-	"gopkg.in/macaroon-bakery.v2/httpbakery/form"
 
 	"github.com/lxc/lxd/client"
 	"github.com/lxc/lxd/lxc/config"
@@ -17,28 +15,62 @@ import (
 	cli "github.com/lxc/lxd/shared/cmd"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/logging"
 	"github.com/lxc/lxd/shared/version"
-
-	schemaform "gopkg.in/juju/environschema.v1/form"
 )
 
 type cmdGlobal struct {
 	conf     *config.Config
 	confPath string
 	cmd      *cobra.Command
+	ret      int
 
 	flagForceLocal bool
 	flagHelp       bool
 	flagHelpAll    bool
 	flagLogDebug   bool
 	flagLogVerbose bool
+	flagProject    string
+	flagQuiet      bool
 	flagVersion    bool
+	flagSubCmds    bool
+}
+
+func usageTemplateSubCmds() string {
+	return `Usage:{{if .Runnable}}
+  {{.UseLine}}{{end}}{{if .HasAvailableSubCommands}}
+  {{.CommandPath}} [command]{{end}}{{if gt (len .Aliases) 0}}
+Aliases:
+  {{.NameAndAliases}}{{end}}{{if .HasExample}}
+
+Examples:
+  {{.Example}}{{end}}{{if .HasAvailableSubCommands}}
+
+Available Commands:{{range .Commands}}{{if (or .IsAvailableCommand (eq .Name "help"))}}
+  {{rpad .Name .NamePadding }}  {{.Short}}{{if .HasSubCommands}}{{range .Commands}}{{if (or .IsAvailableCommand (eq .Name "help"))}}
+    {{rpad .Name .NamePadding }}  {{.Short}}{{if .HasSubCommands}}{{range .Commands}}{{if (or .IsAvailableCommand (eq .Name "help"))}}
+      {{rpad .Name .NamePadding }}  {{.Short}}{{if .HasSubCommands}}{{range .Commands}}{{if (or .IsAvailableCommand (eq .Name "help"))}}
+        {{rpad .Name .NamePadding }}  {{.Short}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{end}}{{if .HasAvailableLocalFlags}}
+
+Flags:
+{{.LocalFlags.FlagUsages | trimTrailingWhitespaces}}{{end}}{{if .HasAvailableInheritedFlags}}
+
+Global Flags:
+  {{.InheritedFlags.FlagUsages | trimTrailingWhitespaces}}{{end}}{{if .HasHelpSubCommands}}
+
+Additional help topics:{{range .Commands}}{{if .IsAdditionalHelpTopicCommand}}
+  {{rpad .CommandPath .CommandPathPadding}} {{.Short}}{{end}}{{end}}{{end}}{{if .HasAvailableSubCommands}}
+
+Use "{{.CommandPath}} [command] --help" for more information about a command.{{end}}
+`
 }
 
 func main() {
 	// Process aliases
-	execIfAliases()
+	err := execIfAliases()
+	if err != nil {
+		fmt.Fprintf(os.Stderr, "Error: %v\n", err)
+		os.Exit(1)
+	}
 
 	// Setup the parser
 	app := &cobra.Command{}
@@ -50,14 +82,19 @@ func main() {
 All of LXD's features can be driven through the various commands below.
 For help with any of those, simply call them with --help.`))
 	app.SilenceUsage = true
+	app.SilenceErrors = true
+	app.CompletionOptions = cobra.CompletionOptions{DisableDefaultCmd: true}
 
 	// Global flags
 	globalCmd := cmdGlobal{cmd: app}
 	app.PersistentFlags().BoolVar(&globalCmd.flagVersion, "version", false, i18n.G("Print version number"))
 	app.PersistentFlags().BoolVarP(&globalCmd.flagHelp, "help", "h", false, i18n.G("Print help"))
 	app.PersistentFlags().BoolVar(&globalCmd.flagForceLocal, "force-local", false, i18n.G("Force using the local unix socket"))
+	app.PersistentFlags().StringVar(&globalCmd.flagProject, "project", "", i18n.G("Override the source project")+"``")
 	app.PersistentFlags().BoolVar(&globalCmd.flagLogDebug, "debug", false, i18n.G("Show all debug messages"))
 	app.PersistentFlags().BoolVarP(&globalCmd.flagLogVerbose, "verbose", "v", false, i18n.G("Show all information messages"))
+	app.PersistentFlags().BoolVarP(&globalCmd.flagQuiet, "quiet", "q", false, i18n.G("Don't show progress information"))
+	app.PersistentFlags().BoolVar(&globalCmd.flagSubCmds, "sub-commands", false, i18n.G("Use with help or --help to view sub-commands"))
 
 	// Wrappers
 	app.PersistentPreRunE = globalCmd.PreRun
@@ -95,10 +132,18 @@ For help with any of those, simply call them with --help.`))
 	execCmd := cmdExec{global: &globalCmd}
 	app.AddCommand(execCmd.Command())
 
+	// export sub-command
+	exportCmd := cmdExport{global: &globalCmd}
+	app.AddCommand(exportCmd.Command())
+
 	// file sub-command
 	fileCmd := cmdFile{global: &globalCmd}
 	app.AddCommand(fileCmd.Command())
 
+	// import sub-command
+	importCmd := cmdImport{global: &globalCmd}
+	app.AddCommand(importCmd.Command())
+
 	// info sub-command
 	infoCmd := cmdInfo{global: &globalCmd}
 	app.AddCommand(infoCmd.Command())
@@ -151,6 +196,10 @@ For help with any of those, simply call them with --help.`))
 	profileCmd := cmdProfile{global: &globalCmd}
 	app.AddCommand(profileCmd.Command())
 
+	// profile sub-command
+	projectCmd := cmdProject{global: &globalCmd}
+	app.AddCommand(projectCmd.Command())
+
 	// query sub-command
 	queryCmd := cmdQuery{global: &globalCmd}
 	app.AddCommand(queryCmd.Command())
@@ -191,6 +240,10 @@ For help with any of those, simply call them with --help.`))
 	versionCmd := cmdVersion{global: &globalCmd}
 	app.AddCommand(versionCmd.Command())
 
+	// warning sub-command
+	warningCmd := cmdWarning{global: &globalCmd}
+	app.AddCommand(warningCmd.Command())
+
 	// Get help command
 	app.InitDefaultHelpCmd()
 	var help *cobra.Command
@@ -205,8 +258,8 @@ For help with any of those, simply call them with --help.`))
 	app.Flags().BoolVar(&globalCmd.flagHelpAll, "all", false, i18n.G("Show less common commands"))
 	help.Flags().BoolVar(&globalCmd.flagHelpAll, "all", false, i18n.G("Show less common commands"))
 
-	// Deal with --all flag
-	err := app.ParseFlags(os.Args[1:])
+	// Deal with --all flag and --sub-commands flag
+	err = app.ParseFlags(os.Args[1:])
 	if err == nil {
 		if globalCmd.flagHelpAll {
 			// Show all commands
@@ -214,12 +267,35 @@ For help with any of those, simply call them with --help.`))
 				cmd.Hidden = false
 			}
 		}
+		if globalCmd.flagSubCmds {
+			app.SetUsageTemplate(usageTemplateSubCmds())
+		}
 	}
 
 	// Run the main command and handle errors
 	err = app.Execute()
 	if err != nil {
-		os.Exit(1)
+		// Handle non-Linux systems
+		if err == config.ErrNotLinux {
+			fmt.Fprintf(os.Stderr, i18n.G(`This client hasn't been configured to use a remote LXD server yet.
+As your platform can't run native Linux instances, you must connect to a remote LXD server.
+
+If you already added a remote server, make it the default with "lxc remote switch NAME".
+To easily setup a local LXD server in a virtual machine, consider using: https://multipass.run`)+"\n")
+			os.Exit(1)
+		}
+
+		// Default error handling
+		fmt.Fprintf(os.Stderr, "Error: %v\n", err)
+
+		// If custom exit status not set, use default error status.
+		if globalCmd.ret == 0 {
+			globalCmd.ret = 1
+		}
+	}
+
+	if globalCmd.ret != 0 {
+		os.Exit(globalCmd.ret)
 	}
 }
 
@@ -260,6 +336,11 @@ func (c *cmdGlobal) PreRun(cmd *cobra.Command, args []string) error {
 		c.conf = config.NewConfig(filepath.Dir(c.confPath), true)
 	}
 
+	// Override the project
+	if c.flagProject != "" {
+		c.conf.ProjectOverride = c.flagProject
+	}
+
 	// Setup password helper
 	c.conf.PromptPassword = func(filename string) (string, error) {
 		return cli.AskPasswordOnce(fmt.Sprintf(i18n.G("Password for %s: "), filename)), nil
@@ -276,43 +357,55 @@ func (c *cmdGlobal) PreRun(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
+		// Attempt to connect to the local server
+		runInit := true
+		d, err := lxd.ConnectLXDUnix("", nil)
+		if err == nil {
+			// Check if server is initialized.
+			info, _, err := d.GetServer()
+			if err == nil && info.Environment.Storage != "" {
+				runInit = false
+			}
+
+			// Detect usable project.
+			names, err := d.GetProjectNames()
+			if err == nil {
+				if len(names) == 1 && names[0] != "default" {
+					remote := c.conf.Remotes["local"]
+					remote.Project = names[0]
+					c.conf.Remotes["local"] = remote
+				}
+			}
+		}
+
+		flush := false
+		if runInit {
+			fmt.Fprintf(os.Stderr, i18n.G("If this is your first time running LXD on this machine, you should also run: lxd init")+"\n")
+			flush = true
+		}
+
+		if !shared.StringInSlice(cmd.Name(), []string{"init", "launch"}) {
+			fmt.Fprintf(os.Stderr, i18n.G(`To start your first container, try: lxc launch ubuntu:22.04
+Or for a virtual machine: lxc launch ubuntu:22.04 --vm`)+"\n")
+			flush = true
+		}
+
+		if flush {
+			fmt.Fprintf(os.Stderr, "\n")
+		}
+
 		// And save the initial configuration
 		err = c.conf.SaveConfig(c.confPath)
 		if err != nil {
 			return err
 		}
-
-		// Attempt to connect to the local server
-		runInit := true
-		d, err := lxd.ConnectLXDUnix("", nil)
-		if err == nil {
-			info, _, err := d.GetServer()
-			if err == nil && info.Environment.Storage != "" {
-				runInit = false
-			}
-		}
-
-		if runInit {
-			fmt.Fprintf(os.Stderr, i18n.G("If this is your first time running LXD on this machine, you should also run: lxd init")+"\n")
-		}
-
-		fmt.Fprintf(os.Stderr, i18n.G("To start your first container, try: lxc launch ubuntu:18.04")+"\n\n")
-	}
-
-	// Only setup macaroons if a config path exists (so the jar can be saved)
-	if shared.PathExists(c.confPath) {
-		// Add interactor for external authentication
-		c.conf.SetAuthInteractor([]httpbakery.Interactor{
-			form.Interactor{Filler: schemaform.IOFiller{}},
-			httpbakery.WebBrowserInteractor{},
-		})
 	}
 
 	// Set the user agent
 	c.conf.UserAgent = version.UserAgent
 
 	// Setup the logger
-	logger.Log, err = logging.GetLogger("", "", c.flagLogVerbose, c.flagLogDebug, nil)
+	err = logger.InitLogger("", "", c.flagLogVerbose, c.flagLogDebug, nil)
 	if err != nil {
 		return err
 	}
@@ -331,12 +424,13 @@ func (c *cmdGlobal) PostRun(cmd *cobra.Command, args []string) error {
 }
 
 type remoteResource struct {
-	server lxd.ContainerServer
+	remote string
+	server lxd.InstanceServer
 	name   string
 }
 
 func (c *cmdGlobal) ParseServers(remotes ...string) ([]remoteResource, error) {
-	servers := map[string]lxd.ContainerServer{}
+	servers := map[string]lxd.InstanceServer{}
 	resources := []remoteResource{}
 
 	for _, remote := range remotes {
@@ -348,7 +442,8 @@ func (c *cmdGlobal) ParseServers(remotes ...string) ([]remoteResource, error) {
 
 		// Setup the struct
 		resource := remoteResource{
-			name: name,
+			remote: remoteName,
+			name:   name,
 		}
 
 		// Look at our cache
@@ -360,7 +455,7 @@ func (c *cmdGlobal) ParseServers(remotes ...string) ([]remoteResource, error) {
 		}
 
 		// New connection
-		d, err := c.conf.GetContainerServer(remoteName)
+		d, err := c.conf.GetInstanceServer(remoteName)
 		if err != nil {
 			return nil, err
 		}
@@ -375,7 +470,7 @@ func (c *cmdGlobal) ParseServers(remotes ...string) ([]remoteResource, error) {
 
 func (c *cmdGlobal) CheckArgs(cmd *cobra.Command, args []string, minArgs int, maxArgs int) (bool, error) {
 	if len(args) < minArgs || (maxArgs != -1 && len(args) > maxArgs) {
-		cmd.Help()
+		_ = cmd.Help()
 
 		if len(args) == 0 {
 			return true, nil
