@@ -2,14 +2,13 @@ package main
 
 import (
 	"fmt"
-	"os"
 	"sort"
 	"strings"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
+	"github.com/lxc/lxd/lxc/utils"
 	cli "github.com/lxc/lxd/shared/cmd"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -20,7 +19,7 @@ type cmdOperation struct {
 
 func (c *cmdOperation) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("operation")
+	cmd.Use = usage("operation")
 	cmd.Short = i18n.G("List, show and delete background operations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List, show and delete background operations`))
@@ -37,10 +36,13 @@ func (c *cmdOperation) Command() *cobra.Command {
 	operationShowCmd := cmdOperationShow{global: c.global, operation: c}
 	cmd.AddCommand(operationShowCmd.Command())
 
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Delete
+// Delete.
 type cmdOperationDelete struct {
 	global    *cmdGlobal
 	operation *cmdOperation
@@ -48,7 +50,7 @@ type cmdOperationDelete struct {
 
 func (c *cmdOperationDelete) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("delete [<remote>:]<operation>")
+	cmd.Use = usage("delete", i18n.G("[<remote>:]<operation>"))
 	cmd.Aliases = []string{"cancel", "rm"}
 	cmd.Short = i18n.G("Delete a background operation (will attempt to cancel)")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -60,7 +62,7 @@ func (c *cmdOperationDelete) Command() *cobra.Command {
 }
 
 func (c *cmdOperationDelete) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -80,23 +82,29 @@ func (c *cmdOperationDelete) Run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	fmt.Printf(i18n.G("Operation %s deleted")+"\n", resource.name)
+	if !c.global.flagQuiet {
+		fmt.Printf(i18n.G("Operation %s deleted")+"\n", resource.name)
+	}
+
 	return nil
 }
 
-// List
+// List.
 type cmdOperationList struct {
 	global    *cmdGlobal
 	operation *cmdOperation
+
+	flagFormat string
 }
 
 func (c *cmdOperationList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list [<remote>:]")
+	cmd.Use = usage("list", i18n.G("[<remote>:]"))
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List background operations")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List background operations`))
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	cmd.RunE = c.Run
 
@@ -104,7 +112,7 @@ func (c *cmdOperationList) Command() *cobra.Command {
 }
 
 func (c *cmdOperationList) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 1)
 	if exit {
 		return err
@@ -140,28 +148,31 @@ func (c *cmdOperationList) Run(cmd *cobra.Command, args []string) error {
 			cancelable = i18n.G("YES")
 		}
 
-		data = append(data, []string{op.ID, strings.ToUpper(op.Class), op.Description, strings.ToUpper(op.Status), cancelable, op.CreatedAt.UTC().Format("2006/01/02 15:04 UTC")})
+		entry := []string{op.ID, strings.ToUpper(op.Class), op.Description, strings.ToUpper(op.Status), cancelable, op.CreatedAt.UTC().Format("2006/01/02 15:04 UTC")}
+		if resource.server.IsClustered() {
+			entry = append(entry, op.Location)
+		}
+
+		data = append(data, entry)
 	}
 
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAutoWrapText(false)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetRowLine(true)
-	table.SetHeader([]string{
+	sort.Sort(utils.ByName(data))
+
+	header := []string{
 		i18n.G("ID"),
 		i18n.G("TYPE"),
 		i18n.G("DESCRIPTION"),
 		i18n.G("STATUS"),
 		i18n.G("CANCELABLE"),
-		i18n.G("CREATED")})
-	sort.Sort(byName(data))
-	table.AppendBulk(data)
-	table.Render()
+		i18n.G("CREATED")}
+	if resource.server.IsClustered() {
+		header = append(header, i18n.G("LOCATION"))
+	}
 
-	return nil
+	return utils.RenderTable(c.flagFormat, header, data, operations)
 }
 
-// Show
+// Show.
 type cmdOperationShow struct {
 	global    *cmdGlobal
 	operation *cmdOperation
@@ -169,7 +180,7 @@ type cmdOperationShow struct {
 
 func (c *cmdOperationShow) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("show [<remote>:]<operation>")
+	cmd.Use = usage("show", i18n.G("[<remote>:]<operation>"))
 	cmd.Short = i18n.G("Show details on a background operation")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show details on a background operation`))
@@ -183,7 +194,7 @@ func (c *cmdOperationShow) Command() *cobra.Command {
 }
 
 func (c *cmdOperationShow) Run(cmd *cobra.Command, args []string) error {
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
