@@ -3,15 +3,16 @@ package main
 import (
 	"fmt"
 	"strings"
+	"time"
 
 	"github.com/spf13/cobra"
 
 	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxc/utils"
+	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	cli "github.com/lxc/lxd/shared/cmd"
 	"github.com/lxc/lxd/shared/i18n"
-
-	"github.com/lxc/lxd/shared"
 )
 
 type cmdPublish struct {
@@ -19,26 +20,24 @@ type cmdPublish struct {
 
 	flagAliases              []string
 	flagCompressionAlgorithm string
+	flagExpiresAt            string
 	flagMakePublic           bool
 	flagForce                bool
 }
 
-func (c *cmdPublish) showByDefault() bool {
-	return true
-}
-
 func (c *cmdPublish) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("publish [<remote>:]<container>[/<snapshot>] [<remote>:] [flags] [key=value...]")
-	cmd.Short = i18n.G("Publish containers as images")
+	cmd.Use = usage("publish", i18n.G("[<remote>:]<instance>[/<snapshot>] [<remote>:] [flags] [key=value...]"))
+	cmd.Short = i18n.G("Publish instances as images")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Publish containers as images`))
+		`Publish instances as images`))
 
 	cmd.RunE = c.Run
 	cmd.Flags().BoolVar(&c.flagMakePublic, "public", false, i18n.G("Make the image public"))
 	cmd.Flags().StringArrayVar(&c.flagAliases, "alias", nil, i18n.G("New alias to define at target")+"``")
-	cmd.Flags().BoolVarP(&c.flagForce, "force", "f", false, i18n.G("Stop the container if currently running"))
-	cmd.Flags().StringVar(&c.flagCompressionAlgorithm, "compression", "", i18n.G("Define a compression algorithm: for image or none")+"``")
+	cmd.Flags().BoolVarP(&c.flagForce, "force", "f", false, i18n.G("Stop the instance if currently running"))
+	cmd.Flags().StringVar(&c.flagCompressionAlgorithm, "compression", "", i18n.G("Compression algorithm to use (`none` for uncompressed)"))
+	cmd.Flags().StringVar(&c.flagExpiresAt, "expire", "", i18n.G("Image expiration date (format: rfc3339)")+"``")
 
 	return cmd
 }
@@ -51,7 +50,7 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 	properties := map[string]string{}
 	firstprop := 1 // first property is arg[2] if arg[1] is image remote, else arg[1]
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, -1)
 	if exit {
 		return err
@@ -76,27 +75,28 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if cName == "" {
-		return fmt.Errorf(i18n.G("Container name is mandatory"))
+		return fmt.Errorf(i18n.G("Instance name is mandatory"))
 	}
+
 	if iName != "" {
 		return fmt.Errorf(i18n.G("There is no \"image name\".  Did you want an alias?"))
 	}
 
-	d, err := conf.GetContainerServer(iRemote)
+	d, err := conf.GetInstanceServer(iRemote)
 	if err != nil {
 		return err
 	}
 
 	s := d
 	if cRemote != iRemote {
-		s, err = conf.GetContainerServer(cRemote)
+		s, err = conf.GetInstanceServer(cRemote)
 		if err != nil {
 			return err
 		}
 	}
 
 	if !shared.IsSnapshot(cName) {
-		ct, etag, err := s.GetContainer(cName)
+		ct, etag, err := s.GetInstance(cName)
 		if err != nil {
 			return err
 		}
@@ -106,12 +106,13 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 
 		if wasRunning {
 			if !c.flagForce {
-				return fmt.Errorf(i18n.G("The container is currently running. Use --force to have it stopped and restarted"))
+				return fmt.Errorf(i18n.G("The instance is currently running. Use --force to have it stopped and restarted"))
 			}
 
 			if ct.Ephemeral {
+				// Clear the ephemeral flag so the instance can be stopped without being destroyed.
 				ct.Ephemeral = false
-				op, err := s.UpdateContainer(cName, ct.Writable(), etag)
+				op, err := s.UpdateInstance(cName, ct.Writable(), etag)
 				if err != nil {
 					return err
 				}
@@ -120,43 +121,45 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 				if err != nil {
 					return err
 				}
-
-				// Refresh the ETag
-				_, etag, err = s.GetContainer(cName)
-				if err != nil {
-					return err
-				}
 			}
 
-			req := api.ContainerStatePut{
+			// Stop the instance.
+			req := api.InstanceStatePut{
 				Action:  string(shared.Stop),
 				Timeout: -1,
 				Force:   true,
 			}
 
-			op, err := s.UpdateContainerState(cName, req, "")
+			op, err := s.UpdateInstanceState(cName, req, "")
 			if err != nil {
 				return err
 			}
 
 			err = op.Wait()
 			if err != nil {
-				return fmt.Errorf(i18n.G("Stopping container failed!"))
+				return fmt.Errorf(i18n.G("Stopping instance failed!"))
 			}
 
+			// Start the instance back up on exit.
 			defer func() {
 				req.Action = string(shared.Start)
-				op, err = s.UpdateContainerState(cName, req, "")
+				op, err = s.UpdateInstanceState(cName, req, "")
 				if err != nil {
 					return
 				}
 
-				op.Wait()
+				_ = op.Wait()
 			}()
 
+			// If we had to clear the ephemeral flag, restore it now.
 			if wasEphemeral {
+				ct, etag, err := s.GetInstance(cName)
+				if err != nil {
+					return err
+				}
+
 				ct.Ephemeral = true
-				op, err := s.UpdateContainer(cName, ct.Writable(), etag)
+				op, err := s.UpdateInstance(cName, ct.Writable(), etag)
 				if err != nil {
 					return err
 				}
@@ -174,6 +177,7 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 		if len(entry) < 2 {
 			return fmt.Errorf(i18n.G("Bad key=value pair: %s"), entry)
 		}
+
 		properties[entry[0]] = entry[1]
 	}
 
@@ -196,30 +200,59 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 	// Create the image
 	req := api.ImagesPost{
 		Source: &api.ImagesPostSource{
-			Type: "container",
+			Type: "instance",
 			Name: cName,
 		},
 		CompressionAlgorithm: c.flagCompressionAlgorithm,
 	}
+
 	req.Properties = properties
 
 	if shared.IsSnapshot(cName) {
 		req.Source.Type = "snapshot"
+	} else if !s.HasExtension("instances") {
+		req.Source.Type = "container"
 	}
 
 	if cRemote == iRemote {
 		req.Public = c.flagMakePublic
 	}
 
+	if c.flagExpiresAt != "" {
+		expiresAt, err := time.Parse(time.RFC3339, c.flagExpiresAt)
+		if err != nil {
+			return fmt.Errorf("Invalid expiration date: %w", err)
+		}
+
+		req.ExpiresAt = expiresAt
+	}
+
 	op, err := s.CreateImage(req, nil)
 	if err != nil {
 		return err
 	}
 
-	err = op.Wait()
+	// Watch the background operation
+	progress := utils.ProgressRenderer{
+		Format: i18n.G("Publishing instance: %s"),
+		Quiet:  c.global.flagQuiet,
+	}
+
+	_, err = op.AddHandler(progress.UpdateOp)
 	if err != nil {
+		progress.Done("")
 		return err
 	}
+
+	// Wait for the copy to complete
+	err = utils.CancelableWait(op, &progress)
+	if err != nil {
+		progress.Done("")
+		return err
+	}
+
+	progress.Done("")
+
 	opAPI := op.Get()
 
 	// Grab the fingerprint
@@ -227,7 +260,7 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 
 	// For remote publish, copy to target now
 	if cRemote != iRemote {
-		defer s.DeleteImage(fingerprint)
+		defer func() { _, _ = s.DeleteImage(fingerprint) }()
 
 		// Get the source image
 		image, _, err := s.GetImage(fingerprint)
@@ -256,7 +289,7 @@ func (c *cmdPublish) Run(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		return err
 	}
-	fmt.Printf(i18n.G("Container published with fingerprint: %s")+"\n", fingerprint)
 
+	fmt.Printf(i18n.G("Instance published with fingerprint: %s")+"\n", fingerprint)
 	return nil
 }
