@@ -3,9 +3,12 @@ package main
 import (
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
+	"net/url"
+	"strings"
 
 	"github.com/spf13/cobra"
 
@@ -25,13 +28,13 @@ type cmdQuery struct {
 
 func (c *cmdQuery) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("query [<remote>:]<API path>")
+	cmd.Use = usage("query", i18n.G("[<remote>:]<API path>"))
 	cmd.Short = i18n.G("Send a raw query to LXD")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Send a raw query to LXD`))
 	cmd.Example = cli.FormatSection("", i18n.G(
-		`lxc query -X DELETE --wait /1.0/containers/c1
-    Delete local container "c1".`))
+		`lxc query -X DELETE --wait /1.0/instances/c1
+    Delete local instance "c1".`))
 	cmd.Hidden = true
 
 	cmd.RunE = c.Run
@@ -43,38 +46,51 @@ func (c *cmdQuery) Command() *cobra.Command {
 	return cmd
 }
 
-func (c *cmdQuery) pretty(input interface{}) string {
-	pretty, err := json.MarshalIndent(input, "", "\t")
+func (c *cmdQuery) pretty(input any) string {
+	pretty := bytes.NewBufferString("")
+	enc := json.NewEncoder(pretty)
+	enc.SetEscapeHTML(false)
+	enc.SetIndent("", "\t")
+	err := enc.Encode(input)
 	if err != nil {
 		return fmt.Sprintf("%v", input)
 	}
 
-	return fmt.Sprintf("%s", pretty)
+	return pretty.String()
 }
 
 func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
 	}
 
+	if c.global.flagProject != "" {
+		return fmt.Errorf(i18n.G("--project cannot be used with the query command"))
+	}
+
 	// Parse the remote
 	remote, path, err := conf.ParseRemote(args[0])
 	if err != nil {
 		return err
 	}
 
+	// Validate path
+	if !strings.HasPrefix(path, "/") {
+		return fmt.Errorf(i18n.G("Query path must start with /"))
+	}
+
 	// Attempt to connect
-	d, err := conf.GetContainerServer(remote)
+	d, err := conf.GetInstanceServer(remote)
 	if err != nil {
 		return err
 	}
 
 	// Guess the encoding of the input
-	var data interface{}
+	var data any
 	err = json.Unmarshal([]byte(c.flagData), &data)
 	if err != nil {
 		data = c.flagData
@@ -83,23 +99,30 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 	// Perform the query
 	resp, _, err := d.RawQuery(c.flagAction, path, data, "")
 	if err != nil {
-		cleanErr := err
+		var jsonSyntaxError *json.SyntaxError
 
-		// Lets assume the endpoint is raw output
-		// Get a raw http client
-		httpClient, err := d.GetHTTPClient()
-		if err != nil {
+		// If not JSON decoding error then fail immediately.
+		if !errors.As(err, &jsonSyntaxError) {
 			return err
 		}
 
+		// If JSON decoding error then try a plain request.
+		cleanErr := err
+
 		// Get the URL prefix
 		httpInfo, err := d.GetConnectionInfo()
 		if err != nil {
 			return err
 		}
 
+		// Setup input.
+		var rs io.ReadSeeker
+		if c.flagData != "" {
+			rs = bytes.NewReader([]byte(c.flagData))
+		}
+
 		// Setup the request
-		req, err := http.NewRequest(c.flagAction, fmt.Sprintf("%s%s", httpInfo.URL, path), bytes.NewReader([]byte(c.flagData)))
+		req, err := http.NewRequest(c.flagAction, fmt.Sprintf("%s%s", httpInfo.URL, path), rs)
 		if err != nil {
 			return err
 		}
@@ -107,7 +130,7 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 		// Set the encoding accordingly
 		req.Header.Set("Content-Type", "plain/text")
 
-		resp, err := httpClient.Do(req)
+		resp, err := d.DoHTTP(req)
 		if err != nil {
 			return err
 		}
@@ -116,7 +139,7 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 			return cleanErr
 		}
 
-		content, err := ioutil.ReadAll(resp.Body)
+		content, err := io.ReadAll(resp.Body)
 		if err != nil {
 			return err
 		}
@@ -127,7 +150,12 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if c.flagRespWait && resp.Operation != "" {
-		resp, _, err = d.RawQuery("GET", fmt.Sprintf("%s/wait", resp.Operation), "", "")
+		uri, err := url.ParseRequestURI(resp.Operation)
+		if err != nil {
+			return err
+		}
+
+		resp, _, err = d.RawQuery("GET", fmt.Sprintf("%s/wait?%s", uri.Path, uri.RawQuery), "", "")
 		if err != nil {
 			return err
 		}
@@ -142,7 +170,7 @@ func (c *cmdQuery) Run(cmd *cobra.Command, args []string) error {
 	if c.flagRespRaw {
 		fmt.Println(c.pretty(resp))
 	} else if resp.Metadata != nil && string(resp.Metadata) != "{}" {
-		var content interface{}
+		var content any
 		err := json.Unmarshal(resp.Metadata, &content)
 		if err != nil {
 			return err
