@@ -5,21 +5,18 @@ import (
 	"encoding/pem"
 	"fmt"
 	"net"
+	"net/http"
 	"net/url"
 	"os"
-	"path/filepath"
 	"sort"
 	"strings"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
-	"golang.org/x/crypto/ssh/terminal"
-	schemaform "gopkg.in/juju/environschema.v1/form"
-	"gopkg.in/macaroon-bakery.v2/httpbakery"
-	"gopkg.in/macaroon-bakery.v2/httpbakery/form"
+	"golang.org/x/term"
 
 	"github.com/lxc/lxd/client"
 	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd/lxc/utils"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	cli "github.com/lxc/lxd/shared/cmd"
@@ -32,7 +29,7 @@ type cmdRemote struct {
 
 func (c *cmdRemote) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("remote")
+	cmd.Use = usage("remote")
 	cmd.Short = i18n.G("Manage the list of remote servers")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Manage the list of remote servers`))
@@ -58,17 +55,20 @@ func (c *cmdRemote) Command() *cobra.Command {
 	cmd.AddCommand(remoteRemoveCmd.Command())
 
 	// Set default
-	remoteSetDefaultCmd := cmdRemoteSetDefault{global: c.global, remote: c}
-	cmd.AddCommand(remoteSetDefaultCmd.Command())
+	remoteSwitchCmd := cmdRemoteSwitch{global: c.global, remote: c}
+	cmd.AddCommand(remoteSwitchCmd.Command())
 
 	// Set URL
 	remoteSetURLCmd := cmdRemoteSetURL{global: c.global, remote: c}
 	cmd.AddCommand(remoteSetURLCmd.Command())
 
+	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
+	cmd.Args = cobra.NoArgs
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// Add
+// Add.
 type cmdRemoteAdd struct {
 	global *cmdGlobal
 	remote *cmdRemote
@@ -79,14 +79,21 @@ type cmdRemoteAdd struct {
 	flagProtocol   string
 	flagAuthType   string
 	flagDomain     string
+	flagProject    string
 }
 
 func (c *cmdRemoteAdd) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("add [<remote>] <IP|FQDN|URL>")
+	cmd.Use = usage("add", i18n.G("[<remote>] <IP|FQDN|URL|token>"))
 	cmd.Short = i18n.G("Add new remote servers")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Add new remote servers`))
+		`Add new remote servers
+
+URL for remote resources must be HTTPS (https://).
+
+Basic authentication can be used when combined with the "simplestreams" protocol:
+  lxc remote add some-name https://LOGIN:PASSWORD@example.com/some/path --protocol=simplestreams
+`))
 
 	cmd.RunE = c.Run
 	cmd.Flags().BoolVar(&c.flagAcceptCert, "accept-certificate", false, i18n.G("Accept certificate"))
@@ -95,14 +102,176 @@ func (c *cmdRemoteAdd) Command() *cobra.Command {
 	cmd.Flags().StringVar(&c.flagAuthType, "auth-type", "", i18n.G("Server authentication type (tls or candid)")+"``")
 	cmd.Flags().BoolVar(&c.flagPublic, "public", false, i18n.G("Public image server"))
 	cmd.Flags().StringVar(&c.flagDomain, "domain", "", i18n.G("Candid domain to use")+"``")
+	cmd.Flags().StringVar(&c.flagProject, "project", "", i18n.G("Project to use for the remote")+"``")
 
 	return cmd
 }
 
+func (c *cmdRemoteAdd) findProject(d lxd.InstanceServer, project string) (string, error) {
+	if project == "" {
+		// Check if we can pull a list of projects.
+		if d.HasExtension("projects") {
+			// Retrieve the allowed projects.
+			names, err := d.GetProjectNames()
+			if err != nil {
+				return "", err
+			}
+
+			if len(names) == 0 {
+				// If no allowed projects, just keep it to the default.
+				return "", nil
+			} else if len(names) == 1 {
+				// If only a single project, use that.
+				return names[0], nil
+			}
+
+			// Deal with multiple projects.
+			if shared.StringInSlice("default", names) {
+				// If we have access to the default project, use it.
+				return "", nil
+			}
+
+			// Let's ask the user.
+			fmt.Println(i18n.G("Available projects:"))
+			for _, name := range names {
+				fmt.Println(" - " + name)
+			}
+
+			return cli.AskChoice(i18n.G("Name of the project to use for this remote:")+" ", names, "")
+		}
+
+		return "", nil
+	}
+
+	_, _, err := d.GetProject(project)
+	if err != nil {
+		return "", err
+	}
+
+	return project, nil
+}
+
+func (c *cmdRemoteAdd) RunToken(server string, token string, rawToken *api.CertificateAddToken) error {
+	conf := c.global.conf
+
+	if !conf.HasClientCertificate() {
+		fmt.Fprintf(os.Stderr, i18n.G("Generating a client certificate. This may take a minute...")+"\n")
+		err := conf.GenerateClientCertificate()
+		if err != nil {
+			return err
+		}
+	}
+
+	for _, addr := range rawToken.Addresses {
+		addr = fmt.Sprintf("https://%s", addr)
+
+		err := c.addRemoteFromToken(addr, server, token, rawToken.Fingerprint)
+		if err != nil {
+			if api.StatusErrorCheck(err, http.StatusServiceUnavailable) {
+				continue
+			}
+
+			return err
+		}
+
+		return nil
+	}
+
+	fmt.Println(i18n.G("All server addresses are unavailable"))
+	fmt.Printf(i18n.G("Please provide an alternate server address (empty to abort):") + " ")
+
+	line, err := shared.ReadStdin()
+	if err != nil {
+		return err
+	}
+
+	if len(line) == 0 {
+		return fmt.Errorf(i18n.G("Failed to add remote"))
+	}
+
+	err = c.addRemoteFromToken(string(line), server, token, rawToken.Fingerprint)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func (c *cmdRemoteAdd) addRemoteFromToken(addr string, server string, token string, fingerprint string) error {
+	conf := c.global.conf
+
+	var certificate *x509.Certificate
+	var err error
+
+	conf.Remotes[server] = config.Remote{Addr: addr, Protocol: c.flagProtocol, AuthType: c.flagAuthType, Domain: c.flagDomain}
+
+	_, err = conf.GetInstanceServer(server)
+	if err != nil {
+		certificate, err = shared.GetRemoteCertificate(addr, c.global.conf.UserAgent)
+		if err != nil {
+			return api.StatusErrorf(http.StatusServiceUnavailable, i18n.G("Unavailable remote server")+": %v", err)
+		}
+
+		certDigest := shared.CertFingerprint(certificate)
+		if fingerprint != certDigest {
+			return fmt.Errorf(i18n.G("Certificate fingerprint mismatch between certificate token and server %q"), addr)
+		}
+
+		dnam := conf.ConfigPath("servercerts")
+		err := os.MkdirAll(dnam, 0750)
+		if err != nil {
+			return fmt.Errorf(i18n.G("Could not create server cert dir"))
+		}
+
+		certf := conf.ServerCertPath(server)
+
+		certOut, err := os.Create(certf)
+		if err != nil {
+			return fmt.Errorf(i18n.G("Failed to create %q: %w"), certf, err)
+		}
+
+		err = pem.Encode(certOut, &pem.Block{Type: "CERTIFICATE", Bytes: certificate.Raw})
+		if err != nil {
+			return fmt.Errorf(i18n.G("Failed to write server cert file %q: %w"), certf, err)
+		}
+
+		err = certOut.Close()
+		if err != nil {
+			return fmt.Errorf(i18n.G("Failed to close server cert file %q: %w"), certf, err)
+		}
+	}
+
+	d, err := conf.GetInstanceServer(server)
+	if err != nil {
+		return api.StatusErrorf(http.StatusServiceUnavailable, i18n.G("Unavailable remote server")+": %v", err)
+	}
+
+	req := api.CertificatesPost{
+		Password: token,
+	}
+
+	err = d.CreateCertificate(req)
+	if err != nil {
+		return fmt.Errorf(i18n.G("Failed to create certificate: %w"), err)
+	}
+
+	// Handle project.
+	remote := conf.Remotes[server]
+	project, err := c.findProject(d, c.flagProject)
+	if err != nil {
+		return fmt.Errorf(i18n.G("Failed to find project: %w"), err)
+	}
+
+	remote.Project = project
+	conf.Remotes[server] = remote
+
+	return conf.SaveConfig(c.global.confPath)
+}
+
 func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 2)
 	if exit {
 		return err
@@ -115,6 +284,11 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 		addr = args[1]
 	}
 
+	// Validate the server name.
+	if strings.Contains(server, ":") {
+		return fmt.Errorf(i18n.G("Remote names may not contain colons"))
+	}
+
 	// Check for existing remote
 	remote, ok := conf.Remotes[server]
 	if ok {
@@ -130,15 +304,16 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 		c.flagProtocol = "lxd"
 	}
 
-	if c.flagAuthType == "" {
-		c.flagAuthType = "tls"
-	}
-
 	// Initialize the remotes list if needed
 	if conf.Remotes == nil {
 		conf.Remotes = map[string]config.Remote{}
 	}
 
+	rawToken, err := shared.CertificateTokenDecode(addr)
+	if err == nil {
+		return c.RunToken(server, addr, rawToken)
+	}
+
 	// Complex remote URL parsing
 	remoteURL, err := url.Parse(addr)
 	if err != nil {
@@ -190,7 +365,7 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 		rHost = host
 		rPort = port
 	} else {
-		rPort = shared.DefaultPort
+		rPort = fmt.Sprintf("%d", shared.HTTPSDefaultPort)
 	}
 
 	if rScheme == "unix" {
@@ -211,7 +386,7 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 	// Finally, actually add the remote, almost...  If the remote is a private
 	// HTTPS server then we need to ensure we have a client certificate before
 	// adding the remote server.
-	if rScheme != "unix" && !c.flagPublic && c.flagAuthType == "tls" {
+	if rScheme != "unix" && !c.flagPublic && (c.flagAuthType == "tls" || c.flagAuthType == "") {
 		if !conf.HasClientCertificate() {
 			fmt.Fprintf(os.Stderr, i18n.G("Generating a client certificate. This may take a minute...")+"\n")
 			err = conf.GenerateClientCertificate()
@@ -220,29 +395,15 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	}
-	conf.Remotes[server] = config.Remote{Addr: addr, Protocol: c.flagProtocol, AuthType: c.flagAuthType}
 
-	conf.SetAuthInteractor([]httpbakery.Interactor{
-		form.Interactor{Filler: schemaform.IOFiller{}},
-		httpbakery.WebBrowserInteractor{
-			OpenWebBrowser: func(uri *url.URL) error {
-				if c.flagDomain != "" {
-					query := uri.Query()
-					query.Set("domain", c.flagDomain)
-					uri.RawQuery = query.Encode()
-				}
-
-				return httpbakery.OpenWebBrowser(uri)
-			},
-		},
-	})
+	conf.Remotes[server] = config.Remote{Addr: addr, Protocol: c.flagProtocol, AuthType: c.flagAuthType, Domain: c.flagDomain}
 
 	// Attempt to connect
 	var d lxd.ImageServer
 	if c.flagPublic {
 		d, err = conf.GetImageServer(server)
 	} else {
-		d, err = conf.GetContainerServer(server)
+		d, err = conf.GetInstanceServer(server)
 	}
 
 	// Handle Unix socket connections
@@ -251,6 +412,18 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
+		remote := conf.Remotes[server]
+		remote.AuthType = "tls"
+
+		// Handle project.
+		project, err := c.findProject(d.(lxd.InstanceServer), c.flagProject)
+		if err != nil {
+			return err
+		}
+
+		remote.Project = project
+
+		conf.Remotes[server] = remote
 		return conf.SaveConfig(c.global.confPath)
 	}
 
@@ -258,7 +431,7 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 	var certificate *x509.Certificate
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = shared.GetRemoteCertificate(addr)
+		certificate, err = shared.GetRemoteCertificate(addr, c.global.conf.UserAgent)
 		if err != nil {
 			return err
 		}
@@ -270,14 +443,18 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 			digest := shared.CertFingerprint(certificate)
 
 			fmt.Printf(i18n.G("Certificate fingerprint: %s")+"\n", digest)
-			fmt.Printf(i18n.G("ok (y/n)?") + " ")
+			fmt.Printf(i18n.G("ok (y/n/[fingerprint])?") + " ")
 			line, err := shared.ReadStdin()
 			if err != nil {
 				return err
 			}
 
-			if len(line) < 1 || strings.ToLower(string(line[0])) != i18n.G("y") {
-				return fmt.Errorf(i18n.G("Server certificate NACKed by user"))
+			if string(line) != digest {
+				if len(line) < 1 || strings.ToLower(string(line[0])) == i18n.G("n") {
+					return fmt.Errorf(i18n.G("Server certificate NACKed by user"))
+				} else if strings.ToLower(string(line[0])) != i18n.G("y") {
+					return fmt.Errorf(i18n.G("Please type 'y', 'n' or the fingerprint:"))
+				}
 			}
 		}
 
@@ -287,20 +464,27 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 			return fmt.Errorf(i18n.G("Could not create server cert dir"))
 		}
 
-		certf := fmt.Sprintf("%s/%s.crt", dnam, server)
+		certf := conf.ServerCertPath(server)
 		certOut, err := os.Create(certf)
 		if err != nil {
 			return err
 		}
 
-		pem.Encode(certOut, &pem.Block{Type: "CERTIFICATE", Bytes: certificate.Raw})
-		certOut.Close()
+		err = pem.Encode(certOut, &pem.Block{Type: "CERTIFICATE", Bytes: certificate.Raw})
+		if err != nil {
+			return fmt.Errorf(i18n.G("Could not write server cert file %q: %w"), certf, err)
+		}
+
+		err = certOut.Close()
+		if err != nil {
+			return fmt.Errorf(i18n.G("Could not close server cert file %q: %w"), certf, err)
+		}
 
 		// Setup a new connection, this time with the remote certificate
 		if c.flagPublic {
 			d, err = conf.GetImageServer(server)
 		} else {
-			d, err = conf.GetContainerServer(server)
+			d, err = conf.GetInstanceServer(server)
 		}
 
 		if err != nil {
@@ -315,15 +499,47 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if c.flagAuthType == "candid" {
-		d.(lxd.ContainerServer).RequireAuthenticated(false)
+		d.(lxd.InstanceServer).RequireAuthenticated(false)
 	}
 
 	// Get server information
-	srv, _, err := d.(lxd.ContainerServer).GetServer()
+	srv, _, err := d.(lxd.InstanceServer).GetServer()
 	if err != nil {
 		return err
 	}
 
+	// If not specified, default authentication to Candid
+	if c.flagAuthType == "" {
+		if !srv.Public && shared.StringInSlice("candid", srv.AuthMethods) {
+			c.flagAuthType = "candid"
+
+			// Update the remote configuration
+			remote := conf.Remotes[server]
+			remote.AuthType = c.flagAuthType
+			conf.Remotes[server] = remote
+
+			// Re-setup the client
+			d, err = conf.GetInstanceServer(server)
+			if err != nil {
+				return err
+			}
+
+			d.(lxd.InstanceServer).RequireAuthenticated(false)
+
+			srv, _, err = d.(lxd.InstanceServer).GetServer()
+			if err != nil {
+				return err
+			}
+		} else {
+			c.flagAuthType = "tls"
+
+			// Update the remote configuration
+			remote := conf.Remotes[server]
+			remote.AuthType = c.flagAuthType
+			conf.Remotes[server] = remote
+		}
+	}
+
 	if !srv.Public && !shared.StringInSlice(c.flagAuthType, srv.AuthMethods) {
 		return fmt.Errorf(i18n.G("Authentication type '%s' not supported by server"), c.flagAuthType)
 	}
@@ -334,60 +550,69 @@ func (c *cmdRemoteAdd) Run(cmd *cobra.Command, args []string) error {
 		return conf.SaveConfig(c.global.confPath)
 	}
 
-	// Check if our cert is already trusted
-	if srv.Auth == "trusted" {
-		return conf.SaveConfig(c.global.confPath)
-	}
-
-	if c.flagAuthType == "tls" {
-		// Prompt for trust password
-		if c.flagPassword == "" {
-			fmt.Printf(i18n.G("Admin password for %s: "), server)
-			pwd, err := terminal.ReadPassword(0)
-			if err != nil {
-				/* We got an error, maybe this isn't a terminal, let's try to
-				 * read it as a file */
-				pwd, err = shared.ReadStdin()
+	// Check if additional authentication is required.
+	if srv.Auth != "trusted" {
+		if c.flagAuthType == "tls" {
+			// Prompt for trust password
+			if c.flagPassword == "" {
+				fmt.Printf(i18n.G("Admin password (or token) for %s:")+" ", server)
+				pwd, err := term.ReadPassword(0)
 				if err != nil {
-					return err
+					/* We got an error, maybe this isn't a terminal, let's try to
+					 * read it as a file */
+					pwd, err = shared.ReadStdin()
+					if err != nil {
+						return err
+					}
 				}
+				fmt.Println("")
+				c.flagPassword = string(pwd)
 			}
-			fmt.Println("")
-			c.flagPassword = string(pwd)
+
+			// Add client certificate to trust store
+			req := api.CertificatesPost{
+				Password: c.flagPassword,
+			}
+
+			req.Type = api.CertificateTypeClient
+
+			err = d.(lxd.InstanceServer).CreateCertificate(req)
+			if err != nil {
+				return err
+			}
+		} else {
+			d.(lxd.InstanceServer).RequireAuthenticated(true)
 		}
 
-		// Add client certificate to trust store
-		req := api.CertificatesPost{
-			Password: c.flagPassword,
-		}
-		req.Type = "client"
-
-		err = d.(lxd.ContainerServer).CreateCertificate(req)
+		// And check if trusted now
+		srv, _, err = d.(lxd.InstanceServer).GetServer()
 		if err != nil {
 			return err
 		}
-	} else {
-		d.(lxd.ContainerServer).RequireAuthenticated(true)
+
+		if srv.Auth != "trusted" {
+			return fmt.Errorf(i18n.G("Server doesn't trust us after authentication"))
+		}
+
+		if c.flagAuthType == "tls" {
+			fmt.Println(i18n.G("Client certificate now trusted by server:"), server)
+		}
 	}
 
-	// And check if trusted now
-	srv, _, err = d.(lxd.ContainerServer).GetServer()
+	// Handle project.
+	remote = conf.Remotes[server]
+	project, err := c.findProject(d.(lxd.InstanceServer), c.flagProject)
 	if err != nil {
 		return err
 	}
 
-	if srv.Auth != "trusted" {
-		return fmt.Errorf(i18n.G("Server doesn't trust us after authentication"))
-	}
-
-	if c.flagAuthType == "tls" {
-		fmt.Println(i18n.G("Client certificate stored at server: "), server)
-	}
+	remote.Project = project
+	conf.Remotes[server] = remote
 
 	return conf.SaveConfig(c.global.confPath)
 }
 
-// Get default
+// Get default.
 type cmdRemoteGetDefault struct {
 	global *cmdGlobal
 	remote *cmdRemote
@@ -395,7 +620,7 @@ type cmdRemoteGetDefault struct {
 
 func (c *cmdRemoteGetDefault) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("get-default")
+	cmd.Use = usage("get-default")
 	cmd.Short = i18n.G("Show the default remote")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show the default remote`))
@@ -408,7 +633,7 @@ func (c *cmdRemoteGetDefault) Command() *cobra.Command {
 func (c *cmdRemoteGetDefault) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 0)
 	if exit {
 		return err
@@ -420,21 +645,24 @@ func (c *cmdRemoteGetDefault) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-// List
+// List.
 type cmdRemoteList struct {
 	global *cmdGlobal
 	remote *cmdRemote
+
+	flagFormat string
 }
 
 func (c *cmdRemoteList) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("list")
+	cmd.Use = usage("list")
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List the available remotes")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`List the available remotes`))
 
 	cmd.RunE = c.Run
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 
 	return cmd
 }
@@ -442,7 +670,7 @@ func (c *cmdRemoteList) Command() *cobra.Command {
 func (c *cmdRemoteList) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 0)
 	if exit {
 		return err
@@ -461,39 +689,53 @@ func (c *cmdRemoteList) Run(cmd *cobra.Command, args []string) error {
 			strStatic = i18n.G("YES")
 		}
 
+		strGlobal := i18n.G("NO")
+		if rc.Global {
+			strGlobal = i18n.G("YES")
+		}
+
 		if rc.Protocol == "" {
 			rc.Protocol = "lxd"
 		}
-		if rc.AuthType == "" && !rc.Public {
-			rc.AuthType = "tls"
+
+		if rc.AuthType == "" {
+			if strings.HasPrefix(rc.Addr, "unix:") {
+				rc.AuthType = "file access"
+			} else if rc.Protocol == "simplestreams" {
+				rc.AuthType = "none"
+			} else {
+				rc.AuthType = "tls"
+			}
+		}
+
+		if rc.AuthType == "candid" && rc.Domain != "" {
+			rc.AuthType = fmt.Sprintf("%s (%s)", rc.AuthType, rc.Domain)
 		}
 
 		strName := name
 		if name == conf.DefaultRemote {
-			strName = fmt.Sprintf("%s (%s)", name, i18n.G("default"))
+			strName = fmt.Sprintf("%s (%s)", name, i18n.G("current"))
 		}
-		data = append(data, []string{strName, rc.Addr, rc.Protocol, rc.AuthType, strPublic, strStatic})
+
+		data = append(data, []string{strName, rc.Addr, rc.Protocol, rc.AuthType, strPublic, strStatic, strGlobal})
 	}
 
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAutoWrapText(false)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetRowLine(true)
-	table.SetHeader([]string{
+	sort.Sort(utils.ByName(data))
+
+	header := []string{
 		i18n.G("NAME"),
 		i18n.G("URL"),
 		i18n.G("PROTOCOL"),
 		i18n.G("AUTH TYPE"),
 		i18n.G("PUBLIC"),
-		i18n.G("STATIC")})
-	sort.Sort(byName(data))
-	table.AppendBulk(data)
-	table.Render()
+		i18n.G("STATIC"),
+		i18n.G("GLOBAL"),
+	}
 
-	return nil
+	return utils.RenderTable(c.flagFormat, header, data, conf.Remotes)
 }
 
-// Rename
+// Rename.
 type cmdRemoteRename struct {
 	global *cmdGlobal
 	remote *cmdRemote
@@ -501,7 +743,7 @@ type cmdRemoteRename struct {
 
 func (c *cmdRemoteRename) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("rename <remote> <new-name>")
+	cmd.Use = usage("rename", i18n.G("<remote> <new-name>"))
 	cmd.Aliases = []string{"mv"}
 	cmd.Short = i18n.G("Rename remotes")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -515,7 +757,7 @@ func (c *cmdRemoteRename) Command() *cobra.Command {
 func (c *cmdRemoteRename) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -531,17 +773,27 @@ func (c *cmdRemoteRename) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Remote %s is static and cannot be modified"), args[0])
 	}
 
-	if _, ok := conf.Remotes[args[1]]; ok {
+	_, ok = conf.Remotes[args[1]]
+	if ok {
 		return fmt.Errorf(i18n.G("Remote %s already exists"), args[1])
 	}
 
 	// Rename the certificate file
-	oldPath := filepath.Join(conf.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[0]))
-	newPath := filepath.Join(conf.ConfigPath("servercerts"), fmt.Sprintf("%s.crt", args[1]))
+	oldPath := conf.ServerCertPath(args[0])
+	newPath := conf.ServerCertPath(args[1])
 	if shared.PathExists(oldPath) {
-		err := os.Rename(oldPath, newPath)
-		if err != nil {
-			return err
+		if conf.Remotes[args[0]].Global {
+			err := conf.CopyGlobalCert(args[0], args[1])
+			if err != nil {
+				return err
+			}
+
+			rc.Global = false
+		} else {
+			err := os.Rename(oldPath, newPath)
+			if err != nil {
+				return err
+			}
 		}
 	}
 
@@ -555,7 +807,7 @@ func (c *cmdRemoteRename) Run(cmd *cobra.Command, args []string) error {
 	return conf.SaveConfig(c.global.confPath)
 }
 
-// Remove
+// Remove.
 type cmdRemoteRemove struct {
 	global *cmdGlobal
 	remote *cmdRemote
@@ -563,7 +815,7 @@ type cmdRemoteRemove struct {
 
 func (c *cmdRemoteRemove) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("remove <remote>")
+	cmd.Use = usage("remove", i18n.G("<remote>"))
 	cmd.Aliases = []string{"rm"}
 	cmd.Short = i18n.G("Remove remotes")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -577,7 +829,7 @@ func (c *cmdRemoteRemove) Command() *cobra.Command {
 func (c *cmdRemoteRemove) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -593,40 +845,45 @@ func (c *cmdRemoteRemove) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Remote %s is static and cannot be modified"), args[0])
 	}
 
+	if rc.Global {
+		return fmt.Errorf(i18n.G("Remote %s is global and cannot be removed"), args[0])
+	}
+
 	if conf.DefaultRemote == args[0] {
 		return fmt.Errorf(i18n.G("Can't remove the default remote"))
 	}
 
 	delete(conf.Remotes, args[0])
 
-	certf := conf.ServerCertPath(args[0])
-	os.Remove(certf)
+	_ = os.Remove(conf.ServerCertPath(args[0]))
+	_ = os.Remove(conf.CookiesPath(args[0]))
 
 	return conf.SaveConfig(c.global.confPath)
 }
 
-// Set default
-type cmdRemoteSetDefault struct {
+// Set default.
+type cmdRemoteSwitch struct {
 	global *cmdGlobal
 	remote *cmdRemote
 }
 
-func (c *cmdRemoteSetDefault) Command() *cobra.Command {
+func (c *cmdRemoteSwitch) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("set-default <remote>")
-	cmd.Short = i18n.G("Set the default remote")
+	cmd.Aliases = []string{"set-default"}
+	cmd.Use = usage("switch", i18n.G("<remote>"))
+	cmd.Short = i18n.G("Switch the default remote")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Set the default remote`))
+		`Switch the default remote`))
 
 	cmd.RunE = c.Run
 
 	return cmd
 }
 
-func (c *cmdRemoteSetDefault) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdRemoteSwitch) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
@@ -643,7 +900,7 @@ func (c *cmdRemoteSetDefault) Run(cmd *cobra.Command, args []string) error {
 	return conf.SaveConfig(c.global.confPath)
 }
 
-// Set URL
+// Set URL.
 type cmdRemoteSetURL struct {
 	global *cmdGlobal
 	remote *cmdRemote
@@ -651,7 +908,7 @@ type cmdRemoteSetURL struct {
 
 func (c *cmdRemoteSetURL) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("set-url <remote> <URL>")
+	cmd.Use = usage("set-url", i18n.G("<remote> <URL>"))
 	cmd.Short = i18n.G("Set the URL for the remote")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Set the URL for the remote`))
@@ -664,7 +921,7 @@ func (c *cmdRemoteSetURL) Command() *cobra.Command {
 func (c *cmdRemoteSetURL) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
@@ -680,7 +937,19 @@ func (c *cmdRemoteSetURL) Run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf(i18n.G("Remote %s is static and cannot be modified"), args[0])
 	}
 
-	conf.Remotes[args[0]] = config.Remote{Addr: args[1]}
+	remote := conf.Remotes[args[0]]
+	if remote.Global {
+		err := conf.CopyGlobalCert(args[0], args[0])
+		if err != nil {
+			return err
+		}
+
+		remote.Global = false
+		conf.Remotes[args[0]] = remote
+	}
+
+	remote.Addr = args[1]
+	conf.Remotes[args[0]] = remote
 
 	return conf.SaveConfig(c.global.confPath)
 }
