@@ -16,10 +16,10 @@ type cmdRename struct {
 
 func (c *cmdRename) Command() *cobra.Command {
 	cmd := &cobra.Command{}
-	cmd.Use = i18n.G("rename [<remote>:]<container>[/<snapshot>] <container>[/<snapshot>]")
-	cmd.Short = i18n.G("Rename containers and snapshots")
+	cmd.Use = usage("rename", i18n.G("[<remote>:]<instance>[/<snapshot>] <instance>[/<snapshot>]"))
+	cmd.Short = i18n.G("Rename instances and snapshots")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Rename containers and snapshots`))
+		`Rename instances and snapshots`))
 	cmd.RunE = c.Run
 
 	return cmd
@@ -28,7 +28,7 @@ func (c *cmdRename) Command() *cobra.Command {
 func (c *cmdRename) Run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
-	// Sanity checks
+	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, 2)
 	if exit {
 		return err
