@@ -2,7 +2,7 @@ package main
 
 import (
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 	"reflect"
 	"sort"
@@ -87,6 +87,28 @@ func profileDeviceAdd(client lxd.InstanceServer, name string, devName string, de
 	return nil
 }
 
+// parseDeviceOverrides parses device overrides of the form "<deviceName>,<key>=<value>" into a device map.
+// The resulting device map is unlikely to contain valid devices as these are simply values to be overridden.
+func parseDeviceOverrides(deviceOverrideArgs []string) (map[string]map[string]string, error) {
+	deviceMap := map[string]map[string]string{}
+	for _, entry := range deviceOverrideArgs {
+		if !strings.Contains(entry, "=") || !strings.Contains(entry, ",") {
+			return nil, fmt.Errorf(i18n.G("Bad device override syntax, expecting <device>,<key>=<value>: %s"), entry)
+		}
+
+		deviceFields := strings.SplitN(entry, ",", 2)
+		keyFields := strings.SplitN(deviceFields[1], "=", 2)
+
+		if deviceMap[deviceFields[0]] == nil {
+			deviceMap[deviceFields[0]] = map[string]string{}
+		}
+
+		deviceMap[deviceFields[0]][keyFields[0]] = keyFields[1]
+	}
+
+	return deviceMap, nil
+}
+
 // Create the specified image aliases, updating those that already exist.
 func ensureImageAliases(client lxd.InstanceServer, aliases []api.ImageAlias, fingerprint string) error {
 	if len(aliases) == 0 {
@@ -143,7 +165,7 @@ func GetExistingAliases(aliases []string, allAliases []api.ImageAliasesEntry) []
 func getConfig(args ...string) (map[string]string, error) {
 	if len(args) == 2 && !strings.Contains(args[0], "=") {
 		if args[1] == "-" && !termios.IsTerminal(getStdinFd()) {
-			buf, err := ioutil.ReadAll(os.Stdin)
+			buf, err := io.ReadAll(os.Stdin)
 			if err != nil {
 				return nil, fmt.Errorf(i18n.G("Can't read from stdin: %w"), err)
 			}
@@ -163,7 +185,7 @@ func getConfig(args ...string) (map[string]string, error) {
 		}
 
 		if fields[1] == "-" && !termios.IsTerminal(getStdinFd()) {
-			buf, err := ioutil.ReadAll(os.Stdin)
+			buf, err := io.ReadAll(os.Stdin)
 			if err != nil {
 				return nil, fmt.Errorf(i18n.G("Can't read from stdin: %w"), err)
 			}
