@@ -10,8 +10,8 @@ import (
 	"github.com/lxc/lxd/shared/i18n"
 )
 
-// CancelableWait waits for an operation and cancel it on SIGINT/SIGTERM
-func CancelableWait(rawOp interface{}, progress *ProgressRenderer) error {
+// CancelableWait waits for an operation and cancel it on SIGINT/SIGTERM.
+func CancelableWait(rawOp any, progress *ProgressRenderer) error {
 	var op lxd.Operation
 	var rop lxd.RemoteOperation
 
@@ -26,7 +26,7 @@ func CancelableWait(rawOp interface{}, progress *ProgressRenderer) error {
 	}
 
 	// Signal handling
-	chSignal := make(chan os.Signal)
+	chSignal := make(chan os.Signal, 1)
 	signal.Notify(chSignal, os.Interrupt)
 
 	// Operation handling
@@ -37,6 +37,7 @@ func CancelableWait(rawOp interface{}, progress *ProgressRenderer) error {
 		} else {
 			chOperation <- rop.Wait()
 		}
+
 		close(chOperation)
 	}()
 
@@ -53,6 +54,7 @@ func CancelableWait(rawOp interface{}, progress *ProgressRenderer) error {
 			} else {
 				err = rop.CancelTarget()
 			}
+
 			if err == nil {
 				return fmt.Errorf(i18n.G("Remote operation canceled by user"))
 			}
