@@ -12,9 +12,10 @@ import (
 	"github.com/lxc/lxd/shared/termios"
 )
 
-// ProgressRenderer tracks the progress information
+// ProgressRenderer tracks the progress information.
 type ProgressRenderer struct {
 	Format string
+	Quiet  bool
 
 	maxLength int
 	wait      time.Time
@@ -31,14 +32,13 @@ func (p *ProgressRenderer) truncate(msg string) string {
 
 	newSize := len(msg)
 	if width < newSize {
-		newSize = width
+		return ""
 	}
 
-	msg = msg[0:newSize]
 	return msg
 }
 
-// Done prints the final status and prevents any update
+// Done prints the final status and prevents any update.
 func (p *ProgressRenderer) Done(msg string) {
 	// Acquire rendering lock
 	p.lock.Lock()
@@ -52,6 +52,11 @@ func (p *ProgressRenderer) Done(msg string) {
 	// Mark this renderer as done
 	p.done = true
 
+	// Handle quiet mode
+	if p.Quiet {
+		msg = ""
+	}
+
 	// Truncate msg to terminal length
 	msg = p.truncate(msg)
 
@@ -75,10 +80,10 @@ func (p *ProgressRenderer) Done(msg string) {
 	fmt.Print(msg)
 }
 
-// Update changes the status message to the provided string
+// Update changes the status message to the provided string.
 func (p *ProgressRenderer) Update(status string) {
 	// Wait if needed
-	timeout := p.wait.Sub(time.Now())
+	timeout := time.Until(p.wait)
 	if timeout.Seconds() > 0 {
 		time.Sleep(timeout)
 	}
@@ -92,6 +97,11 @@ func (p *ProgressRenderer) Update(status string) {
 		return
 	}
 
+	// Handle quiet mode
+	if p.Quiet {
+		return
+	}
+
 	// Skip status updates when not dealing with a terminal
 	if p.terminal == 0 {
 		if !termios.IsTerminal(int(os.Stdout.Fd())) {
@@ -130,7 +140,7 @@ func (p *ProgressRenderer) Update(status string) {
 	fmt.Print(msg)
 }
 
-// Warn shows a temporary message instead of the status
+// Warn shows a temporary message instead of the status.
 func (p *ProgressRenderer) Warn(status string, timeout time.Duration) {
 	// Acquire rendering lock
 	p.lock.Lock()
@@ -143,7 +153,7 @@ func (p *ProgressRenderer) Warn(status string, timeout time.Duration) {
 
 	// Render the new message
 	p.wait = time.Now().Add(timeout)
-	msg := fmt.Sprintf("%s", status)
+	msg := status
 
 	// Truncate msg to terminal length
 	msg = "\r" + p.truncate(msg)
@@ -162,12 +172,12 @@ func (p *ProgressRenderer) Warn(status string, timeout time.Duration) {
 	fmt.Print(msg)
 }
 
-// UpdateProgress is a helper to update the status using an iopgress instance
+// UpdateProgress is a helper to update the status using an iopgress instance.
 func (p *ProgressRenderer) UpdateProgress(progress ioprogress.ProgressData) {
 	p.Update(progress.Text)
 }
 
-// UpdateOp is a helper to update the status using a LXD API operation
+// UpdateOp is a helper to update the status using a LXD API operation.
 func (p *ProgressRenderer) UpdateOp(op api.Operation) {
 	if op.Metadata == nil {
 		return
