@@ -225,12 +225,9 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 	srv.Auth = "trusted"
 
-	address, err := node.HTTPSAddress(d.db.Node)
-	if err != nil {
-		return response.InternalError(err)
-	}
+	localHTTPSAddress := s.LocalConfig.HTTPSAddress()
 
-	addresses, err := util.ListenAddresses(address)
+	addresses, err := util.ListenAddresses(localHTTPSAddress)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -408,6 +405,9 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
+	// Don't apply changes to settings until daemon is fully started.
+	<-d.waitReady.Done()
+
 	req := api.ServerPut{}
 	err := json.NewDecoder(r.Body).Decode(&req)
 	if err != nil {
@@ -417,7 +417,7 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 	// If this is a notification from a cluster node, just run the triggers
 	// for reacting to the values that changed.
 	if isClusterNotification(r) {
-		logger.Debugf("Handling config changed notification")
+		logger.Debug("Handling config changed notification")
 		changed := make(map[string]string)
 		for key, value := range req.Config {
 			changed[key] = value.(string)
@@ -425,9 +425,9 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 
 		// Get the current (updated) config.
 		var config *clusterConfig.Config
-		err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err := d.db.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
-			config, err = clusterConfig.Load(tx)
+			config, err = clusterConfig.Load(ctx, tx)
 			return err
 		})
 		if err != nil {
@@ -502,6 +502,9 @@ func api10Patch(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
+	// Don't apply changes to settings until daemon is fully started.
+	<-d.waitReady.Done()
+
 	render, err := daemonConfigRender(d.State())
 	if err != nil {
 		return response.InternalError(err)
@@ -546,16 +549,16 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 
 	nodeChanged := map[string]string{}
 	var newNodeConfig *node.Config
-	err = d.db.Node.Transaction(func(tx *db.NodeTx) error {
+	err = d.db.Node.Transaction(r.Context(), func(ctx context.Context, tx *db.NodeTx) error {
 		var err error
-		newNodeConfig, err = node.ConfigLoad(tx)
+		newNodeConfig, err = node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return fmt.Errorf("Failed to load node config: %w", err)
 		}
 
 		// We currently don't allow changing the cluster.https_address once it's set.
 		if clustered {
-			curConfig, err := tx.Config()
+			curConfig, err := tx.Config(ctx)
 			if err != nil {
 				return fmt.Errorf("Cannot fetch node config from database: %w", err)
 			}
@@ -576,14 +579,14 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		if nodeValues["storage.backups_volume"] != nil && nodeValues["storage.backups_volume"] != newNodeConfig.StorageBackupsVolume() {
 			err := daemonStorageValidate(s, nodeValues["storage.backups_volume"].(string))
 			if err != nil {
-				return err
+				return fmt.Errorf("Failed validation of %q: %w", "storage.backups_volume", err)
 			}
 		}
 
 		if nodeValues["storage.images_volume"] != nil && nodeValues["storage.images_volume"] != newNodeConfig.StorageImagesVolume() {
 			err := daemonStorageValidate(s, nodeValues["storage.images_volume"].(string))
 			if err != nil {
-				return err
+				return fmt.Errorf("Failed validation of %q: %w", "storage.images_volume", err)
 			}
 		}
 
@@ -626,9 +629,9 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 	// Then deal with cluster wide configuration
 	var clusterChanged map[string]string
 	var newClusterConfig *clusterConfig.Config
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.db.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
-		newClusterConfig, err = clusterConfig.Load(tx)
+		newClusterConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return fmt.Errorf("Failed to load cluster config: %w", err)
 		}
@@ -672,13 +675,14 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		return client.UpdateServer(serverPut, etag)
 	})
 	if err != nil {
-		logger.Debugf("Failed to notify other nodes about config change: %v", err)
+		logger.Error("Failed to notify other members about config change", logger.Ctx{"err": err})
 		return response.SmartError(err)
 	}
 
 	// Update the daemon config.
 	d.globalConfigMu.Lock()
 	d.globalConfig = newClusterConfig
+	d.localConfig = newNodeConfig
 	d.globalConfigMu.Unlock()
 
 	// Run any update triggers.
@@ -693,9 +697,6 @@ func doApi10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 }
 
 func doApi10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, nodeConfig *node.Config, clusterConfig *clusterConfig.Config) error {
-	// Don't apply changes to settings until daemon is full started.
-	<-d.readyChan
-
 	s := d.State()
 
 	maasChanged := false
