@@ -7,7 +7,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"net/url"
 	"os"
@@ -151,13 +151,13 @@ type internalWarningCreatePost struct {
 
 // internalCreateWarning creates a warning, and is used for testing only.
 func internalCreateWarning(d *Daemon, r *http.Request) response.Response {
-	body, err := ioutil.ReadAll(r.Body)
+	body, err := io.ReadAll(r.Body)
 	if err != nil {
 		return response.InternalError(err)
 	}
 
-	rdr1 := ioutil.NopCloser(bytes.NewBuffer(body))
-	rdr2 := ioutil.NopCloser(bytes.NewBuffer(body))
+	rdr1 := io.NopCloser(bytes.NewBuffer(body))
+	rdr2 := io.NopCloser(bytes.NewBuffer(body))
 
 	reqRaw := shared.Jmap{}
 	err = json.NewDecoder(rdr1).Decode(&reqRaw)
@@ -221,9 +221,7 @@ func internalWaitReady(d *Daemon, r *http.Request) response.Response {
 		return response.Unavailable(fmt.Errorf("LXD daemon is shutting down"))
 	}
 
-	select {
-	case <-d.readyChan:
-	default:
+	if d.waitReady.Err() == nil {
 		return response.Unavailable(fmt.Errorf("LXD daemon not ready yet"))
 	}
 
@@ -239,13 +237,14 @@ func internalShutdown(d *Daemon, r *http.Request) response.Response {
 	}
 
 	forceCtx, forceCtxCancel := context.WithCancel(context.Background())
-	defer forceCtxCancel()
 
 	if force == "true" {
 		forceCtxCancel() // Don't wait for operations to finish.
 	}
 
 	return response.ManualResponse(func(w http.ResponseWriter) error {
+		defer forceCtxCancel()
+
 		<-d.setupChan // Wait for daemon to start.
 
 		// Run shutdown sequence synchronously.
@@ -293,13 +292,17 @@ func internalContainerHookLoadFromReference(s *state.State, r *http.Request) (in
 	} else {
 		inst, err = instance.LoadByProjectAndName(s, projectName, instanceRef)
 		if err != nil {
+			if api.StatusErrorCheck(err, http.StatusNotFound) {
+				return nil, err
+			}
+
 			// If DB not available, try loading from backup file.
 			logger.Warn("Failed loading instance from database, trying backup file", logger.Ctx{"project": projectName, "instance": instanceRef, "err": err})
 
 			instancePath := filepath.Join(shared.VarPath("containers"), project.Instance(projectName, instanceRef))
 			inst, err = instance.LoadFromBackup(s, projectName, instancePath, false)
 			if err != nil {
-				return inst, err
+				return nil, err
 			}
 		}
 	}
@@ -688,13 +691,21 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 	}
 
 	// Check if a storage volume entry for the instance already exists.
-	_, volume, ctVolErr := d.db.Cluster.GetLocalStoragePoolVolume(projectName, backupConf.Container.Name, instanceDBVolType, pool.ID())
-	if ctVolErr != nil && !response.IsNotFoundError(ctVolErr) {
-		return ctVolErr
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, backupConf.Container.Name, true)
+		if err != nil && !response.IsNotFoundError(err) {
+			return err
+		}
+
+		return nil
+	})
+	if err != nil {
+		return err
 	}
 
 	// If a storage volume entry exists only proceed if force was specified.
-	if ctVolErr == nil && !force {
+	if dbVolume != nil && !force {
 		return fmt.Errorf(`Storage volume for instance %q already exists in the database. Set "force" to overwrite`, backupConf.Container.Name)
 	}
 
@@ -713,13 +724,13 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		return fmt.Errorf(`No storage volume struct in the backup file found. The storage volume needs to be recovered manually`)
 	}
 
-	if ctVolErr == nil {
-		if volume.Name != backupConf.Volume.Name {
-			return fmt.Errorf(`The name %q of the storage volume is not identical to the instance's name "%s"`, volume.Name, backupConf.Container.Name)
+	if dbVolume != nil {
+		if dbVolume.Name != backupConf.Volume.Name {
+			return fmt.Errorf(`The name %q of the storage volume is not identical to the instance's name "%s"`, dbVolume.Name, backupConf.Container.Name)
 		}
 
-		if volume.Type != backupConf.Volume.Type {
-			return fmt.Errorf(`The type %q of the storage volume is not identical to the instance's type %q`, volume.Type, backupConf.Volume.Type)
+		if dbVolume.Type != backupConf.Volume.Type {
+			return fmt.Errorf(`The type %q of the storage volume is not identical to the instance's type %q`, dbVolume.Type, backupConf.Volume.Type)
 		}
 
 		// Remove the storage volume db entry for the instance since force was specified.
@@ -799,13 +810,21 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 		}
 
 		// Check if a storage volume entry for the snapshot already exists.
-		_, _, csVolErr := d.db.Cluster.GetLocalStoragePoolVolume(projectName, snapInstName, instanceDBVolType, pool.ID())
-		if csVolErr != nil && !response.IsNotFoundError(csVolErr) {
-			return csVolErr
+		var dbVolume *db.StorageVolume
+		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, snapInstName, true)
+			if err != nil && !response.IsNotFoundError(err) {
+				return err
+			}
+
+			return nil
+		})
+		if err != nil {
+			return err
 		}
 
 		// If a storage volume entry exists only proceed if force was specified.
-		if csVolErr == nil && !force {
+		if dbVolume != nil && !force {
 			return fmt.Errorf(`Storage volume for snapshot %q already exists in the database. Set "force" to overwrite`, snapInstName)
 		}
 
@@ -816,7 +835,7 @@ func internalImportFromBackup(d *Daemon, projectName string, instName string, fo
 			}
 		}
 
-		if csVolErr == nil {
+		if dbVolume != nil {
 			err := d.db.Cluster.RemoveStoragePoolVolume(projectName, snapInstName, instanceDBVolType, pool.ID())
 			if err != nil {
 				return err
