@@ -5,7 +5,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net"
 	"net/http"
 	"net/url"
@@ -140,8 +140,7 @@ func projectsGet(d *Daemon, r *http.Request) response.Response {
 
 	var result any
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		filter := cluster.ProjectFilter{}
-		projects, err := cluster.GetProjects(ctx, tx.Tx(), filter)
+		projects, err := cluster.GetProjects(ctx, tx.Tx())
 		if err != nil {
 			return err
 		}
@@ -219,17 +218,17 @@ func projectUsedBy(ctx context.Context, tx *db.ClusterTx, project *cluster.Proje
 		usedBy = append(usedBy, apiImage.URL(version.APIVersion, project.Name).String())
 	}
 
-	volumes, err := tx.GetStorageVolumeURIs(project.Name)
+	volumes, err := tx.GetStorageVolumeURIs(ctx, project.Name)
 	if err != nil {
 		return nil, err
 	}
 
-	networks, err := tx.GetNetworkURIs(project.ID, project.Name)
+	networks, err := tx.GetNetworkURIs(ctx, project.ID, project.Name)
 	if err != nil {
 		return nil, err
 	}
 
-	acls, err := tx.GetNetworkACLURIs(project.ID, project.Name)
+	acls, err := tx.GetNetworkACLURIs(ctx, project.ID, project.Name)
 	if err != nil {
 		return nil, err
 	}
@@ -277,10 +276,10 @@ func projectsPost(d *Daemon, r *http.Request) response.Response {
 		project.Config = map[string]string{}
 	}
 
-	for _, feature := range cluster.ProjectFeaturesDefaults {
-		_, ok := project.Config[feature]
-		if !ok {
-			project.Config[feature] = "true"
+	for featureName, featureInfo := range cluster.ProjectFeatures {
+		_, ok := project.Config[featureName]
+		if !ok && featureInfo.DefaultEnabled {
+			project.Config[featureName] = "true"
 		}
 	}
 
@@ -600,13 +599,13 @@ func projectPatch(d *Daemon, r *http.Request) response.Response {
 		return response.PreconditionFailed(err)
 	}
 
-	body, err := ioutil.ReadAll(r.Body)
+	body, err := io.ReadAll(r.Body)
 	if err != nil {
 		return response.InternalError(err)
 	}
 
-	rdr1 := ioutil.NopCloser(bytes.NewBuffer(body))
-	rdr2 := ioutil.NopCloser(bytes.NewBuffer(body))
+	rdr1 := io.NopCloser(bytes.NewBuffer(body))
+	rdr2 := io.NopCloser(bytes.NewBuffer(body))
 
 	reqRaw := shared.Jmap{}
 	err = json.NewDecoder(rdr1).Decode(&reqRaw)
@@ -661,24 +660,39 @@ func projectChange(d *Daemon, project *api.Project, req api.ProjectPut) response
 		}
 	}
 
-	// Flag indicating if any feature has changed.
-	featuresChanged := false
-	for _, featureKey := range cluster.ProjectFeatures {
-		if shared.StringInSlice(featureKey, configChanged) {
-			featuresChanged = true
-			break
+	// Record which features have been changed.
+	var featuresChanged []string
+	for _, configKeyChanged := range configChanged {
+		_, isFeature := cluster.ProjectFeatures[configKeyChanged]
+		if isFeature {
+			featuresChanged = append(featuresChanged, configKeyChanged)
 		}
 	}
 
 	// Quick checks.
-	if project.Name == projecthelpers.Default && featuresChanged {
-		return response.BadRequest(fmt.Errorf("You can't change the features of the default project"))
-	}
+	if len(featuresChanged) > 0 {
+		if project.Name == projecthelpers.Default {
+			return response.BadRequest(fmt.Errorf("You can't change the features of the default project"))
+		}
 
-	if featuresChanged && len(project.UsedBy) > 0 {
 		// Consider the project empty if it is only used by the default profile.
-		if len(project.UsedBy) > 1 || !strings.Contains(project.UsedBy[0], "/profiles/default") {
-			return response.BadRequest(fmt.Errorf("Features can only be changed on empty projects"))
+		usedByLen := len(project.UsedBy)
+		projectInUse := usedByLen > 1 || (usedByLen == 1 && !strings.Contains(project.UsedBy[0], "/profiles/default"))
+		if projectInUse {
+			// Check if feature is allowed to be changed.
+			for _, featureChanged := range featuresChanged {
+				// If feature is currently enabled, and it is being changed in the request, it
+				// must be being disabled. So prevent it on non-empty projects.
+				if shared.IsTrue(project.Config[featureChanged]) {
+					return response.BadRequest(fmt.Errorf("Project feature %q cannot be disabled on non-empty projects", featureChanged))
+				}
+
+				// If feature is currently disabled, and it is being changed in the request, it
+				// must be being enabled. So check if feature can be enabled on non-empty projects.
+				if shared.IsFalse(project.Config[featureChanged]) && !cluster.ProjectFeatures[featureChanged].CanEnableNonEmpty {
+					return response.BadRequest(fmt.Errorf("Project feature %q cannot be enabled on non-empty projects", featureChanged))
+				}
+			}
 		}
 	}
 
@@ -695,7 +709,7 @@ func projectChange(d *Daemon, project *api.Project, req api.ProjectPut) response
 			return err
 		}
 
-		err = cluster.UpdateProject(context.TODO(), tx.Tx(), project.Name, req)
+		err = cluster.UpdateProject(ctx, tx.Tx(), project.Name, req)
 		if err != nil {
 			return fmt.Errorf("Persist profile changes: %w", err)
 		}
@@ -708,13 +722,20 @@ func projectChange(d *Daemon, project *api.Project, req api.ProjectPut) response
 				}
 			} else {
 				// Delete the project-specific default profile.
-				err = cluster.DeleteProfile(context.TODO(), tx.Tx(), project.Name, projecthelpers.Default)
+				err = cluster.DeleteProfile(ctx, tx.Tx(), project.Name, projecthelpers.Default)
 				if err != nil {
 					return fmt.Errorf("Delete project default profile: %w", err)
 				}
 			}
 		}
 
+		if shared.StringInSlice("features.images", configChanged) && shared.IsFalse(req.Config["features.images"]) && shared.IsTrue(req.Config["features.profiles"]) {
+			err = cluster.InitProjectWithoutImages(ctx, tx.Tx(), project.Name)
+			if err != nil {
+				return err
+			}
+		}
+
 		return nil
 	})
 
@@ -744,8 +765,8 @@ func projectChange(d *Daemon, project *api.Project, req api.ProjectPut) response
 //     schema:
 //       $ref: "#/definitions/ProjectPost"
 // responses:
-//   "200":
-//     $ref: "#/responses/EmptySyncResponse"
+//   "202":
+//     $ref: "#/responses/Operation"
 //   "400":
 //     $ref: "#/responses/BadRequest"
 //   "403":
@@ -955,7 +976,7 @@ func projectStateGet(d *Daemon, r *http.Request) response.Response {
 
 	// Get current limits and usage.
 	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		result, err := projecthelpers.GetCurrentAllocations(tx, name)
+		result, err := projecthelpers.GetCurrentAllocations(ctx, tx, name)
 		if err != nil {
 			return err
 		}
@@ -1005,7 +1026,7 @@ func projectIsEmpty(ctx context.Context, project *cluster.Project, tx *db.Cluste
 		return false, nil
 	}
 
-	volumes, err := tx.GetStorageVolumeURIs(project.Name)
+	volumes, err := tx.GetStorageVolumeURIs(ctx, project.Name)
 	if err != nil {
 		return false, err
 	}
@@ -1014,7 +1035,7 @@ func projectIsEmpty(ctx context.Context, project *cluster.Project, tx *db.Cluste
 		return false, nil
 	}
 
-	networks, err := tx.GetNetworkURIs(project.ID, project.Name)
+	networks, err := tx.GetNetworkURIs(ctx, project.ID, project.Name)
 	if err != nil {
 		return false, err
 	}
@@ -1023,7 +1044,7 @@ func projectIsEmpty(ctx context.Context, project *cluster.Project, tx *db.Cluste
 		return false, nil
 	}
 
-	acls, err := tx.GetNetworkACLURIs(project.ID, project.Name)
+	acls, err := tx.GetNetworkACLURIs(ctx, project.ID, project.Name)
 	if err != nil {
 		return false, err
 	}
@@ -1086,6 +1107,7 @@ func projectValidateConfig(s *state.State, config map[string]string) error {
 		"restricted.devices.disk.paths":        validate.Optional(validate.IsListOf(validate.IsAbsFilePath)),
 		"restricted.idmap.uid":                 validate.Optional(validate.IsListOf(validate.IsUint32Range)),
 		"restricted.idmap.gid":                 validate.Optional(validate.IsListOf(validate.IsUint32Range)),
+		"restricted.networks.access":           validate.Optional(validate.IsListOf(validate.IsAny)),
 		"restricted.networks.uplinks":          validate.Optional(validate.IsListOf(validate.IsAny)),
 		"restricted.networks.subnets": validate.Optional(func(value string) error {
 			return projectValidateRestrictedSubnets(s, value)
