@@ -8,7 +8,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"net/url"
@@ -25,6 +24,7 @@ import (
 	"github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/response"
 	"github.com/lxc/lxd/lxd/revert"
+	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
@@ -41,7 +41,7 @@ import (
 // After creation, the Daemon is expected to expose whatever http handlers the
 // HandlerFuncs method returns and to access the dqlite cluster using the
 // dialer returned by the DialFunc method.
-func NewGateway(shutdownCtx context.Context, db *db.Node, networkCert *shared.CertInfo, serverCert func() *shared.CertInfo, options ...Option) (*Gateway, error) {
+func NewGateway(shutdownCtx context.Context, db *db.Node, networkCert *shared.CertInfo, stateFunc func() *state.State, options ...Option) (*Gateway, error) {
 	ctx, cancel := context.WithCancel(context.Background())
 
 	o := newOptions()
@@ -53,13 +53,13 @@ func NewGateway(shutdownCtx context.Context, db *db.Node, networkCert *shared.Ce
 		shutdownCtx: shutdownCtx,
 		db:          db,
 		networkCert: networkCert,
-		serverCert:  serverCert,
 		options:     o,
 		ctx:         ctx,
 		cancel:      cancel,
 		upgradeCh:   make(chan struct{}),
 		acceptCh:    make(chan net.Conn),
 		store:       &dqliteNodeStore{},
+		state:       stateFunc,
 	}
 
 	err := gateway.init(false)
@@ -82,7 +82,6 @@ type HeartbeatHandler func(w http.ResponseWriter, r *http.Request, isLeader bool
 type Gateway struct {
 	db          *db.Node
 	networkCert *shared.CertInfo
-	serverCert  func() *shared.CertInfo
 	options     *options
 
 	// The raft instance to use for creating the dqlite driver. It's nil if
@@ -133,6 +132,9 @@ type Gateway struct {
 
 	// Abstract unix socket that the local dqlite task is listening to.
 	bindAddress string
+
+	// State function.
+	state func() *state.State
 }
 
 // Current dqlite protocol version.
@@ -155,7 +157,7 @@ func setDqliteVersionHeader(request *http.Request) {
 func (g *Gateway) HandlerFuncs(heartbeatHandler HeartbeatHandler, trustedCerts func() map[cluster.CertificateType]map[string]x509.Certificate) map[string]http.HandlerFunc {
 	database := func(w http.ResponseWriter, r *http.Request) {
 		g.lock.RLock()
-		if !tlsCheckCert(r, g.networkCert, g.serverCert(), trustedCerts()) {
+		if !tlsCheckCert(r, g.networkCert, g.state().ServerCert(), trustedCerts()) {
 			g.lock.RUnlock()
 			http.Error(w, "403 invalid client certificate", http.StatusForbidden)
 			return
@@ -487,7 +489,7 @@ func (g *Gateway) TransferLeadership() error {
 			return err
 		}
 
-		if !HasConnectivity(g.networkCert, g.serverCert(), address) {
+		if !HasConnectivity(g.networkCert, g.state().ServerCert(), address) {
 			continue
 		}
 
@@ -576,7 +578,7 @@ func (g *Gateway) Sync() {
 	dir := filepath.Join(g.db.Dir(), "global")
 	for _, file := range files {
 		path := filepath.Join(dir, file.Name)
-		err := ioutil.WriteFile(path, file.Data, 0600)
+		err := os.WriteFile(path, file.Data, 0600)
 		if err != nil {
 			logger.Warnf("Failed to dump database file %s: %v", file.Name, err)
 		}
@@ -601,7 +603,7 @@ func (g *Gateway) Reset(networkCert *shared.CertInfo) error {
 		return err
 	}
 
-	err = g.db.Transaction(func(tx *db.NodeTx) error {
+	err = g.db.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		return tx.ReplaceRaftNodes(nil)
 	})
 	if err != nil {
@@ -661,14 +663,14 @@ func (g *Gateway) LeaderAddress() (string, error) {
 
 	// If this isn't a raft node, contact a raft node and ask for the
 	// address of the current leader.
-	config, err := tlsClientConfig(g.networkCert, g.serverCert())
+	config, err := tlsClientConfig(g.networkCert, g.state().ServerCert())
 	if err != nil {
 		return "", err
 	}
 
 	addresses := []string{}
-	err = g.db.Transaction(func(tx *db.NodeTx) error {
-		nodes, err := tx.GetRaftNodes()
+	err = g.db.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		nodes, err := tx.GetRaftNodes(ctx)
 		if err != nil {
 			return err
 		}
@@ -957,23 +959,23 @@ func (g *Gateway) currentRaftNodes() ([]db.RaftNode, error) {
 	// Get the names of the raft nodes from the global database.
 	if g.Cluster != nil {
 		err = g.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			nodes, err := tx.GetNodes()
+			members, err := tx.GetNodes(ctx)
 			if err != nil {
-				return fmt.Errorf("Failed loading cluster members: %w", err)
+				return fmt.Errorf("Failed getting cluster members: %w", err)
 			}
 
-			nodesByAddress := make(map[string]db.NodeInfo, len(nodes))
-			for _, node := range nodes {
-				nodesByAddress[node.Address] = node
+			membersByAddress := make(map[string]db.NodeInfo, len(members))
+			for _, member := range members {
+				membersByAddress[member.Address] = member
 			}
 
 			for i, server := range servers {
-				node, found := nodesByAddress[server.Address]
+				member, found := membersByAddress[server.Address]
 				if !found {
 					logger.Warn("Cluster member info not found", logger.Ctx{"address": server.Address})
 				}
 
-				raftNodes[i].Name = node.Name
+				raftNodes[i].Name = member.Name
 			}
 
 			return nil
@@ -994,9 +996,9 @@ func (g *Gateway) nodeAddress(raftAddress string) (string, error) {
 	}
 
 	var address string
-	err := g.db.Transaction(func(tx *db.NodeTx) error {
+	err := g.db.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		var err error
-		address, err = tx.GetRaftNodeAddress(1)
+		address, err = tx.GetRaftNodeAddress(ctx, 1)
 		if err != nil {
 			if !response.IsNotFoundError(err) {
 				return fmt.Errorf("Failed to fetch raft server address: %w", err)
@@ -1017,7 +1019,7 @@ func (g *Gateway) nodeAddress(raftAddress string) (string, error) {
 }
 
 func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway) (net.Conn, error) {
-	config, err := tlsClientConfig(g.networkCert, g.serverCert())
+	config, err := tlsClientConfig(g.networkCert, g.state().ServerCert())
 	if err != nil {
 		return nil, err
 	}
@@ -1063,7 +1065,7 @@ func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway
 	if err != nil {
 		l.Error("Failed extracting TCP connection from remote connection", logger.Ctx{"err": err})
 	} else {
-		err := tcp.SetTimeouts(remoteTCP)
+		err := tcp.SetTimeouts(remoteTCP, time.Second*30)
 		if err != nil {
 			l.Error("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
 		}
@@ -1156,7 +1158,7 @@ func dqliteProxy(name string, stopCh chan struct{}, remote net.Conn, local net.C
 	if err != nil {
 		l.Error("Failed extracting TCP connection from remote connection", logger.Ctx{"err": err})
 	} else {
-		err := tcp.SetTimeouts(remoteTCP)
+		err := tcp.SetTimeouts(remoteTCP, time.Second*30)
 		if err != nil {
 			l.Error("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
 		}
