@@ -15,7 +15,6 @@ import (
 	"github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/lxc/lxd/lxd/db/warningtype"
-	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/response"
 	"github.com/lxc/lxd/lxd/task"
 	"github.com/lxc/lxd/lxd/warnings"
@@ -129,7 +128,7 @@ func (hbState *APIHeartbeat) Update(fullStateList bool, raftNodes []db.RaftNode,
 	if len(raftNodeMap) > 0 && hbState.cluster != nil {
 		_ = hbState.cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			for addr, raftNode := range raftNodeMap {
-				_, err := tx.GetPendingNodeByAddress(addr)
+				_, err := tx.GetPendingNodeByAddress(ctx, addr)
 				if err != nil {
 					logger.Errorf("Unaccounted raft node(s) not found in 'nodes' table for heartbeat: %+v", raftNode)
 				}
@@ -320,15 +319,16 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 	}
 
 	// Address of this node.
-	localAddress, err := node.ClusterAddress(g.db)
-	if err != nil {
-		logger.Error("Failed to fetch local cluster address", logger.Ctx{"err": err})
+	var localClusterAddress string
+	s := g.state()
+
+	if s.LocalConfig != nil {
+		localClusterAddress = s.LocalConfig.ClusterAddress()
 	}
 
-	var allNodes []db.NodeInfo
+	var members []db.NodeInfo
 	err = g.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		var err error
-		allNodes, err = tx.GetNodes()
+		members, err = tx.GetNodes(ctx)
 		if err != nil {
 			return err
 		}
@@ -350,10 +350,10 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 
 	if mode != hearbeatNormal {
 		// Log unscheduled heartbeats with a higher level than normal heartbeats.
-		logger.Info("Starting heartbeat round", logger.Ctx{"mode": modeStr, "local": localAddress})
+		logger.Info("Starting heartbeat round", logger.Ctx{"mode": modeStr, "local": localClusterAddress})
 	} else {
 		// Don't spam the normal log with regular heartbeat messages.
-		logger.Debug("Starting heartbeat round", logger.Ctx{"mode": modeStr, "local": localAddress})
+		logger.Debug("Starting heartbeat round", logger.Ctx{"mode": modeStr, "local": localClusterAddress})
 	}
 
 	// Replace the local raft_nodes table immediately because it
@@ -361,15 +361,15 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 	// been elected leader before the former leader had chance to
 	// send us a fresh update through the heartbeat pool.
 	logger.Debug("Heartbeat updating local raft members", logger.Ctx{"members": raftNodes})
-	err = g.db.Transaction(func(tx *db.NodeTx) error {
+	err = g.db.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		return tx.ReplaceRaftNodes(raftNodes)
 	})
 	if err != nil {
-		logger.Warn("Failed to replace local raft members", logger.Ctx{"err": err, "mode": modeStr, "local": localAddress})
+		logger.Warn("Failed to replace local raft members", logger.Ctx{"err": err, "mode": modeStr, "local": localClusterAddress})
 		return
 	}
 
-	if localAddress == "" {
+	if localClusterAddress == "" {
 		logger.Error("No local address set, aborting heartbeat round", logger.Ctx{"mode": modeStr})
 		return
 	}
@@ -388,19 +388,21 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 		spreadDuration = heartbeatInterval
 	}
 
+	serverCert := g.state().ServerCert()
+
 	// If this leader node hasn't sent a heartbeat recently, then its node state records
 	// are likely out of date, this can happen when a node becomes a leader.
 	// Send stale set to all nodes in database to get a fresh set of active nodes.
 	if mode == hearbeatInitial {
-		hbState.Update(false, raftNodes, allNodes, g.HeartbeatOfflineThreshold)
-		hbState.Send(ctx, g.networkCert, g.serverCert(), localAddress, allNodes, spreadDuration)
+		hbState.Update(false, raftNodes, members, g.HeartbeatOfflineThreshold)
+		hbState.Send(ctx, g.networkCert, serverCert, localClusterAddress, members, spreadDuration)
 
 		// We have the latest set of node states now, lets send that state set to all nodes.
 		hbState.FullStateList = true
-		hbState.Send(ctx, g.networkCert, g.serverCert(), localAddress, allNodes, spreadDuration)
+		hbState.Send(ctx, g.networkCert, serverCert, localClusterAddress, members, spreadDuration)
 	} else {
-		hbState.Update(true, raftNodes, allNodes, g.HeartbeatOfflineThreshold)
-		hbState.Send(ctx, g.networkCert, g.serverCert(), localAddress, allNodes, spreadDuration)
+		hbState.Update(true, raftNodes, members, g.HeartbeatOfflineThreshold)
+		hbState.Send(ctx, g.networkCert, serverCert, localClusterAddress, members, spreadDuration)
 	}
 
 	// Check if context has been cancelled.
@@ -408,10 +410,10 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 
 	// Look for any new node which appeared since sending last heartbeat.
 	if ctxErr == nil {
-		var currentNodes []db.NodeInfo
+		var currentMembers []db.NodeInfo
 		err = g.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
-			currentNodes, err = tx.GetNodes()
+			currentMembers, err = tx.GetNodes(ctx)
 			if err != nil {
 				return err
 			}
@@ -419,15 +421,15 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 			return nil
 		})
 		if err != nil {
-			logger.Warn("Failed to get current cluster members", logger.Ctx{"err": err, "mode": modeStr, "local": localAddress})
+			logger.Warn("Failed to get current cluster members", logger.Ctx{"err": err, "mode": modeStr, "local": localClusterAddress})
 			return
 		}
 
-		newNodes := []db.NodeInfo{}
-		for _, currentNode := range currentNodes {
+		newMembers := []db.NodeInfo{}
+		for _, currentMember := range currentMembers {
 			existing := false
-			for _, node := range allNodes {
-				if node.Address == currentNode.Address && node.ID == currentNode.ID {
+			for _, member := range members {
+				if member.Address == currentMember.Address && member.ID == currentMember.ID {
 					existing = true
 					break
 				}
@@ -435,15 +437,15 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 
 			if !existing {
 				// We found a new node
-				allNodes = append(allNodes, currentNode)
-				newNodes = append(newNodes, currentNode)
+				members = append(members, currentMember)
+				newMembers = append(newMembers, currentMember)
 			}
 		}
 
 		// If any new nodes found, send heartbeat to just them (with full node state).
-		if len(newNodes) > 0 {
-			hbState.Update(true, raftNodes, allNodes, g.HeartbeatOfflineThreshold)
-			hbState.Send(ctx, g.networkCert, g.serverCert(), localAddress, newNodes, 0)
+		if len(newMembers) > 0 {
+			hbState.Update(true, raftNodes, members, g.HeartbeatOfflineThreshold)
+			hbState.Send(ctx, g.networkCert, serverCert, localClusterAddress, newMembers, 0)
 		}
 	}
 
@@ -483,7 +485,7 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 
 	// If the context has been cancelled, return prematurely after saving the members we did manage to ping.
 	if ctxErr != nil {
-		logger.Warn("Aborting heartbeat round", logger.Ctx{"err": ctxErr, "mode": modeStr, "local": localAddress})
+		logger.Warn("Aborting heartbeat round", logger.Ctx{"err": ctxErr, "mode": modeStr, "local": localClusterAddress})
 		return
 	}
 
@@ -499,10 +501,10 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 
 	if mode != hearbeatNormal {
 		// Log unscheduled heartbeats with a higher level than normal heartbeats.
-		logger.Info("Completed heartbeat round", logger.Ctx{"duration": duration, "local": localAddress})
+		logger.Info("Completed heartbeat round", logger.Ctx{"duration": duration, "local": localClusterAddress})
 	} else {
 		// Don't spam the normal log with regular heartbeat messages.
-		logger.Debug("Completed heartbeat round", logger.Ctx{"duration": duration, "local": localAddress})
+		logger.Debug("Completed heartbeat round", logger.Ctx{"duration": duration, "local": localClusterAddress})
 	}
 }
 
