@@ -16,6 +16,7 @@ import (
 	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
 	"github.com/lxc/lxd/lxd/db"
 	clusterDB "github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/osarch"
@@ -39,10 +40,10 @@ func TestHeartbeat(t *testing.T) {
 
 	// Artificially mark all nodes as down
 	err := leaderState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		require.NoError(t, err)
-		for _, node := range nodes {
-			err := tx.SetNodeHeartbeat(node.Address, time.Now().Add(-time.Minute))
+		for _, member := range members {
+			err := tx.SetNodeHeartbeat(member.Address, time.Now().Add(-time.Minute))
 			require.NoError(t, err)
 		}
 
@@ -58,14 +59,14 @@ func TestHeartbeat(t *testing.T) {
 
 	// The heartbeat timestamps of all nodes got updated
 	err = leaderState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		require.NoError(t, err)
 
-		offlineThreshold, err := tx.GetNodeOfflineThreshold()
+		offlineThreshold, err := tx.GetNodeOfflineThreshold(ctx)
 		require.NoError(t, err)
 
-		for _, node := range nodes {
-			assert.False(t, node.IsOffline(offlineThreshold))
+		for _, member := range members {
+			assert.False(t, member.IsOffline(offlineThreshold))
 		}
 
 		return nil
@@ -206,7 +207,8 @@ func (f *heartbeatFixture) node() (*state.State, *cluster.Gateway, string) {
 
 	serverCert := shared.TestingKeyPair()
 	state.ServerCert = func() *shared.CertInfo { return serverCert }
-	gateway := newGateway(f.t, state.DB.Node, serverCert, serverCert)
+
+	gateway := newGateway(f.t, state.DB.Node, serverCert, state)
 	f.cleanups = append(f.cleanups, func() { _ = gateway.Shutdown() })
 
 	mux := http.NewServeMux()
@@ -232,13 +234,13 @@ func (f *heartbeatFixture) node() (*state.State, *cluster.Gateway, string) {
 	require.NoError(f.t, err)
 
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		state.GlobalConfig, err = clusterConfig.Load(tx)
+		state.GlobalConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return err
 		}
 
 		// Get the local node (will be used if clustered).
-		state.ServerName, err = tx.GetLocalNodeName()
+		state.ServerName, err = tx.GetLocalNodeName(ctx)
 		if err != nil {
 			return err
 		}
@@ -247,6 +249,12 @@ func (f *heartbeatFixture) node() (*state.State, *cluster.Gateway, string) {
 	})
 	require.NoError(f.t, err)
 
+	err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		state.LocalConfig, err = node.ConfigLoad(ctx, tx)
+		return err
+	})
+	require.NoError(f.t, err)
+
 	f.gateways[len(f.gateways)] = gateway
 	f.states[gateway] = state
 	f.servers[gateway] = server
