@@ -39,25 +39,25 @@ func Bootstrap(state *state.State, gateway *Gateway, serverName string) error {
 		return err
 	}
 
-	var address string
+	var localClusterAddress string
 
-	err = state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		// Fetch current network address and raft nodes
-		config, err := node.ConfigLoad(tx)
+		config, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return fmt.Errorf("Failed to fetch node configuration: %w", err)
 		}
 
-		address = config.ClusterAddress()
+		localClusterAddress = config.ClusterAddress()
 
 		// Make sure node-local database state is in order.
-		err = membershipCheckNodeStateForBootstrapOrJoin(tx, address)
+		err = membershipCheckNodeStateForBootstrapOrJoin(ctx, tx, localClusterAddress)
 		if err != nil {
 			return err
 		}
 
 		// Add ourselves as first raft node
-		err = tx.CreateFirstRaftNode(address, serverName)
+		err = tx.CreateFirstRaftNode(localClusterAddress, serverName)
 		if err != nil {
 			return fmt.Errorf("Failed to insert first raft node: %w", err)
 		}
@@ -71,13 +71,13 @@ func Bootstrap(state *state.State, gateway *Gateway, serverName string) error {
 	// Update our own entry in the nodes table.
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Make sure cluster database state is in order.
-		err := membershipCheckClusterStateForBootstrapOrJoin(tx)
+		err := membershipCheckClusterStateForBootstrapOrJoin(ctx, tx)
 		if err != nil {
 			return err
 		}
 
 		// Add ourselves to the nodes table.
-		err = tx.BootstrapNode(serverName, address)
+		err = tx.BootstrapNode(serverName, localClusterAddress)
 		if err != nil {
 			return fmt.Errorf("Failed updating cluster member: %w", err)
 		}
@@ -151,8 +151,12 @@ func Bootstrap(state *state.State, gateway *Gateway, serverName string) error {
 	// connection, so new queries will be executed over the new network
 	// connection.
 	err = state.DB.Cluster.ExitExclusive(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		_, err := tx.GetNodes()
-		return err
+		_, err := tx.GetNodes(ctx)
+		if err != nil {
+			return fmt.Errorf("Failed getting cluster members: %w", err)
+		}
+
+		return nil
 	})
 	if err != nil {
 		return fmt.Errorf("Cluster database initialization failed: %w", err)
@@ -230,7 +234,7 @@ func Accept(state *state.State, gateway *Gateway, name, address string, schema,
 	var id int64
 	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Check that the node can be accepted with these parameters.
-		err := membershipCheckClusterStateForAccept(tx, name, address, schema, api)
+		err := membershipCheckClusterStateForAccept(ctx, tx, name, address, schema, api)
 		if err != nil {
 			return err
 		}
@@ -307,18 +311,18 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 		return fmt.Errorf("Member name must not be empty")
 	}
 
-	var address string
-	err := state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	var localClusterAddress string
+	err := state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		// Fetch current network address and raft nodes
-		config, err := node.ConfigLoad(tx)
+		config, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return fmt.Errorf("Failed to fetch node configuration: %w", err)
 		}
 
-		address = config.ClusterAddress()
+		localClusterAddress = config.ClusterAddress()
 
 		// Make sure node-local database state is in order.
-		err = membershipCheckNodeStateForBootstrapOrJoin(tx, address)
+		err = membershipCheckNodeStateForBootstrapOrJoin(ctx, tx, localClusterAddress)
 		if err != nil {
 			return err
 		}
@@ -345,12 +349,12 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 	var operations []cluster.Operation
 
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		pools, err = tx.GetStoragePoolsLocalConfig()
+		pools, err = tx.GetStoragePoolsLocalConfig(ctx)
 		if err != nil {
 			return err
 		}
 
-		networks, err = tx.GetNetworksLocalConfig()
+		networks, err = tx.GetNetworksLocalConfig(ctx)
 		if err != nil {
 			return err
 		}
@@ -400,7 +404,7 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 	// If we are listed among the database nodes, join the raft cluster.
 	var info *db.RaftNode
 	for _, node := range raftNodes {
-		if node.Address == address {
+		if node.Address == localClusterAddress {
 			info = &node
 		}
 	}
@@ -439,7 +443,7 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 	// tables with our local configuration.
 	logger.Info("Migrate local data to cluster database")
 	err = state.DB.Cluster.ExitExclusive(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		node, err := tx.GetPendingNodeByAddress(address)
+		node, err := tx.GetPendingNodeByAddress(ctx, localClusterAddress)
 		if err != nil {
 			return fmt.Errorf("Failed to get ID of joining node: %w", err)
 		}
@@ -448,7 +452,7 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 		tx.NodeID(node.ID)
 
 		// Storage pools.
-		ids, err := tx.GetNonPendingStoragePoolsNamesToIDs()
+		ids, err := tx.GetNonPendingStoragePoolsNamesToIDs(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed to get cluster storage pool IDs: %w", err)
 		}
@@ -459,7 +463,7 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 				return fmt.Errorf("Failed to add joining node's to the pool: %w", err)
 			}
 
-			driver, err := tx.GetStoragePoolDriver(id)
+			driver, err := tx.GetStoragePoolDriver(ctx, id)
 			if err != nil {
 				return fmt.Errorf("Failed to get storage pool driver: %w", err)
 			}
@@ -467,7 +471,7 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 			if shared.StringInSlice(driver, []string{"ceph", "cephfs"}) {
 				// For ceph pools we have to create volume
 				// entries for the joining node.
-				err := tx.UpdateCephStoragePoolAfterNodeJoin(id, node.ID)
+				err := tx.UpdateCephStoragePoolAfterNodeJoin(ctx, id, node.ID)
 				if err != nil {
 					return fmt.Errorf("Failed to create ceph volumes for joining node: %w", err)
 				}
@@ -486,7 +490,7 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 		}
 
 		// Networks.
-		netids, err := tx.GetNonPendingNetworkIDs()
+		netids, err := tx.GetNonPendingNetworkIDs(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed to get cluster network IDs: %w", err)
 		}
@@ -571,30 +575,30 @@ func NotifyHeartbeat(state *state.State, gateway *Gateway) {
 
 	var err error
 	var raftNodes []db.RaftNode
-	var localAddress string
-	err = state.DB.Node.Transaction(func(tx *db.NodeTx) error {
-		raftNodes, err = tx.GetRaftNodes()
+	var localClusterAddress string
+	err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		raftNodes, err = tx.GetRaftNodes(ctx)
 		if err != nil {
 			return err
 		}
 
-		config, err := node.ConfigLoad(tx)
+		config, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
 		}
 
-		localAddress = config.ClusterAddress()
+		localClusterAddress = config.ClusterAddress()
 
 		return nil
 	})
 	if err != nil {
-		logger.Warn("Failed to get current raft members", logger.Ctx{"err": err, "local": localAddress})
+		logger.Warn("Failed to get current raft members", logger.Ctx{"err": err, "local": localClusterAddress})
 		return
 	}
 
-	var allNodes []db.NodeInfo
+	var members []db.NodeInfo
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		allNodes, err = tx.GetNodes()
+		members, err = tx.GetNodes(ctx)
 		if err != nil {
 			return err
 		}
@@ -602,12 +606,12 @@ func NotifyHeartbeat(state *state.State, gateway *Gateway) {
 		return nil
 	})
 	if err != nil {
-		logger.Warn("Failed to get current cluster members", logger.Ctx{"err": err, "local": localAddress})
+		logger.Warn("Failed to get current cluster members", logger.Ctx{"err": err, "local": localClusterAddress})
 		return
 	}
 
 	// Setup a full-state notification heartbeat.
-	hbState.Update(true, raftNodes, allNodes, gateway.HeartbeatOfflineThreshold)
+	hbState.Update(true, raftNodes, members, gateway.HeartbeatOfflineThreshold)
 
 	var wg sync.WaitGroup
 
@@ -619,9 +623,9 @@ func NotifyHeartbeat(state *state.State, gateway *Gateway) {
 	}()
 
 	// Notify all other members of the change in membership.
-	logger.Info("Sending member change notification heartbeat to all members", logger.Ctx{"local": localAddress})
-	for _, node := range allNodes {
-		if node.Address == localAddress {
+	logger.Info("Sending member change notification heartbeat to all members", logger.Ctx{"local": localClusterAddress})
+	for _, member := range members {
+		if member.Address == localClusterAddress {
 			continue
 		}
 
@@ -629,7 +633,7 @@ func NotifyHeartbeat(state *state.State, gateway *Gateway) {
 		go func(address string) {
 			_ = HeartbeatNode(context.Background(), address, state.Endpoints.NetworkCert(), state.ServerCert(), hbState)
 			wg.Done()
-		}(node.Address)
+		}(member.Address)
 	}
 
 	// Wait until all members have been notified (or at least have had a change to be notified).
@@ -665,14 +669,11 @@ func Rebalance(state *state.State, gateway *Gateway, unavailableMembers []string
 		return "", nodes, nil
 	}
 
-	address, err := node.ClusterAddress(state.DB.Node)
-	if err != nil {
-		return "", nil, err
-	}
+	localClusterAddress := state.LocalConfig.ClusterAddress()
 
 	// Check if we have a spare node that we can promote to the missing role.
 	candidateAddress := candidates[0].Address
-	logger.Info("Found cluster member whose role needs to be changed", logger.Ctx{"candidateAddress": candidateAddress, "newRole": role, "local": address})
+	logger.Info("Found cluster member whose role needs to be changed", logger.Ctx{"candidateAddress": candidateAddress, "newRole": role, "local": localClusterAddress})
 
 	for i, node := range nodes {
 		if node.Address == candidateAddress {
@@ -690,7 +691,7 @@ func Assign(state *state.State, gateway *Gateway, nodes []db.RaftNode) error {
 	address := ""
 	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
-		address, err = tx.GetLocalNodeAddress()
+		address, err = tx.GetLocalNodeAddress(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed to fetch the address of this cluster member: %w", err)
 		}
@@ -721,7 +722,7 @@ func Assign(state *state.State, gateway *Gateway, nodes []db.RaftNode) error {
 
 	// Replace our local list of raft nodes with the given one (which
 	// includes ourselves).
-	err = state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		err = tx.ReplaceRaftNodes(nodes)
 		if err != nil {
 			return fmt.Errorf("Failed to set raft nodes: %w", err)
@@ -893,14 +894,14 @@ func Leave(state *state.State, gateway *Gateway, name string, force bool) (strin
 	var address string
 	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get the node (if it doesn't exists an error is returned).
-		node, err := tx.GetNodeByName(name)
+		node, err := tx.GetNodeByName(ctx, name)
 		if err != nil {
 			return err
 		}
 
 		// Check that the node is eligeable for leaving.
 		if !force {
-			err := membershipCheckClusterStateForLeave(tx, node.ID)
+			err := membershipCheckClusterStateForLeave(ctx, tx, node.ID)
 			if err != nil {
 				return err
 			}
@@ -1001,7 +1002,7 @@ func newRolesChanges(state *state.State, gateway *Gateway, nodes []db.RaftNode,
 	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		domains, err = tx.GetNodesFailureDomains()
+		domains, err = tx.GetNodesFailureDomains(ctx)
 		if err != nil {
 			return fmt.Errorf("Load failure domains: %w", err)
 		}
@@ -1015,7 +1016,7 @@ func newRolesChanges(state *state.State, gateway *Gateway, nodes []db.RaftNode,
 	cluster := map[client.NodeInfo]*client.NodeMetadata{}
 
 	for _, node := range nodes {
-		if !shared.StringInSlice(node.Address, unavailableMembers) && HasConnectivity(gateway.networkCert, gateway.serverCert(), node.Address) {
+		if !shared.StringInSlice(node.Address, unavailableMembers) && HasConnectivity(gateway.networkCert, gateway.state().ServerCert(), node.Address) {
 			cluster[node.NodeInfo] = &client.NodeMetadata{
 				FailureDomain: domains[node.Address],
 			}
@@ -1041,12 +1042,12 @@ func Purge(c *db.Cluster, name string) error {
 
 	return c.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get the node (if it doesn't exists an error is returned).
-		node, err := tx.GetNodeByName(name)
+		node, err := tx.GetNodeByName(ctx, name)
 		if err != nil {
 			return fmt.Errorf("Failed to get member %q: %w", name, err)
 		}
 
-		err = tx.ClearNode(node.ID)
+		err = tx.ClearNode(ctx, node.ID)
 		if err != nil {
 			return fmt.Errorf("Failed to clear member %q: %w", name, err)
 		}
@@ -1071,7 +1072,7 @@ func Count(state *state.State) (int, error) {
 	var count int
 	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
-		count, err = tx.GetNodesCount()
+		count, err = tx.GetNodesCount(ctx)
 		return err
 	})
 
@@ -1082,8 +1083,8 @@ func Count(state *state.State) (int, error) {
 // node.
 func Enabled(node *db.Node) (bool, error) {
 	enabled := false
-	err := node.Transaction(func(tx *db.NodeTx) error {
-		addresses, err := tx.GetRaftNodeAddresses()
+	err := node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		addresses, err := tx.GetRaftNodeAddresses(ctx)
 		if err != nil {
 			return err
 		}
@@ -1097,8 +1098,8 @@ func Enabled(node *db.Node) (bool, error) {
 
 // Check that node-related preconditions are met for bootstrapping or joining a
 // cluster.
-func membershipCheckNodeStateForBootstrapOrJoin(tx *db.NodeTx, address string) error {
-	nodes, err := tx.GetRaftNodes()
+func membershipCheckNodeStateForBootstrapOrJoin(ctx context.Context, tx *db.NodeTx, address string) error {
+	nodes, err := tx.GetRaftNodes(ctx)
 	if err != nil {
 		return fmt.Errorf("Failed to fetch current raft nodes: %w", err)
 	}
@@ -1125,44 +1126,44 @@ func membershipCheckNodeStateForBootstrapOrJoin(tx *db.NodeTx, address string) e
 
 // Check that cluster-related preconditions are met for bootstrapping or
 // joining a cluster.
-func membershipCheckClusterStateForBootstrapOrJoin(tx *db.ClusterTx) error {
-	nodes, err := tx.GetNodes()
+func membershipCheckClusterStateForBootstrapOrJoin(ctx context.Context, tx *db.ClusterTx) error {
+	members, err := tx.GetNodes(ctx)
 	if err != nil {
-		return fmt.Errorf("Failed to fetch current cluster nodes: %w", err)
+		return fmt.Errorf("Failed getting cluster members: %w", err)
 	}
 
-	if len(nodes) != 1 {
-		return fmt.Errorf("Inconsistent state: found leftover entries in nodes")
+	if len(members) != 1 {
+		return fmt.Errorf("Inconsistent state: Found leftover entries in cluster members")
 	}
 
 	return nil
 }
 
 // Check that cluster-related preconditions are met for accepting a new node.
-func membershipCheckClusterStateForAccept(tx *db.ClusterTx, name string, address string, schema int, api int) error {
-	nodes, err := tx.GetNodes()
+func membershipCheckClusterStateForAccept(ctx context.Context, tx *db.ClusterTx, name string, address string, schema int, api int) error {
+	members, err := tx.GetNodes(ctx)
 	if err != nil {
-		return fmt.Errorf("Failed to fetch current cluster nodes: %w", err)
+		return fmt.Errorf("Failed getting cluster members: %w", err)
 	}
 
-	if len(nodes) == 1 && nodes[0].Address == "0.0.0.0" {
+	if len(members) == 1 && members[0].Address == "0.0.0.0" {
 		return fmt.Errorf("Clustering isn't enabled")
 	}
 
-	for _, node := range nodes {
-		if node.Name == name {
+	for _, member := range members {
+		if member.Name == name {
 			return fmt.Errorf("The cluster already has a member with name: %s", name)
 		}
 
-		if node.Address == address {
+		if member.Address == address {
 			return fmt.Errorf("The cluster already has a member with address: %s", address)
 		}
 
-		if node.Schema != schema {
+		if member.Schema != schema {
 			return fmt.Errorf("The joining server version doesn't match (expected %s with DB schema %v)", version.Version, schema)
 		}
 
-		if node.APIExtensions != api {
+		if member.APIExtensions != api {
 			return fmt.Errorf("The joining server version doesn't match (expected %s with API count %v)", version.Version, api)
 		}
 	}
@@ -1171,9 +1172,9 @@ func membershipCheckClusterStateForAccept(tx *db.ClusterTx, name string, address
 }
 
 // Check that cluster-related preconditions are met for leaving a cluster.
-func membershipCheckClusterStateForLeave(tx *db.ClusterTx, nodeID int64) error {
+func membershipCheckClusterStateForLeave(ctx context.Context, tx *db.ClusterTx, nodeID int64) error {
 	// Check that it has no containers or images.
-	message, err := tx.NodeIsEmpty(nodeID)
+	message, err := tx.NodeIsEmpty(ctx, nodeID)
 	if err != nil {
 		return err
 	}
@@ -1182,13 +1183,13 @@ func membershipCheckClusterStateForLeave(tx *db.ClusterTx, nodeID int64) error {
 		return fmt.Errorf(message)
 	}
 
-	// Check that it's not the last node.
-	nodes, err := tx.GetNodes()
+	// Check that it's not the last member.
+	members, err := tx.GetNodes(ctx)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed getting cluster members: %w", err)
 	}
 
-	if len(nodes) == 1 {
+	if len(members) == 1 {
 		return fmt.Errorf("Member is the only member in the cluster")
 	}
 
