@@ -4,8 +4,8 @@ import (
 	"context"
 	"crypto/x509"
 	"fmt"
-	"io/ioutil"
 	"net/http"
+	"os"
 	"path/filepath"
 	"testing"
 	"time"
@@ -35,7 +35,7 @@ func TestBootstrap_UnmetPreconditions(t *testing.T) {
 				f.ClusterAddress("1.2.3.4:666")
 				f.RaftNode("5.6.7.8:666")
 				filename := filepath.Join(f.state.OS.VarDir, "cluster.crt")
-				_ = ioutil.WriteFile(filename, []byte{}, 0644)
+				_ = os.WriteFile(filename, []byte{}, 0644)
 			},
 			"Inconsistent state: found leftover cluster certificate",
 		},
@@ -61,7 +61,7 @@ func TestBootstrap_UnmetPreconditions(t *testing.T) {
 				f.ClusterAddress("1.2.3.4:666")
 				f.ClusterNode("5.6.7.8:666")
 			},
-			"Inconsistent state: found leftover entries in nodes",
+			"Inconsistent state: Found leftover entries in cluster members",
 		},
 	}
 
@@ -74,7 +74,8 @@ func TestBootstrap_UnmetPreconditions(t *testing.T) {
 
 			serverCert := shared.TestingKeyPair()
 			state.ServerCert = func() *shared.CertInfo { return serverCert }
-			gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+
+			gateway := newGateway(t, state.DB.Node, serverCert, state)
 			defer func() { _ = gateway.Shutdown() }()
 
 			err := cluster.Bootstrap(state, gateway, "buzz")
@@ -88,8 +89,9 @@ func TestBootstrap(t *testing.T) {
 	defer cleanup()
 
 	serverCert := shared.TestingKeyPair()
-	gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
 	state.ServerCert = func() *shared.CertInfo { return serverCert }
+
+	gateway := newGateway(t, state.DB.Node, serverCert, state)
 	defer func() { _ = gateway.Shutdown() }()
 
 	mux := http.NewServeMux()
@@ -104,8 +106,8 @@ func TestBootstrap(t *testing.T) {
 	require.NoError(t, err)
 
 	// The node-local database has now an entry in the raft_nodes table
-	err = state.DB.Node.Transaction(func(tx *db.NodeTx) error {
-		nodes, err := tx.GetRaftNodes()
+	err = state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
+		nodes, err := tx.GetRaftNodes(ctx)
 		require.NoError(t, err)
 		require.Len(t, nodes, 1)
 		assert.Equal(t, uint64(1), nodes[0].ID)
@@ -116,11 +118,11 @@ func TestBootstrap(t *testing.T) {
 
 	// The cluster database has now an entry in the nodes table
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		require.NoError(t, err)
-		require.Len(t, nodes, 1)
-		assert.Equal(t, "buzz", nodes[0].Name)
-		assert.Equal(t, address, nodes[0].Address)
+		require.Len(t, members, 1)
+		assert.Equal(t, "buzz", members[0].Name)
+		assert.Equal(t, address, members[0].Address)
 		return nil
 	})
 	require.NoError(t, err)
@@ -212,7 +214,9 @@ func TestAccept_UnmetPreconditions(t *testing.T) {
 			defer cleanup()
 
 			serverCert := shared.TestingKeyPair()
-			gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+			state.ServerCert = func() *shared.CertInfo { return serverCert }
+
+			gateway := newGateway(t, state.DB.Node, serverCert, state)
 			defer func() { _ = gateway.Shutdown() }()
 
 			c.setup(&membershipFixtures{t: t, state: state})
@@ -229,7 +233,9 @@ func TestAccept(t *testing.T) {
 	defer cleanup()
 
 	serverCert := shared.TestingKeyPair()
-	gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+	state.ServerCert = func() *shared.CertInfo { return serverCert }
+
+	gateway := newGateway(t, state.DB.Node, serverCert, state)
 	defer func() { _ = gateway.Shutdown() }()
 
 	f := &membershipFixtures{t: t, state: state}
@@ -239,13 +245,13 @@ func TestAccept(t *testing.T) {
 	err := state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		state.GlobalConfig, err = clusterConfig.Load(tx)
+		state.GlobalConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return err
 		}
 
 		// Get the local node (will be used if clustered).
-		state.ServerName, err = tx.GetLocalNodeName()
+		state.ServerName, err = tx.GetLocalNodeName(ctx)
 		if err != nil {
 			return err
 		}
@@ -274,7 +280,9 @@ func TestJoin(t *testing.T) {
 	targetState, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	targetGateway := newGateway(t, targetState.DB.Node, targetCert, targetCert)
+	targetState.ServerCert = func() *shared.CertInfo { return targetCert }
+
+	targetGateway := newGateway(t, targetState.DB.Node, targetCert, targetState)
 	defer func() { _ = targetGateway.Shutdown() }()
 
 	altServerCert := shared.TestingAltKeyPair()
@@ -305,13 +313,13 @@ func TestJoin(t *testing.T) {
 	require.NoError(t, err)
 
 	err = targetState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		targetState.GlobalConfig, err = clusterConfig.Load(tx)
+		targetState.GlobalConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return err
 		}
 
 		// Get the local node (will be used if clustered).
-		targetState.ServerName, err = tx.GetLocalNodeName()
+		targetState.ServerName, err = tx.GetLocalNodeName(ctx)
 		if err != nil {
 			return err
 		}
@@ -339,7 +347,9 @@ func TestJoin(t *testing.T) {
 	state, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	gateway := newGateway(t, state.DB.Node, targetCert, altServerCert)
+	state.ServerCert = func() *shared.CertInfo { return altServerCert }
+
+	gateway := newGateway(t, state.DB.Node, targetCert, state)
 
 	defer func() { _ = gateway.Shutdown() }()
 
@@ -358,13 +368,13 @@ func TestJoin(t *testing.T) {
 	require.NoError(t, err)
 
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		state.GlobalConfig, err = clusterConfig.Load(tx)
+		state.GlobalConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return err
 		}
 
 		// Get the local node (will be used if clustered).
-		state.ServerName, err = tx.GetLocalNodeName()
+		state.ServerName, err = tx.GetLocalNodeName(ctx)
 		if err != nil {
 			return err
 		}
@@ -417,9 +427,9 @@ func TestJoin(t *testing.T) {
 
 	// The node has gone from the cluster db.
 	err = targetState.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		require.NoError(t, err)
-		assert.Len(t, nodes, 1)
+		assert.Len(t, members, 1)
 		return nil
 	})
 	require.NoError(t, err)
@@ -438,7 +448,7 @@ type membershipFixtures struct {
 
 // Set core.https_address to the given value.
 func (h *membershipFixtures) CoreAddress(address string) {
-	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	err := h.state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
 		config := map[string]string{
 			"core.https_address": address,
 		}
@@ -450,7 +460,7 @@ func (h *membershipFixtures) CoreAddress(address string) {
 
 // Set cluster.https_address to the given value.
 func (h *membershipFixtures) ClusterAddress(address string) {
-	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	err := h.state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
 		config := map[string]string{
 			"cluster.https_address": address,
 		}
@@ -462,7 +472,7 @@ func (h *membershipFixtures) ClusterAddress(address string) {
 
 // Add the given address to the raft_nodes table.
 func (h *membershipFixtures) RaftNode(address string) {
-	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	err := h.state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
 		_, err := tx.CreateRaftNode(address, "rusp")
 		return err
 	})
@@ -472,9 +482,9 @@ func (h *membershipFixtures) RaftNode(address string) {
 // Get the current list of the raft nodes in the raft_nodes table.
 func (h *membershipFixtures) RaftNodes() []db.RaftNode {
 	var nodes []db.RaftNode
-	err := h.state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	err := h.state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
 		var err error
-		nodes, err = tx.GetRaftNodes()
+		nodes, err = tx.GetRaftNodes(ctx)
 		return err
 	})
 	require.NoError(h.t, err)
