@@ -8,7 +8,6 @@ import (
 
 	"github.com/lxc/lxd/client"
 	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
@@ -32,28 +31,26 @@ const (
 // NewNotifier builds a Notifier that can be used to notify other peers using
 // the given policy.
 func NewNotifier(state *state.State, networkCert *shared.CertInfo, serverCert *shared.CertInfo, policy NotifierPolicy) (Notifier, error) {
-	address, err := node.ClusterAddress(state.DB.Node)
-	if err != nil {
-		return nil, fmt.Errorf("failed to fetch node address: %w", err)
-	}
+	localClusterAddress := state.LocalConfig.ClusterAddress()
 
 	// Fast-track the case where we're not clustered at all.
-	if address == "" {
+	if localClusterAddress == "" {
 		nullNotifier := func(func(lxd.InstanceServer) error) error { return nil }
 		return nullNotifier, nil
 	}
 
-	var nodes []db.NodeInfo
+	var err error
+	var members []db.NodeInfo
 	var offlineThreshold time.Duration
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		offlineThreshold, err = tx.GetNodeOfflineThreshold()
+		offlineThreshold, err = tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
 			return err
 		}
 
-		nodes, err = tx.GetNodes()
+		members, err = tx.GetNodes(ctx)
 		if err != nil {
-			return err
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
 		return nil
@@ -63,20 +60,20 @@ func NewNotifier(state *state.State, networkCert *shared.CertInfo, serverCert *s
 	}
 
 	peers := []string{}
-	for _, node := range nodes {
-		if node.Address == address || node.Address == "0.0.0.0" {
+	for _, member := range members {
+		if member.Address == localClusterAddress || member.Address == "0.0.0.0" {
 			continue // Exclude ourselves
 		}
 
-		if node.IsOffline(offlineThreshold) {
+		if member.IsOffline(offlineThreshold) {
 			// Even if the heartbeat timestamp is not recent
 			// enough, let's try to connect to the node, just in
 			// case the heartbeat is lagging behind for some reason
 			// and the node is actually up.
-			if !HasConnectivity(networkCert, serverCert, node.Address) {
+			if !HasConnectivity(networkCert, serverCert, member.Address) {
 				switch policy {
 				case NotifyAll:
-					return nil, fmt.Errorf("peer node %s is down", node.Address)
+					return nil, fmt.Errorf("peer node %s is down", member.Address)
 				case NotifyAlive:
 					continue // Just skip this node
 				case NotifyTryAll:
@@ -84,7 +81,7 @@ func NewNotifier(state *state.State, networkCert *shared.CertInfo, serverCert *s
 			}
 		}
 
-		peers = append(peers, node.Address)
+		peers = append(peers, member.Address)
 	}
 
 	notifier := func(hook func(lxd.InstanceServer) error) error {
