@@ -4,7 +4,6 @@ import (
 	"context"
 	"crypto/x509"
 	"fmt"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"path/filepath"
@@ -20,6 +19,7 @@ import (
 	"github.com/lxc/lxd/lxd/cluster"
 	"github.com/lxc/lxd/lxd/db"
 	clusterDB "github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
 )
@@ -42,8 +42,20 @@ func TestNotifyUpgradeCompleted(t *testing.T) {
 	}()
 
 	state0 := f.State(gateway0)
+
+	// Populate state.LocalConfig after nodes created above.
+	var err error
+	var nodeConfig *node.Config
+	err = state0.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		nodeConfig, err = node.ConfigLoad(ctx, tx)
+		return err
+	})
+	require.NoError(t, err)
+
+	state0.LocalConfig = nodeConfig
+
 	serverCert0 := gateway0.ServerCert()
-	err := cluster.NotifyUpgradeCompleted(state0, serverCert0, serverCert0)
+	err = cluster.NotifyUpgradeCompleted(state0, serverCert0, serverCert0)
 	require.NoError(t, err)
 
 	wg.Wait()
@@ -52,7 +64,7 @@ func TestNotifyUpgradeCompleted(t *testing.T) {
 // The task function checks if the node is out of date and runs whatever is in
 // LXD_CLUSTER_UPDATE if so.
 func TestMaybeUpdate_Upgrade(t *testing.T) {
-	dir, err := ioutil.TempDir("", "")
+	dir, err := os.MkdirTemp("", "")
 	require.NoError(t, err)
 
 	defer func() { _ = os.RemoveAll(dir) }()
@@ -61,13 +73,13 @@ func TestMaybeUpdate_Upgrade(t *testing.T) {
 	stamp := filepath.Join(dir, "stamp")
 	script := filepath.Join(dir, "cluster-upgrade")
 	data := []byte(fmt.Sprintf("#!/bin/sh\ntouch %s\n", stamp))
-	err = ioutil.WriteFile(script, data, 0755)
+	err = os.WriteFile(script, data, 0755)
 	require.NoError(t, err)
 
 	state, cleanup := state.NewTestState(t)
 	defer cleanup()
 
-	_ = state.DB.Node.Transaction(func(tx *db.NodeTx) error {
+	_ = state.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
 		nodes := []db.RaftNode{
 			{NodeInfo: client.NodeInfo{ID: 1, Address: "0.0.0.0:666"}},
 			{NodeInfo: client.NodeInfo{ID: 2, Address: "1.2.3.4:666"}},
@@ -82,7 +94,7 @@ func TestMaybeUpdate_Upgrade(t *testing.T) {
 		id, err := tx.CreateNode("buzz", "1.2.3.4:666")
 		require.NoError(t, err)
 
-		node, err := tx.GetNodeByName("buzz")
+		node, err := tx.GetNodeByName(ctx, "buzz")
 		require.NoError(t, err)
 
 		version := node.Version()
@@ -105,7 +117,7 @@ func TestMaybeUpdate_Upgrade(t *testing.T) {
 
 // If the node is up-to-date, nothing is done.
 func TestMaybeUpdate_NothingToDo(t *testing.T) {
-	dir, err := ioutil.TempDir("", "")
+	dir, err := os.MkdirTemp("", "")
 	require.NoError(t, err)
 
 	defer func() { _ = os.RemoveAll(dir) }()
@@ -114,7 +126,7 @@ func TestMaybeUpdate_NothingToDo(t *testing.T) {
 	stamp := filepath.Join(dir, "stamp")
 	script := filepath.Join(dir, "cluster-upgrade")
 	data := []byte(fmt.Sprintf("#!/bin/sh\ntouch %s\n", stamp))
-	err = ioutil.WriteFile(script, data, 0755)
+	err = os.WriteFile(script, data, 0755)
 	require.NoError(t, err)
 
 	state, cleanup := state.NewTestState(t)
@@ -141,7 +153,9 @@ func TestUpgradeMembersWithoutRole(t *testing.T) {
 	address := server.Listener.Addr().String()
 	setRaftRole(t, state.DB.Node, address)
 
-	gateway := newGateway(t, state.DB.Node, serverCert, serverCert)
+	state.ServerCert = func() *shared.CertInfo { return serverCert }
+
+	gateway := newGateway(t, state.DB.Node, serverCert, state)
 	defer func() { _ = gateway.Shutdown() }()
 
 	trustedCerts := func() map[clusterDB.CertificateType]map[string]x509.Certificate {
@@ -167,7 +181,7 @@ func TestUpgradeMembersWithoutRole(t *testing.T) {
 		require.NoError(t, err)
 		_, err = tx.CreateNode("bar", "5.6.7.8")
 		require.NoError(t, err)
-		members, err = tx.GetNodes()
+		members, err = tx.GetNodes(ctx)
 		require.NoError(t, err)
 		return nil
 	})
