@@ -37,7 +37,7 @@ func Load(schema Schema, values map[string]string) (Map, error) {
 //
 // Return a map of key/value pairs that were actually changed. If some keys
 // fail to apply, details are included in the returned ErrorList.
-func (m *Map) Change(changes map[string]interface{}) (map[string]string, error) {
+func (m *Map) Change(changes map[string]any) (map[string]string, error) {
 	values := make(map[string]string, len(m.schema))
 
 	errors := ErrorList{}
@@ -56,7 +56,7 @@ func (m *Map) Change(changes map[string]interface{}) (map[string]string, error)
 			change = ""
 		}
 
-		// Sanity check that we were actually passed a string.
+		// Ensure that we were actually passed a string.
 		s := reflect.ValueOf(change)
 		if s.Kind() != reflect.String {
 			errors.add(name, nil, fmt.Sprintf("invalid type %s", s.Kind()))
@@ -84,6 +84,7 @@ func (m *Map) Change(changes map[string]interface{}) (map[string]string, error)
 	for _, name := range names {
 		changed[name] = m.GetRaw(name)
 	}
+
 	return changed, err
 }
 
@@ -92,17 +93,24 @@ func (m *Map) Change(changes map[string]interface{}) (map[string]string, error)
 // Keys that match their default value will not be included in the dump. Also,
 // if a Key has its Hidden attribute set to true, it will be rendered as
 // "true", for obfuscating the actual value.
-func (m *Map) Dump() map[string]interface{} {
-	values := map[string]interface{}{}
+func (m *Map) Dump() map[string]any {
+	values := map[string]any{}
 
-	for name, key := range m.schema {
-		value := m.GetRaw(name)
-		if value != key.Default {
-			if key.Hidden {
-				values[name] = true
-			} else {
-				values[name] = value
+	for name, value := range m.values {
+		key, ok := m.schema[name]
+		if ok {
+			// Schema key
+			value := m.GetRaw(name)
+			if value != key.Default {
+				if key.Hidden {
+					values[name] = true
+				} else {
+					values[name] = value
+				}
 			}
+		} else if shared.IsUserConfig(name) {
+			// User key, just include it as is
+			values[name] = value
 		}
 	}
 
@@ -111,17 +119,26 @@ func (m *Map) Dump() map[string]interface{} {
 
 // GetRaw returns the value of the given key, which must be of type String.
 func (m *Map) GetRaw(name string) string {
-	key := m.schema.mustGetKey(name)
 	value, ok := m.values[name]
+	// User key?
+	if shared.IsUserConfig(name) {
+		return value
+	}
+	// Schema key
+	key := m.schema.mustGetKey(name)
 	if !ok {
 		value = key.Default
 	}
+
 	return value
 }
 
 // GetString returns the value of the given key, which must be of type String.
 func (m *Map) GetString(name string) string {
-	m.schema.assertKeyType(name, String)
+	if !shared.IsUserConfig(name) {
+		m.schema.assertKeyType(name, String)
+	}
+
 	return m.GetRaw(name)
 }
 
@@ -138,6 +155,7 @@ func (m *Map) GetInt64(name string) int64 {
 	if err != nil {
 		panic(fmt.Sprintf("cannot convert to int64: %v", err))
 	}
+
 	return n
 }
 
@@ -163,6 +181,7 @@ func (m *Map) update(values map[string]string) ([]string, error) {
 			errors.add(name, value, err.Error())
 			continue
 		}
+
 		if changed {
 			names = append(names, name)
 		}
@@ -182,6 +201,23 @@ func (m *Map) update(values map[string]string) ([]string, error) {
 // effectively revert it to the default. Return a boolean indicating whether
 // the value has changed, and error if something went wrong.
 func (m *Map) set(name string, value string, initial bool) (bool, error) {
+	// Bypass schema for user.* keys
+	if shared.IsUserConfig(name) {
+		current, ok := m.values[name]
+		if ok && value == current {
+			// Value is unchanged
+			return false, nil
+		}
+
+		if value == "" {
+			delete(m.values, name)
+		} else {
+			m.values[name] = value
+		}
+
+		return true, nil
+	}
+
 	key, ok := m.schema[name]
 	if !ok {
 		return false, fmt.Errorf("unknown key")
@@ -228,5 +264,6 @@ func normalizeBool(value string) string {
 	if shared.IsTrue(value) {
 		return "true"
 	}
+
 	return "false"
 }
