@@ -5,10 +5,10 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/lxc/lxd/lxd/config"
-	"github.com/mpvl/subtest"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+
+	"github.com/lxc/lxd/lxd/config"
 )
 
 // Loading a config Map initializes it with the given values.
@@ -47,7 +47,7 @@ func TestLoad(t *testing.T) {
 	}
 
 	for _, c := range cases {
-		subtest.Run(t, c.title, func(t *testing.T) {
+		t.Run(c.title, func(t *testing.T) {
 			m, err := config.Load(schema, c.values)
 			require.NoError(t, err)
 
@@ -86,8 +86,9 @@ func TestLoad_Error(t *testing.T) {
 			"cannot set 'bar' to '': unknown key (and 1 more errors)",
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, c.title, func(t *testing.T) {
+		t.Run(c.title, func(t *testing.T) {
 			_, err := config.Load(c.schema, c.values)
 			assert.EqualError(t, err, c.message)
 		})
@@ -103,6 +104,7 @@ func TestChange(t *testing.T) {
 		"yuk": {Type: config.Bool, Default: "true"},
 		"xyz": {Hidden: true},
 	}
+
 	values := map[string]string{ // Initial values
 		"foo": "hello",
 		"bar": "x",
@@ -111,48 +113,48 @@ func TestChange(t *testing.T) {
 
 	cases := []struct {
 		title  string
-		values map[string]interface{} // New values
-		result map[string]string      // Expected values after change
+		values map[string]any    // New values
+		result map[string]string // Expected values after change
 	}{
 		{
 			`plain change of regular key`,
-			map[string]interface{}{"foo": "world"},
+			map[string]any{"foo": "world"},
 			map[string]string{"foo": "world"},
 		},
 		{
 			`key setter is honored`,
-			map[string]interface{}{"bar": "y"},
+			map[string]any{"bar": "y"},
 			map[string]string{"bar": "Y"},
 		},
 		{
 			`bool true values are normalized`,
-			map[string]interface{}{"egg": "yes"},
+			map[string]any{"egg": "yes"},
 			map[string]string{"egg": "true"},
 		},
 		{
 			`bool false values are normalized`,
-			map[string]interface{}{"yuk": "0"},
+			map[string]any{"yuk": "0"},
 			map[string]string{"yuk": "false"},
 		},
 		{
 			`the special value 'true' is a passthrough for hidden keys`,
-			map[string]interface{}{"xyz": true},
+			map[string]any{"xyz": true},
 			map[string]string{"xyz": "sekret"},
 		},
 		{
 			`the special value nil is converted to empty string`,
-			map[string]interface{}{"foo": nil},
+			map[string]any{"foo": nil},
 			map[string]string{"foo": ""},
 		},
 		{
 			`multiple values are all mutated`,
-			map[string]interface{}{"foo": "x", "bar": "hey", "egg": "0"},
+			map[string]any{"foo": "x", "bar": "hey", "egg": "0"},
 			map[string]string{"foo": "x", "bar": "HEY", "egg": ""},
 		},
 	}
 
 	for _, c := range cases {
-		subtest.Run(t, c.title, func(t *testing.T) {
+		t.Run(c.title, func(t *testing.T) {
 			m, err := config.Load(schema, values)
 			require.NoError(t, err)
 
@@ -172,41 +174,43 @@ func TestMap_ChangeReturnsChangedKeys(t *testing.T) {
 		"foo": {Type: config.Bool},
 		"bar": {Default: "egg"},
 	}
+
 	values := map[string]string{"foo": "true"} // Initial values
 
 	cases := []struct {
 		title   string
-		changes map[string]interface{} // New values
-		changed map[string]string      // Keys that should have actually changed
+		changes map[string]any    // New values
+		changed map[string]string // Keys that should have actually changed
 	}{
 		{
 			`plain single change`,
-			map[string]interface{}{"foo": "no"},
+			map[string]any{"foo": "no"},
 			map[string]string{"foo": "false"},
 		},
 		{
 			`unchanged boolean value, even if it's spelled 'yes' and not 'true'`,
-			map[string]interface{}{"foo": "yes"},
+			map[string]any{"foo": "yes"},
 			map[string]string{},
 		},
 		{
 			`unset value`,
-			map[string]interface{}{"foo": ""},
+			map[string]any{"foo": ""},
 			map[string]string{"foo": "false"},
 		},
 		{
 			`unchanged value, since it matches the default`,
-			map[string]interface{}{"foo": "true", "bar": "egg"},
+			map[string]any{"foo": "true", "bar": "egg"},
 			map[string]string{},
 		},
 		{
 			`multiple changes`,
-			map[string]interface{}{"foo": "false", "bar": "baz"},
+			map[string]any{"foo": "false", "bar": "baz"},
 			map[string]string{"foo": "false", "bar": "baz"},
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, c.title, func(t *testing.T) {
+		t.Run(c.title, func(t *testing.T) {
 			m, err := config.Load(schema, values)
 			assert.NoError(t, err)
 
@@ -227,33 +231,33 @@ func TestMap_ChangeError(t *testing.T) {
 
 	var cases = []struct {
 		title   string
-		changes map[string]interface{}
+		changes map[string]any
 		message string
 	}{
 		{
 			`schema has no key with the given name`,
-			map[string]interface{}{"xxx": ""},
+			map[string]any{"xxx": ""},
 			"cannot set 'xxx' to '': unknown key",
 		},
 		{
 			`validation fails`,
-			map[string]interface{}{"foo": "yyy"},
+			map[string]any{"foo": "yyy"},
 			"cannot set 'foo' to 'yyy': invalid boolean",
 		},
 		{
 			`custom setter fails`,
-			map[string]interface{}{"egg": "xxx"},
+			map[string]any{"egg": "xxx"},
 			"cannot set 'egg' to 'xxx': boom",
 		},
 		{
 			`non string value`,
-			map[string]interface{}{"egg": 123},
+			map[string]any{"egg": 123},
 			"cannot set 'egg': invalid type int",
 		},
 	}
 
 	for _, c := range cases {
-		subtest.Run(t, c.message, func(t *testing.T) {
+		t.Run(c.message, func(t *testing.T) {
 			m, err := config.Load(schema, nil)
 			assert.NoError(t, err)
 
@@ -271,18 +275,21 @@ func TestMap_Dump(t *testing.T) {
 		"bar": {Default: "x"},
 		"egg": {Hidden: true},
 	}
+
 	values := map[string]string{
 		"foo": "hello",
 		"bar": "x",
 		"egg": "123",
 	}
+
 	m, err := config.Load(schema, values)
 	assert.NoError(t, err)
 
-	dump := map[string]interface{}{
+	dump := map[string]any{
 		"foo": "hello",
 		"egg": true,
 	}
+
 	assert.Equal(t, dump, m.Dump())
 }
 
@@ -293,6 +300,7 @@ func TestMap_Getters(t *testing.T) {
 		"bar": {Type: config.Bool},
 		"egg": {Type: config.Int64},
 	}
+
 	values := map[string]string{
 		"foo": "hello",
 		"bar": "true",
