@@ -3,8 +3,6 @@ package config
 import (
 	"fmt"
 
-	log "github.com/lxc/lxd/shared/log15"
-
 	"github.com/lxc/lxd/shared/logger"
 )
 
@@ -18,9 +16,10 @@ func SafeLoad(schema Schema, values map[string]string) (Map, error) {
 		if !ok {
 			return m, err
 		}
+
 		for _, error := range errors {
 			message := fmt.Sprintf("Invalid configuration key: %s", error.Reason)
-			logger.Error(message, log.Ctx{"key": error.Name})
+			logger.Error(message, logger.Ctx{"key": error.Name})
 		}
 	}
 
