@@ -13,7 +13,7 @@ import (
 // and options for their values, expressed using Key objects.
 type Schema map[string]Key
 
-// Keys returns all keys defined in the schema
+// Keys returns all keys defined in the schema.
 func (s Schema) Keys() []string {
 	keys := make([]string, len(s))
 	i := 0
@@ -21,17 +21,19 @@ func (s Schema) Keys() []string {
 		keys[i] = key
 		i++
 	}
+
 	sort.Strings(keys)
 	return keys
 }
 
 // Defaults returns a map of all key names in the schema along with their default
 // values.
-func (s Schema) Defaults() map[string]interface{} {
-	values := make(map[string]interface{}, len(s))
+func (s Schema) Defaults() map[string]any {
+	values := make(map[string]any, len(s))
 	for name, key := range s {
 		values[name] = key.Default
 	}
+
 	return values
 }
 
@@ -41,6 +43,7 @@ func (s Schema) mustGetKey(name string) Key {
 	if !ok {
 		panic(fmt.Sprintf("attempt to access unknown key '%s'", name))
 	}
+
 	return key
 }
 
@@ -102,11 +105,13 @@ func (v *Key) validate(value string) error {
 		if !shared.StringInSlice(strings.ToLower(value), booleans) {
 			return fmt.Errorf("invalid boolean")
 		}
+
 	case Int64:
 		_, err := strconv.ParseInt(value, 10, 64)
 		if err != nil {
 			return fmt.Errorf("invalid integer")
 		}
+
 	default:
 		panic(fmt.Sprintf("unexpected value type: %d", v.Type))
 	}
