@@ -20,13 +20,13 @@ import (
 	"github.com/canonical/candid/candidclient"
 	dqliteClient "github.com/canonical/go-dqlite/client"
 	"github.com/canonical/go-dqlite/driver"
+	"github.com/go-macaroon-bakery/macaroon-bakery/v3/bakery"
+	"github.com/go-macaroon-bakery/macaroon-bakery/v3/bakery/checkers"
+	"github.com/go-macaroon-bakery/macaroon-bakery/v3/bakery/identchecker"
+	"github.com/go-macaroon-bakery/macaroon-bakery/v3/httpbakery"
 	"github.com/gorilla/mux"
 	liblxc "github.com/lxc/go-lxc"
 	"golang.org/x/sys/unix"
-	"gopkg.in/macaroon-bakery.v3/bakery"
-	"gopkg.in/macaroon-bakery.v3/bakery/checkers"
-	"gopkg.in/macaroon-bakery.v3/bakery/identchecker"
-	"gopkg.in/macaroon-bakery.v3/httpbakery"
 
 	"github.com/lxc/lxd/lxd/bgp"
 	"github.com/lxc/lxd/lxd/cluster"
@@ -60,6 +60,7 @@ import (
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/lxd/warnings"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/cancel"
 	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/version"
@@ -77,8 +78,9 @@ type Daemon struct {
 	rbac        *rbac.Server
 
 	// Event servers
-	devlxdEvents *events.DevLXDServer
-	events       *events.Server
+	devlxdEvents     *events.DevLXDServer
+	events           *events.Server
+	internalListener *events.InternalListener
 
 	// Tasks registry for long-running background tasks
 	// Keep clustering tasks separate as they cause a lot of CPU wakeups
@@ -116,7 +118,7 @@ type Daemon struct {
 
 	// Status control.
 	setupChan      chan struct{}      // Closed when basic Daemon setup is completed
-	readyChan      chan struct{}      // Closed when LXD is fully ready
+	waitReady      *cancel.Canceller  // Cancelled when LXD is fully ready
 	shutdownCtx    context.Context    // Cancelled when shutdown starts.
 	shutdownCancel context.CancelFunc // Cancels the shutdownCtx to indicate shutdown starting.
 	shutdownDoneCh chan error         // Receives the result of the d.Stop() function and tells LXD to end.
@@ -129,6 +131,7 @@ type Daemon struct {
 
 	// Configuration.
 	globalConfig   *clusterConfig.Config
+	localConfig    *node.Config
 	globalConfigMu sync.Mutex
 
 	// Cluster.
@@ -195,7 +198,7 @@ func newDaemon(config *DaemonConfig, os *sys.OS) *Daemon {
 		db:             &db.DB{},
 		os:             os,
 		setupChan:      make(chan struct{}),
-		readyChan:      make(chan struct{}),
+		waitReady:      cancel.New(context.Background()),
 		shutdownCtx:    shutdownCtx,
 		shutdownCancel: shutdownCancel,
 		shutdownDoneCh: make(chan error),
@@ -439,6 +442,7 @@ func (d *Daemon) State() *state.State {
 
 	d.globalConfigMu.Lock()
 	globalConfig := d.globalConfig
+	localConfig := d.localConfig
 	d.globalConfigMu.Unlock()
 
 	return &state.State{
@@ -458,6 +462,7 @@ func (d *Daemon) State() *state.State {
 		InstanceTypes:          instanceTypes,
 		DevMonitor:             d.devmonitor,
 		GlobalConfig:           globalConfig,
+		LocalConfig:            localConfig,
 		ServerName:             d.serverName,
 	}
 }
@@ -477,8 +482,10 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 	var uri string
 	if c.Path == "" {
 		uri = fmt.Sprintf("/%s", version)
-	} else {
+	} else if version != "" {
 		uri = fmt.Sprintf("/%s/%s", version, c.Path)
+	} else {
+		uri = fmt.Sprintf("/%s", c.Path)
 	}
 
 	route := restAPI.HandleFunc(uri, func(w http.ResponseWriter, r *http.Request) {
@@ -699,9 +706,9 @@ func (d *Daemon) createCmd(restAPI *mux.Router, version string, c APIEndpoint) {
 		// Handle errors
 		err = resp.Render(w)
 		if err != nil {
-			err := response.InternalError(err).Render(w)
+			writeErr := response.InternalError(err).Render(w)
 			if err != nil {
-				logger.Error("Failed writing error for HTTP response", logger.Ctx{"url": uri, "err": err})
+				logger.Error("Failed writing error for HTTP response", logger.Ctx{"url": uri, "err": err, "writeErr": writeErr})
 			}
 		}
 	})
@@ -775,6 +782,9 @@ func (d *Daemon) init() error {
 	// Setup logger
 	events.LoggingServer = d.events
 
+	// Setup internal event listener
+	d.internalListener = events.NewInternalListener(d.shutdownCtx, d.events)
+
 	// Lets check if there's an existing LXD running
 	err := endpoints.CheckAlreadyRunning(d.UnixSocket())
 	if err != nil {
@@ -1046,7 +1056,7 @@ func (d *Daemon) init() error {
 		d.shutdownCtx,
 		d.db.Node,
 		networkCert,
-		d.serverCert,
+		d.State,
 		cluster.Latency(d.config.RaftLatency),
 		cluster.LogLevel(clusterLogLevel))
 	if err != nil {
@@ -1073,25 +1083,19 @@ func (d *Daemon) init() error {
 		}
 	}
 
-	address, err := node.HTTPSAddress(d.db.Node)
+	logger.Info("Loading daemon configuration")
+	err = d.db.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		d.localConfig, err = node.ConfigLoad(ctx, tx)
+		return err
+	})
 	if err != nil {
-		return fmt.Errorf("Failed to fetch node address: %w", err)
+		return err
 	}
 
-	clusterAddress, err := node.ClusterAddress(d.db.Node)
-	if err != nil {
-		return fmt.Errorf("Failed to fetch cluster address: %w", err)
-	}
-
-	debugAddress, err := node.DebugAddress(d.db.Node)
-	if err != nil {
-		return fmt.Errorf("Failed to fetch debug address: %w", err)
-	}
-
-	metricsAddress, err := node.MetricsAddress(d.db.Node)
-	if err != nil {
-		return fmt.Errorf("Failed to fetch metrics address: %w", err)
-	}
+	localHTTPAddress := d.localConfig.HTTPSAddress()
+	localClusterAddress := d.localConfig.ClusterAddress()
+	debugAddress := d.localConfig.DebugAddress()
+	metricsAddress := d.localConfig.MetricsAddress()
 
 	if os.Getenv("LISTEN_PID") != "" {
 		d.systemdSocketActivated = true
@@ -1105,8 +1109,8 @@ func (d *Daemon) init() error {
 		RestServer:           restServer(d),
 		DevLxdServer:         devLxdServer(d),
 		LocalUnixSocketGroup: d.config.Group,
-		NetworkAddress:       address,
-		ClusterAddress:       clusterAddress,
+		NetworkAddress:       localHTTPAddress,
+		ClusterAddress:       localClusterAddress,
 		DebugAddress:         debugAddress,
 		MetricsAddress:       metricsAddress,
 		MetricsServer:        metricsServer(d),
@@ -1155,7 +1159,7 @@ func (d *Daemon) init() error {
 			options = append(options, driver.WithTracing(dqliteClient.LogDebug))
 		}
 
-		d.db.Cluster, err = db.OpenCluster(context.Background(), "db.bin", store, clusterAddress, dir, d.config.DqliteSetupTimeout, nil, options...)
+		d.db.Cluster, err = db.OpenCluster(context.Background(), "db.bin", store, localClusterAddress, dir, d.config.DqliteSetupTimeout, nil, options...)
 		if err == nil {
 			logger.Info("Initialized global database")
 			break
@@ -1257,8 +1261,8 @@ func (d *Daemon) init() error {
 	}
 
 	// Get daemon configuration.
-	bgpAddress := ""
-	bgpRouterID := ""
+	bgpAddress := d.localConfig.BGPAddress()
+	bgpRouterID := d.localConfig.BGPRouterID()
 	bgpASN := int64(0)
 
 	candidAPIURL := ""
@@ -1266,7 +1270,7 @@ func (d *Daemon) init() error {
 	candidDomains := ""
 	candidExpiry := int64(0)
 
-	dnsAddress := ""
+	dnsAddress := d.localConfig.DNSAddress()
 
 	rbacAPIURL := ""
 	rbacAPIKey := ""
@@ -1278,33 +1282,16 @@ func (d *Daemon) init() error {
 
 	maasAPIURL := ""
 	maasAPIKey := ""
-	maasMachine := ""
-
-	logger.Info("Loading daemon configuration")
-	err = d.db.Node.Transaction(func(tx *db.NodeTx) error {
-		config, err := node.ConfigLoad(tx)
-		if err != nil {
-			return err
-		}
-
-		maasMachine = config.MAASMachine()
-		bgpAddress = config.BGPAddress()
-		bgpRouterID = config.BGPRouterID()
-		dnsAddress = config.DNSAddress()
-		return nil
-	})
-	if err != nil {
-		return err
-	}
+	maasMachine := d.localConfig.MAASMachine()
 
 	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		config, err := clusterConfig.Load(tx)
+		config, err := clusterConfig.Load(ctx, tx)
 		if err != nil {
 			return err
 		}
 
 		// Get the local node (will be used if clustered).
-		serverName, err := tx.GetLocalNodeName()
+		serverName, err := tx.GetLocalNodeName(ctx)
 		if err != nil {
 			return err
 		}
@@ -1323,9 +1310,7 @@ func (d *Daemon) init() error {
 	d.globalConfigMu.Lock()
 	bgpASN = d.globalConfig.BGPASN()
 
-	d.proxy = shared.ProxyFromConfig(
-		d.globalConfig.ProxyHTTPS(), d.globalConfig.ProxyHTTP(), d.globalConfig.ProxyIgnoreHosts(),
-	)
+	d.proxy = shared.ProxyFromConfig(d.globalConfig.ProxyHTTPS(), d.globalConfig.ProxyHTTP(), d.globalConfig.ProxyIgnoreHosts())
 
 	candidAPIURL, candidAPIKey, candidExpiry, candidDomains = d.globalConfig.CandidServer()
 	maasAPIURL, maasAPIKey = d.globalConfig.MAASController()
@@ -1423,6 +1408,8 @@ func (d *Daemon) init() error {
 	// Cleanup leftover images.
 	pruneLeftoverImages(d)
 
+	var instances []instance.Instance
+
 	if !d.os.MockMode {
 		// Start the scheduler
 		go deviceEventListener(d.State())
@@ -1439,9 +1426,15 @@ func (d *Daemon) init() error {
 			return err
 		}
 
+		// Must occur after d.devmonitor has been initialised.
+		instances, err = instance.LoadNodeAll(d.State(), instancetype.Any)
+		if err != nil {
+			return fmt.Errorf("Failed loading local instances: %w", err)
+		}
+
 		// Register devices on running instances to receive events and reconnect to VM monitor sockets.
 		// This should come after the event handler go routines have been started.
-		devicesRegister(d.State())
+		devicesRegister(instances)
 
 		// Setup seccomp handler
 		if d.os.SeccompListener {
@@ -1506,47 +1499,7 @@ func (d *Daemon) init() error {
 		logger.Warn("Failed to resolve warnings", logger.Ctx{"err": err})
 	}
 
-	// Run the post initialization actions
-	err = d.Ready()
-	if err != nil {
-		return err
-	}
-
-	logger.Info("Daemon started")
-
-	return nil
-}
-
-func (d *Daemon) startClusterTasks() {
-	// Add initial event listeners from global database members.
-	// Run asynchronously so that connecting to remote members doesn't delay starting up other cluster tasks.
-	go cluster.EventsUpdateListeners(d.endpoints, d.db.Cluster, d.serverCert, nil, d.events.Inject)
-
-	// Heartbeats
-	d.taskClusterHeartbeat = d.clusterTasks.Add(cluster.HeartbeatTask(d.gateway))
-
-	// Auto-sync images across the cluster (hourly)
-	d.clusterTasks.Add(autoSyncImagesTask(d))
-
-	// Remove orphaned operations
-	d.clusterTasks.Add(autoRemoveOrphanedOperationsTask(d))
-
-	// Start all background tasks
-	d.clusterTasks.Start(d.shutdownCtx)
-}
-
-func (d *Daemon) stopClusterTasks() {
-	_ = d.clusterTasks.Stop(3 * time.Second)
-	d.clusterTasks = task.Group{}
-}
-
-func (d *Daemon) Ready() error {
-	// Check if clustered
-	clustered, err := cluster.Enabled(d.db.Node)
-	if err != nil {
-		return err
-	}
-
+	// Start cluster tasks if needed.
 	if clustered {
 		d.startClusterTasks()
 	}
@@ -1587,33 +1540,53 @@ func (d *Daemon) Ready() error {
 
 		// Remove resolved warnings (daily)
 		d.tasks.Add(pruneResolvedWarningsTask(d))
+
+		// Remove expired tokens (hourly)
+		d.tasks.Add(autoRemoveExpiredTokensTask(d))
 	}
 
 	// Start all background tasks
 	d.tasks.Start(d.shutdownCtx)
 
-	// Get daemon state struct
-	s := d.State()
-
 	// Restore instances
 	if !d.db.Cluster.LocalNodeIsEvacuated() {
-		instances, err := instance.LoadNodeAll(s, instancetype.Any)
-		if err != nil {
-			return fmt.Errorf("Failed loading instances to restore: %w", err)
-		}
-
-		instancesStart(s, instances)
+		instancesStart(d.State(), instances)
 	}
 
 	// Re-balance in case things changed while LXD was down
-	deviceTaskBalance(s)
+	deviceTaskBalance(d.State())
 
 	// Unblock incoming requests
-	close(d.readyChan)
+	d.waitReady.Cancel()
+
+	logger.Info("Daemon started")
 
 	return nil
 }
 
+func (d *Daemon) startClusterTasks() {
+	// Add initial event listeners from global database members.
+	// Run asynchronously so that connecting to remote members doesn't delay starting up other cluster tasks.
+	go cluster.EventsUpdateListeners(d.endpoints, d.db.Cluster, d.serverCert, nil, d.events.Inject)
+
+	// Heartbeats
+	d.taskClusterHeartbeat = d.clusterTasks.Add(cluster.HeartbeatTask(d.gateway))
+
+	// Auto-sync images across the cluster (hourly)
+	d.clusterTasks.Add(autoSyncImagesTask(d))
+
+	// Remove orphaned operations
+	d.clusterTasks.Add(autoRemoveOrphanedOperationsTask(d))
+
+	// Start all background tasks
+	d.clusterTasks.Start(d.shutdownCtx)
+}
+
+func (d *Daemon) stopClusterTasks() {
+	_ = d.clusterTasks.Stop(3 * time.Second)
+	d.clusterTasks = task.Group{}
+}
+
 // numRunningInstances returns the number of running instances.
 func (d *Daemon) numRunningInstances(instances []instance.Instance) int {
 	count := 0
@@ -1675,7 +1648,6 @@ func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
 			// waitForOperations will block until all operations are done, or it's forced to shut down.
 			// For the latter case, we re-use the shutdown channel which is filled when a shutdown is
 			// initiated using `lxd shutdown`.
-			logger.Info("Waiting for operations to finish")
 			waitForOperations(ctx, d.db.Cluster, s.GlobalConfig.ShutdownTimeout())
 		}
 
@@ -1700,7 +1672,6 @@ func (d *Daemon) Stop(ctx context.Context, sig os.Signal) error {
 
 		// Full shutdown requested.
 		if sig == unix.SIGPWR {
-			logger.Info("Stopping instances")
 			instancesShutdown(s, instances)
 
 			logger.Info("Stopping networks")
@@ -2070,7 +2041,7 @@ func (d *Daemon) heartbeatHandler(w http.ResponseWriter, r *http.Request, isLead
 
 	// Accept raft node list from any heartbeat type so that we get freshest data quickly.
 	logger.Debug("Replace current raft nodes", logger.Ctx{"raftMembers": raftNodes})
-	err = d.db.Node.Transaction(func(tx *db.NodeTx) error {
+	err = d.db.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		return tx.ReplaceRaftNodes(raftNodes)
 	})
 	if err != nil {
@@ -2079,7 +2050,7 @@ func (d *Daemon) heartbeatHandler(w http.ResponseWriter, r *http.Request, isLead
 		return
 	}
 
-	localAddress, _ := node.ClusterAddress(d.db.Node)
+	localClusterAddress := d.State().LocalConfig.ClusterAddress()
 
 	if hbData.FullStateList {
 		// If there is an ongoing heartbeat round (and by implication this is the leader), then this could
@@ -2100,7 +2071,7 @@ func (d *Daemon) heartbeatHandler(w http.ResponseWriter, r *http.Request, isLead
 			return
 		}
 
-		logger.Info("Partial heartbeat received", logger.Ctx{"local": localAddress})
+		logger.Info("Partial heartbeat received", logger.Ctx{"local": localClusterAddress})
 	}
 }
 
@@ -2118,10 +2089,10 @@ func (d *Daemon) nodeRefreshTask(heartbeatData *cluster.APIHeartbeat, isLeader b
 		return
 	}
 
-	localAddress, _ := node.ClusterAddress(d.db.Node)
+	localClusterAddress := d.State().LocalConfig.ClusterAddress()
 
 	if !heartbeatData.FullStateList || len(heartbeatData.Members) <= 0 {
-		logger.Error("Heartbeat member refresh task called with partial state list", logger.Ctx{"local": localAddress})
+		logger.Error("Heartbeat member refresh task called with partial state list", logger.Ctx{"local": localClusterAddress})
 		return
 	}
 
@@ -2137,14 +2108,14 @@ func (d *Daemon) nodeRefreshTask(heartbeatData *cluster.APIHeartbeat, isLeader b
 	stateChangeTaskFailure := false // Records whether any of the state change tasks failed.
 
 	// Handle potential OVN chassis changes.
-	err := networkUpdateOVNChassis(s, heartbeatData, localAddress)
+	err := networkUpdateOVNChassis(s, heartbeatData, localClusterAddress)
 	if err != nil {
 		stateChangeTaskFailure = true
 		logger.Error("Error restarting OVN networks", logger.Ctx{"err": err})
 	}
 
 	if d.hasMemberStateChanged(heartbeatData) {
-		logger.Info("Cluster member state has changed", logger.Ctx{"local": localAddress})
+		logger.Info("Cluster member state has changed", logger.Ctx{"local": localClusterAddress})
 
 		// Refresh cluster certificates cached.
 		updateCertificateCache(d)
@@ -2153,7 +2124,7 @@ func (d *Daemon) nodeRefreshTask(heartbeatData *cluster.APIHeartbeat, isLeader b
 		err := networkUpdateForkdnsServersTask(s, heartbeatData)
 		if err != nil {
 			stateChangeTaskFailure = true
-			logger.Error("Error refreshing forkdns", logger.Ctx{"err": err, "local": localAddress})
+			logger.Error("Error refreshing forkdns", logger.Ctx{"err": err, "local": localClusterAddress})
 		}
 	}
 
@@ -2209,10 +2180,10 @@ func (d *Daemon) nodeRefreshTask(heartbeatData *cluster.APIHeartbeat, isLeader b
 		// can upgrade some member.
 		if isDegraded || onlineVoters < int(maxVoters) || onlineStandbys < int(maxStandBy) {
 			d.clusterMembershipMutex.Lock()
-			logger.Debug("Rebalancing member roles in heartbeat", logger.Ctx{"local": localAddress})
+			logger.Debug("Rebalancing member roles in heartbeat", logger.Ctx{"local": localClusterAddress})
 			err := rebalanceMemberRoles(d, nil, unavailableMembers)
 			if err != nil && !errors.Is(err, cluster.ErrNotLeader) {
-				logger.Warn("Could not rebalance cluster member roles", logger.Ctx{"err": err, "local": localAddress})
+				logger.Warn("Could not rebalance cluster member roles", logger.Ctx{"err": err, "local": localClusterAddress})
 			}
 
 			d.clusterMembershipMutex.Unlock()
@@ -2220,10 +2191,10 @@ func (d *Daemon) nodeRefreshTask(heartbeatData *cluster.APIHeartbeat, isLeader b
 
 		if hasNodesNotPartOfRaft {
 			d.clusterMembershipMutex.Lock()
-			logger.Debug("Upgrading members without raft role in heartbeat", logger.Ctx{"local": localAddress})
+			logger.Debug("Upgrading members without raft role in heartbeat", logger.Ctx{"local": localClusterAddress})
 			err := upgradeNodesWithoutRaftRole(d)
 			if err != nil && !errors.Is(err, cluster.ErrNotLeader) {
-				logger.Warn("Failed upgrading raft roles:", logger.Ctx{"err": err, "local": localAddress})
+				logger.Warn("Failed upgrading raft roles:", logger.Ctx{"err": err, "local": localClusterAddress})
 			}
 
 			d.clusterMembershipMutex.Unlock()
