@@ -1,6 +1,7 @@
 package main
 
 import (
+	"context"
 	"crypto/sha256"
 	"fmt"
 	"io"
@@ -10,6 +11,7 @@ import (
 	"time"
 
 	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/lifecycle"
 	"github.com/lxc/lxd/lxd/locking"
@@ -50,7 +52,7 @@ func (d *Daemon) imageOperationLock(fingerprint string) locking.UnlockFunc {
 	l.Debug("Acquiring lock for image")
 	defer l.Debug("Lock acquired for image")
 
-	return locking.Lock(fmt.Sprintf("ImageOperation_%s", fingerprint))
+	return locking.Lock(context.TODO(), fmt.Sprintf("ImageOperation_%s", fingerprint))
 }
 
 // ImageDownload resolves the image fingerprint and if not in the database, downloads it.
@@ -132,12 +134,19 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 	interval := d.State().GlobalConfig.ImagesAutoUpdateIntervalHours()
 
 	if args.PreferCached && interval > 0 && alias != fp {
-		for _, architecture := range d.os.Architectures {
-			cachedFingerprint, err := d.db.Cluster.GetCachedImageSourceFingerprint(args.Server, args.Protocol, alias, args.Type, architecture)
-			if err == nil && cachedFingerprint != fp {
-				fp = cachedFingerprint
-				break
+		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			for _, architecture := range d.os.Architectures {
+				cachedFingerprint, err := tx.GetCachedImageSourceFingerprint(ctx, args.Server, args.Protocol, alias, args.Type, architecture)
+				if err == nil && cachedFingerprint != fp {
+					fp = cachedFingerprint
+					break
+				}
 			}
+
+			return nil
+		})
+		if err != nil {
+			return nil, err
 		}
 	}
 
@@ -180,6 +189,14 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 				return nil, err
 			}
 
+			// Mark the image as "cached" if downloading for an instance.
+			if args.SetCached {
+				err := d.db.Cluster.SetImageCachedAndLastUseDate(args.ProjectName, imgInfo.Fingerprint, time.Now().UTC())
+				if err != nil {
+					return nil, fmt.Errorf("Failed setting cached flag and last use date: %w", err)
+				}
+			}
+
 			var id int
 			id, imgInfo, err = d.db.Cluster.GetImage(fp, cluster.ImageFilter{Project: &args.ProjectName})
 			if err != nil {
@@ -537,9 +554,9 @@ func (d *Daemon) ImageDownload(r *http.Request, op *operations.Operation, args *
 
 	// Mark the image as "cached" if downloading for an instance
 	if args.SetCached {
-		err := d.db.Cluster.InitImageLastUseDate(fp)
+		err := d.db.Cluster.SetImageCachedAndLastUseDate(args.ProjectName, fp, time.Now().UTC())
 		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("Failed setting cached flag and last use date: %w", err)
 		}
 	}
 
