@@ -1,8 +1,8 @@
 package main
 
 import (
+	"context"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -21,8 +21,8 @@ func daemonStorageVolumesUnmount(s *state.State) error {
 	var storageBackups string
 	var storageImages string
 
-	err := s.DB.Node.Transaction(func(tx *db.NodeTx) error {
-		nodeConfig, err := node.ConfigLoad(tx)
+	err := s.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		nodeConfig, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
 		}
@@ -77,8 +77,8 @@ func daemonStorageVolumesUnmount(s *state.State) error {
 func daemonStorageMount(s *state.State) error {
 	var storageBackups string
 	var storageImages string
-	err := s.DB.Node.Transaction(func(tx *db.NodeTx) error {
-		nodeConfig, err := node.ConfigLoad(tx)
+	err := s.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		nodeConfig, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
 		}
@@ -160,14 +160,25 @@ func daemonStorageValidate(s *state.State, target string) error {
 	}
 
 	// Confirm volume exists.
-	_, _, err = s.DB.Cluster.GetLocalStoragePoolVolume(project.Default, volumeName, db.StoragePoolVolumeTypeCustom, poolID)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVol, err := tx.GetStoragePoolVolume(ctx, poolID, project.Default, db.StoragePoolVolumeTypeCustom, volumeName, true)
+		if err != nil {
+			return fmt.Errorf("Failed loading storage volume %q in %q project: %w", target, project.Default, err)
+		}
+
+		if dbVol.ContentType != db.StoragePoolVolumeContentTypeNameFS {
+			return fmt.Errorf("Storage volume %q in %q project is not filesystem content type", target, project.Default)
+		}
+
+		return nil
+	})
 	if err != nil {
-		return fmt.Errorf("Unable to load storage volume %q: %w", target, err)
+		return err
 	}
 
 	snapshots, err := s.DB.Cluster.GetLocalStoragePoolVolumeSnapshotsWithType(project.Default, volumeName, db.StoragePoolVolumeTypeCustom, poolID)
 	if err != nil {
-		return fmt.Errorf("Unable to load storage volume snapshots %q: %w", target, err)
+		return fmt.Errorf("Unable to load storage volume snapshots %q in %q project: %w", target, project.Default, err)
 	}
 
 	if len(snapshots) != 0 {
@@ -191,7 +202,7 @@ func daemonStorageValidate(s *state.State, target string) error {
 	volStorageName := project.StorageVolume(project.Default, volumeName)
 	mountpoint := storageDrivers.GetVolumeMountPath(poolName, storageDrivers.VolumeTypeCustom, volStorageName)
 
-	entries, err := ioutil.ReadDir(mountpoint)
+	entries, err := os.ReadDir(mountpoint)
 	if err != nil {
 		return fmt.Errorf("Failed to list %q: %w", mountpoint, err)
 	}
@@ -239,7 +250,7 @@ func daemonStorageMove(s *state.State, storageType string, target string) error
 		}
 
 		// Remove the source content.
-		entries, err := ioutil.ReadDir(source)
+		entries, err := os.ReadDir(source)
 		if err != nil {
 			return err
 		}
