@@ -4,7 +4,6 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"testing"
@@ -183,7 +182,7 @@ INSERT INTO nodes(name, address, schema, api_extensions, arch, description) VALU
 func assertNode(t *testing.T, db *sql.DB, address string, schema int, apiExtensions int) {
 	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		where := "address=? AND schema=? AND api_extensions=?"
-		n, err := query.Count(tx, "nodes", where, address, schema, apiExtensions)
+		n, err := query.Count(ctx, tx, "nodes", where, address, schema, apiExtensions)
 		assert.Equal(t, 1, n, "node does not have expected version")
 		return err
 	})
@@ -194,7 +193,7 @@ func assertNode(t *testing.T, db *sql.DB, address string, schema int, apiExtensi
 func newDir(t *testing.T) (string, func()) {
 	t.Helper()
 
-	dir, err := ioutil.TempDir("", "dqlite-replication-test-")
+	dir, err := os.MkdirTemp("", "dqlite-replication-test-")
 	assert.NoError(t, err)
 
 	cleanup := func() {
