@@ -3,12 +3,14 @@
 package db
 
 import (
+	"context"
+
 	"github.com/lxc/lxd/lxd/db/query"
 )
 
 // Config fetches all LXD node-level config keys.
-func (n *NodeTx) Config() (map[string]string, error) {
-	return query.SelectConfig(n.tx, "config", "")
+func (n *NodeTx) Config(ctx context.Context) (map[string]string, error) {
+	return query.SelectConfig(ctx, n.tx, "config", "")
 }
 
 // UpdateConfig updates the given LXD node-level configuration keys in the
@@ -18,8 +20,8 @@ func (n *NodeTx) UpdateConfig(values map[string]string) error {
 }
 
 // Config fetches all LXD cluster config keys.
-func (c *ClusterTx) Config() (map[string]string, error) {
-	return query.SelectConfig(c.tx, "config", "")
+func (c *ClusterTx) Config(ctx context.Context) (map[string]string, error) {
+	return query.SelectConfig(ctx, c.tx, "config", "")
 }
 
 // UpdateClusterConfig updates the given LXD cluster configuration keys in the
