@@ -3,6 +3,7 @@
 package db_test
 
 import (
+	"context"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
@@ -15,7 +16,7 @@ import (
 func TestTx_Config(t *testing.T) {
 	tx, cleanup := db.NewTestNodeTx(t)
 	defer cleanup()
-	values, err := tx.Config()
+	values, err := tx.Config(context.Background())
 	require.NoError(t, err)
 	assert.Equal(t, map[string]string{}, values)
 	assert.NoError(t, err)
@@ -29,7 +30,7 @@ func TestTx_UpdateConfig(t *testing.T) {
 	err := tx.UpdateConfig(map[string]string{"foo": "x", "bar": "y"})
 	require.NoError(t, err)
 
-	values, err := tx.Config()
+	values, err := tx.Config(context.Background())
 	require.NoError(t, err)
 	assert.Equal(t, map[string]string{"foo": "x", "bar": "y"}, values)
 }
@@ -44,7 +45,7 @@ func TestTx_UpdateConfigUnsetKeys(t *testing.T) {
 	err = tx.UpdateConfig(map[string]string{"foo": "x", "bar": ""})
 	require.NoError(t, err)
 
-	values, err := tx.Config()
+	values, err := tx.Config(context.Background())
 	require.NoError(t, err)
 	assert.Equal(t, map[string]string{"foo": "x"}, values)
 }
