@@ -97,11 +97,11 @@ func (n *Node) Dir() string {
 // node-level database interactions invoked by the given function. If the
 // function returns no error, all database changes are committed to the
 // node-level database, otherwise they are rolled back.
-func (n *Node) Transaction(f func(*NodeTx) error) error {
+func (n *Node) Transaction(ctx context.Context, f func(context.Context, *NodeTx) error) error {
 	nodeTx := &NodeTx{}
-	return query.Transaction(context.TODO(), n.db, func(ctx context.Context, tx *sql.Tx) error {
+	return query.Transaction(ctx, n.db, func(ctx context.Context, tx *sql.Tx) error {
 		nodeTx.tx = tx
-		return f(nodeTx)
+		return f(ctx, nodeTx)
 	})
 }
 
@@ -245,33 +245,33 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 
 	err = clusterDB.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		// Figure out the ID of this node.
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed to fetch nodes: %w", err)
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
-		nodeID := int64(-1)
-		if len(nodes) == 1 && nodes[0].Address == "0.0.0.0" {
+		memberID := int64(-1)
+		if len(members) == 1 && members[0].Address == "0.0.0.0" {
 			// We're not clustered
-			nodeID = 1
+			memberID = 1
 		} else {
-			for _, node := range nodes {
-				if node.Address == address {
-					nodeID = node.ID
+			for _, member := range members {
+				if member.Address == address {
+					memberID = member.ID
 					break
 				}
 			}
 		}
 
-		if nodeID < 0 {
+		if memberID < 0 {
 			return fmt.Errorf("No node registered with address %s", address)
 		}
 
 		// Set the local member ID
-		clusterDB.NodeID(nodeID)
+		clusterDB.NodeID(memberID)
 
 		// Delete any operation tied to this member
-		err = cluster.DeleteOperations(ctx, tx.tx, nodeID)
+		err = cluster.DeleteOperations(ctx, tx.tx, memberID)
 		if err != nil {
 			return err
 		}
@@ -392,7 +392,7 @@ func (c *Cluster) retry(f func() error) error {
 	return query.Retry(f)
 }
 
-// NodeID sets the the node NodeID associated with this cluster instance. It's used for
+// NodeID sets the node NodeID associated with this cluster instance. It's used for
 // backward-compatibility of all db-related APIs that were written before
 // clustering and don't accept a node NodeID, so in those cases we automatically
 // use this value as implicit node NodeID.
@@ -500,17 +500,17 @@ func DqliteLatestSegment() (string, error) {
 func dbQueryRowScan(c *Cluster, q string, args []any, outargs []any) error {
 	return c.retry(func() error {
 		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
-			return tx.QueryRow(q, args...).Scan(outargs...)
+			return tx.QueryRowContext(ctx, q, args...).Scan(outargs...)
 		})
 	})
 }
 
-func doDbQueryScan(c *Cluster, q string, args []any, outargs []any) ([][]any, error) {
+func doDbScan(c *Cluster, q string, args []any, outargs []any) ([][]any, error) {
 	result := [][]any{}
 
 	err := c.retry(func() error {
 		return query.Transaction(context.TODO(), c.db, func(ctx context.Context, tx *sql.Tx) error {
-			rows, err := tx.Query(q, args...)
+			rows, err := tx.QueryContext(ctx, q, args...)
 			if err != nil {
 				return err
 			}
@@ -589,7 +589,7 @@ func doDbQueryScan(c *Cluster, q string, args []any, outargs []any) ([][]any, er
  * of interfaces, containing pointers to the actual output arguments.
  */
 func queryScan(c *Cluster, q string, inargs []any, outfmt []any) ([][]any, error) {
-	return doDbQueryScan(c, q, inargs, outfmt)
+	return doDbScan(c, q, inargs, outfmt)
 }
 
 func exec(c *Cluster, q string, args ...any) error {
