@@ -7,6 +7,7 @@ import (
 	"go/ast"
 	"go/build"
 	"os"
+	"path/filepath"
 	"strings"
 
 	"github.com/lxc/lxd/lxd/db/generate/file"
@@ -46,6 +47,10 @@ func NewMethod(database, pkg, entity, kind string, config map[string]string) (*M
 		return nil, err
 	}
 
+	if database != "" {
+		database = filepath.Base(database)
+	}
+
 	method := &Method{
 		db:     database,
 		entity: entity,
@@ -80,8 +85,6 @@ func (m *Method) Generate(buf *file.Buffer) error {
 	}
 
 	switch operation(m.kind) {
-	case "URIs":
-		return m.uris(buf)
 	case "GetMany":
 		return m.getMany(buf)
 	case "GetOne":
@@ -130,108 +133,17 @@ func (m *Method) GenerateSignature(buf *file.Buffer) error {
 	return m.signature(buf, true)
 }
 
-func (m *Method) uris(buf *file.Buffer) error {
-	mapping, err := Parse(m.pkg, lex.Camel(m.entity), m.kind)
-	if err != nil {
-		return fmt.Errorf("Parse entity struct: %w", err)
-	}
-
-	// Go type name the objects to return (e.g. api.Foo).
-	typ := lex.Camel(m.entity)
-
-	err = m.signature(buf, false)
-	if err != nil {
-		return err
-	}
-
-	defer m.end(buf)
-
-	buf.L("var err error")
-	buf.N()
-	buf.L("// Result slice.")
-	buf.L("objects := make(%s, 0)", lex.Slice(typ))
-	buf.N()
-	filters, ignoredFilters := FiltersFromStmt(m.pkg, "objects", m.entity, mapping.Filters)
-	buf.N()
-	buf.L("// Pick the prepared statement and arguments to use based on active criteria.")
-	buf.L("var stmt *sql.Stmt")
-	buf.L("var args []any")
-	buf.N()
-
-	for i, filter := range filters {
-		branch := "if"
-		if i > 0 {
-			branch = "} else if"
-		}
-
-		buf.L("%s %s {", branch, activeCriteria(filter, ignoredFilters[i]))
-
-		if m.db == "" {
-			buf.L("stmt = Stmt(tx, %s)", stmtCodeVar(m.entity, "objects", filter...))
-		} else {
-			buf.L("stmt = %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects", filter...))
-		}
-
-		buf.L("args = []any{")
-		for _, name := range filter {
-			if name == "Parent" {
-				buf.L("len(filter.Parent)+1,")
-				buf.L("filter.%s+\"/\",", name)
-			} else {
-				buf.L("filter.%s,", name)
-			}
-		}
-
-		buf.L("}")
-	}
-
-	branch := "if"
-	if len(filters) > 0 {
-		branch = "} else if"
-	}
-
-	buf.L("%s %s {", branch, activeCriteria([]string{}, FieldNames(mapping.Filters)))
-	if m.db == "" {
-		buf.L("stmt = Stmt(tx, %s)", stmtCodeVar(m.entity, "objects"))
-	} else {
-		buf.L("stmt = %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects"))
-	}
-
-	buf.L("args = []any{}")
-	buf.L("} else {")
-	buf.L("return nil, fmt.Errorf(\"No statement exists for the given Filter\")")
-	buf.L("}")
-	buf.N()
-	buf.L("// Dest function for scanning a row.")
-	buf.L("dest := %s", destFunc("objects", typ, mapping.ColumnFields()))
-	buf.N()
-	buf.L("// Select.")
-	buf.L("err = query.SelectObjects(stmt, dest, args...)")
-	m.ifErrNotNil(buf, true, "nil", fmt.Sprintf("fmt.Errorf(\"Failed to fetch from \\\"%s\\\" table: %%w\", err)", entityTable(m.entity, m.config["table"])))
-	buf.L("uris := make([]string, len(objects))")
-	buf.L("for i := range objects {")
-	name := mapping.Identifier().Name
-	buf.L("uri := api.NewURL().Path(version.APIVersion, \"%s\", objects[i].%s)", lex.Plural(m.entity), name)
-	for _, field := range mapping.NaturalKey() {
-		if field.Name != name {
-			buf.L("uri.%s(objects[i].%s)", field.Name, field.Name)
-		}
-	}
-	buf.N()
-	buf.L("uris[i] = uri.String()")
-	buf.L("}")
-	buf.N()
-	buf.L("return uris, nil")
-
-	return nil
-}
-
 func (m *Method) getMany(buf *file.Buffer) error {
 	mapping, err := Parse(m.pkg, lex.Camel(m.entity), m.kind)
 	if err != nil {
 		return fmt.Errorf("Parse entity struct: %w", err)
 	}
 
+	err = m.getManyTemplateFuncs(buf, mapping)
+	if err != nil {
+		return err
+	}
+
 	if m.config["references"] != "" {
 		refFields := strings.Split(m.config["references"], ",")
 		for _, fieldName := range refFields {
@@ -268,31 +180,80 @@ func (m *Method) getMany(buf *file.Buffer) error {
 		buf.L("fillParent[i] = strings.Replace(parent, \"_\", \"s_\", -1) + \"s\"")
 		buf.L("}")
 		buf.N()
-		if m.db == "" {
-			buf.L("sqlStmt, err := prepare(tx, fmt.Sprintf(%s, fillParent...))", stmtLocal)
-		} else {
-			buf.L("sqlStmt, err := %s.Prepare(tx, fmt.Sprintf(%s, fillParent...))", m.db, stmtLocal)
+		buf.L("queryStr := fmt.Sprintf(%s, fillParent...)", stmtLocal)
+		buf.L("queryParts := strings.SplitN(queryStr, \"ORDER BY\", 2)")
+		buf.L("args := []any{}")
+
+		buf.N()
+		buf.L("for i, filter := range filters {")
+		buf.L("var cond string")
+		buf.L("if i == 0 {")
+		buf.L("cond = \" WHERE ( %%s )\"")
+		buf.L("} else {")
+		buf.L("cond = \" OR ( %%s )\"")
+		buf.L("}")
+		buf.N()
+		buf.L("entries := []string{}")
+		for _, filter := range mapping.Filters {
+			// Skip over filter fields that are themselves filters for a referenced table.
+			found := false
+			for _, refField := range mapping.RefFields() {
+				if filter.Type.Name == entityFilter(refField.Name) {
+					found = true
+					break
+				}
+			}
+
+			if found {
+				continue
+			}
+
+			buf.L("if filter.%s != nil {", filter.Name)
+			buf.L("entries = append(entries, \"%s = ?\")", lex.Snake(filter.Name))
+			buf.L("args = append(args, filter.%s)", filter.Name)
+			buf.L("}")
+			buf.N()
 		}
 
-		m.ifErrNotNil(buf, true, "nil", "err")
-		buf.L("args := []any{}")
+		buf.L("if len(entries) == 0 {")
+		buf.L("return nil, fmt.Errorf(\"Cannot filter on empty %s\")", entityFilter(mapping.Name))
+		buf.L("}")
+		buf.N()
+		buf.L("queryParts[0] += fmt.Sprintf(cond, strings.Join(entries, \" AND \"))")
+		buf.L("}")
+		buf.N()
+		buf.L("queryStr = strings.Join(queryParts, \" ORDER BY\")")
 	} else if mapping.Type == AssociationTable {
 		filter := m.config["struct"] + "ID"
 		if m.db == "" {
-			buf.L("sqlStmt := Stmt(tx, %s)", stmtCodeVar(m.entity, "objects", filter))
+			buf.L("sqlStmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, "objects", filter))
 		} else {
-			buf.L("sqlStmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects", filter))
+			buf.L("sqlStmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects", filter))
 		}
 
+		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "objects", filter)))
+
 		buf.L("args := []any{%sID}", lex.Minuscule(m.config["struct"]))
 	} else {
 		filters, ignoredFilters := FiltersFromStmt(m.pkg, "objects", m.entity, mapping.Filters)
 		buf.N()
 		buf.L("// Pick the prepared statement and arguments to use based on active criteria.")
 		buf.L("var sqlStmt *sql.Stmt")
-		buf.L("var args []any")
+		buf.L("args := []any{}")
+		buf.L("queryParts := [2]string{}")
 		buf.N()
 
+		buf.L("if len(filters) == 0 {")
+		if m.db == "" {
+			buf.L("sqlStmt, err = Stmt(tx, %s)", stmtCodeVar(m.entity, "objects"))
+		} else {
+			buf.L("sqlStmt, err = %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects"))
+		}
+
+		m.ifErrNotNil(buf, false, "nil", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "objects")))
+		buf.L("}")
+		buf.N()
+		buf.L("for i, filter := range filters {")
 		for i, filter := range filters {
 			branch := "if"
 			if i > 0 {
@@ -300,25 +261,38 @@ func (m *Method) getMany(buf *file.Buffer) error {
 			}
 
 			buf.L("%s %s {", branch, activeCriteria(filter, ignoredFilters[i]))
-
-			if m.db == "" {
-				buf.L("sqlStmt = Stmt(tx, %s)", stmtCodeVar(m.entity, "objects", filter...))
-			} else {
-				buf.L("sqlStmt = %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects", filter...))
-			}
-
-			buf.L("args = []any{")
-
+			var args string
 			for _, name := range filter {
-				if name == "Parent" {
-					buf.L("len(filter.Parent)+1,")
-					buf.L("filter.%s+\"/\",", name)
-				} else {
-					buf.L("filter.%s,", name)
-				}
+				args += fmt.Sprintf("filter.%s,", name)
 			}
 
+			buf.L("args = append(args, []any{%s}...)", args)
+			buf.L("if len(filters) == 1 {")
+			if m.db == "" {
+				buf.L("sqlStmt, err = Stmt(tx, %s)", stmtCodeVar(m.entity, "objects", filter...))
+			} else {
+				buf.L("sqlStmt, err = %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects", filter...))
+			}
+
+			m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "objects", filter...)))
+			buf.L("break")
 			buf.L("}")
+			buf.N()
+			if m.db != "" {
+				buf.L("query, err := %s.StmtString(%s)", m.db, stmtCodeVar(m.entity, "objects", filter...))
+			} else {
+				buf.L("query, err := StmtString(%s)", stmtCodeVar(m.entity, "objects", filter...))
+			}
+
+			m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "objects")))
+			buf.L("parts := strings.SplitN(query, \"ORDER BY\", 2)")
+			buf.L("if i == 0 {")
+			buf.L("copy(queryParts[:], parts)")
+			buf.L("continue")
+			buf.L("}")
+			buf.N()
+			buf.L("_, where, _ := strings.Cut(parts[0], \"WHERE\")")
+			buf.L("queryParts[0] += \"OR\" + where")
 		}
 
 		branch := "if"
@@ -327,27 +301,32 @@ func (m *Method) getMany(buf *file.Buffer) error {
 		}
 
 		buf.L("%s %s {", branch, activeCriteria([]string{}, FieldNames(mapping.Filters)))
-		if m.db == "" {
-			buf.L("sqlStmt = Stmt(tx, %s)", stmtCodeVar(m.entity, "objects"))
-		} else {
-			buf.L("sqlStmt = %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "objects"))
-		}
-
-		buf.L("args = []any{}")
+		buf.L("return nil, fmt.Errorf(\"Cannot filter on empty %s\")", entityFilter(mapping.Name))
 		buf.L("} else {")
 		buf.L("return nil, fmt.Errorf(\"No statement exists for the given Filter\")")
 		buf.L("}")
+		buf.L("}")
+		buf.N()
 	}
 
-	buf.N()
-	buf.L("// Dest function for scanning a row.")
-	buf.L("dest := %s", destFunc("objects", typ, mapping.ColumnFields()))
-	buf.N()
-	buf.L("// Select.")
-	buf.L("err = query.SelectObjects(sqlStmt, dest, args...)")
-	if mapping.Type == ReferenceTable || mapping.Type == MapTable {
+	if mapping.Type == EntityTable {
+		buf.L("// Select.")
+		buf.L("if sqlStmt != nil {")
+		buf.L("objects, err = get%s(ctx, sqlStmt, args...)", lex.Plural(mapping.Name))
+		buf.L("} else {")
+		buf.L("queryStr := strings.Join(queryParts[:], \"ORDER BY\")")
+		buf.L("objects, err = get%sRaw(ctx, tx, queryStr, args...)", lex.Plural(mapping.Name))
+		buf.L("}")
+		buf.N()
+		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%s\" table: %%w", err)`, entityTable(m.entity, m.config["table"])))
+	} else if mapping.Type == ReferenceTable || mapping.Type == MapTable {
+		buf.L("// Select.")
+		buf.L("objects, err = get%sRaw(ctx, tx, queryStr, parent, args...)", lex.Plural(mapping.Name))
 		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%%s_%s\" table: %%w", parent, err)`, entityTable(m.entity, m.config["table"])))
 	} else {
+		buf.N()
+		buf.L("// Select.")
+		buf.L("objects, err = get%s(ctx, sqlStmt, args...)", lex.Plural(mapping.Name))
 		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%s\" table: %%w", err)`, entityTable(m.entity, m.config["table"])))
 	}
 
@@ -384,12 +363,24 @@ func (m *Method) getMany(buf *file.Buffer) error {
 			buf.L("}")
 			buf.L("}")
 		case ReferenceTable:
+			buf.L("%sFilters := []%s{}", refVar, entityFilter(refStruct))
+			buf.L("for _, f := range filters {")
+			buf.L("filter := f.%s", refStruct)
+			buf.L("if filter != nil {")
+			buf.L("if %s {", activeCriteria(nil, FieldNames(refMapping.Filters)))
+			buf.L("return nil, fmt.Errorf(\"Cannot filter on empty %s\")", entityFilter(refMapping.Name))
+			buf.L("}")
+			buf.N()
+			buf.L("%sFilters = append(%sFilters, *filter)", refVar, refVar)
+			buf.L("}")
+			buf.L("}")
+			buf.N()
 			if mapping.Type == ReferenceTable {
 				// A reference table should let its child reference know about its parent.
-				buf.L("%s, err := Get%s(ctx, tx, parent+\"_%s\")", refSlice, lex.Plural(refStruct), m.entity)
+				buf.L("%s, err := Get%s(ctx, tx, parent+\"_%s\", %sFilters...)", refSlice, lex.Plural(refStruct), m.entity, refVar)
 				m.ifErrNotNil(buf, true, "nil", "err")
 			} else {
-				buf.L("%s, err := Get%s(ctx, tx, \"%s\")", refSlice, lex.Plural(refStruct), m.entity)
+				buf.L("%s, err := Get%s(ctx, tx, \"%s\", %sFilters...)", refSlice, lex.Plural(refStruct), m.entity, refVar)
 				m.ifErrNotNil(buf, true, "nil", "err")
 			}
 
@@ -410,12 +401,24 @@ func (m *Method) getMany(buf *file.Buffer) error {
 
 			buf.L("}")
 		case MapTable:
+			buf.L("%sFilters := []%s{}", refVar, entityFilter(refStruct))
+			buf.L("for _, f := range filters {")
+			buf.L("filter := f.%s", refStruct)
+			buf.L("if filter != nil {")
+			buf.L("if %s {", activeCriteria(nil, FieldNames(refMapping.Filters)))
+			buf.L("return nil, fmt.Errorf(\"Cannot filter on empty %s\")", entityFilter(refMapping.Name))
+			buf.L("}")
+			buf.N()
+			buf.L("%sFilters = append(%sFilters, *filter)", refVar, refVar)
+			buf.L("}")
+			buf.L("}")
+			buf.N()
 			if mapping.Type == ReferenceTable {
 				// A reference table should let its child reference know about its parent.
-				buf.L("%s, err := Get%s(ctx, tx, parent+\"_%s\")", refSlice, lex.Plural(refStruct), m.entity)
+				buf.L("%s, err := Get%s(ctx, tx, parent+\"_%s\", %sFilters...)", refSlice, lex.Plural(refStruct), m.entity, refVar)
 				m.ifErrNotNil(buf, true, "nil", "err")
 			} else {
-				buf.L("%s, err := Get%s(ctx, tx, \"%s\")", refSlice, lex.Plural(refStruct), m.entity)
+				buf.L("%s, err := Get%s(ctx, tx, \"%s\", %sFilters...)", refSlice, lex.Plural(refStruct), m.entity, refVar)
 				m.ifErrNotNil(buf, true, "nil", "err")
 			}
 
@@ -495,7 +498,7 @@ func (m *Method) getRefs(buf *file.Buffer, refMapping *Mapping) error {
 
 	switch refMapping.Type {
 	case ReferenceTable:
-		buf.L("%s, err := Get%s(ctx, tx, \"%s\")", refParentList, lex.Plural(refStruct), m.entity)
+		buf.L("%s, err := Get%s(ctx, tx, \"%s\", filters...)", refParentList, lex.Plural(refStruct), m.entity)
 		m.ifErrNotNil(buf, true, "nil", "err")
 		buf.L("%s := map[string]%s{}", refList, refStruct)
 		buf.L("for _, ref := range %s[%sID] {", refParentList, refParent)
@@ -508,7 +511,7 @@ func (m *Method) getRefs(buf *file.Buffer, refMapping *Mapping) error {
 		buf.L("}")
 		buf.N()
 	case MapTable:
-		buf.L("%s, err := Get%s(ctx, tx, \"%s\")", refParentList, lex.Plural(refStruct), m.entity)
+		buf.L("%s, err := Get%s(ctx, tx, \"%s\", filters...)", refParentList, lex.Plural(refStruct), m.entity)
 		m.ifErrNotNil(buf, true, "nil", "err")
 		buf.L("%s, ok := %s[%sID]", refList, refParentList, refParent)
 		buf.L("if !ok {")
@@ -584,29 +587,20 @@ func (m *Method) id(buf *file.Buffer) error {
 	defer m.end(buf)
 
 	if m.db == "" {
-		buf.L("stmt := Stmt(tx, %s)", stmtCodeVar(m.entity, "ID"))
+		buf.L("stmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, "ID"))
 	} else {
-		buf.L("stmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "ID"))
+		buf.L("stmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "ID"))
 	}
 
-	buf.L("rows, err := stmt.Query(%s)", mapping.FieldParams(nk))
-	m.ifErrNotNil(buf, true, "-1", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" ID: %%w", err)`, entityTable(m.entity, m.config["table"])))
-	buf.L("defer func() { _ = rows.Close() }()")
-	buf.N()
-	buf.L("// Ensure we read one and only one row.")
-	buf.L("if !rows.Next() {")
-	buf.L(`        return -1, api.StatusErrorf(http.StatusNotFound, "%s not found")`, lex.Camel(m.entity))
-	buf.L("}")
-	buf.N()
+	m.ifErrNotNil(buf, true, "-1", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "ID")))
+	buf.L("row := stmt.QueryRowContext(ctx, %s)", mapping.FieldParams(nk))
 	buf.L("var id int64")
-	buf.L("err = rows.Scan(&id)")
-	m.ifErrNotNil(buf, true, "-1", "fmt.Errorf(\"Failed to scan ID: %w\", err)")
-	buf.L("if rows.Next() {")
-	buf.L("        return -1, fmt.Errorf(\"More than one row returned\")")
+	buf.L("err = row.Scan(&id)")
+	buf.L("if errors.Is(err, sql.ErrNoRows) {")
+	buf.L(`return -1, api.StatusErrorf(http.StatusNotFound, "%s not found")`, lex.Camel(m.entity))
 	buf.L("}")
 	buf.N()
-	buf.L("err = rows.Err()")
-	m.ifErrNotNil(buf, true, "-1", "fmt.Errorf(\"Result set failure: %w\", err)")
+	m.ifErrNotNil(buf, true, "-1", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" ID: %%w", err)`, entityTable(m.entity, m.config["table"])))
 	buf.L("return id, nil")
 
 	return nil
@@ -689,8 +683,7 @@ func (m *Method) create(buf *file.Buffer, replace bool) error {
 		buf.L("fillParent[i] = strings.Replace(parent, \"_\", \"s_\", -1) + \"s\"")
 		buf.L("}")
 		buf.N()
-		buf.L("stmt, err := %sprepare(tx, fmt.Sprintf(%s, fillParent...))", m.db, stmtLocal)
-		m.ifErrNotNil(buf, true, "err")
+		buf.L("queryStr := fmt.Sprintf(%s, fillParent...)", stmtLocal)
 		createParams := ""
 		columnFields := mapping.ColumnFields("ID")
 		if mapping.Type == ReferenceTable {
@@ -706,10 +699,10 @@ func (m *Method) create(buf *file.Buffer, replace bool) error {
 
 		refFields := mapping.RefFields()
 		if len(refFields) == 0 {
-			buf.L("_, err = stmt.Exec(%s)", createParams)
+			buf.L("_, err := tx.Exec(queryStr, %s)", createParams)
 			m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Insert failed for \"%%s_%s\" table: %%w", parent, err)`, lex.Plural(m.entity)))
 		} else {
-			buf.L("result, err := stmt.Exec(%s)", createParams)
+			buf.L("result, err := tx.Exec(queryStr, %s)", createParams)
 			m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Insert failed for \"%%s_%s\" table: %%w", parent, err)`, lex.Plural(m.entity)))
 			buf.L("id, err := result.LastInsertId()")
 			m.ifErrNotNil(buf, true, "fmt.Errorf(\"Failed to fetch ID: %w\", err)")
@@ -753,18 +746,19 @@ func (m *Method) create(buf *file.Buffer, replace bool) error {
 
 		buf.L("// Prepared statement to use. ")
 		if m.db == "" {
-			buf.L("stmt := Stmt(tx, %s)", stmtCodeVar(m.entity, kind))
+			buf.L("stmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, kind))
 		} else {
-			buf.L("stmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, kind))
+			buf.L("stmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, kind))
 		}
 
-		buf.N()
-		buf.L("// Execute the statement. ")
-
 		if mapping.Type == AssociationTable {
-			buf.L("_, err := stmt.Exec(args...)")
+			m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, kind)))
+			buf.L("// Execute the statement. ")
+			buf.L("_, err = stmt.Exec(args...)")
 			m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Failed to create \"%s\" entry: %%w", err)`, entityTable(m.entity, m.config["table"])))
 		} else {
+			m.ifErrNotNil(buf, true, "-1", fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, kind)))
+			buf.L("// Execute the statement. ")
 			buf.L("result, err := stmt.Exec(args...)")
 			m.ifErrNotNil(buf, true, "-1", fmt.Sprintf(`fmt.Errorf("Failed to create \"%s\" entry: %%w", err)`, entityTable(m.entity, m.config["table"])))
 			buf.L("id, err := result.LastInsertId()")
@@ -893,11 +887,12 @@ func (m *Method) rename(buf *file.Buffer) error {
 	defer m.end(buf)
 
 	if m.db == "" {
-		buf.L("stmt := Stmt(tx, %s)", stmtCodeVar(m.entity, "rename"))
+		buf.L("stmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, "rename"))
 	} else {
-		buf.L("stmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "rename"))
+		buf.L("stmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "rename"))
 	}
 
+	m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "rename")))
 	buf.L("result, err := stmt.Exec(%s)", "to, "+mapping.FieldParams(nk))
 	m.ifErrNotNil(buf, true, fmt.Sprintf("fmt.Errorf(\"Rename %s failed: %%w\", err)", mapping.Name))
 	buf.L("n, err := result.RowsAffected()")
@@ -1013,11 +1008,12 @@ func (m *Method) update(buf *file.Buffer) error {
 		buf.L("id, err := Get%sID(ctx, tx, %s)", lex.Camel(m.entity), mapping.FieldParams(nk))
 		m.ifErrNotNil(buf, true, "err")
 		if m.db == "" {
-			buf.L("stmt := Stmt(tx, %s)", stmtCodeVar(m.entity, "update"))
+			buf.L("stmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, "update"))
 		} else {
-			buf.L("stmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "update"))
+			buf.L("stmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "update"))
 		}
 
+		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "update")))
 		buf.L("result, err := stmt.Exec(%s)", strings.Join(params, ", ")+", id")
 		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Update \"%s\" entry failed: %%w", err)`, entityTable(m.entity, m.config["table"])))
 		buf.L("n, err := result.RowsAffected()")
@@ -1100,11 +1096,12 @@ func (m *Method) delete(buf *file.Buffer, deleteOne bool) error {
 	defer m.end(buf)
 	if mapping.Type == AssociationTable {
 		if m.db == "" {
-			buf.L("stmt := Stmt(tx, %s)", stmtCodeVar(m.entity, "delete", m.config["struct"]+"ID"))
+			buf.L("stmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, "delete", m.config["struct"]+"ID"))
 		} else {
-			buf.L("stmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "delete", m.config["struct"]+"ID"))
+			buf.L("stmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "delete", m.config["struct"]+"ID"))
 		}
 
+		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "delete", m.config["struct"]+"ID")))
 		buf.L("result, err := stmt.Exec(int(%sID))", lex.Minuscule(m.config["struct"]))
 		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Delete \"%s\" entry failed: %%w", err)`, entityTable(m.entity, m.config["table"])))
 	} else if mapping.Type == ReferenceTable || mapping.Type == MapTable {
@@ -1116,18 +1113,18 @@ func (m *Method) delete(buf *file.Buffer, deleteOne bool) error {
 		buf.L("fillParent[i] = strings.Replace(parent, \"_\", \"s_\", -1) + \"s\"")
 		buf.L("}")
 		buf.N()
-		buf.L("stmt, err := %sprepare(tx, fmt.Sprintf(%s, fillParent...))", m.db, stmtLocal)
-		m.ifErrNotNil(buf, true, "err")
-		buf.L("result, err := stmt.Exec(referenceID)")
+		buf.L("queryStr := fmt.Sprintf(%s, fillParent...)", stmtLocal)
+		buf.L("result, err := tx.Exec(queryStr, referenceID)")
 		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Delete entry for \"%%s_%s\" failed: %%w", parent, err)`, m.entity))
 	} else {
 		activeFilters := mapping.ActiveFilters(m.kind)
 		if m.db == "" {
-			buf.L("stmt := Stmt(tx, %s)", stmtCodeVar(m.entity, "delete", FieldNames(activeFilters)...))
+			buf.L("stmt, err := Stmt(tx, %s)", stmtCodeVar(m.entity, "delete", FieldNames(activeFilters)...))
 		} else {
-			buf.L("stmt := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "delete", FieldNames(activeFilters)...))
+			buf.L("stmt, err := %s.Stmt(tx, %s)", m.db, stmtCodeVar(m.entity, "delete", FieldNames(activeFilters)...))
 		}
 
+		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Failed to get \"%s\" prepared statement: %%w", err)`, stmtCodeVar(m.entity, "delete", FieldNames(activeFilters)...)))
 		buf.L("result, err := stmt.Exec(%s)", mapping.FieldParams(activeFilters))
 		m.ifErrNotNil(buf, true, fmt.Sprintf(`fmt.Errorf("Delete \"%s\": %%w", err)`, entityTable(m.entity, m.config["table"])))
 	}
@@ -1196,7 +1193,7 @@ func (m *Method) signature(buf *file.Buffer, isInterface bool) error {
 		switch operation(m.kind) {
 		case "GetMany":
 			comment = fmt.Sprintf("returns all available %s for the parent entity.", lex.Plural(m.entity))
-			args += "parent string"
+			args += fmt.Sprintf("parent string, filters ...%s", entityFilter(m.entity))
 			rets = fmt.Sprintf("(map[int][]%s, error)", mapping.Name)
 		case "Create":
 			comment = fmt.Sprintf("adds a new %s to the database.", m.entity)
@@ -1218,7 +1215,7 @@ func (m *Method) signature(buf *file.Buffer, isInterface bool) error {
 		switch operation(m.kind) {
 		case "GetMany":
 			comment = fmt.Sprintf("returns all available %s.", lex.Plural(m.entity))
-			args += "parent string"
+			args += fmt.Sprintf("parent string, filters ...%s", entityFilter(m.entity))
 			rets = "(map[int]map[string]string, error)"
 		case "Create":
 			comment = fmt.Sprintf("adds a new %s to the database.", m.entity)
@@ -1245,11 +1242,11 @@ func (m *Method) signature(buf *file.Buffer, isInterface bool) error {
 		case "GetMany":
 			if m.ref == "" {
 				comment = fmt.Sprintf("returns all available %s.", lex.Plural(m.entity))
-				args += fmt.Sprintf("filter %s", entityFilter(m.entity))
+				args += fmt.Sprintf("filters ...%s", entityFilter(m.entity))
 				rets = fmt.Sprintf("(%s, error)", lex.Slice(lex.Camel(m.entity)))
 			} else {
 				comment = fmt.Sprintf("returns all available %s %s", mapping.Name, lex.Plural(m.ref))
-				args += fmt.Sprintf("%sID int", lex.Minuscule(mapping.Name))
+				args += fmt.Sprintf("%sID int, filters ...%s", lex.Minuscule(mapping.Name), entityFilter(m.ref))
 				refMapping, err := Parse(m.pkg, m.ref, "")
 				if err != nil {
 					return fmt.Errorf("Parse entity struct: %w", err)
@@ -1447,3 +1444,81 @@ func (m *Method) ifErrNotNil(buf *file.Buffer, newLine bool, rets ...string) {
 func (m *Method) end(buf *file.Buffer) {
 	buf.L("}")
 }
+
+// getManyTemplateFuncs returns two functions that can be used to perform generic queries without validation, and return
+// a slice of objects matching the entity. One function will accept pre-registered statements, and the other will accept
+// raw queries.
+func (m *Method) getManyTemplateFuncs(buf *file.Buffer, mapping *Mapping) error {
+	if mapping.Type == AssociationTable {
+		if m.config["struct"] != "" && strings.HasSuffix(mapping.Name, m.config["struct"]) {
+			return nil
+		}
+	}
+
+	tableName := mapping.TableName(m.entity, m.config["table"])
+	// Create a function to get the column names to use with SELECT statements for the entity.
+	buf.L("// %sColumns returns a string of column names to be used with a SELECT statement for the entity.", lex.Minuscule(mapping.Name))
+	buf.L("// Use this function when building statements to retrieve database entries matching the %s entity.", mapping.Name)
+	buf.L("func %sColumns() string {", lex.Minuscule(mapping.Name))
+	columns := make([]string, len(mapping.Fields))
+	for i, field := range mapping.Fields {
+		column, err := field.SelectColumn(mapping, tableName)
+		if err != nil {
+			return err
+		}
+
+		columns[i] = column
+	}
+
+	buf.L("return \"%s\"", strings.Join(columns, ", "))
+	buf.L("}")
+	buf.N()
+
+	// Create a function supporting prepared statements.
+	buf.L("// get%s can be used to run handwritten sql.Stmts to return a slice of objects.", lex.Plural(mapping.Name))
+	if mapping.Type != ReferenceTable && mapping.Type != MapTable {
+		buf.L("func get%s(ctx context.Context, stmt *sql.Stmt, args ...any) ([]%s, error) {", lex.Plural(mapping.Name), mapping.Name)
+	} else {
+		buf.L("func get%s(ctx context.Context, stmt *sql.Stmt, parent string, args ...any) ([]%s, error) {", lex.Plural(mapping.Name), mapping.Name)
+	}
+
+	buf.L("objects := make([]%s, 0)", mapping.Name)
+	buf.N()
+	buf.L("dest := %s", destFunc("objects", lex.Camel(m.entity), mapping.ColumnFields()))
+	buf.N()
+	buf.L("err := query.SelectObjects(ctx, stmt, dest, args...)")
+	if mapping.Type != ReferenceTable && mapping.Type != MapTable {
+		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%s\" table: %%w", err)`, tableName))
+	} else {
+		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%s\" table: %%w", parent, err)`, tableName))
+	}
+
+	buf.L("	return objects, nil")
+	buf.L("}")
+	buf.N()
+
+	// Create a function supporting raw queries.
+	buf.L("// get%s can be used to run handwritten query strings to return a slice of objects.", lex.Plural(mapping.Name))
+	if mapping.Type != ReferenceTable && mapping.Type != MapTable {
+		buf.L("func get%sRaw(ctx context.Context, tx *sql.Tx, sql string, args ...any) ([]%s, error) {", lex.Plural(mapping.Name), mapping.Name)
+	} else {
+		buf.L("func get%sRaw(ctx context.Context, tx *sql.Tx, sql string, parent string, args ...any) ([]%s, error) {", lex.Plural(mapping.Name), mapping.Name)
+	}
+
+	buf.L("objects := make([]%s, 0)", mapping.Name)
+	buf.N()
+	buf.L("dest := %s", destFunc("objects", lex.Camel(m.entity), mapping.ColumnFields()))
+	buf.N()
+	buf.L("err := query.Scan(ctx, tx, sql, dest, args...)")
+	if mapping.Type != ReferenceTable && mapping.Type != MapTable {
+		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%s\" table: %%w", err)`, tableName))
+	} else {
+		m.ifErrNotNil(buf, true, "nil", fmt.Sprintf(`fmt.Errorf("Failed to fetch from \"%s\" table: %%w", parent, err)`, tableName))
+	}
+
+	buf.L("	return objects, nil")
+	buf.L("}")
+	buf.N()
+
+	return nil
+}
