@@ -26,51 +26,40 @@ var ImageSourceProtocol = map[int]string{
 }
 
 // GetLocalImagesFingerprints returns the fingerprints of all local images.
-func (c *ClusterTx) GetLocalImagesFingerprints() ([]string, error) {
+func (c *ClusterTx) GetLocalImagesFingerprints(ctx context.Context) ([]string, error) {
 	q := `
 SELECT images.fingerprint
   FROM images_nodes
   JOIN images ON images.id = images_nodes.image_id
  WHERE node_id = ?
 `
-	return query.SelectStrings(c.tx, q, c.nodeID)
+	return query.SelectStrings(ctx, c.tx, q, c.nodeID)
 }
 
 // GetImageSource returns the image source with the given ID.
-func (c *ClusterTx) GetImageSource(imageID int) (int, api.ImageSource, error) {
+func (c *ClusterTx) GetImageSource(ctx context.Context, imageID int) (int, api.ImageSource, error) {
 	q := `SELECT id, server, protocol, certificate, alias FROM images_source WHERE image_id=?`
-	sources := []struct {
+	type imagesSource struct {
 		ID          int
 		Server      string
 		Protocol    int
 		Certificate string
 		Alias       string
-	}{}
-	dest := func(i int) []any {
-		sources = append(sources, struct {
-			ID          int
-			Server      string
-			Protocol    int
-			Certificate string
-			Alias       string
-		}{})
-		return []any{
-			&sources[i].ID,
-			&sources[i].Server,
-			&sources[i].Protocol,
-			&sources[i].Certificate,
-			&sources[i].Alias,
+	}
+
+	sources := []imagesSource{}
+	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
+		s := imagesSource{}
+
+		err := scan(&s.ID, &s.Server, &s.Protocol, &s.Certificate, &s.Alias)
+		if err != nil {
+			return err
 		}
-	}
 
-	stmt, err := c.tx.Prepare(q)
-	if err != nil {
-		return -1, api.ImageSource{}, err
-	}
+		sources = append(sources, s)
 
-	defer func() { _ = stmt.Close() }()
-
-	err = query.SelectObjects(stmt, dest, imageID)
+		return nil
+	}, imageID)
 	if err != nil {
 		return -1, api.ImageSource{}, err
 	}
@@ -98,7 +87,7 @@ func (c *ClusterTx) GetImageSource(imageID int) (int, api.ImageSource, error) {
 
 // Fill extra image fields such as properties and alias. This is called after
 // fetching a single row from the images table.
-func (c *ClusterTx) imageFill(id int, image *api.Image, create, expire, used, upload *time.Time, arch int, imageType int) error {
+func (c *ClusterTx) imageFill(ctx context.Context, id int, image *api.Image, create, expire, used, upload *time.Time, arch int, imageType int) error {
 	// Some of the dates can be nil in the DB, let's process them.
 	if create != nil {
 		image.CreatedAt = *create
@@ -125,39 +114,35 @@ func (c *ClusterTx) imageFill(id int, image *api.Image, create, expire, used, up
 	image.UploadedAt = *upload
 
 	// Get the properties
-	properties, err := query.SelectConfig(c.tx, "images_properties", "image_id=?", id)
+	properties, err := query.SelectConfig(ctx, c.tx, "images_properties", "image_id=?", id)
 	if err != nil {
 		return err
 	}
 
 	image.Properties = properties
 
+	q := "SELECT name, description FROM images_aliases WHERE image_id=?"
+
 	// Get the aliases
 	aliases := []api.ImageAlias{}
-	dest := func(i int) []any {
-		aliases = append(aliases, api.ImageAlias{})
-		return []any{
-			&aliases[i].Name,
-			&aliases[i].Description,
+	err = query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
+		alias := api.ImageAlias{}
+
+		err := scan(&alias.Name, &alias.Description)
+		if err != nil {
+			return err
 		}
-	}
 
-	q := "SELECT name, description FROM images_aliases WHERE image_id=?"
-	stmt, err := c.tx.Prepare(q)
-	if err != nil {
-		return err
-	}
-
-	defer func() { _ = stmt.Close() }()
-
-	err = query.SelectObjects(stmt, dest, id)
+		aliases = append(aliases, alias)
+		return nil
+	}, id)
 	if err != nil {
 		return err
 	}
 
 	image.Aliases = aliases
 
-	_, source, err := c.GetImageSource(id)
+	_, source, err := c.GetImageSource(ctx, id)
 	if err == nil {
 		image.UpdateSource = &source
 	}
@@ -165,7 +150,7 @@ func (c *ClusterTx) imageFill(id int, image *api.Image, create, expire, used, up
 	return nil
 }
 
-func (c *ClusterTx) imageFillProfiles(id int, image *api.Image, project string) error {
+func (c *ClusterTx) imageFillProfiles(ctx context.Context, id int, image *api.Image, project string) error {
 	// Check which project name to use
 	enabled, err := cluster.ProjectHasProfiles(context.Background(), c.tx, project)
 	if err != nil {
@@ -183,7 +168,7 @@ SELECT profiles.name FROM profiles
 	JOIN projects ON profiles.project_id = projects.id
 WHERE images_profiles.image_id = ? AND projects.name = ?
 `
-	profiles, err := query.SelectStrings(c.tx, q, id, project)
+	profiles, err := query.SelectStrings(ctx, c.tx, q, id, project)
 	if err != nil {
 		return err
 	}
@@ -194,7 +179,7 @@ WHERE images_profiles.image_id = ? AND projects.name = ?
 }
 
 // GetImagesFingerprints returns the names of all images (optionally only the public ones).
-func (c *Cluster) GetImagesFingerprints(project string, publicOnly bool) ([]string, error) {
+func (c *ClusterTx) GetImagesFingerprints(ctx context.Context, projectName string, publicOnly bool) ([]string, error) {
 	q := `
 SELECT fingerprint
   FROM images
@@ -207,19 +192,16 @@ SELECT fingerprint
 
 	var fingerprints []string
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		enabled, err := cluster.ProjectHasImages(context.Background(), tx.tx, project)
-		if err != nil {
-			return fmt.Errorf("Check if project has images: %w", err)
-		}
+	enabled, err := cluster.ProjectHasImages(ctx, c.tx, projectName)
+	if err != nil {
+		return nil, fmt.Errorf("Check if project has images: %w", err)
+	}
 
-		if !enabled {
-			project = "default"
-		}
+	if !enabled {
+		projectName = "default"
+	}
 
-		fingerprints, err = query.SelectStrings(tx.tx, q, project)
-		return err
-	})
+	fingerprints, err = query.SelectStrings(ctx, c.tx, q, projectName)
 	if err != nil {
 		return nil, err
 	}
@@ -298,7 +280,7 @@ func (c *Cluster) CreateImageSource(id int, server string, protocol string, cert
 // GetCachedImageSourceFingerprint tries to find a source entry of a locally
 // cached image that matches the given remote details (server, protocol and
 // alias). Return the fingerprint linked to the matching entry, if any.
-func (c *Cluster) GetCachedImageSourceFingerprint(server string, protocol string, alias string, typeName string, architecture int) (string, error) {
+func (c *ClusterTx) GetCachedImageSourceFingerprint(ctx context.Context, server string, protocol string, alias string, typeName string, architecture int) (string, error) {
 	imageType := instancetype.Any
 	if typeName != "" {
 		var err error
@@ -334,12 +316,7 @@ func (c *Cluster) GetCachedImageSourceFingerprint(server string, protocol string
 
 	q += "ORDER BY creation_date DESC"
 
-	var fingerprints []string
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		var err error
-		fingerprints, err = query.SelectStrings(tx.tx, q, args...)
-		return err
-	})
+	fingerprints, err := query.SelectStrings(ctx, c.tx, q, args...)
 	if err != nil {
 		return "", err
 	}
@@ -367,7 +344,7 @@ func (c *Cluster) ImageExists(project string, fingerprint string) (bool, error)
 			project = "default"
 		}
 
-		count, err := query.Count(tx.tx, table, where, project, fingerprint)
+		count, err := query.Count(ctx, tx.tx, table, where, project, fingerprint)
 		if err != nil {
 			return err
 		}
@@ -399,7 +376,7 @@ func (c *Cluster) ImageIsReferencedByOtherProjects(project string, fingerprint s
 			project = "default"
 		}
 
-		count, err := query.Count(tx.tx, table, where, project, fingerprint)
+		count, err := query.Count(ctx, tx.tx, table, where, project, fingerprint)
 		if err != nil {
 			return err
 		}
@@ -466,7 +443,7 @@ func (c *ClusterTx) GetImageByFingerprintPrefix(ctx context.Context, fingerprint
 		filter.Project = &project
 	}
 
-	images, err := c.getImagesByFingerprintPrefix(fingerprintPrefix, filter)
+	images, err := c.getImagesByFingerprintPrefix(ctx, fingerprintPrefix, filter)
 	if err != nil {
 		return -1, nil, fmt.Errorf("Failed to fetch images: %w", err)
 	}
@@ -488,14 +465,14 @@ func (c *ClusterTx) GetImageByFingerprintPrefix(ctx context.Context, fingerprint
 	image.AutoUpdate = object.AutoUpdate
 
 	err = c.imageFill(
-		object.ID, &image,
+		ctx, object.ID, &image,
 		&object.CreationDate.Time, &object.ExpiryDate.Time, &object.LastUseDate.Time,
 		&object.UploadDate, object.Architecture, object.Type)
 	if err != nil {
 		return -1, nil, fmt.Errorf("Fill image details: %w", err)
 	}
 
-	err = c.imageFillProfiles(object.ID, &image, profileProject)
+	err = c.imageFillProfiles(ctx, object.ID, &image, profileProject)
 	if err != nil {
 		return -1, nil, fmt.Errorf("Fill image profiles: %w", err)
 	}
@@ -511,7 +488,7 @@ func (c *Cluster) GetImageFromAnyProject(fingerprint string) (int, *api.Image, e
 	var object cluster.Image
 
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		images, err := tx.getImagesByFingerprintPrefix(fingerprint, cluster.ImageFilter{})
+		images, err := tx.getImagesByFingerprintPrefix(ctx, fingerprint, cluster.ImageFilter{})
 		if err != nil {
 			return fmt.Errorf("Failed to fetch images: %w", err)
 		}
@@ -530,7 +507,7 @@ func (c *Cluster) GetImageFromAnyProject(fingerprint string) (int, *api.Image, e
 		image.AutoUpdate = object.AutoUpdate
 
 		err = tx.imageFill(
-			object.ID, &image,
+			ctx, object.ID, &image,
 			&object.CreationDate.Time, &object.ExpiryDate.Time, &object.LastUseDate.Time,
 			&object.UploadDate, object.Architecture, object.Type)
 		if err != nil {
@@ -548,7 +525,7 @@ func (c *Cluster) GetImageFromAnyProject(fingerprint string) (int, *api.Image, e
 
 // getImagesByFingerprintPrefix returns the images with fingerprints matching the prefix.
 // Optional filters 'project' and 'public' will be included if not nil.
-func (c *ClusterTx) getImagesByFingerprintPrefix(fingerprintPrefix string, filter cluster.ImageFilter) ([]cluster.Image, error) {
+func (c *ClusterTx) getImagesByFingerprintPrefix(ctx context.Context, fingerprintPrefix string, filter cluster.ImageFilter) ([]cluster.Image, error) {
 	sql := `
 SELECT images.id, projects.name AS project, images.fingerprint, images.type, images.filename, images.size, images.public, images.architecture, images.creation_date, images.expiry_date, images.upload_date, images.cached, images.last_use_date, images.auto_update
 FROM images
@@ -573,7 +550,7 @@ WHERE images.fingerprint LIKE ?
 
 	images := make([]cluster.Image, 0)
 
-	err := c.QueryScan(sql, func(scan func(dest ...any) error) error {
+	err := query.Scan(ctx, c.Tx(), sql, func(scan func(dest ...any) error) error {
 		var img cluster.Image
 
 		err := scan(
@@ -623,23 +600,23 @@ WHERE images.fingerprint = ?
 	var addresses []string  // Addresses of online nodes with the image
 
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		offlineThreshold, err := tx.GetNodeOfflineThreshold()
+		offlineThreshold, err := tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
 			return err
 		}
 
-		localAddress, err = tx.GetLocalNodeAddress()
+		localAddress, err = tx.GetLocalNodeAddress(ctx)
 		if err != nil {
 			return err
 		}
 
-		allAddresses, err := query.SelectStrings(tx.tx, stmt, fingerprint)
+		allAddresses, err := query.SelectStrings(ctx, tx.tx, stmt, fingerprint)
 		if err != nil {
 			return err
 		}
 
 		for _, address := range allAddresses {
-			node, err := tx.GetNodeByAddress(address)
+			node, err := tx.GetNodeByAddress(ctx, address)
 			if err != nil {
 				return err
 			}
@@ -702,7 +679,7 @@ func (c *Cluster) DeleteImage(id int) error {
 }
 
 // GetImageAliases returns the names of the aliases of all images.
-func (c *Cluster) GetImageAliases(project string) ([]string, error) {
+func (c *ClusterTx) GetImageAliases(ctx context.Context, projectName string) ([]string, error) {
 	var names []string
 	q := `
 SELECT images_aliases.name
@@ -711,19 +688,16 @@ SELECT images_aliases.name
  WHERE projects.name=?
 `
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		enabled, err := cluster.ProjectHasImages(context.Background(), tx.tx, project)
-		if err != nil {
-			return fmt.Errorf("Check if project has images: %w", err)
-		}
+	enabled, err := cluster.ProjectHasImages(ctx, c.tx, projectName)
+	if err != nil {
+		return nil, fmt.Errorf("Check if project has images: %w", err)
+	}
 
-		if !enabled {
-			project = "default"
-		}
+	if !enabled {
+		projectName = "default"
+	}
 
-		names, err = query.SelectStrings(tx.tx, q, project)
-		return err
-	})
+	names, err = query.SelectStrings(ctx, c.tx, q, projectName)
 	if err != nil {
 		return nil, err
 	}
@@ -732,7 +706,7 @@ SELECT images_aliases.name
 }
 
 // GetImageAlias returns the alias with the given name in the given project.
-func (c *Cluster) GetImageAlias(project, name string, isTrustedClient bool) (int, api.ImageAliasesEntry, error) {
+func (c *ClusterTx) GetImageAlias(ctx context.Context, projectName string, imageName string, isTrustedClient bool) (int, api.ImageAliasesEntry, error) {
 	id := -1
 	entry := api.ImageAliasesEntry{}
 	q := `SELECT images_aliases.id, images.fingerprint, images.type, images_aliases.description
@@ -746,41 +720,34 @@ func (c *Cluster) GetImageAlias(project, name string, isTrustedClient bool) (int
 		q = q + ` AND images.public=1`
 	}
 
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		enabled, err := cluster.ProjectHasImages(context.Background(), tx.tx, project)
-		if err != nil {
-			return fmt.Errorf("Check if project has images: %w", err)
-		}
-
-		if !enabled {
-			project = "default"
-		}
-
-		var fingerprint, description string
-		var imageType int
-
-		arg1 := []any{project, name}
-		arg2 := []any{&id, &fingerprint, &imageType, &description}
-		err = tx.tx.QueryRow(q, arg1...).Scan(arg2...)
-		if err != nil {
-			if err == sql.ErrNoRows {
-				return api.StatusErrorf(http.StatusNotFound, "Image alias not found")
-			}
-
-			return err
-		}
-
-		entry.Name = name
-		entry.Target = fingerprint
-		entry.Description = description
-		entry.Type = instancetype.Type(imageType).String()
-
-		return nil
-	})
+	enabled, err := cluster.ProjectHasImages(ctx, c.tx, projectName)
 	if err != nil {
-		return -1, entry, err
+		return -1, api.ImageAliasesEntry{}, fmt.Errorf("Check if project has images: %w", err)
 	}
 
+	if !enabled {
+		projectName = "default"
+	}
+
+	var fingerprint, description string
+	var imageType int
+
+	arg1 := []any{projectName, imageName}
+	arg2 := []any{&id, &fingerprint, &imageType, &description}
+	err = c.tx.QueryRowContext(ctx, q, arg1...).Scan(arg2...)
+	if err != nil {
+		if err == sql.ErrNoRows {
+			return -1, api.ImageAliasesEntry{}, api.StatusErrorf(http.StatusNotFound, "Image alias not found")
+		}
+
+		return 0, entry, err
+	}
+
+	entry.Name = imageName
+	entry.Target = fingerprint
+	entry.Description = description
+	entry.Type = instancetype.Type(imageType).String()
+
 	return id, entry, nil
 }
 
@@ -795,25 +762,22 @@ func (c *Cluster) RenameImageAlias(id int, name string) error {
 }
 
 // DeleteImageAlias deletes the alias with the given name.
-func (c *Cluster) DeleteImageAlias(project, name string) error {
+func (c *ClusterTx) DeleteImageAlias(ctx context.Context, projectName string, name string) error {
 	q := `
 DELETE
   FROM images_aliases
  WHERE project_id = (SELECT id FROM projects WHERE name = ?) AND name = ?
 `
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		enabled, err := cluster.ProjectHasImages(context.Background(), tx.tx, project)
-		if err != nil {
-			return fmt.Errorf("Check if project has images: %w", err)
-		}
+	enabled, err := cluster.ProjectHasImages(ctx, c.tx, projectName)
+	if err != nil {
+		return fmt.Errorf("Check if project has images: %w", err)
+	}
 
-		if !enabled {
-			project = "default"
-		}
+	if !enabled {
+		projectName = "default"
+	}
 
-		_, err = tx.tx.Exec(q, project, name)
-		return err
-	})
+	_, err = c.tx.ExecContext(ctx, q, projectName, name)
 	if err != nil {
 		return err
 	}
@@ -832,24 +796,20 @@ func (c *Cluster) MoveImageAlias(source int, destination int) error {
 }
 
 // CreateImageAlias inserts an alias ento the database.
-func (c *Cluster) CreateImageAlias(project, name string, imageID int, desc string) error {
-	stmt := `
-INSERT INTO images_aliases (name, image_id, description, project_id)
-     VALUES (?, ?, ?, (SELECT id FROM projects WHERE name = ?))
+func (c *ClusterTx) CreateImageAlias(ctx context.Context, projectName, aliasName string, imageID int, desc string) error {
+	stmt := `INSERT INTO images_aliases (name, image_id, description, project_id)
+VALUES (?, ?, ?, (SELECT id FROM projects WHERE name = ?))
 `
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		enabled, err := cluster.ProjectHasImages(context.Background(), tx.tx, project)
-		if err != nil {
-			return fmt.Errorf("Check if project has images: %w", err)
-		}
+	enabled, err := cluster.ProjectHasImages(ctx, c.tx, projectName)
+	if err != nil {
+		return fmt.Errorf("Check if project has images: %w", err)
+	}
 
-		if !enabled {
-			project = "default"
-		}
+	if !enabled {
+		projectName = "default"
+	}
 
-		_, err = tx.tx.Exec(stmt, name, imageID, desc, project)
-		return err
-	})
+	_, err = c.tx.Exec(stmt, aliasName, imageID, desc, projectName)
 	if err != nil {
 		return err
 	}
@@ -858,12 +818,9 @@ INSERT INTO images_aliases (name, image_id, description, project_id)
 }
 
 // UpdateImageAlias updates the alias with the given ID.
-func (c *Cluster) UpdateImageAlias(id int, imageID int, desc string) error {
+func (c *ClusterTx) UpdateImageAlias(ctx context.Context, aliasID int, imageID int, desc string) error {
 	stmt := `UPDATE images_aliases SET image_id=?, description=? WHERE id=?`
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec(stmt, imageID, desc, id)
-		return err
-	})
+	_, err := c.tx.ExecContext(ctx, stmt, imageID, desc, aliasID)
 	return err
 }
 
@@ -893,20 +850,20 @@ func (c *Cluster) CopyDefaultImageProfiles(id int, newID int) error {
 
 // UpdateImageLastUseDate updates the last_use_date field of the image with the
 // given fingerprint.
-func (c *Cluster) UpdateImageLastUseDate(fingerprint string, date time.Time) error {
-	stmt := `UPDATE images SET last_use_date=? WHERE fingerprint=?`
+func (c *Cluster) UpdateImageLastUseDate(projectName string, fingerprint string, lastUsed time.Time) error {
+	stmt := `UPDATE images SET last_use_date=? WHERE fingerprint=? AND project_id = (SELECT id FROM projects WHERE name = ? LIMIT 1)`
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec(stmt, date, fingerprint)
+		_, err := tx.tx.Exec(stmt, lastUsed, fingerprint, projectName)
 		return err
 	})
 	return err
 }
 
-// InitImageLastUseDate inits the last_use_date field of the image with the given fingerprint.
-func (c *Cluster) InitImageLastUseDate(fingerprint string) error {
-	stmt := `UPDATE images SET cached=1, last_use_date=strftime("%s") WHERE fingerprint=?`
+// SetImageCachedAndLastUseDate sets the cached and last_use_date field of the image with the given fingerprint.
+func (c *Cluster) SetImageCachedAndLastUseDate(projectName string, fingerprint string, lastUsed time.Time) error {
+	stmt := `UPDATE images SET cached=1, last_use_date=? WHERE fingerprint=? AND project_id = (SELECT id FROM projects WHERE name = ? LIMIT 1)`
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		_, err := tx.tx.Exec(stmt, fingerprint)
+		_, err := tx.tx.Exec(stmt, lastUsed, fingerprint, projectName)
 		return err
 	})
 	return err
@@ -930,14 +887,8 @@ func (c *Cluster) UpdateImage(id int, fname string, sz int64, public bool, autoU
 			autoUpdateInt = 1
 		}
 
-		stmt, err := tx.tx.Prepare(`UPDATE images SET filename=?, size=?, public=?, auto_update=?, architecture=?, creation_date=?, expiry_date=? WHERE id=?`)
-		if err != nil {
-			return err
-		}
-
-		defer func() { _ = stmt.Close() }()
-
-		_, err = stmt.Exec(fname, sz, publicInt, autoUpdateInt, arch, createdAt, expiresAt, id)
+		sql := `UPDATE images SET filename=?, size=?, public=?, auto_update=?, architecture=?, creation_date=?, expiry_date=? WHERE id=?`
+		_, err = tx.tx.Exec(sql, fname, sz, publicInt, autoUpdateInt, arch, createdAt, expiresAt, id)
 		if err != nil {
 			return err
 		}
@@ -947,19 +898,13 @@ func (c *Cluster) UpdateImage(id int, fname string, sz int64, public bool, autoU
 			return err
 		}
 
-		stmt2, err := tx.tx.Prepare(`INSERT INTO images_properties (image_id, type, key, value) VALUES (?, ?, ?, ?)`)
-		if err != nil {
-			return err
-		}
-
-		defer func() { _ = stmt2.Close() }()
-
+		sql = `INSERT INTO images_properties (image_id, type, key, value) VALUES (?, ?, ?, ?)`
 		for key, value := range properties {
 			if value == "" {
 				continue
 			}
 
-			_, err = stmt2.Exec(id, 0, key, value)
+			_, err = tx.tx.Exec(sql, id, 0, key, value)
 			if err != nil {
 				return err
 			}
@@ -986,15 +931,9 @@ func (c *Cluster) UpdateImage(id int, fname string, sz int64, public bool, autoU
 				return err
 			}
 
-			stmt3, err := tx.tx.Prepare(`INSERT INTO images_profiles (image_id, profile_id) VALUES (?, ?)`)
-			if err != nil {
-				return err
-			}
-
-			defer func() { _ = stmt3.Close() }()
-
+			sql = `INSERT INTO images_profiles (image_id, profile_id) VALUES (?, ?)`
 			for _, profileID := range profileIds {
-				_, err = stmt3.Exec(id, profileID)
+				_, err = tx.tx.Exec(sql, id, profileID)
 				if err != nil {
 					return err
 				}
@@ -1007,7 +946,7 @@ func (c *Cluster) UpdateImage(id int, fname string, sz int64, public bool, autoU
 }
 
 // CreateImage creates a new image.
-func (c *Cluster) CreateImage(project, fp string, fname string, sz int64, public bool, autoUpdate bool, architecture string, createdAt time.Time, expiresAt time.Time, properties map[string]string, typeName string) error {
+func (c *Cluster) CreateImage(project string, fp string, fname string, sz int64, public bool, autoUpdate bool, architecture string, createdAt time.Time, expiresAt time.Time, properties map[string]string, typeName string) error {
 	arch, err := osarch.ArchitectureId(architecture)
 	if err != nil {
 		arch = 0
@@ -1028,7 +967,7 @@ func (c *Cluster) CreateImage(project, fp string, fname string, sz int64, public
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		imageProject := project
-		enabled, err := cluster.ProjectHasImages(context.Background(), tx.tx, imageProject)
+		enabled, err := cluster.ProjectHasImages(ctx, tx.tx, imageProject)
 		if err != nil {
 			return fmt.Errorf("Check if project has images: %w", err)
 		}
@@ -1047,14 +986,8 @@ func (c *Cluster) CreateImage(project, fp string, fname string, sz int64, public
 			autoUpdateInt = 1
 		}
 
-		stmt, err := tx.tx.Prepare(`INSERT INTO images (project_id, fingerprint, filename, size, public, auto_update, architecture, creation_date, expiry_date, upload_date, type) VALUES ((SELECT id FROM projects WHERE name = ?), ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`)
-		if err != nil {
-			return err
-		}
-
-		defer func() { _ = stmt.Close() }()
-
-		result, err := stmt.Exec(imageProject, fp, fname, sz, publicInt, autoUpdateInt, arch, createdAt, expiresAt, time.Now().UTC(), imageType)
+		sql := `INSERT INTO images (project_id, fingerprint, filename, size, public, auto_update, architecture, creation_date, expiry_date, upload_date, type) VALUES ((SELECT id FROM projects WHERE name = ?), ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`
+		result, err := tx.tx.Exec(sql, imageProject, fp, fname, sz, publicInt, autoUpdateInt, arch, createdAt, expiresAt, time.Now().UTC(), imageType)
 		if err != nil {
 			return err
 		}
@@ -1067,17 +1000,11 @@ func (c *Cluster) CreateImage(project, fp string, fname string, sz int64, public
 		id := int(id64)
 
 		if len(properties) > 0 {
-			pstmt, err := tx.tx.Prepare(`INSERT INTO images_properties (image_id, type, key, value) VALUES (?, 0, ?, ?)`)
-			if err != nil {
-				return err
-			}
-
-			defer func() { _ = pstmt.Close() }()
-
+			sql = `INSERT INTO images_properties (image_id, type, key, value) VALUES (?, 0, ?, ?)`
 			for k, v := range properties {
 				// we can assume, that there is just one
 				// value per key
-				_, err = pstmt.Exec(id, k, v)
+				_, err = tx.tx.Exec(sql, id, k, v)
 				if err != nil {
 					return err
 				}
@@ -1098,6 +1025,25 @@ func (c *Cluster) CreateImage(project, fp string, fname string, sz int64, public
 			return err
 		}
 
+		// All projects with features.images=false can use all images added to the "default" project.
+		// If these projects also have features.profiles=true, their default profiles should be associated
+		// with all created images.
+		if imageProject == "default" {
+			_, err = tx.tx.Exec(
+				`INSERT OR IGNORE INTO images_profiles(image_id, profile_id)
+					SELECT ?, profiles.id FROM profiles
+						JOIN projects_config AS t1 ON t1.project_id = profiles.project_id
+							AND t1.key = "features.images"
+							AND t1.value = "false"
+						JOIN projects_config AS t2 ON t2.project_id = profiles.project_id
+							AND t2.key = "features.profiles"
+							AND t2.value = "true"
+						WHERE profiles.name = "default"`, id)
+			if err != nil {
+				return err
+			}
+		}
+
 		_, err = tx.tx.Exec("INSERT INTO images_nodes(image_id, node_id) VALUES(?, ?)", id, c.nodeID)
 		if err != nil {
 			return err
@@ -1114,7 +1060,7 @@ func (c *Cluster) GetPoolsWithImage(imageFingerprint string) ([]int64, error) {
 	var ids []int
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		var err error
-		ids, err = query.SelectIntegers(tx.tx, q, c.nodeID, imageFingerprint, StoragePoolVolumeTypeImage)
+		ids, err = query.SelectIntegers(ctx, tx.tx, q, c.nodeID, imageFingerprint, StoragePoolVolumeTypeImage)
 		return err
 	})
 	if err != nil {
@@ -1144,7 +1090,7 @@ func (c *Cluster) GetPoolNamesFromIDs(poolIDs []int64) ([]string, error) {
 
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		var err error
-		poolNames, err = query.SelectStrings(tx.tx, q, args...)
+		poolNames, err = query.SelectStrings(ctx, tx.tx, q, args...)
 		return err
 	})
 	if err != nil {
@@ -1166,7 +1112,7 @@ func (c *Cluster) GetImages() (map[string][]string, error) {
     SELECT images.fingerprint, projects.name FROM images
       LEFT JOIN projects ON images.project_id = projects.id
 		`
-		rows, err := tx.tx.Query(stmt)
+		rows, err := tx.tx.QueryContext(ctx, stmt)
 		if err != nil {
 			return err
 		}
@@ -1203,7 +1149,7 @@ func (c *Cluster) GetImagesOnNode(id int64) (map[string][]string, error) {
 			LEFT JOIN projects ON images.project_id = projects.id
     WHERE nodes.id = ?
 		`
-		rows, err := tx.tx.Query(stmt, id)
+		rows, err := tx.tx.QueryContext(ctx, stmt, id)
 		if err != nil {
 			return err
 		}
@@ -1261,7 +1207,7 @@ SELECT DISTINCT nodes.address FROM nodes WHERE nodes.address NOT IN (
 func (c *Cluster) getNodesByImageFingerprint(stmt, fingerprint string, autoUpdate *bool) ([]string, error) {
 	var addresses []string // Addresses of online nodes with the image
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		offlineThreshold, err := tx.GetNodeOfflineThreshold()
+		offlineThreshold, err := tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
 			return err
 		}
@@ -1269,9 +1215,9 @@ func (c *Cluster) getNodesByImageFingerprint(stmt, fingerprint string, autoUpdat
 		var allAddresses []string
 
 		if autoUpdate == nil {
-			allAddresses, err = query.SelectStrings(tx.tx, stmt, fingerprint)
+			allAddresses, err = query.SelectStrings(ctx, tx.tx, stmt, fingerprint)
 		} else {
-			allAddresses, err = query.SelectStrings(tx.tx, stmt, fingerprint, autoUpdate)
+			allAddresses, err = query.SelectStrings(ctx, tx.tx, stmt, fingerprint, autoUpdate)
 		}
 
 		if err != nil {
@@ -1279,7 +1225,7 @@ func (c *Cluster) getNodesByImageFingerprint(stmt, fingerprint string, autoUpdat
 		}
 
 		for _, address := range allAddresses {
-			node, err := tx.GetNodeByAddress(address)
+			node, err := tx.GetNodeByAddress(ctx, address)
 			if err != nil {
 				return err
 			}
@@ -1297,7 +1243,7 @@ func (c *Cluster) getNodesByImageFingerprint(stmt, fingerprint string, autoUpdat
 }
 
 // GetProjectsUsingImage get the project names using an image by fingerprint.
-func (c *ClusterTx) GetProjectsUsingImage(fingerprint string) ([]string, error) {
+func (c *ClusterTx) GetProjectsUsingImage(ctx context.Context, fingerprint string) ([]string, error) {
 	var err error
 	var imgProjectNames []string
 
@@ -1307,7 +1253,7 @@ func (c *ClusterTx) GetProjectsUsingImage(fingerprint string) ([]string, error)
 		JOIN projects ON projects.id=images.project_id
 		WHERE fingerprint = ?
 	`
-	err = c.QueryScan(q, func(scan func(dest ...any) error) error {
+	err = query.Scan(ctx, c.Tx(), q, func(scan func(dest ...any) error) error {
 		var imgProjectName string
 		err = scan(&imgProjectName)
 		if err != nil {
