@@ -44,31 +44,21 @@ type InstanceArgs struct {
 	ExpiryDate   time.Time
 }
 
-// InstanceTypeFilter returns an InstanceFilter populated with a valid instance type,
-// or an empty filter if instance type is 'Any'.
-func InstanceTypeFilter(instanceType instancetype.Type) cluster.InstanceFilter {
-	if instanceType != instancetype.Any {
-		return cluster.InstanceFilter{Type: &instanceType}
-	}
-
-	return cluster.InstanceFilter{}
-}
-
 // GetInstanceNames returns the names of all containers the given project.
-func (c *ClusterTx) GetInstanceNames(project string) ([]string, error) {
+func (c *ClusterTx) GetInstanceNames(ctx context.Context, project string) ([]string, error) {
 	stmt := `
 SELECT instances.name FROM instances
   JOIN projects ON projects.id = instances.project_id
   WHERE projects.name = ? AND instances.type = ?
 `
-	return query.SelectStrings(c.tx, stmt, project, instancetype.Any)
+	return query.SelectStrings(ctx, c.tx, stmt, project, instancetype.Any)
 }
 
 // GetNodeAddressOfInstance returns the address of the node hosting the
 // instance with the given name in the given project.
 //
 // It returns the empty string if the container is hosted on this node.
-func (c *ClusterTx) GetNodeAddressOfInstance(project string, name string, filter cluster.InstanceFilter) (string, error) {
+func (c *ClusterTx) GetNodeAddressOfInstance(ctx context.Context, project string, name string, instType instancetype.Type) (string, error) {
 	var stmt string
 
 	args := make([]any, 0, 4) // Expect up to 4 filters.
@@ -79,9 +69,9 @@ func (c *ClusterTx) GetNodeAddressOfInstance(project string, name string, filter
 	args = append(args, project)
 
 	// Instance type filter.
-	if filter.Type != nil {
+	if instType != instancetype.Any {
 		filters.WriteString(" AND instances.type = ?")
-		args = append(args, *filter.Type)
+		args = append(args, instType)
 	}
 
 	if strings.Contains(name, shared.SnapshotDelimiter) {
@@ -119,7 +109,7 @@ SELECT nodes.id, nodes.address
 
 	var address string
 	var id int64
-	rows, err := c.tx.Query(stmt, args...)
+	rows, err := c.tx.QueryContext(ctx, stmt, args...)
 	if err != nil {
 		return "", err
 	}
@@ -159,8 +149,8 @@ SELECT nodes.id, nodes.address
 // string, to distinguish it from remote nodes.
 //
 // Instances whose node is down are added to the special address "0.0.0.0".
-func (c *ClusterTx) GetProjectAndInstanceNamesByNodeAddress(projects []string, filter cluster.InstanceFilter) (map[string][][2]string, error) {
-	offlineThreshold, err := c.GetNodeOfflineThreshold()
+func (c *ClusterTx) GetProjectAndInstanceNamesByNodeAddress(ctx context.Context, projects []string, instType instancetype.Type) (map[string][][2]string, error) {
+	offlineThreshold, err := c.GetNodeOfflineThreshold(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -175,9 +165,9 @@ func (c *ClusterTx) GetProjectAndInstanceNamesByNodeAddress(projects []string, f
 	}
 
 	// Instance type filter.
-	if filter.Type != nil {
+	if instType != instancetype.Any {
 		filters.WriteString(" AND instances.type = ?")
-		args = append(args, *filter.Type)
+		args = append(args, instType)
 	}
 
 	stmt := fmt.Sprintf(`
@@ -189,7 +179,7 @@ SELECT instances.name, nodes.id, nodes.address, nodes.heartbeat, projects.name
   ORDER BY instances.id
 `, filters.String())
 
-	rows, err := c.tx.Query(stmt, args...)
+	rows, err := c.tx.QueryContext(ctx, stmt, args...)
 	if err != nil {
 		return nil, err
 	}
@@ -230,29 +220,33 @@ SELECT instances.name, nodes.id, nodes.address, nodes.heartbeat, projects.name
 var ErrInstanceListStop = fmt.Errorf("search stopped")
 
 // InstanceList loads all instances across all projects and for each instance runs the instanceFunc passing in the
-// instance and it's project and profiles. Accepts optional filter argument to specify a subset of instances.
-func (c *Cluster) InstanceList(filter *cluster.InstanceFilter, instanceFunc func(inst InstanceArgs, project api.Project) error) error {
+// instance and it's project and profiles. Accepts optional filter arguments to specify a subset of instances.
+func (c *Cluster) InstanceList(ctx context.Context, instanceFunc func(inst InstanceArgs, project api.Project) error, filters ...cluster.InstanceFilter) error {
 	projectsByName := make(map[string]*api.Project)
 	var instances map[int]InstanceArgs
 
-	if filter == nil {
-		filter = &cluster.InstanceFilter{}
-	}
+	emptyFilter := cluster.InstanceFilter{}
+	validFilters := []cluster.InstanceFilter{}
+	for _, filter := range filters {
+		if filter.Type != nil && *filter.Type == instancetype.Any {
+			filter.Type = nil
+		}
 
-	if filter.Type != nil && *filter.Type == instancetype.Any {
-		filter.Type = nil
+		if filter != emptyFilter {
+			validFilters = append(validFilters, filter)
+		}
 	}
 
 	// Retrieve required info from the database in single transaction for performance.
-	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
+	err := c.Transaction(ctx, func(ctx context.Context, tx *ClusterTx) error {
 		// Get all projects.
-		projects, err := cluster.GetProjects(ctx, tx.tx, cluster.ProjectFilter{})
+		projects, err := cluster.GetProjects(ctx, tx.tx)
 		if err != nil {
 			return fmt.Errorf("Failed loading projects: %w", err)
 		}
 
 		// Get all instances using supplied filter.
-		dbInstances, err := cluster.GetInstances(ctx, tx.tx, *filter)
+		dbInstances, err := cluster.GetInstances(ctx, tx.tx, validFilters...)
 		if err != nil {
 			return fmt.Errorf("Failed loading instances: %w", err)
 		}
@@ -311,7 +305,7 @@ func (c *Cluster) InstanceList(filter *cluster.InstanceFilter, instanceFunc func
 
 // instanceConfigFill function loads config for all specified instances in a single query and then updates
 // the entries in the instances map.
-func (c *ClusterTx) instanceConfigFill(snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
+func (c *ClusterTx) instanceConfigFill(ctx context.Context, snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
 	instances := *instanceArgs
 
 	// Don't use query parameters for the IN statement to workaround an issue in Dqlite (apparently)
@@ -350,7 +344,7 @@ func (c *ClusterTx) instanceConfigFill(snapshotsMode bool, instanceArgs *map[int
 
 	q.WriteString(`)`)
 
-	return c.QueryScan(q.String(), func(scan func(dest ...any) error) error {
+	return query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
 		var instanceID int
 		var key, value string
 
@@ -383,7 +377,7 @@ func (c *ClusterTx) instanceConfigFill(snapshotsMode bool, instanceArgs *map[int
 
 // instanceDevicesFill loads the device config for all instances specified in a single query and then updates
 // the entries in the instances map.
-func (c *ClusterTx) instanceDevicesFill(snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
+func (c *ClusterTx) instanceDevicesFill(ctx context.Context, snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
 	instances := *instanceArgs
 
 	// Don't use query parameters for the IN statement to workaround an issue in Dqlite (apparently)
@@ -430,7 +424,7 @@ func (c *ClusterTx) instanceDevicesFill(snapshotsMode bool, instanceArgs *map[in
 
 	q.WriteString(`)`)
 
-	return c.QueryScan(q.String(), func(scan func(dest ...any) error) error {
+	return query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
 		var instanceID int
 		var deviceType cluster.DeviceType
 		var deviceName, key, value string
@@ -473,7 +467,7 @@ func (c *ClusterTx) instanceDevicesFill(snapshotsMode bool, instanceArgs *map[in
 
 // instanceProfiles loads the profile IDs to apply to an instance (in the application order) for all
 // instanceIDs in a single query and then updates the instanceApplyProfileIDs and profilesByID maps.
-func (c *ClusterTx) instanceProfilesFill(snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
+func (c *ClusterTx) instanceProfilesFill(ctx context.Context, snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
 	instances := *instanceArgs
 
 	// Get profiles referenced by instances.
@@ -518,7 +512,7 @@ func (c *ClusterTx) instanceProfilesFill(snapshotsMode bool, instanceArgs *map[i
 	profilesByID := make(map[int]*api.Profile)
 	instanceApplyProfileIDs := make(map[int64][]int, len(instances))
 
-	err := c.QueryScan(q.String(), func(scan func(dest ...any) error) error {
+	err := query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
 		var instanceID int64
 		var profileID int
 
@@ -542,7 +536,7 @@ func (c *ClusterTx) instanceProfilesFill(snapshotsMode bool, instanceArgs *map[i
 	}
 
 	// Get all profiles.
-	profiles, err := cluster.GetProfiles(context.TODO(), c.Tx(), cluster.ProfileFilter{})
+	profiles, err := cluster.GetProfiles(context.TODO(), c.Tx())
 	if err != nil {
 		return fmt.Errorf("Failed loading profiles: %w", err)
 	}
@@ -622,20 +616,20 @@ func (c *ClusterTx) InstancesToInstanceArgs(ctx context.Context, fillProfiles bo
 	}
 
 	// Populate instance config.
-	err := c.instanceConfigFill(snapshotCount > 0, &instanceArgs)
+	err := c.instanceConfigFill(ctx, snapshotCount > 0, &instanceArgs)
 	if err != nil {
 		return nil, fmt.Errorf("Failed loading instance config: %w", err)
 	}
 
 	// Populate instance devices.
-	err = c.instanceDevicesFill(snapshotCount > 0, &instanceArgs)
+	err = c.instanceDevicesFill(ctx, snapshotCount > 0, &instanceArgs)
 	if err != nil {
 		return nil, fmt.Errorf("Failed loading instance devices: %w", err)
 	}
 
 	// Populate instance profiles if requested.
 	if fillProfiles {
-		err = c.instanceProfilesFill(snapshotCount > 0, &instanceArgs)
+		err = c.instanceProfilesFill(ctx, snapshotCount > 0, &instanceArgs)
 		if err != nil {
 			return nil, fmt.Errorf("Failed loading instance profiles: %w", err)
 		}
@@ -646,7 +640,7 @@ func (c *ClusterTx) InstancesToInstanceArgs(ctx context.Context, fillProfiles bo
 
 // GetProjectInstanceToNodeMap returns a map associating the project (key element 0) and name (key element 1) of each
 // instance in the given projects to the name of the node hosting the instance.
-func (c *ClusterTx) GetProjectInstanceToNodeMap(projects []string, filter cluster.InstanceFilter) (map[[2]string]string, error) {
+func (c *ClusterTx) GetProjectInstanceToNodeMap(ctx context.Context, projects []string, instType instancetype.Type) (map[[2]string]string, error) {
 	args := make([]any, 0, 2) // Expect up to 2 filters.
 	var filters strings.Builder
 
@@ -657,9 +651,9 @@ func (c *ClusterTx) GetProjectInstanceToNodeMap(projects []string, filter cluste
 	}
 
 	// Instance type filter.
-	if filter.Type != nil {
+	if instType != instancetype.Any {
 		filters.WriteString(" AND instances.type = ?")
-		args = append(args, *filter.Type)
+		args = append(args, instType)
 	}
 
 	stmt := fmt.Sprintf(`
@@ -670,7 +664,7 @@ SELECT instances.name, nodes.name, projects.name
   WHERE %s
 `, filters.String())
 
-	rows, err := c.tx.Query(stmt, args...)
+	rows, err := c.tx.QueryContext(ctx, stmt, args...)
 	if err != nil {
 		return nil, err
 	}
@@ -704,17 +698,17 @@ SELECT instances.name, nodes.name, projects.name
 func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project, oldName string, newName string, newNode string, volumeType int) error {
 	// First check that the container to be moved is backed by a ceph
 	// volume.
-	poolName, err := c.GetInstancePool(project, oldName)
+	poolName, err := c.GetInstancePool(ctx, project, oldName)
 	if err != nil {
 		return fmt.Errorf("Failed to get instance's storage pool name: %w", err)
 	}
 
-	poolID, err := c.GetStoragePoolID(poolName)
+	poolID, err := c.GetStoragePoolID(ctx, poolName)
 	if err != nil {
 		return fmt.Errorf("Failed to get instance's storage pool ID: %w", err)
 	}
 
-	poolDriver, err := c.GetStoragePoolDriver(poolID)
+	poolDriver, err := c.GetStoragePoolDriver(ctx, poolID)
 	if err != nil {
 		return fmt.Errorf("Failed to get instance's storage pool driver: %w", err)
 	}
@@ -730,7 +724,7 @@ func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project, oldName str
 		return fmt.Errorf("Failed to get instance's ID: %w", err)
 	}
 
-	node, err := c.GetNodeByName(newNode)
+	node, err := c.GetNodeByName(ctx, newNode)
 	if err != nil {
 		return fmt.Errorf("Failed to get new node's info: %w", err)
 	}
@@ -776,7 +770,7 @@ func (c *ClusterTx) UpdateInstanceNode(ctx context.Context, project, oldName str
 // GetLocalInstancesInProject retuurns all instances of the given type on the local member in the given project.
 // If projectName is empty then all instances in all projects are returned.
 func (c *ClusterTx) GetLocalInstancesInProject(ctx context.Context, filter cluster.InstanceFilter) ([]cluster.Instance, error) {
-	node, err := c.GetLocalNodeName()
+	node, err := c.GetLocalNodeName(ctx)
 	if err != nil {
 		return nil, fmt.Errorf("Local node name: %w", err)
 	}
@@ -857,18 +851,11 @@ func (c *ClusterTx) DeleteInstanceConfigKey(id int64, key string) error {
 	return err
 }
 
-// UpdateInstancePowerState sets the the power state of the container with the given ID.
+// UpdateInstancePowerState sets the power state of the container with the given ID.
 func (c *ClusterTx) UpdateInstancePowerState(id int, state string) error {
 	// Set the new value
 	str := "INSERT OR REPLACE INTO instances_config (instance_id, key, value) VALUES (?, 'volatile.last_state.power', ?)"
-	stmt, err := c.tx.Prepare(str)
-	if err != nil {
-		return err
-	}
-
-	defer func() { _ = stmt.Close() }()
-
-	_, err = stmt.Exec(id, state)
+	_, err := c.tx.Exec(str, id, state)
 	if err != nil {
 		return err
 	}
@@ -880,14 +867,7 @@ func (c *ClusterTx) UpdateInstancePowerState(id int, state string) error {
 // with the given ID.
 func (c *ClusterTx) UpdateInstanceLastUsedDate(id int, date time.Time) error {
 	str := `UPDATE instances SET last_use_date=? WHERE id=?`
-	stmt, err := c.tx.Prepare(str)
-	if err != nil {
-		return err
-	}
-
-	defer func() { _ = stmt.Close() }()
-
-	_, err = stmt.Exec(date, id)
+	_, err := c.tx.Exec(str, date, id)
 	if err != nil {
 		return err
 	}
@@ -925,7 +905,7 @@ func (c *ClusterTx) GetInstanceSnapshotsWithName(ctx context.Context, project st
 }
 
 // GetLocalInstanceWithVsockID returns all available instances with the given config key and value.
-func (c *ClusterTx) GetLocalInstanceWithVsockID(vsockID int) (*cluster.Instance, error) {
+func (c *ClusterTx) GetLocalInstanceWithVsockID(ctx context.Context, vsockID int) (*cluster.Instance, error) {
 	q := `
 SELECT instances.id, projects.name AS project, instances.name, nodes.name AS node, instances.type, instances.architecture, instances.ephemeral, instances.creation_date, instances.stateful, instances.last_use_date, coalesce(instances.description, ''), instances.expiry_date
   FROM instances JOIN projects ON instances.project_id = projects.id JOIN nodes ON instances.node_id = nodes.id JOIN instances_config ON instances.id = instances_config.instance_id
@@ -935,7 +915,7 @@ SELECT instances.id, projects.name AS project, instances.name, nodes.name AS nod
 	inargs := []any{c.nodeID, instancetype.VM, vsockID}
 	inst := cluster.Instance{}
 
-	err := c.tx.QueryRow(q, inargs...).Scan(&inst.ID, &inst.Project, &inst.Name, &inst.Node, &inst.Type, &inst.Architecture, &inst.Ephemeral, &inst.CreationDate, &inst.Stateful, &inst.LastUseDate, &inst.Description, &inst.ExpiryDate)
+	err := c.tx.QueryRowContext(ctx, q, inargs...).Scan(&inst.ID, &inst.Project, &inst.Name, &inst.Node, &inst.Type, &inst.Architecture, &inst.Ephemeral, &inst.CreationDate, &inst.Stateful, &inst.LastUseDate, &inst.Description, &inst.ExpiryDate)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return nil, api.StatusErrorf(http.StatusNotFound, "Instance not found")
@@ -948,7 +928,7 @@ SELECT instances.id, projects.name AS project, instances.name, nodes.name AS nod
 }
 
 // GetInstancePool returns the storage pool of a given instance (or snapshot).
-func (c *ClusterTx) GetInstancePool(projectName string, instanceName string) (string, error) {
+func (c *ClusterTx) GetInstancePool(ctx context.Context, projectName string, instanceName string) (string, error) {
 	// Strip snapshot name if supplied in instanceName, and lookup the storage pool of the parent instance
 	// as that must always be the same as the snapshot's storage pool.
 	instanceName, _, _ = api.GetParentAndSnapshotName(instanceName)
@@ -976,7 +956,7 @@ SELECT storage_pools.name FROM storage_pools
 		inargs = append(inargs, driver)
 	}
 
-	err := c.tx.QueryRow(query, inargs...).Scan(outargs...)
+	err := c.tx.QueryRowContext(ctx, query, inargs...).Scan(outargs...)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return "", api.StatusErrorf(http.StatusNotFound, "Instance storage pool not found")
@@ -1014,7 +994,7 @@ SELECT projects.name, instances.name
 WHERE instances.id=?
 `
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return tx.tx.QueryRow(q, id).Scan(&project, &name)
+		return tx.tx.QueryRowContext(ctx, q, id).Scan(&project, &name)
 	})
 
 	if err == sql.ErrNoRows {
@@ -1041,7 +1021,7 @@ func (c *Cluster) GetInstanceConfig(id int, key string) (string, error) {
 	q := "SELECT value FROM instances_config WHERE instance_id=? AND key=?"
 	value := ""
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return tx.tx.QueryRow(q, id, key).Scan(&value)
+		return tx.tx.QueryRowContext(ctx, q, id, key).Scan(&value)
 	})
 	if err == sql.ErrNoRows {
 		return "", api.StatusErrorf(http.StatusNotFound, "Instance config not found")
@@ -1081,6 +1061,7 @@ func (c *Cluster) UpdateInstanceSnapshotCreationDate(instanceID int, date time.T
 
 // GetInstanceSnapshotsNames returns the names of all snapshots of the instance
 // in the given project with the given name.
+// Returns snapshots slice ordered by when they were created, oldest first.
 func (c *Cluster) GetInstanceSnapshotsNames(project, name string) ([]string, error) {
 	result := []string{}
 
@@ -1090,7 +1071,7 @@ SELECT instances_snapshots.name
   JOIN instances ON instances.id = instances_snapshots.instance_id
   JOIN projects ON projects.id = instances.project_id
 WHERE projects.name=? AND instances.name=?
-ORDER BY date(instances_snapshots.creation_date)
+ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 `
 	inargs := []any{project, name}
 	outfmt := []any{name}
@@ -1114,7 +1095,9 @@ SELECT instances_snapshots.name
   FROM instances_snapshots
   JOIN instances ON instances.id = instances_snapshots.instance_id
   JOIN projects ON projects.id = instances.project_id
-WHERE projects.name=? AND instances.name=?`
+WHERE projects.name=? AND instances.name=?
+ORDER BY instances_snapshots.creation_date, instances_snapshots.id
+`
 	var numstr string
 	inargs := []any{project, name}
 	outfmt := []any{numstr}
@@ -1150,7 +1133,7 @@ func (c *Cluster) GetInstancePool(project, instanceName string) (string, error)
 	var poolName string
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		var err error
-		poolName, err = tx.GetInstancePool(project, instanceName)
+		poolName, err = tx.GetInstancePool(ctx, project, instanceName)
 		return err
 	})
 	return poolName, err
@@ -1158,19 +1141,13 @@ func (c *Cluster) GetInstancePool(project, instanceName string) (string, error)
 
 // CreateInstanceConfig inserts a new config for the instance with the given ID.
 func CreateInstanceConfig(tx *sql.Tx, id int, config map[string]string) error {
-	stmt, err := tx.Prepare("INSERT INTO instances_config (instance_id, key, value) values (?, ?, ?)")
-	if err != nil {
-		return err
-	}
-
-	defer func() { _ = stmt.Close() }()
-
+	sql := "INSERT INTO instances_config (instance_id, key, value) values (?, ?, ?)"
 	for k, v := range config {
 		if v == "" {
 			continue
 		}
 
-		_, err := stmt.Exec(id, k, v)
+		_, err := tx.Exec(sql, id, k, v)
 		if err != nil {
 			return fmt.Errorf("Error adding configuration item %q = %q to instance %d: %w", k, v, id, err)
 		}
@@ -1184,22 +1161,16 @@ func CreateInstanceConfig(tx *sql.Tx, id int, config map[string]string) error {
 func UpdateInstance(tx *sql.Tx, id int, description string, architecture int, ephemeral bool,
 	expiryDate time.Time) error {
 	str := "UPDATE instances SET description=?, architecture=?, ephemeral=?, expiry_date=? WHERE id=?"
-	stmt, err := tx.Prepare(str)
-	if err != nil {
-		return err
-	}
-
-	defer func() { _ = stmt.Close() }()
-
 	ephemeralInt := 0
 	if ephemeral {
 		ephemeralInt = 1
 	}
 
+	var err error
 	if expiryDate.IsZero() {
-		_, err = stmt.Exec(description, architecture, ephemeralInt, "", id)
+		_, err = tx.Exec(str, description, architecture, ephemeralInt, "", id)
 	} else {
-		_, err = stmt.Exec(description, architecture, ephemeralInt, expiryDate, id)
+		_, err = tx.Exec(str, description, architecture, ephemeralInt, expiryDate, id)
 	}
 
 	if err != nil {
