@@ -3,10 +3,12 @@
 package db
 
 import (
+	"context"
 	"database/sql"
 	"fmt"
 	"strings"
 
+	"github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/db/query"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
@@ -19,7 +21,7 @@ import (
 // upgraded from a version without clustering to a version that supports
 // clustering, since in those version all data lives in the cluster database
 // (regardless of whether clustering is actually on or off).
-func LoadPreClusteringData(tx *sql.Tx) (*Dump, error) {
+func LoadPreClusteringData(ctx context.Context, tx *sql.Tx) (*Dump, error) {
 	// Sanitize broken foreign key references that might be around from the
 	// time where we didn't enforce foreign key constraints.
 	_, err := tx.Exec(`
@@ -54,7 +56,7 @@ DELETE FROM storage_volumes_config WHERE storage_volume_id NOT IN (SELECT id FRO
 		data := [][]any{}
 		stmt := fmt.Sprintf("SELECT * FROM %s", table)
 
-		rows, err := tx.Query(stmt)
+		rows, err := tx.QueryContext(ctx, stmt)
 		if err != nil {
 			return nil, fmt.Errorf("failed to fetch rows from %s: %w", table, err)
 		}
@@ -117,23 +119,18 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 	}
 
 	// Insert an entry for node 1.
-	stmt := `
-INSERT INTO nodes(id, name, address, schema, api_extensions) VALUES(1, 'none', '0.0.0.0', 14, 1)
-`
-	_, err = tx.Exec(stmt)
-	if err != nil {
-		return err
-	}
+	var stmt strings.Builder
+	stmt.WriteString(`INSERT INTO nodes(id, name, address, schema, api_extensions) VALUES(1, 'none', '0.0.0.0', 14, 1);`)
 
 	// Default project
-	stmt = `
-INSERT INTO projects (name, description) VALUES ('default', 'Default LXD project');
-INSERT INTO projects_config (project_id, key, value) VALUES (1, 'features.images', 'true');
-INSERT INTO projects_config (project_id, key, value) VALUES (1, 'features.networks', 'true');
-INSERT INTO projects_config (project_id, key, value) VALUES (1, 'features.profiles', 'true');
-INSERT INTO projects_config (project_id, key, value) VALUES (1, 'features.storage.volumes', 'true');
-`
-	_, err = tx.Exec(stmt)
+	stmt.WriteString(`INSERT INTO projects (name, description) VALUES ('default', 'Default LXD project');`)
+
+	// Enable all features for default project.
+	for featureName := range cluster.ProjectFeatures {
+		stmt.WriteString(fmt.Sprintf(`INSERT INTO projects_config (project_id, key, value) VALUES (1, '%s', 'true');`, featureName))
+	}
+
+	_, err = tx.Exec(stmt.String())
 	if err != nil {
 		return err
 	}
