@@ -20,8 +20,8 @@ import (
 // GetNetworksLocalConfig returns a map associating each network name to its
 // node-specific config values on the local member (i.e. the ones where node_id
 // equals the ID of the local member).
-func (c *ClusterTx) GetNetworksLocalConfig() (map[string]map[string]string, error) {
-	names, err := query.SelectStrings(c.tx, "SELECT name FROM networks")
+func (c *ClusterTx) GetNetworksLocalConfig(ctx context.Context) (map[string]map[string]string, error) {
+	names, err := query.SelectStrings(ctx, c.tx, "SELECT name FROM networks")
 	if err != nil {
 		return nil, err
 	}
@@ -29,8 +29,7 @@ func (c *ClusterTx) GetNetworksLocalConfig() (map[string]map[string]string, erro
 	networks := make(map[string]map[string]string, len(names))
 	for _, name := range names {
 		table := "networks_config JOIN networks ON networks.id=networks_config.network_id"
-		config, err := query.SelectConfig(
-			c.tx, table, "networks.name=? AND networks_config.node_id=?",
+		config, err := query.SelectConfig(ctx, c.tx, table, "networks.name=? AND networks_config.node_id=?",
 			name, c.nodeID)
 		if err != nil {
 			return nil, err
@@ -45,30 +44,27 @@ func (c *ClusterTx) GetNetworksLocalConfig() (map[string]map[string]string, erro
 // GetNonPendingNetworkIDs returns a map associating each network name to its ID.
 //
 // Pending networks are skipped.
-func (c *ClusterTx) GetNonPendingNetworkIDs() (map[string]map[string]int64, error) {
-	networks := []struct {
+func (c *ClusterTx) GetNonPendingNetworkIDs(ctx context.Context) (map[string]map[string]int64, error) {
+	type network struct {
 		id          int64
 		name        string
 		projectName string
-	}{}
-
-	dest := func(i int) []any {
-		networks = append(networks, struct {
-			id          int64
-			name        string
-			projectName string
-		}{})
-		return []any{&networks[i].id, &networks[i].name, &networks[i].projectName}
 	}
 
-	stmt, err := c.tx.Prepare("SELECT networks.id, networks.name, projects.name FROM networks JOIN projects on projects.id = networks.project_id WHERE NOT networks.state=?")
-	if err != nil {
-		return nil, err
-	}
+	networks := []network{}
+	sql := "SELECT networks.id, networks.name, projects.name FROM networks JOIN projects on projects.id = networks.project_id WHERE NOT networks.state=?"
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		n := network{}
 
-	defer func() { _ = stmt.Close() }()
+		err := scan(&n.id, &n.name, &n.projectName)
+		if err != nil {
+			return err
+		}
 
-	err = query.SelectObjects(stmt, dest, networkPending)
+		networks = append(networks, n)
+
+		return nil
+	}, networkPending)
 	if err != nil {
 		return nil, err
 	}
@@ -87,14 +83,14 @@ func (c *ClusterTx) GetNonPendingNetworkIDs() (map[string]map[string]int64, erro
 
 // GetCreatedNetworks returns a map of api.Network associated to project and network ID.
 // Only networks that have are in state networkCreated are returned.
-func (c *ClusterTx) GetCreatedNetworks() (map[string]map[int64]api.Network, error) {
-	return c.getCreatedNetworks("")
+func (c *ClusterTx) GetCreatedNetworks(ctx context.Context) (map[string]map[int64]api.Network, error) {
+	return c.getCreatedNetworks(ctx, "")
 }
 
 // GetCreatedNetworksByProject returns a map of api.Network in a project associated to network ID.
 // Only networks that have are in state networkCreated are returned.
-func (c *ClusterTx) GetCreatedNetworksByProject(projectName string) (map[int64]api.Network, error) {
-	nets, err := c.getCreatedNetworks(projectName)
+func (c *ClusterTx) GetCreatedNetworksByProject(ctx context.Context, projectName string) (map[int64]api.Network, error) {
+	nets, err := c.getCreatedNetworks(ctx, projectName)
 	if err != nil {
 		return nil, err
 	}
@@ -104,7 +100,7 @@ func (c *ClusterTx) GetCreatedNetworksByProject(projectName string) (map[int64]a
 
 // getCreatedNetworks returns a map of api.Network associated to project and network ID.
 // Supports an optional projectName filter. If projectName is empty, all networks in created state are returned.
-func (c *ClusterTx) getCreatedNetworks(projectName string) (map[string]map[int64]api.Network, error) {
+func (c *ClusterTx) getCreatedNetworks(ctx context.Context, projectName string) (map[string]map[int64]api.Network, error) {
 	var sb strings.Builder
 	sb.WriteString(`SELECT projects.name, networks.id, networks.name, coalesce(networks.description, ''), networks.type, networks.state
 	FROM networks
@@ -119,7 +115,7 @@ func (c *ClusterTx) getCreatedNetworks(projectName string) (map[string]map[int64
 		args = append(args, projectName)
 	}
 
-	rows, err := c.tx.Query(sb.String(), args...)
+	rows, err := c.tx.QueryContext(ctx, sb.String(), args...)
 	if err != nil {
 		return nil, err
 	}
@@ -161,14 +157,14 @@ func (c *ClusterTx) getCreatedNetworks(projectName string) (map[string]map[int64
 	// Populate config.
 	for projectName, networks := range projectNetworks {
 		for networkID, network := range networks {
-			networkConfig, err := query.SelectConfig(c.tx, "networks_config", "network_id=? AND (node_id=? OR node_id IS NULL)", networkID, c.nodeID)
+			networkConfig, err := query.SelectConfig(ctx, c.tx, "networks_config", "network_id=? AND (node_id=? OR node_id IS NULL)", networkID, c.nodeID)
 			if err != nil {
 				return nil, err
 			}
 
 			network.Config = networkConfig
 
-			nodes, err := c.NetworkNodes(networkID)
+			nodes, err := c.NetworkNodes(ctx, networkID)
 			if err != nil {
 				return nil, err
 			}
@@ -185,9 +181,9 @@ func (c *ClusterTx) getCreatedNetworks(projectName string) (map[string]map[int64
 }
 
 // GetNetworkID returns the ID of the network with the given name.
-func (c *ClusterTx) GetNetworkID(projectName string, name string) (int64, error) {
+func (c *ClusterTx) GetNetworkID(ctx context.Context, projectName string, name string) (int64, error) {
 	stmt := "SELECT id FROM networks WHERE project_id = (SELECT id FROM projects WHERE name = ?) AND name=?"
-	ids, err := query.SelectIntegers(c.tx, stmt, projectName, name)
+	ids, err := query.SelectIntegers(ctx, c.tx, stmt, projectName, name)
 	if err != nil {
 		return -1, err
 	}
@@ -246,9 +242,9 @@ func (c *ClusterTx) NetworkNodeJoin(networkID, nodeID int64) error {
 // nodes grouped by node name, for the given networkID.
 //
 // If the network is not defined on all nodes, an error is returned.
-func (c *ClusterTx) NetworkNodeConfigs(networkID int64) (map[string]map[string]string, error) {
+func (c *ClusterTx) NetworkNodeConfigs(ctx context.Context, networkID int64) (map[string]map[string]string, error) {
 	// Fetch all nodes.
-	nodes, err := c.GetNodes()
+	nodes, err := c.GetNodes(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -260,7 +256,7 @@ SELECT nodes.name FROM nodes
   LEFT JOIN networks ON networks_nodes.network_id = networks.id
 WHERE networks.id = ? AND networks.state = ?
 `
-	defined, err := query.SelectStrings(c.tx, stmt, networkID, networkPending)
+	defined, err := query.SelectStrings(ctx, c.tx, stmt, networkID, networkPending)
 	if err != nil {
 		return nil, err
 	}
@@ -279,7 +275,7 @@ WHERE networks.id = ? AND networks.state = ?
 
 	configs := map[string]map[string]string{}
 	for _, node := range nodes {
-		config, err := query.SelectConfig(c.tx, "networks_config", "node_id=?", node.ID)
+		config, err := query.SelectConfig(ctx, c.tx, "networks_config", "node_id=?", node.ID)
 		if err != nil {
 			return nil, err
 		}
@@ -291,7 +287,7 @@ WHERE networks.id = ? AND networks.state = ?
 }
 
 // CreatePendingNetwork creates a new pending network on the node with the given name.
-func (c *ClusterTx) CreatePendingNetwork(node string, projectName string, name string, netType NetworkType, conf map[string]string) error {
+func (c *ClusterTx) CreatePendingNetwork(ctx context.Context, node string, projectName string, name string, netType NetworkType, conf map[string]string) error {
 	// First check if a network with the given name exists, and, if so, that it's in the pending state.
 	network := struct {
 		id      int64
@@ -299,32 +295,22 @@ func (c *ClusterTx) CreatePendingNetwork(node string, projectName string, name s
 		netType NetworkType
 	}{}
 
-	var errConsistency error
-	dest := func(i int) []any {
+	sql := "SELECT id, state, type FROM networks WHERE project_id = (SELECT id FROM projects WHERE name = ?) AND name=?"
+	count := 0
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
 		// Ensure that there is at most one network with the given name.
-		if i != 0 {
-			errConsistency = fmt.Errorf("More than one network exists with the given name")
+		if count != 0 {
+			return fmt.Errorf("More than one network exists with the given name")
 		}
 
-		return []any{&network.id, &network.state, &network.netType}
-	}
+		count++
 
-	stmt, err := c.tx.Prepare("SELECT id, state, type FROM networks WHERE project_id = (SELECT id FROM projects WHERE name = ?) AND name=?")
+		return scan(&network.id, &network.state, &network.netType)
+	}, projectName, name)
 	if err != nil {
 		return err
 	}
 
-	defer func() { _ = stmt.Close() }()
-
-	err = query.SelectObjects(stmt, dest, projectName, name)
-	if err != nil {
-		return err
-	}
-
-	if errConsistency != nil {
-		return errConsistency
-	}
-
 	var networkID = network.id
 	if networkID == 0 {
 		projectID, err := cluster.GetProjectID(context.Background(), c.tx, projectName)
@@ -352,13 +338,13 @@ func (c *ClusterTx) CreatePendingNetwork(node string, projectName string, name s
 	}
 
 	// Get the ID of the node with the given name.
-	nodeInfo, err := c.GetNodeByName(node)
+	nodeInfo, err := c.GetNodeByName(ctx, node)
 	if err != nil {
 		return err
 	}
 
 	// Check that no network entry for this node and network exists yet.
-	count, err := query.Count(c.tx, "networks_nodes", "network_id=? AND node_id=?", networkID, nodeInfo.ID)
+	count, err = query.Count(ctx, c.tx, "networks_nodes", "network_id=? AND node_id=?", networkID, nodeInfo.ID)
 	if err != nil {
 		return err
 	}
@@ -458,25 +444,26 @@ func (c *ClusterTx) UpdateNetwork(id int64, description string, config map[strin
 }
 
 // NetworkNodes returns the nodes keyed by node ID that the given network is defined on.
-func (c *ClusterTx) NetworkNodes(networkID int64) (map[int64]NetworkNode, error) {
+func (c *ClusterTx) NetworkNodes(ctx context.Context, networkID int64) (map[int64]NetworkNode, error) {
 	nodes := []NetworkNode{}
-	dest := func(i int) []any {
-		nodes = append(nodes, NetworkNode{})
-		return []any{&nodes[i].ID, &nodes[i].Name, &nodes[i].State}
-	}
 
-	stmt, err := c.tx.Prepare(`
+	sql := `
 		SELECT nodes.id, nodes.name, networks_nodes.state FROM nodes
 		JOIN networks_nodes ON networks_nodes.node_id = nodes.id
 		WHERE networks_nodes.network_id = ?
-	`)
-	if err != nil {
-		return nil, err
-	}
+	`
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		node := NetworkNode{}
 
-	defer func() { _ = stmt.Close() }()
+		err := scan(&node.ID, &node.Name, &node.State)
+		if err != nil {
+			return err
+		}
 
-	err = query.SelectObjects(stmt, dest, networkID)
+		nodes = append(nodes, node)
+
+		return nil
+	}, networkID)
 	if err != nil {
 		return nil, err
 	}
@@ -490,10 +477,10 @@ func (c *ClusterTx) NetworkNodes(networkID int64) (map[int64]NetworkNode, error)
 }
 
 // GetNetworkURIs returns the URIs for the networks with the given project.
-func (c *ClusterTx) GetNetworkURIs(projectID int, project string) ([]string, error) {
+func (c *ClusterTx) GetNetworkURIs(ctx context.Context, projectID int, project string) ([]string, error) {
 	sql := `SELECT networks.name from networks WHERE networks.project_id = ?`
 
-	names, err := query.SelectStrings(c.tx, sql, projectID)
+	names, err := query.SelectStrings(ctx, c.tx, sql, projectID)
 	if err != nil {
 		return nil, fmt.Errorf("Unable to get URIs for network: %w", err)
 	}
@@ -588,12 +575,12 @@ func (c *Cluster) getNetworkByProjectAndName(projectName string, networkName str
 	var nodes map[int64]NetworkNode
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		networkID, networkState, networkType, network, err = c.getPartialNetworkByProjectAndName(tx, projectName, networkName, stateFilter)
+		networkID, networkState, networkType, network, err = c.getPartialNetworkByProjectAndName(ctx, tx, projectName, networkName, stateFilter)
 		if err != nil {
 			return err
 		}
 
-		nodes, err = c.networkPopulatePeerInfo(tx, networkID, network, networkState, networkType)
+		nodes, err = c.networkPopulatePeerInfo(ctx, tx, networkID, network, networkState, networkType)
 		if err != nil {
 			return err
 		}
@@ -610,7 +597,7 @@ func (c *Cluster) getNetworkByProjectAndName(projectName string, networkName str
 // getPartialNetworkByProjectAndName gets the network with the given project, name and state.
 // If stateFilter is -1, then a network can be in any state.
 // Returns network ID, network state, network type, and partially populated network info.
-func (c *Cluster) getPartialNetworkByProjectAndName(tx *ClusterTx, projectName string, networkName string, stateFilter NetworkState) (int64, NetworkState, NetworkType, *api.Network, error) {
+func (c *Cluster) getPartialNetworkByProjectAndName(ctx context.Context, tx *ClusterTx, projectName string, networkName string, stateFilter NetworkState) (int64, NetworkState, NetworkType, *api.Network, error) {
 	var err error
 	var networkID int64 = int64(-1)
 	var network api.Network
@@ -636,7 +623,7 @@ func (c *Cluster) getPartialNetworkByProjectAndName(tx *ClusterTx, projectName s
 
 	q.WriteString(" LIMIT 1")
 
-	err = tx.tx.QueryRow(q.String(), args...).Scan(&networkID, &network.Name, &network.Description, &networkState, &networkType)
+	err = tx.tx.QueryRowContext(ctx, q.String(), args...).Scan(&networkID, &network.Name, &network.Description, &networkState, &networkType)
 	if err != nil {
 		if errors.Is(err, sql.ErrNoRows) {
 			return -1, -1, -1, nil, api.StatusErrorf(http.StatusNotFound, "Network not found")
@@ -650,20 +637,20 @@ func (c *Cluster) getPartialNetworkByProjectAndName(tx *ClusterTx, projectName s
 
 // networkPopulatePeerInfo takes a pointer to partially populated network info struct and enriches it.
 // Returns the network cluster member info.
-func (c *Cluster) networkPopulatePeerInfo(tx *ClusterTx, networkID int64, network *api.Network, networkState NetworkState, networkType NetworkType) (map[int64]NetworkNode, error) {
+func (c *Cluster) networkPopulatePeerInfo(ctx context.Context, tx *ClusterTx, networkID int64, network *api.Network, networkState NetworkState, networkType NetworkType) (map[int64]NetworkNode, error) {
 	var err error
 
 	// Populate Status and Type fields by converting from DB values.
 	network.Status = NetworkStateToAPIStatus(networkState)
 	networkFillType(network, networkType)
 
-	err = c.getNetworkConfig(tx, networkID, network)
+	err = c.getNetworkConfig(ctx, tx, networkID, network)
 	if err != nil {
 		return nil, err
 	}
 
 	// Populate Location field.
-	nodes, err := tx.NetworkNodes(networkID)
+	nodes, err := tx.NetworkNodes(ctx, networkID)
 	if err != nil {
 		return nil, err
 	}
@@ -707,26 +694,6 @@ func networkFillType(network *api.Network, netType NetworkType) {
 	}
 }
 
-// NetworkNodes returns the nodes keyed by node ID that the given network is defined on.
-func (c *Cluster) NetworkNodes(networkID int64) (map[int64]NetworkNode, error) {
-	var nodes map[int64]NetworkNode
-	var err error
-
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		nodes, err = tx.NetworkNodes(networkID)
-		if err != nil {
-			return err
-		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
-	}
-
-	return nodes, nil
-}
-
 // GetNetworkWithInterface returns the network associated with the interface with the given name.
 func (c *Cluster) GetNetworkWithInterface(devName string) (int64, *api.Network, error) {
 	id := int64(-1)
@@ -763,7 +730,7 @@ func (c *Cluster) GetNetworkWithInterface(devName string) (int64, *api.Network,
 	}
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		return c.getNetworkConfig(tx, id, &network)
+		return c.getNetworkConfig(ctx, tx, id, &network)
 	})
 	if err != nil {
 		return -1, nil, err
@@ -773,7 +740,7 @@ func (c *Cluster) GetNetworkWithInterface(devName string) (int64, *api.Network,
 }
 
 // getNetworkConfig populates the config map of the Network with the given ID.
-func (c *Cluster) getNetworkConfig(tx *ClusterTx, networkID int64, network *api.Network) error {
+func (c *Cluster) getNetworkConfig(ctx context.Context, tx *ClusterTx, networkID int64, network *api.Network) error {
 	q := `
         SELECT key, value
         FROM networks_config
@@ -783,7 +750,7 @@ func (c *Cluster) getNetworkConfig(tx *ClusterTx, networkID int64, network *api.
 
 	network.Config = map[string]string{}
 
-	return tx.QueryScan(q, func(scan func(dest ...any) error) error {
+	return query.Scan(ctx, tx.Tx(), q, func(scan func(dest ...any) error) error {
 		var key, value string
 
 		err := scan(&key, &value)
