@@ -78,69 +78,32 @@ func (n NodeInfo) IsOffline(threshold time.Duration) bool {
 	return nodeIsOffline(threshold, n.Heartbeat)
 }
 
+// NodeInfoArgs provides information about the cluster environment for use with NodeInfo.ToAPI().
+type NodeInfoArgs struct {
+	LeaderAddress        string
+	FailureDomains       map[uint64]string
+	MemberFailureDomains map[string]uint64
+	OfflineThreshold     time.Duration
+	MaxMemberVersion     [2]int
+	RaftNodes            []RaftNode
+}
+
 // ToAPI returns a LXD API entry.
-func (n NodeInfo) ToAPI(cluster *Cluster, node *Node, leader string) (*api.ClusterMember, error) {
-	// Load some needed data.
+func (n NodeInfo) ToAPI(ctx context.Context, tx *ClusterTx, args NodeInfoArgs) (*api.ClusterMember, error) {
 	var err error
-	var offlineThreshold time.Duration
 	var maxVersion [2]int
 	var failureDomain string
 
-	// From cluster database.
-	err = cluster.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		// Get offline threshold.
-		offlineThreshold, err = tx.GetNodeOfflineThreshold()
-		if err != nil {
-			return fmt.Errorf("Load offline threshold config: %w", err)
-		}
-
-		// Get failure domains.
-		nodesDomains, err := tx.GetNodesFailureDomains()
-		if err != nil {
-			return fmt.Errorf("Load nodes failure domains: %w", err)
-		}
-
-		domainsNames, err := tx.GetFailureDomainsNames()
-		if err != nil {
-			return fmt.Errorf("Load failure domains names: %w", err)
-		}
-
-		domainID := nodesDomains[n.Address]
-		failureDomain = domainsNames[domainID]
-
-		// Get the highest schema and API versions.
-		maxVersion, err = tx.GetNodeMaxVersion()
-		if err != nil {
-			return fmt.Errorf("Get max version: %w", err)
-		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
-	}
+	domainID := args.MemberFailureDomains[n.Address]
+	failureDomain = args.FailureDomains[domainID]
 
 	// From local database.
 	var raftNode *RaftNode
-	err = node.Transaction(func(tx *NodeTx) error {
-		nodes, err := tx.GetRaftNodes()
-		if err != nil {
-			return fmt.Errorf("Load offline threshold config: %w", err)
-		}
-
-		for _, node := range nodes {
-			if node.Address != n.Address {
-				continue
-			}
-
+	for _, node := range args.RaftNodes {
+		if node.Address == n.Address {
 			raftNode = &node
 			break
 		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
 	}
 
 	// Fill in the struct.
@@ -158,8 +121,8 @@ func (n NodeInfo) ToAPI(cluster *Cluster, node *Node, leader string) (*api.Clust
 
 	result.Groups = n.Groups
 
-	// Check if node is the leader node
-	if leader == n.Address {
+	// Check if member is the leader.
+	if args.LeaderAddress == n.Address {
 		result.Roles = append(result.Roles, string(ClusterRoleDatabaseLeader))
 		result.Database = true
 	}
@@ -188,8 +151,9 @@ func (n NodeInfo) ToAPI(cluster *Cluster, node *Node, leader string) (*api.Clust
 	if n.State == ClusterMemberStateEvacuated {
 		result.Status = "Evacuated"
 		result.Message = "Unavailable due to maintenance"
-	} else if n.IsOffline(offlineThreshold) {
+	} else if n.IsOffline(args.OfflineThreshold) {
 		result.Status = "Offline"
+		result.Message = fmt.Sprintf("No heartbeat for %s (%s)", time.Since(n.Heartbeat), n.Heartbeat)
 	} else {
 		// Check if up to date.
 		n, err := util.CompareVersions(maxVersion, n.Version())
@@ -203,10 +167,6 @@ func (n NodeInfo) ToAPI(cluster *Cluster, node *Node, leader string) (*api.Clust
 		}
 	}
 
-	if n.IsOffline(offlineThreshold) {
-		result.Message = fmt.Sprintf("No heartbeat for %s (%s)", time.Since(n.Heartbeat), n.Heartbeat)
-	}
-
 	return &result, nil
 }
 
@@ -217,9 +177,9 @@ func (n NodeInfo) Version() [2]int {
 }
 
 // GetNodeByAddress returns the node with the given network address.
-func (c *ClusterTx) GetNodeByAddress(address string) (NodeInfo, error) {
+func (c *ClusterTx) GetNodeByAddress(ctx context.Context, address string) (NodeInfo, error) {
 	null := NodeInfo{}
-	nodes, err := c.nodes(false /* not pending */, "address=?", address)
+	nodes, err := c.nodes(ctx, false /* not pending */, "address=?", address)
 	if err != nil {
 		return null, err
 	}
@@ -234,13 +194,13 @@ func (c *ClusterTx) GetNodeByAddress(address string) (NodeInfo, error) {
 	}
 }
 
-// GetNodeMaxVersion returns the highest version possible on the cluster.
-func (c *ClusterTx) GetNodeMaxVersion() ([2]int, error) {
+// GetNodeMaxVersion returns the highest schema and API versions possible on the cluster.
+func (c *ClusterTx) GetNodeMaxVersion(ctx context.Context) ([2]int, error) {
 	version := [2]int{}
 
 	// Get the maximum DB schema.
 	var maxSchema int
-	row := c.tx.QueryRow("SELECT MAX(schema) FROM nodes")
+	row := c.tx.QueryRowContext(ctx, "SELECT MAX(schema) FROM nodes")
 	err := row.Scan(&maxSchema)
 	if err != nil {
 		return version, err
@@ -248,7 +208,7 @@ func (c *ClusterTx) GetNodeMaxVersion() ([2]int, error) {
 
 	// Get the maximum API extension.
 	var maxAPI int
-	row = c.tx.QueryRow("SELECT MAX(api_extensions) FROM nodes")
+	row = c.tx.QueryRowContext(ctx, "SELECT MAX(api_extensions) FROM nodes")
 	err = row.Scan(&maxAPI)
 	if err != nil {
 		return version, err
@@ -261,9 +221,9 @@ func (c *ClusterTx) GetNodeMaxVersion() ([2]int, error) {
 }
 
 // GetNodeWithID returns the node with the given ID.
-func (c *ClusterTx) GetNodeWithID(nodeID int) (NodeInfo, error) {
+func (c *ClusterTx) GetNodeWithID(ctx context.Context, nodeID int) (NodeInfo, error) {
 	null := NodeInfo{}
-	nodes, err := c.nodes(false /* not pending */, "id=?", nodeID)
+	nodes, err := c.nodes(ctx, false /* not pending */, "id=?", nodeID)
 	if err != nil {
 		return null, err
 	}
@@ -279,9 +239,9 @@ func (c *ClusterTx) GetNodeWithID(nodeID int) (NodeInfo, error) {
 }
 
 // GetPendingNodeByAddress returns the pending node with the given network address.
-func (c *ClusterTx) GetPendingNodeByAddress(address string) (NodeInfo, error) {
+func (c *ClusterTx) GetPendingNodeByAddress(ctx context.Context, address string) (NodeInfo, error) {
 	null := NodeInfo{}
-	nodes, err := c.nodes(true /*pending */, "address=?", address)
+	nodes, err := c.nodes(ctx, true /*pending */, "address=?", address)
 	if err != nil {
 		return null, err
 	}
@@ -297,9 +257,9 @@ func (c *ClusterTx) GetPendingNodeByAddress(address string) (NodeInfo, error) {
 }
 
 // GetNodeByName returns the node with the given name.
-func (c *ClusterTx) GetNodeByName(name string) (NodeInfo, error) {
+func (c *ClusterTx) GetNodeByName(ctx context.Context, name string) (NodeInfo, error) {
 	null := NodeInfo{}
-	nodes, err := c.nodes(false /* not pending */, "name=?", name)
+	nodes, err := c.nodes(ctx, false /* not pending */, "name=?", name)
 	if err != nil {
 		return null, err
 	}
@@ -316,9 +276,9 @@ func (c *ClusterTx) GetNodeByName(name string) (NodeInfo, error) {
 
 // GetLocalNodeName returns the name of the node this method is invoked on.
 // Usually you should not use this function directly but instead use the cached State.ServerName value.
-func (c *ClusterTx) GetLocalNodeName() (string, error) {
+func (c *ClusterTx) GetLocalNodeName(ctx context.Context) (string, error) {
 	stmt := "SELECT name FROM nodes WHERE id=?"
-	names, err := query.SelectStrings(c.tx, stmt, c.nodeID)
+	names, err := query.SelectStrings(ctx, c.tx, stmt, c.nodeID)
 	if err != nil {
 		return "", err
 	}
@@ -334,9 +294,9 @@ func (c *ClusterTx) GetLocalNodeName() (string, error) {
 }
 
 // GetLocalNodeAddress returns the address of the node this method is invoked on.
-func (c *ClusterTx) GetLocalNodeAddress() (string, error) {
+func (c *ClusterTx) GetLocalNodeAddress(ctx context.Context) (string, error) {
 	stmt := "SELECT address FROM nodes WHERE id=?"
-	addresses, err := query.SelectStrings(c.tx, stmt, c.nodeID)
+	addresses, err := query.SelectStrings(ctx, c.tx, stmt, c.nodeID)
 	if err != nil {
 		return "", err
 	}
@@ -353,8 +313,8 @@ func (c *ClusterTx) GetLocalNodeAddress() (string, error) {
 
 // NodeIsOutdated returns true if there's some cluster node having an API or
 // schema version greater than the node this method is invoked on.
-func (c *ClusterTx) NodeIsOutdated() (bool, error) {
-	nodes, err := c.nodes(false /* not pending */, "")
+func (c *ClusterTx) NodeIsOutdated(ctx context.Context) (bool, error) {
+	nodes, err := c.nodes(ctx, false /* not pending */, "")
 	if err != nil {
 		return false, fmt.Errorf("Failed to fetch nodes: %w", err)
 	}
@@ -394,16 +354,16 @@ func (c *ClusterTx) NodeIsOutdated() (bool, error) {
 //
 // If this LXD instance is not clustered, a list with a single node whose
 // address is 0.0.0.0 is returned.
-func (c *ClusterTx) GetNodes() ([]NodeInfo, error) {
-	return c.nodes(false /* not pending */, "")
+func (c *ClusterTx) GetNodes(ctx context.Context) ([]NodeInfo, error) {
+	return c.nodes(ctx, false /* not pending */, "")
 }
 
 // GetNodesCount returns the number of nodes in the LXD cluster.
 //
 // Since there's always at least one node row, even when not-clustered, the
 // return value is greater than zero.
-func (c *ClusterTx) GetNodesCount() (int, error) {
-	count, err := query.Count(c.tx, "nodes", "")
+func (c *ClusterTx) GetNodesCount(ctx context.Context) (int, error) {
+	count, err := query.Count(ctx, c.tx, "nodes", "")
 	if err != nil {
 		return 0, fmt.Errorf("failed to count existing nodes: %w", err)
 	}
@@ -414,8 +374,8 @@ func (c *ClusterTx) GetNodesCount() (int, error) {
 // RenameNode changes the name of an existing node.
 //
 // Return an error if a node with the same name already exists.
-func (c *ClusterTx) RenameNode(old, new string) error {
-	count, err := query.Count(c.tx, "nodes", "name=?", new)
+func (c *ClusterTx) RenameNode(ctx context.Context, old string, new string) error {
+	count, err := query.Count(ctx, c.tx, "nodes", "name=?", new)
 	if err != nil {
 		return fmt.Errorf("failed to check existing nodes: %w", err)
 	}
@@ -463,12 +423,12 @@ func (c *ClusterTx) SetDescription(id int64, description string) error {
 }
 
 // Nodes returns all LXD nodes part of the cluster.
-func (c *ClusterTx) nodes(pending bool, where string, args ...any) ([]NodeInfo, error) {
+func (c *ClusterTx) nodes(ctx context.Context, pending bool, where string, args ...any) ([]NodeInfo, error) {
 	// Get node roles
 	sql := "SELECT node_id, role FROM nodes_roles"
 
 	nodeRoles := map[int64][]ClusterRole{}
-	err := c.QueryScan(sql, func(scan func(dest ...any) error) error {
+	err := query.Scan(ctx, c.Tx(), sql, func(scan func(dest ...any) error) error {
 		var nodeID int64
 		var role int
 
@@ -496,7 +456,7 @@ func (c *ClusterTx) nodes(pending bool, where string, args ...any) ([]NodeInfo,
 JOIN cluster_groups ON cluster_groups.id = nodes_cluster_groups.group_id`
 	nodeGroups := map[int64][]string{}
 
-	err = c.QueryScan(sql, func(scan func(dest ...any) error) error {
+	err = query.Scan(ctx, c.Tx(), sql, func(scan func(dest ...any) error) error {
 		var nodeID int64
 		var group string
 
@@ -518,23 +478,6 @@ JOIN cluster_groups ON cluster_groups.id = nodes_cluster_groups.group_id`
 		return nil, err
 	}
 
-	// Process node entries
-	nodes := []NodeInfo{}
-	dest := func(i int) []any {
-		nodes = append(nodes, NodeInfo{})
-		return []any{
-			&nodes[i].ID,
-			&nodes[i].Name,
-			&nodes[i].Address,
-			&nodes[i].Description,
-			&nodes[i].Schema,
-			&nodes[i].APIExtensions,
-			&nodes[i].Heartbeat,
-			&nodes[i].Architecture,
-			&nodes[i].State,
-		}
-	}
-
 	// Get the node entries
 	sql = "SELECT id, name, address, description, schema, api_extensions, heartbeat, arch, state FROM nodes "
 
@@ -554,14 +497,19 @@ JOIN cluster_groups ON cluster_groups.id = nodes_cluster_groups.group_id`
 
 	sql += "ORDER BY id"
 
-	stmt, err := c.tx.Prepare(sql)
-	if err != nil {
-		return nil, err
-	}
+	// Process node entries
+	nodes := []NodeInfo{}
+	err = query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		node := NodeInfo{}
+		err := scan(&node.ID, &node.Name, &node.Address, &node.Description, &node.Schema, &node.APIExtensions, &node.Heartbeat, &node.Architecture, &node.State)
+		if err != nil {
+			return err
+		}
 
-	defer func() { _ = stmt.Close() }()
+		nodes = append(nodes, node)
 
-	err = query.SelectObjects(stmt, dest, args...)
+		return nil
+	}, args...)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to fetch nodes: %w", err)
 	}
@@ -718,13 +666,13 @@ func (c *ClusterTx) UpdateNodeRoles(id int64, roles []ClusterRole) error {
 }
 
 // UpdateNodeClusterGroups changes the list of cluster groups the member belongs to.
-func (c *ClusterTx) UpdateNodeClusterGroups(id int64, groups []string) error {
-	nodeInfo, err := c.GetNodeWithID(int(id))
+func (c *ClusterTx) UpdateNodeClusterGroups(ctx context.Context, id int64, groups []string) error {
+	nodeInfo, err := c.GetNodeWithID(ctx, int(id))
 	if err != nil {
 		return err
 	}
 
-	oldGroups, err := c.GetClusterGroupsWithNode(nodeInfo.Name)
+	oldGroups, err := c.GetClusterGroupsWithNode(ctx, nodeInfo.Name)
 	if err != nil {
 		return err
 	}
@@ -740,7 +688,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(id int64, groups []string) error {
 		}
 
 		// Add node to new group.
-		err = c.AddNodeToClusterGroup(newGroup, nodeInfo.Name)
+		err = c.AddNodeToClusterGroup(ctx, newGroup, nodeInfo.Name)
 		if err != nil {
 			return fmt.Errorf("Failed to add member to cluster group: %w", err)
 		}
@@ -752,7 +700,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(id int64, groups []string) error {
 		}
 
 		// Remove node from group.
-		err = c.RemoveNodeFromClusterGroup(oldGroup, nodeInfo.Name)
+		err = c.RemoveNodeFromClusterGroup(ctx, oldGroup, nodeInfo.Name)
 		if err != nil {
 			return fmt.Errorf("Failed to remove member from cluster group: %w", err)
 		}
@@ -762,7 +710,7 @@ func (c *ClusterTx) UpdateNodeClusterGroups(id int64, groups []string) error {
 }
 
 // UpdateNodeFailureDomain changes the failure domain of a node.
-func (c *ClusterTx) UpdateNodeFailureDomain(id int64, domain string) error {
+func (c *ClusterTx) UpdateNodeFailureDomain(ctx context.Context, id int64, domain string) error {
 	var domainID any
 
 	if domain == "" {
@@ -772,7 +720,7 @@ func (c *ClusterTx) UpdateNodeFailureDomain(id int64, domain string) error {
 	if domain == "default" {
 		domainID = nil
 	} else {
-		row := c.tx.QueryRow("SELECT id FROM nodes_failure_domains WHERE name=?", domain)
+		row := c.tx.QueryRowContext(ctx, "SELECT id FROM nodes_failure_domains WHERE name=?", domain)
 		err := row.Scan(&domainID)
 		if err != nil {
 			if err != sql.ErrNoRows {
@@ -828,7 +776,7 @@ func (c *ClusterTx) UpdateNodeStatus(id int64, state int) error {
 }
 
 // GetNodeFailureDomain returns the failure domain associated with the node with the given ID.
-func (c *ClusterTx) GetNodeFailureDomain(id int64) (string, error) {
+func (c *ClusterTx) GetNodeFailureDomain(ctx context.Context, id int64) (string, error) {
 	stmt := `
 SELECT coalesce(nodes_failure_domains.name,'default')
   FROM nodes LEFT JOIN nodes_failure_domains ON nodes.failure_domain_id = nodes_failure_domains.id
@@ -836,7 +784,7 @@ SELECT coalesce(nodes_failure_domains.name,'default')
 `
 	var domain string
 
-	err := c.tx.QueryRow(stmt, id).Scan(&domain)
+	err := c.tx.QueryRowContext(ctx, stmt, id).Scan(&domain)
 	if err != nil {
 		return "", err
 	}
@@ -846,26 +794,25 @@ SELECT coalesce(nodes_failure_domains.name,'default')
 
 // GetNodesFailureDomains returns a map associating each node address with its
 // failure domain code.
-func (c *ClusterTx) GetNodesFailureDomains() (map[string]uint64, error) {
-	stmt, err := c.tx.Prepare("SELECT address, coalesce(failure_domain_id, 0) FROM nodes")
-	if err != nil {
-		return nil, err
-	}
-
-	rows := []struct {
+func (c *ClusterTx) GetNodesFailureDomains(ctx context.Context) (map[string]uint64, error) {
+	sql := "SELECT address, coalesce(failure_domain_id, 0) FROM nodes"
+	type failureDomain struct {
 		Address         string
 		FailureDomainID int64
-	}{}
-
-	dest := func(i int) []any {
-		rows = append(rows, struct {
-			Address         string
-			FailureDomainID int64
-		}{})
-		return []any{&rows[len(rows)-1].Address, &rows[len(rows)-1].FailureDomainID}
 	}
 
-	err = query.SelectObjects(stmt, dest)
+	rows := []failureDomain{}
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		fd := failureDomain{}
+		err := scan(&fd.Address, &fd.FailureDomainID)
+		if err != nil {
+			return err
+		}
+
+		rows = append(rows, fd)
+
+		return nil
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -881,26 +828,26 @@ func (c *ClusterTx) GetNodesFailureDomains() (map[string]uint64, error) {
 
 // GetFailureDomainsNames return a map associating failure domain IDs to their
 // names.
-func (c *ClusterTx) GetFailureDomainsNames() (map[uint64]string, error) {
-	stmt, err := c.tx.Prepare("SELECT id, name FROM nodes_failure_domains")
-	if err != nil {
-		return nil, err
-	}
+func (c *ClusterTx) GetFailureDomainsNames(ctx context.Context) (map[uint64]string, error) {
+	sql := "SELECT id, name FROM nodes_failure_domains"
 
-	rows := []struct {
+	type failureDomain struct {
 		ID   int64
 		Name string
-	}{}
-
-	dest := func(i int) []any {
-		rows = append(rows, struct {
-			ID   int64
-			Name string
-		}{})
-		return []any{&rows[len(rows)-1].ID, &rows[len(rows)-1].Name}
 	}
 
-	err = query.SelectObjects(stmt, dest)
+	rows := []failureDomain{}
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		fd := failureDomain{}
+		err := scan(&fd.ID, &fd.Name)
+		if err != nil {
+			return err
+		}
+
+		rows = append(rows, fd)
+
+		return nil
+	})
 	if err != nil {
 		return nil, err
 	}
@@ -960,9 +907,9 @@ func (c *ClusterTx) SetNodeHeartbeat(address string, heartbeat time.Time) error
 // NodeIsEmpty returns an empty string if the node with the given ID has no
 // containers or images associated with it. Otherwise, it returns a message
 // say what's left.
-func (c *ClusterTx) NodeIsEmpty(id int64) (string, error) {
+func (c *ClusterTx) NodeIsEmpty(ctx context.Context, id int64) (string, error) {
 	// Check if the node has any instances.
-	containers, err := query.SelectStrings(c.tx, "SELECT name FROM instances WHERE node_id=?", id)
+	containers, err := query.SelectStrings(ctx, c.tx, "SELECT name FROM instances WHERE node_id=?", id)
 	if err != nil {
 		return "", fmt.Errorf("Failed to get instances for node %d: %w", id, err)
 	}
@@ -974,26 +921,24 @@ func (c *ClusterTx) NodeIsEmpty(id int64) (string, error) {
 	}
 
 	// Check if the node has any images available only in it.
-	images := []struct {
+	type image struct {
 		fingerprint string
 		nodeID      int64
-	}{}
-	dest := func(i int) []any {
-		images = append(images, struct {
-			fingerprint string
-			nodeID      int64
-		}{})
-		return []any{&images[i].fingerprint, &images[i].nodeID}
 	}
 
-	stmt, err := c.tx.Prepare(`
-SELECT fingerprint, node_id FROM images JOIN images_nodes ON images.id=images_nodes.image_id`)
-	if err != nil {
-		return "", err
-	}
+	images := []image{}
+	sql := `SELECT fingerprint, node_id FROM images JOIN images_nodes ON images.id=images_nodes.image_id`
+	err = query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		img := image{}
+		err := scan(&img.fingerprint, &img.nodeID)
+		if err != nil {
+			return err
+		}
 
-	defer func() { _ = stmt.Close() }()
-	err = query.SelectObjects(stmt, dest)
+		images = append(images, img)
+
+		return nil
+	})
 	if err != nil {
 		return "", fmt.Errorf("Failed to get image list for node %d: %w", id, err)
 	}
@@ -1021,9 +966,13 @@ SELECT fingerprint, node_id FROM images JOIN images_nodes ON images.id=images_no
 	}
 
 	// Check if the node has any custom volumes.
-	volumes, err := query.SelectStrings(
-		c.tx, "SELECT storage_volumes.name FROM storage_volumes JOIN storage_pools ON storage_volumes.storage_pool_id=storage_pools.id WHERE storage_volumes.node_id=? AND storage_volumes.type=? AND storage_pools.driver NOT IN ('ceph', 'cephfs')",
-		id, StoragePoolVolumeTypeCustom)
+	sql = `
+SELECT storage_volumes.name
+  FROM storage_volumes
+  JOIN storage_pools ON storage_volumes.storage_pool_id=storage_pools.id
+  WHERE storage_volumes.node_id=? AND storage_volumes.type=? AND storage_pools.driver NOT IN ('ceph', 'cephfs')
+`
+	volumes, err := query.SelectStrings(ctx, c.tx, sql, id, StoragePoolVolumeTypeCustom)
 	if err != nil {
 		return "", fmt.Errorf("Failed to get custom volumes for node %d: %w", id, err)
 	}
@@ -1038,14 +987,14 @@ SELECT fingerprint, node_id FROM images JOIN images_nodes ON images.id=images_no
 }
 
 // ClearNode removes any instance or image associated with this node.
-func (c *ClusterTx) ClearNode(id int64) error {
+func (c *ClusterTx) ClearNode(ctx context.Context, id int64) error {
 	_, err := c.tx.Exec("DELETE FROM instances WHERE node_id=?", id)
 	if err != nil {
 		return err
 	}
 
 	// Get the IDs of the images this node is hosting.
-	ids, err := query.SelectIntegers(c.tx, "SELECT image_id FROM images_nodes WHERE node_id=?", id)
+	ids, err := query.SelectIntegers(ctx, c.tx, "SELECT image_id FROM images_nodes WHERE node_id=?", id)
 	if err != nil {
 		return err
 	}
@@ -1058,7 +1007,7 @@ func (c *ClusterTx) ClearNode(id int64) error {
 
 	// Delete the image as well if this was the only node with it.
 	for _, id := range ids {
-		count, err := query.Count(c.tx, "images_nodes", "image_id=?", id)
+		count, err := query.Count(ctx, c.tx, "images_nodes", "image_id=?", id)
 		if err != nil {
 			return err
 		}
@@ -1079,10 +1028,9 @@ func (c *ClusterTx) ClearNode(id int64) error {
 // GetNodeOfflineThreshold returns the amount of time that needs to elapse after
 // which a series of unsuccessful heartbeat will make the node be considered
 // offline.
-func (c *ClusterTx) GetNodeOfflineThreshold() (time.Duration, error) {
+func (c *ClusterTx) GetNodeOfflineThreshold(ctx context.Context) (time.Duration, error) {
 	threshold := time.Duration(DefaultOfflineThreshold) * time.Second
-	values, err := query.SelectStrings(
-		c.tx, "SELECT value FROM config WHERE key='cluster.offline_threshold'")
+	values, err := query.SelectStrings(ctx, c.tx, "SELECT value FROM config WHERE key='cluster.offline_threshold'")
 	if err != nil {
 		return -1, err
 	}
@@ -1099,50 +1047,38 @@ func (c *ClusterTx) GetNodeOfflineThreshold() (time.Duration, error) {
 	return threshold, nil
 }
 
-// GetNodeWithLeastInstances returns the name of the non-offline node with with
-// the least number of containers (either already created or being created with
-// an operation). If archs is not empty, then return only nodes with an
-// architecture in that list.
-func (c *ClusterTx) GetNodeWithLeastInstances(archs []int, defaultArch int, group string, allowedGroups []string) (string, error) {
-	threshold, err := c.GetNodeOfflineThreshold()
-	if err != nil {
-		return "", fmt.Errorf("Failed to get offline threshold: %w", err)
-	}
+// GetCandidateMembers returns cluster members that are online, in created state and don't need manual targeting.
+// It excludes members that do not support any of the targetArchitectures (if non-nil) or not in targetClusterGroup
+// (if non-empty). It also takes into account any restrictions on allowedClusterGroups (if non-nil).
+func (c *ClusterTx) GetCandidateMembers(ctx context.Context, allMembers []NodeInfo, targetArchitectures []int, targetClusterGroup string, allowedClusterGroups []string, offlineThreshold time.Duration) ([]NodeInfo, error) {
+	var candidateMembers []NodeInfo
 
-	nodes, err := c.GetNodes()
-	if err != nil {
-		return "", fmt.Errorf("Failed to get current cluster members: %w", err)
-	}
-
-	name := ""
-	containers := -1
-	isDefaultArchChosen := false
-	for _, node := range nodes {
-		// Skip evacuated members.
-		if node.State == ClusterMemberStateEvacuated || node.IsOffline(threshold) {
+	for _, member := range allMembers {
+		// Skip pending, evacuated or offline members.
+		if member.State != ClusterMemberStateCreated || member.IsOffline(offlineThreshold) {
 			continue
 		}
 
 		// Skip manually targeted members.
-		if node.Config["scheduler.instance"] == "manual" {
+		if member.Config["scheduler.instance"] == "manual" {
 			continue
 		}
 
-		// Skip group-only members if targeted group doesn't match.
-		if node.Config["scheduler.instance"] == "group" && !shared.StringInSlice(group, node.Groups) {
+		// Skip group-only members if targeted cluster group doesn't match.
+		if member.Config["scheduler.instance"] == "group" && !shared.StringInSlice(targetClusterGroup, member.Groups) {
 			continue
 		}
 
 		// Skip if a group is requested and member isn't part of it.
-		if group != "" && !shared.StringInSlice(group, node.Groups) {
+		if targetClusterGroup != "" && !shared.StringInSlice(targetClusterGroup, member.Groups) {
 			continue
 		}
 
-		// Skip if working with a restricted set of groups and member isn't part of any.
-		if allowedGroups != nil {
+		// Skip if working with a restricted set of cluster groups and member isn't part of any.
+		if allowedClusterGroups != nil {
 			found := false
-			for _, allowedGroup := range allowedGroups {
-				if shared.StringInSlice(allowedGroup, node.Groups) {
+			for _, allowedClusterGroup := range allowedClusterGroups {
+				if shared.StringInSlice(allowedClusterGroup, member.Groups) {
 					found = true
 					break
 				}
@@ -1153,58 +1089,57 @@ func (c *ClusterTx) GetNodeWithLeastInstances(archs []int, defaultArch int, grou
 			}
 		}
 
-		// Get member personalities too.
-		personalities, err := osarch.ArchitecturePersonalities(node.Architecture)
-		if err != nil {
-			return "", err
-		}
-
-		supported := []int{node.Architecture}
-		supported = append(supported, personalities...)
-
-		match := false
-		isDefaultArch := false
-		for _, entry := range supported {
-			if shared.IntInSlice(entry, archs) {
-				match = true
+		// Consider target architectures if specified.
+		if targetArchitectures != nil {
+			// Get member personalities too.
+			personalities, err := osarch.ArchitecturePersonalities(member.Architecture)
+			if err != nil {
+				return nil, err
 			}
 
-			if entry == defaultArch {
-				isDefaultArch = true
-			}
-		}
-		if len(archs) > 0 && !match {
-			continue
-		}
-
-		if !isDefaultArch && isDefaultArchChosen {
-			continue
-		}
-
-		// Fetch the number of instances already created on this node.
-		created, err := query.Count(c.tx, "instances", "node_id=?", node.ID)
-		if err != nil {
-			return "", fmt.Errorf("Failed to get instances count: %w", err)
-		}
-
-		// Fetch the number of instances currently being created on this node.
-		pending, err := query.Count(
-			c.tx, "operations", "node_id=? AND type=?", node.ID, operationtype.InstanceCreate)
-		if err != nil {
-			return "", fmt.Errorf("Failed to get pending instances count: %w", err)
-		}
-
-		count := created + pending
-		if containers == -1 || count < containers || (isDefaultArch && !isDefaultArchChosen) {
-			containers = count
-			name = node.Name
-			if isDefaultArch {
-				isDefaultArchChosen = true
+			supportedArchitectures := append([]int{member.Architecture}, personalities...)
+			for _, supportedArchitecture := range supportedArchitectures {
+				if shared.IntInSlice(supportedArchitecture, targetArchitectures) {
+					candidateMembers = append(candidateMembers, member)
+					break
+				}
 			}
+		} else {
+			// Otherwise consider member a candidate irrespective of architecture.
+			candidateMembers = append(candidateMembers, member)
 		}
 	}
 
-	return name, nil
+	return candidateMembers, nil
+}
+
+// GetNodeWithLeastInstances returns the name of the member with the least number of instances that are either
+// already created or being created with an operation.
+func (c *ClusterTx) GetNodeWithLeastInstances(ctx context.Context, members []NodeInfo) (*NodeInfo, error) {
+	var member *NodeInfo
+	var lowestInstanceCount = -1
+
+	for i := range members {
+		// Fetch the number of instances already created on this member.
+		created, err := query.Count(ctx, c.tx, "instances", "node_id=?", members[i].ID)
+		if err != nil {
+			return nil, fmt.Errorf("Failed to get instances count: %w", err)
+		}
+
+		// Fetch the number of instances currently being created on this member.
+		pending, err := query.Count(ctx, c.tx, "operations", "node_id=? AND type=?", members[i].ID, operationtype.InstanceCreate)
+		if err != nil {
+			return nil, fmt.Errorf("Failed to get pending instances count: %w", err)
+		}
+
+		memberInstanceCount := created + pending
+		if lowestInstanceCount == -1 || memberInstanceCount < lowestInstanceCount {
+			lowestInstanceCount = memberInstanceCount
+			member = &members[i]
+		}
+	}
+
+	return member, nil
 }
 
 // SetNodeVersion updates the schema and API version of the node with the
@@ -1240,12 +1175,12 @@ func (c *Cluster) LocalNodeIsEvacuated() bool {
 	isEvacuated := false
 
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		name, err := tx.GetLocalNodeName()
+		name, err := tx.GetLocalNodeName(ctx)
 		if err != nil {
 			return err
 		}
 
-		node, err := tx.GetNodeByName(name)
+		node, err := tx.GetNodeByName(ctx, name)
 		if err != nil {
 			return nil
 		}
