@@ -1,6 +1,7 @@
 package node
 
 import (
+	"context"
 	"database/sql"
 	"encoding/hex"
 	"fmt"
@@ -105,7 +106,7 @@ const UpdateFromPreClustering = 36
 
 // Schema updates begin here
 
-func updateFromV41(tx *sql.Tx) error {
+func updateFromV41(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 	ALTER TABLE raft_nodes ADD COLUMN name TEXT NOT NULL default "";
 	`
@@ -113,7 +114,7 @@ func updateFromV41(tx *sql.Tx) error {
 	return err
 }
 
-func updateFromV40(tx *sql.Tx) error {
+func updateFromV40(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE certificates (
 	id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -130,27 +131,26 @@ CREATE TABLE certificates (
 
 // Fix the address of the bootstrap node being set to "0" in the raft_nodes
 // table.
-func updateFromV39(tx *sql.Tx) error {
-	nodes := []struct {
+func updateFromV39(ctx context.Context, tx *sql.Tx) error {
+	type node struct {
 		ID      uint64
 		Address string
-	}{}
-	dest := func(i int) []any {
-		nodes = append(nodes, struct {
-			ID      uint64
-			Address string
-		}{})
-		return []any{&nodes[i].ID, &nodes[i].Address}
 	}
 
-	stmt, err := tx.Prepare("SELECT id, address FROM raft_nodes")
-	if err != nil {
-		return err
-	}
+	sql := "SELECT id, address FROM raft_nodes"
+	nodes := []node{}
+	err := query.Scan(ctx, tx, sql, func(scan func(dest ...any) error) error {
+		n := node{}
 
-	defer func() { _ = stmt.Close() }()
+		err := scan(&n.ID, &n.Address)
+		if err != nil {
+			return err
+		}
 
-	err = query.SelectObjects(stmt, dest)
+		nodes = append(nodes, n)
+
+		return nil
+	})
 	if err != nil {
 		return fmt.Errorf("Failed to fetch raft nodes: %w", err)
 	}
@@ -164,7 +164,7 @@ func updateFromV39(tx *sql.Tx) error {
 		return nil
 	}
 
-	config, err := query.SelectConfig(tx, "config", "")
+	config, err := query.SelectConfig(ctx, tx, "config", "")
 	if err != nil {
 		return err
 	}
@@ -182,7 +182,7 @@ func updateFromV39(tx *sql.Tx) error {
 
 // Add role column to raft_nodes table. All existing entries will have role "0"
 // which means voter.
-func updateFromV38(tx *sql.Tx) error {
+func updateFromV38(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 ALTER TABLE raft_nodes ADD COLUMN role INTEGER NOT NULL DEFAULT 0;
 `
@@ -192,8 +192,8 @@ ALTER TABLE raft_nodes ADD COLUMN role INTEGER NOT NULL DEFAULT 0;
 
 // Copy core.https_address to cluster.https_address in case this node is
 // clustered.
-func updateFromV37(tx *sql.Tx) error {
-	count, err := query.Count(tx, "raft_nodes", "")
+func updateFromV37(ctx context.Context, tx *sql.Tx) error {
+	count, err := query.Count(ctx, tx, "raft_nodes", "")
 	if err != nil {
 		return fmt.Errorf("Fetch count of Raft nodes: %w", err)
 	}
@@ -232,7 +232,7 @@ INSERT INTO config (key, value)
 // cluster, regardless of whether they are part of the raft cluster or not, and
 // all nodes will consult this table when they need to find out a leader to
 // send SQL queries to.
-func updateFromV36(tx *sql.Tx) error {
+func updateFromV36(ctx context.Context, tx *sql.Tx) error {
 	stmts := `
 CREATE TABLE raft_nodes (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -265,7 +265,7 @@ DROP TABLE storage_pools;
 	return err
 }
 
-func updateFromV35(tx *sql.Tx) error {
+func updateFromV35(ctx context.Context, tx *sql.Tx) error {
 	stmts := `
 CREATE TABLE tmp (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -290,7 +290,7 @@ ALTER TABLE containers ADD COLUMN description TEXT;
 	return err
 }
 
-func updateFromV34(tx *sql.Tx) error {
+func updateFromV34(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE IF NOT EXISTS storage_pools (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -326,7 +326,7 @@ CREATE TABLE IF NOT EXISTS storage_volumes_config (
 	return err
 }
 
-func updateFromV33(tx *sql.Tx) error {
+func updateFromV33(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE IF NOT EXISTS networks (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -345,12 +345,12 @@ CREATE TABLE IF NOT EXISTS networks_config (
 	return err
 }
 
-func updateFromV32(tx *sql.Tx) error {
+func updateFromV32(ctx context.Context, tx *sql.Tx) error {
 	_, err := tx.Exec("ALTER TABLE containers ADD COLUMN last_use_date DATETIME;")
 	return err
 }
 
-func updateFromV31(tx *sql.Tx) error {
+func updateFromV31(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE IF NOT EXISTS patches (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -362,13 +362,13 @@ CREATE TABLE IF NOT EXISTS patches (
 	return err
 }
 
-func updateFromV30(tx *sql.Tx) error {
+func updateFromV30(ctx context.Context, tx *sql.Tx) error {
 	// NOTE: this database update contained daemon-level logic which
 	//       was been moved to patchUpdateFromV15 in patches.go.
 	return nil
 }
 
-func updateFromV29(tx *sql.Tx) error {
+func updateFromV29(ctx context.Context, tx *sql.Tx) error {
 	if shared.PathExists(shared.VarPath("zfs.img")) {
 		err := os.Chmod(shared.VarPath("zfs.img"), 0600)
 		if err != nil {
@@ -379,7 +379,7 @@ func updateFromV29(tx *sql.Tx) error {
 	return nil
 }
 
-func updateFromV28(tx *sql.Tx) error {
+func updateFromV28(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 INSERT INTO profiles_devices (profile_id, name, type) SELECT id, "aadisable", 2 FROM profiles WHERE name="docker";
 INSERT INTO profiles_devices_config (profile_device_id, key, value) SELECT profiles_devices.id, "source", "/dev/null" FROM profiles_devices LEFT JOIN profiles WHERE profiles_devices.profile_id = profiles.id AND profiles.name = "docker" AND profiles_devices.name = "aadisable";
@@ -389,12 +389,12 @@ INSERT INTO profiles_devices_config (profile_device_id, key, value) SELECT profi
 	return nil
 }
 
-func updateFromV27(tx *sql.Tx) error {
+func updateFromV27(ctx context.Context, tx *sql.Tx) error {
 	_, err := tx.Exec("UPDATE profiles_devices SET type=3 WHERE type='unix-char';")
 	return err
 }
 
-func updateFromV26(tx *sql.Tx) error {
+func updateFromV26(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 ALTER TABLE images ADD COLUMN auto_update INTEGER NOT NULL DEFAULT 0;
 CREATE TABLE IF NOT EXISTS images_source (
@@ -410,7 +410,7 @@ CREATE TABLE IF NOT EXISTS images_source (
 	return err
 }
 
-func updateFromV25(tx *sql.Tx) error {
+func updateFromV25(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 INSERT INTO profiles (name, description) VALUES ("docker", "Profile supporting docker in containers");
 INSERT INTO profiles_config (profile_id, key, value) SELECT id, "security.nesting", "true" FROM profiles WHERE name="docker";
@@ -422,17 +422,17 @@ INSERT INTO profiles_devices_config (profile_device_id, key, value) SELECT profi
 	return nil
 }
 
-func updateFromV24(tx *sql.Tx) error {
+func updateFromV24(ctx context.Context, tx *sql.Tx) error {
 	_, err := tx.Exec("ALTER TABLE containers ADD COLUMN stateful INTEGER NOT NULL DEFAULT 0;")
 	return err
 }
 
-func updateFromV23(tx *sql.Tx) error {
+func updateFromV23(ctx context.Context, tx *sql.Tx) error {
 	_, err := tx.Exec("ALTER TABLE profiles ADD COLUMN description TEXT;")
 	return err
 }
 
-func updateFromV22(tx *sql.Tx) error {
+func updateFromV22(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 DELETE FROM containers_devices_config WHERE key='type';
 DELETE FROM profiles_devices_config WHERE key='type';`
@@ -440,12 +440,12 @@ DELETE FROM profiles_devices_config WHERE key='type';`
 	return err
 }
 
-func updateFromV21(tx *sql.Tx) error {
+func updateFromV21(ctx context.Context, tx *sql.Tx) error {
 	_, err := tx.Exec("ALTER TABLE containers ADD COLUMN creation_date DATETIME NOT NULL DEFAULT 0;")
 	return err
 }
 
-func updateFromV20(tx *sql.Tx) error {
+func updateFromV20(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 UPDATE containers_devices SET name='__lxd_upgrade_root' WHERE name='root';
 UPDATE profiles_devices SET name='__lxd_upgrade_root' WHERE name='root';
@@ -457,7 +457,7 @@ INSERT INTO containers_devices_config (container_device_id, key, value) SELECT i
 	return err
 }
 
-func updateFromV19(tx *sql.Tx) error {
+func updateFromV19(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 DELETE FROM containers_config WHERE container_id NOT IN (SELECT id FROM containers);
 DELETE FROM containers_devices_config WHERE container_device_id NOT IN (SELECT id FROM containers_devices WHERE container_id IN (SELECT id FROM containers));
@@ -469,12 +469,12 @@ DELETE FROM images_properties WHERE image_id NOT IN (SELECT id FROM images);`
 	return err
 }
 
-func updateFromV18(tx *sql.Tx) error {
+func updateFromV18(ctx context.Context, tx *sql.Tx) error {
 	var id int
 	var value string
 
 	// Update container config
-	rows, err := tx.Query("SELECT id, value FROM containers_config WHERE key='limits.memory'")
+	rows, err := tx.QueryContext(ctx, "SELECT id, value FROM containers_config WHERE key='limits.memory'")
 	if err != nil {
 		return err
 	}
@@ -520,7 +520,7 @@ func updateFromV18(tx *sql.Tx) error {
 	}
 
 	// Update profiles config
-	rows, err = tx.Query("SELECT id, value FROM profiles_config WHERE key='limits.memory'")
+	rows, err = tx.QueryContext(ctx, "SELECT id, value FROM profiles_config WHERE key='limits.memory'")
 	if err != nil {
 		return err
 	}
@@ -568,7 +568,7 @@ func updateFromV18(tx *sql.Tx) error {
 	return nil
 }
 
-func updateFromV17(tx *sql.Tx) error {
+func updateFromV17(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 DELETE FROM profiles_config WHERE key LIKE 'volatile.%';
 UPDATE containers_config SET key='limits.cpu' WHERE key='limits.cpus';
@@ -577,7 +577,7 @@ UPDATE profiles_config SET key='limits.cpu' WHERE key='limits.cpus';`
 	return err
 }
 
-func updateFromV16(tx *sql.Tx) error {
+func updateFromV16(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 UPDATE config SET key='storage.lvm_vg_name' WHERE key = 'core.lvm_vg_name';
 UPDATE config SET key='storage.lvm_thinpool_name' WHERE key = 'core.lvm_thinpool_name';`
@@ -585,13 +585,13 @@ UPDATE config SET key='storage.lvm_thinpool_name' WHERE key = 'core.lvm_thinpool
 	return err
 }
 
-func updateFromV15(tx *sql.Tx) error {
+func updateFromV15(ctx context.Context, tx *sql.Tx) error {
 	// NOTE: this database update contained daemon-level logic which
 	//       was been moved to patchUpdateFromV15 in patches.go.
 	return nil
 }
 
-func updateFromV14(tx *sql.Tx) error {
+func updateFromV14(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 PRAGMA foreign_keys=OFF; -- So that integrity doesn't get in the way for now
 
@@ -624,14 +624,14 @@ PRAGMA foreign_keys=ON; -- Make sure we turn integrity checks back on.`
 	return err
 }
 
-func updateFromV13(tx *sql.Tx) error {
+func updateFromV13(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 UPDATE containers_config SET key='volatile.base_image' WHERE key = 'volatile.baseImage';`
 	_, err := tx.Exec(stmt)
 	return err
 }
 
-func updateFromV12(tx *sql.Tx) error {
+func updateFromV12(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 ALTER TABLE images ADD COLUMN cached INTEGER NOT NULL DEFAULT 0;
 ALTER TABLE images ADD COLUMN last_use_date DATETIME;`
@@ -639,19 +639,19 @@ ALTER TABLE images ADD COLUMN last_use_date DATETIME;`
 	return err
 }
 
-func updateFromV11(tx *sql.Tx) error {
+func updateFromV11(ctx context.Context, tx *sql.Tx) error {
 	// NOTE: this database update contained daemon-level logic which
 	//       was been moved to patchUpdateFromV15 in patches.go.
 	return nil
 }
 
-func updateFromV10(tx *sql.Tx) error {
+func updateFromV10(ctx context.Context, tx *sql.Tx) error {
 	// NOTE: this database update contained daemon-level logic which
 	//       was been moved to patchUpdateFromV10 in patches.go.
 	return nil
 }
 
-func updateFromV9(tx *sql.Tx) error {
+func updateFromV9(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE tmp (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
@@ -694,14 +694,14 @@ DROP TABLE tmp;`
 	return err
 }
 
-func updateFromV8(tx *sql.Tx) error {
+func updateFromV8(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 UPDATE certificates SET fingerprint = replace(fingerprint, " ", "");`
 	_, err := tx.Exec(stmt)
 	return err
 }
 
-func updateFromV7(tx *sql.Tx) error {
+func updateFromV7(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 UPDATE config SET key='core.trust_password' WHERE key IN ('password', 'trust_password', 'trust-password', 'core.trust-password');
 DELETE FROM config WHERE key != 'core.trust_password';`
@@ -709,7 +709,7 @@ DELETE FROM config WHERE key != 'core.trust_password';`
 	return err
 }
 
-func updateFromV6(tx *sql.Tx) error {
+func updateFromV6(ctx context.Context, tx *sql.Tx) error {
 	// This update recreates the schemas that need an ON DELETE CASCADE foreign
 	// key.
 	stmt := `
@@ -840,7 +840,7 @@ PRAGMA foreign_keys=ON; -- Make sure we turn integrity checks back on.`
 	}
 
 	// Get the rows with broken foreign keys an nuke them
-	rows, err := tx.Query("PRAGMA foreign_key_check;")
+	rows, err := tx.QueryContext(ctx, "PRAGMA foreign_key_check;")
 	if err != nil {
 		return err
 	}
@@ -875,7 +875,7 @@ PRAGMA foreign_keys=ON; -- Make sure we turn integrity checks back on.`
 	return nil
 }
 
-func updateFromV5(tx *sql.Tx) error {
+func updateFromV5(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 ALTER TABLE containers ADD COLUMN power_state INTEGER NOT NULL DEFAULT 0;
 ALTER TABLE containers ADD COLUMN ephemeral INTEGER NOT NULL DEFAULT 0;`
@@ -883,7 +883,7 @@ ALTER TABLE containers ADD COLUMN ephemeral INTEGER NOT NULL DEFAULT 0;`
 	return err
 }
 
-func updateFromV4(tx *sql.Tx) error {
+func updateFromV4(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE IF NOT EXISTS config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
@@ -922,14 +922,14 @@ CREATE TABLE IF NOT EXISTS config (
 	return nil
 }
 
-func updateFromV3(tx *sql.Tx) error {
+func updateFromV3(ctx context.Context, tx *sql.Tx) error {
 	// Attempt to create a default profile (but don't fail if already there)
 	_, _ = tx.Exec("INSERT INTO profiles (name) VALUES (\"default\");")
 
 	return nil
 }
 
-func updateFromV2(tx *sql.Tx) error {
+func updateFromV2(ctx context.Context, tx *sql.Tx) error {
 	stmt := `
 CREATE TABLE IF NOT EXISTS containers_devices (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
@@ -989,7 +989,7 @@ CREATE TABLE IF NOT EXISTS profiles_devices_config (
 	return err
 }
 
-func updateFromV1(tx *sql.Tx) error {
+func updateFromV1(ctx context.Context, tx *sql.Tx) error {
 	// v1..v2 adds images aliases
 	stmt := `
 CREATE TABLE IF NOT EXISTS images_aliases (
@@ -1004,7 +1004,7 @@ CREATE TABLE IF NOT EXISTS images_aliases (
 	return err
 }
 
-func updateFromV0(tx *sql.Tx) error {
+func updateFromV0(ctx context.Context, tx *sql.Tx) error {
 	// v0..v1 the dawn of containers
 	stmt := `
 CREATE TABLE certificates (
