@@ -20,8 +20,8 @@ var StorageRemoteDriverNames func() []string
 
 // GetStoragePoolsLocalConfig returns a map associating each storage pool name to
 // its node-specific config values (i.e. the ones where node_id is not NULL).
-func (c *ClusterTx) GetStoragePoolsLocalConfig() (map[string]map[string]string, error) {
-	names, err := query.SelectStrings(c.tx, "SELECT name FROM storage_pools")
+func (c *ClusterTx) GetStoragePoolsLocalConfig(ctx context.Context) (map[string]map[string]string, error) {
+	names, err := query.SelectStrings(ctx, c.tx, "SELECT name FROM storage_pools")
 	if err != nil {
 		return nil, err
 	}
@@ -31,8 +31,7 @@ func (c *ClusterTx) GetStoragePoolsLocalConfig() (map[string]map[string]string,
 		table := `
 storage_pools_config JOIN storage_pools ON storage_pools.id=storage_pools_config.storage_pool_id
 `
-		config, err := query.SelectConfig(
-			c.tx, table, "storage_pools.name=? AND storage_pools_config.node_id=?",
+		config, err := query.SelectConfig(ctx, c.tx, table, "storage_pools.name=? AND storage_pools_config.node_id=?",
 			name, c.nodeID)
 		if err != nil {
 			return nil, err
@@ -45,9 +44,9 @@ storage_pools_config JOIN storage_pools ON storage_pools.id=storage_pools_config
 }
 
 // GetStoragePoolID returns the ID of the pool with the given name.
-func (c *ClusterTx) GetStoragePoolID(name string) (int64, error) {
+func (c *ClusterTx) GetStoragePoolID(ctx context.Context, name string) (int64, error) {
 	stmt := "SELECT id FROM storage_pools WHERE name=?"
-	ids, err := query.SelectIntegers(c.tx, stmt, name)
+	ids, err := query.SelectIntegers(ctx, c.tx, stmt, name)
 	if err != nil {
 		return -1, err
 	}
@@ -58,14 +57,14 @@ func (c *ClusterTx) GetStoragePoolID(name string) (int64, error) {
 	case 1:
 		return int64(ids[0]), nil
 	default:
-		return -1, fmt.Errorf("more than one pool has the given name")
+		return -1, fmt.Errorf("More than one pool has the given name")
 	}
 }
 
 // GetStoragePoolDriver returns the driver of the pool with the given ID.
-func (c *ClusterTx) GetStoragePoolDriver(id int64) (string, error) {
+func (c *ClusterTx) GetStoragePoolDriver(ctx context.Context, id int64) (string, error) {
 	stmt := "SELECT driver FROM storage_pools WHERE id=?"
-	drivers, err := query.SelectStrings(c.tx, stmt, id)
+	drivers, err := query.SelectStrings(ctx, c.tx, stmt, id)
 	if err != nil {
 		return "", err
 	}
@@ -76,33 +75,32 @@ func (c *ClusterTx) GetStoragePoolDriver(id int64) (string, error) {
 	case 1:
 		return drivers[0], nil
 	default:
-		return "", fmt.Errorf("more than one pool has the given id")
+		return "", fmt.Errorf("More than one pool has the given id")
 	}
 }
 
 // GetNonPendingStoragePoolsNamesToIDs returns a map associating each storage pool name to its ID.
 //
 // Pending storage pools are skipped.
-func (c *ClusterTx) GetNonPendingStoragePoolsNamesToIDs() (map[string]int64, error) {
-	pools := []struct {
+func (c *ClusterTx) GetNonPendingStoragePoolsNamesToIDs(ctx context.Context) (map[string]int64, error) {
+	type pool struct {
 		id   int64
 		name string
-	}{}
-	dest := func(i int) []any {
-		pools = append(pools, struct {
-			id   int64
-			name string
-		}{})
-		return []any{&pools[i].id, &pools[i].name}
 	}
 
-	stmt, err := c.tx.Prepare("SELECT id, name FROM storage_pools WHERE NOT state=?")
-	if err != nil {
-		return nil, err
-	}
+	sql := "SELECT id, name FROM storage_pools WHERE NOT state=?"
+	pools := []pool{}
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		var p pool
+		err := scan(&p.id, &p.name)
+		if err != nil {
+			return err
+		}
 
-	defer func() { _ = stmt.Close() }()
-	err = query.SelectObjects(stmt, dest, storagePoolPending)
+		pools = append(pools, p)
+
+		return nil
+	}, storagePoolPending)
 	if err != nil {
 		return nil, err
 	}
@@ -134,11 +132,11 @@ func (c *ClusterTx) UpdateStoragePoolAfterNodeJoin(poolID, nodeID int64) error {
 
 // UpdateCephStoragePoolAfterNodeJoin updates internal state to reflect that nodeID is
 // joining a cluster where poolID is a ceph pool.
-func (c *ClusterTx) UpdateCephStoragePoolAfterNodeJoin(poolID, nodeID int64) error {
+func (c *ClusterTx) UpdateCephStoragePoolAfterNodeJoin(ctx context.Context, poolID int64, nodeID int64) error {
 	// Get the IDs of the other nodes (they should be all linked to
 	// the pool).
 	stmt := "SELECT node_id FROM storage_pools_nodes WHERE storage_pool_id=?"
-	nodeIDs, err := query.SelectIntegers(c.tx, stmt, poolID)
+	nodeIDs, err := query.SelectIntegers(ctx, c.tx, stmt, poolID)
 	if err != nil {
 		return fmt.Errorf("failed to fetch IDs of nodes with ceph pool: %w", err)
 	}
@@ -164,12 +162,12 @@ INSERT INTO storage_volumes(name, storage_pool_id, node_id, type, description, p
 SELECT id FROM storage_volumes WHERE storage_pool_id=? AND node_id=?
   ORDER BY name, type
 `
-	volumeIDs, err := query.SelectIntegers(c.tx, stmt, poolID, nodeID)
+	volumeIDs, err := query.SelectIntegers(ctx, c.tx, stmt, poolID, nodeID)
 	if err != nil {
 		return fmt.Errorf("failed to get joining node's ceph volume IDs: %w", err)
 	}
 
-	otherVolumeIDs, err := query.SelectIntegers(c.tx, stmt, poolID, otherNodeID)
+	otherVolumeIDs, err := query.SelectIntegers(ctx, c.tx, stmt, poolID, otherNodeID)
 	if err != nil {
 		return fmt.Errorf("failed to get other node's ceph volume IDs: %w", err)
 	}
@@ -180,8 +178,7 @@ SELECT id FROM storage_volumes WHERE storage_pool_id=? AND node_id=?
 
 	for i, otherVolumeID := range otherVolumeIDs {
 		volumeID := volumeIDs[i]
-		config, err := query.SelectConfig(
-			c.tx, "storage_volumes_config", "storage_volume_id=?", otherVolumeID)
+		config, err := query.SelectConfig(ctx, c.tx, "storage_volumes_config", "storage_volume_id=?", otherVolumeID)
 		if err != nil {
 			return fmt.Errorf("failed to get storage volume config: %w", err)
 		}
@@ -196,8 +193,7 @@ INSERT INTO storage_volumes_config(storage_volume_id, key, value) VALUES(?, ?, ?
 		}
 
 		// Copy volume snapshots as well.
-		otherSnapshotIDs, err := query.SelectIntegers(c.tx,
-			"SELECT id FROM storage_volumes_snapshots WHERE storage_volume_id = ?",
+		otherSnapshotIDs, err := query.SelectIntegers(ctx, c.tx, "SELECT id FROM storage_volumes_snapshots WHERE storage_volume_id = ?",
 			otherVolumeID)
 		if err != nil {
 			return err
@@ -210,7 +206,7 @@ INSERT INTO storage_volumes_config(storage_volume_id, key, value) VALUES(?, ?, ?
 				return fmt.Errorf("Increment storage volumes sequence: %w", err)
 			}
 
-			row := c.tx.QueryRow("SELECT seq FROM sqlite_sequence WHERE name = 'storage_volumes' LIMIT 1")
+			row := c.tx.QueryRowContext(ctx, "SELECT seq FROM sqlite_sequence WHERE name = 'storage_volumes' LIMIT 1")
 			err = row.Scan(&snapshotID)
 			if err != nil {
 				return fmt.Errorf("Fetch next storage volume ID: %w", err)
@@ -264,7 +260,7 @@ type StoragePoolNode struct {
 
 // CreatePendingStoragePool creates a new pending storage pool on the node with
 // the given name.
-func (c *ClusterTx) CreatePendingStoragePool(node, name, driver string, conf map[string]string) error {
+func (c *ClusterTx) CreatePendingStoragePool(ctx context.Context, node string, name string, driver string, conf map[string]string) error {
 	// First check if a storage pool with the given name exists, and, if
 	// so, that it has a matching driver and it's in the pending state.
 	pool := struct {
@@ -273,31 +269,22 @@ func (c *ClusterTx) CreatePendingStoragePool(node, name, driver string, conf map
 		state  StoragePoolState
 	}{}
 
-	var errConsistency error
-	dest := func(i int) []any {
+	sql := "SELECT id, driver, state FROM storage_pools WHERE name=?"
+	count := 0
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
 		// Ensure that there is at most one pool with the given name.
-		if i != 0 {
-			errConsistency = fmt.Errorf("more than one pool exists with the given name")
+		if count != 0 {
+			return fmt.Errorf("more than one pool exists with the given name")
 		}
 
-		return []any{&pool.id, &pool.driver, &pool.state}
-	}
+		count++
 
-	stmt, err := c.tx.Prepare("SELECT id, driver, state FROM storage_pools WHERE name=?")
+		return scan(&pool.id, &pool.driver, &pool.state)
+	}, name)
 	if err != nil {
 		return err
 	}
 
-	defer func() { _ = stmt.Close() }()
-	err = query.SelectObjects(stmt, dest, name)
-	if err != nil {
-		return err
-	}
-
-	if errConsistency != nil {
-		return errConsistency
-	}
-
 	var poolID = pool.id
 	if poolID == 0 {
 		// No existing pool with the given name was found, let's create
@@ -321,14 +308,13 @@ func (c *ClusterTx) CreatePendingStoragePool(node, name, driver string, conf map
 	}
 
 	// Get the ID of the node with the given name.
-	nodeInfo, err := c.GetNodeByName(node)
+	nodeInfo, err := c.GetNodeByName(ctx, node)
 	if err != nil {
 		return err
 	}
 
 	// Check that no storage_pool entry of this node and pool exists yet.
-	count, err := query.Count(
-		c.tx, "storage_pools_nodes", "storage_pool_id=? AND node_id=?", poolID, nodeInfo.ID)
+	count, err = query.Count(ctx, c.tx, "storage_pools_nodes", "storage_pool_id=? AND node_id=?", poolID, nodeInfo.ID)
 	if err != nil {
 		return err
 	}
@@ -383,25 +369,26 @@ func (c *ClusterTx) storagePoolState(name string, state StoragePoolState) error
 }
 
 // storagePoolNodes returns the nodes keyed by node ID that the given storage pool is defined on.
-func (c *ClusterTx) storagePoolNodes(poolID int64) (map[int64]StoragePoolNode, error) {
+func (c *ClusterTx) storagePoolNodes(ctx context.Context, poolID int64) (map[int64]StoragePoolNode, error) {
 	nodes := []StoragePoolNode{}
-	dest := func(i int) []any {
-		nodes = append(nodes, StoragePoolNode{})
-		return []any{&nodes[i].ID, &nodes[i].Name, &nodes[i].State}
-	}
-
-	stmt, err := c.tx.Prepare(`
+	sql := `
 		SELECT nodes.id, nodes.name, storage_pools_nodes.state FROM nodes
 		JOIN storage_pools_nodes ON storage_pools_nodes.node_id = nodes.id
 		WHERE storage_pools_nodes.storage_pool_id = ?
-	`)
-	if err != nil {
-		return nil, err
-	}
+	`
 
-	defer func() { _ = stmt.Close() }()
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		node := StoragePoolNode{}
 
-	err = query.SelectObjects(stmt, dest, poolID)
+		err := scan(&node.ID, &node.Name, &node.State)
+		if err != nil {
+			return err
+		}
+
+		nodes = append(nodes, node)
+
+		return nil
+	}, poolID)
 	if err != nil {
 		return nil, err
 	}
@@ -443,9 +430,9 @@ func (c *ClusterTx) storagePoolNodeState(poolID int64, state StoragePoolState) e
 // nodes grouped by node name, for the given poolID.
 //
 // If the storage pool is not defined on all nodes, an error is returned.
-func (c *ClusterTx) GetStoragePoolNodeConfigs(poolID int64) (map[string]map[string]string, error) {
+func (c *ClusterTx) GetStoragePoolNodeConfigs(ctx context.Context, poolID int64) (map[string]map[string]string, error) {
 	// Fetch all nodes.
-	nodes, err := c.GetNodes()
+	nodes, err := c.GetNodes(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -457,7 +444,7 @@ SELECT nodes.name FROM nodes
   LEFT JOIN storage_pools ON storage_pools_nodes.storage_pool_id = storage_pools.id
 WHERE storage_pools.id = ? AND storage_pools.state = ?
 `
-	defined, err := query.SelectStrings(c.tx, stmt, poolID, storagePoolPending)
+	defined, err := query.SelectStrings(ctx, c.tx, stmt, poolID, storagePoolPending)
 	if err != nil {
 		return nil, err
 	}
@@ -476,8 +463,7 @@ WHERE storage_pools.id = ? AND storage_pools.state = ?
 
 	configs := map[string]map[string]string{}
 	for _, node := range nodes {
-		config, err := query.SelectConfig(
-			c.tx, "storage_pools_config", "storage_pool_id=? AND node_id=?", poolID, node.ID)
+		config, err := query.SelectConfig(ctx, c.tx, "storage_pools_config", "storage_pool_id=? AND node_id=?", poolID, node.ID)
 		if err != nil {
 			return nil, err
 		}
@@ -607,19 +593,19 @@ func (c *Cluster) getStoragePool(onlyCreated bool, where string, args ...any) (i
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		var state StoragePoolState
 
-		err = tx.tx.QueryRow(q.String(), args...).Scan(&poolID, &pool.Name, &pool.Driver, &pool.Description, &state)
+		err = tx.tx.QueryRowContext(ctx, q.String(), args...).Scan(&poolID, &pool.Name, &pool.Driver, &pool.Description, &state)
 		if err != nil {
 			return err
 		}
 
 		pool.Status = StoragePoolStateToAPIStatus(state)
 
-		err = c.getStoragePoolConfig(tx, poolID, &pool)
+		err = c.getStoragePoolConfig(ctx, tx, poolID, &pool)
 		if err != nil {
 			return err
 		}
 
-		nodes, err = tx.storagePoolNodes(poolID)
+		nodes, err = tx.storagePoolNodes(ctx, poolID)
 		if err != nil {
 			return err
 		}
@@ -656,33 +642,13 @@ func StoragePoolStateToAPIStatus(state StoragePoolState) string {
 	}
 }
 
-// StoragePoolNodes returns the nodes keyed by node ID that the given storage pool is defined on.
-func (c *Cluster) StoragePoolNodes(poolID int64) (map[int64]StoragePoolNode, error) {
-	var nodes map[int64]StoragePoolNode
-	var err error
-
-	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		nodes, err = tx.storagePoolNodes(poolID)
-		if err != nil {
-			return err
-		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
-	}
-
-	return nodes, nil
-}
-
 // getStoragePoolConfig populates the config map of the Storage pool with the given ID.
-func (c *Cluster) getStoragePoolConfig(tx *ClusterTx, poolID int64, pool *api.StoragePool) error {
+func (c *Cluster) getStoragePoolConfig(ctx context.Context, tx *ClusterTx, poolID int64, pool *api.StoragePool) error {
 	q := "SELECT key, value FROM storage_pools_config WHERE storage_pool_id=? AND (node_id=? OR node_id IS NULL)"
 
 	pool.Config = map[string]string{}
 
-	return tx.QueryScan(q, func(scan func(dest ...any) error) error {
+	return query.Scan(ctx, tx.Tx(), q, func(scan func(dest ...any) error) error {
 		var key, value string
 
 		err := scan(&key, &value)
@@ -768,24 +734,6 @@ func storagePoolConfigAdd(tx *sql.Tx, poolID, nodeID int64, poolConfig map[strin
 	return nil
 }
 
-// StoragePoolDriver returns the driver of the pool with the given ID.
-func storagePoolDriverGet(tx *sql.Tx, id int64) (string, error) {
-	stmt := "SELECT driver FROM storage_pools WHERE id=?"
-	drivers, err := query.SelectStrings(tx, stmt, id)
-	if err != nil {
-		return "", err
-	}
-
-	switch len(drivers) {
-	case 0:
-		return "", api.StatusErrorf(http.StatusNotFound, "Storage pool not found")
-	case 1:
-		return drivers[0], nil
-	default:
-		return "", fmt.Errorf("more than one pool has the given ID")
-	}
-}
-
 // UpdateStoragePool updates a storage pool.
 func (c *Cluster) UpdateStoragePool(poolName, description string, poolConfig map[string]string) error {
 	poolID, _, _, err := c.GetStoragePoolInAnyState(poolName)
@@ -861,7 +809,7 @@ func (c *Cluster) IsRemoteStorage(poolID int64) (bool, error) {
 	isRemoteStorage := false
 
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		driver, err := tx.GetStoragePoolDriver(poolID)
+		driver, err := tx.GetStoragePoolDriver(ctx, poolID)
 		if err != nil {
 			return err
 		}
