@@ -18,7 +18,7 @@ import (
 )
 
 // GetStoragePoolVolumesWithType return a list of all volumes of the given type.
-func (c *ClusterTx) GetStoragePoolVolumesWithType(volumeType int) ([]StorageVolumeArgs, error) {
+func (c *ClusterTx) GetStoragePoolVolumesWithType(ctx context.Context, volumeType int) ([]StorageVolumeArgs, error) {
 	stmt := `
 SELECT storage_volumes.id, storage_volumes.name, storage_volumes.description, storage_pools.name, projects.name, IFNULL(storage_volumes.node_id, -1)
 FROM storage_volumes
@@ -28,7 +28,7 @@ WHERE storage_volumes.type = ?
 `
 
 	result := []StorageVolumeArgs{}
-	err := c.QueryScan(stmt, func(scan func(dest ...any) error) error {
+	err := query.Scan(ctx, c.Tx(), stmt, func(scan func(dest ...any) error) error {
 		entry := StorageVolumeArgs{}
 
 		err := scan(&entry.ID, &entry.Name, &entry.Description, &entry.PoolName, &entry.ProjectName, &entry.NodeID)
@@ -44,7 +44,7 @@ WHERE storage_volumes.type = ?
 	}
 
 	for i := range result {
-		result[i].Config, err = c.storageVolumeConfigGet(result[i].ID, false)
+		result[i].Config, err = c.storageVolumeConfigGet(ctx, result[i].ID, false)
 		if err != nil {
 			return nil, err
 		}
@@ -105,7 +105,7 @@ type StorageVolume struct {
 // If there are no volumes, it returns an empty list and no error.
 // Accepts filters for narrowing down the results returned. If memberSpecific is true, then the search is
 // restricted to volumes that belong to this member or belong to all members.
-func (c *ClusterTx) GetStoragePoolVolumes(poolID int64, memberSpecific bool, filters ...StorageVolumeFilter) ([]*StorageVolume, error) {
+func (c *ClusterTx) GetStoragePoolVolumes(ctx context.Context, poolID int64, memberSpecific bool, filters ...StorageVolumeFilter) ([]*StorageVolume, error) {
 	var q *strings.Builder = &strings.Builder{}
 	args := []any{poolID}
 
@@ -176,7 +176,7 @@ func (c *ClusterTx) GetStoragePoolVolumes(poolID int64, memberSpecific bool, fil
 	var err error
 	var volumes []*StorageVolume
 
-	err = c.QueryScan(q.String(), func(scan func(dest ...any) error) error {
+	err = query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
 		var volumeType int = int(-1)
 		var contentType int = int(-1)
 		var vol StorageVolume
@@ -206,7 +206,7 @@ func (c *ClusterTx) GetStoragePoolVolumes(poolID int64, memberSpecific bool, fil
 
 	// Populate config.
 	for _, volume := range volumes {
-		volume.Config, err = c.storageVolumeConfigGet(volume.ID, shared.IsSnapshot(volume.Name))
+		volume.Config, err = c.storageVolumeConfigGet(ctx, volume.ID, shared.IsSnapshot(volume.Name))
 		if err != nil {
 			return nil, fmt.Errorf("Failed loading volume config for %q: %w", volume.Name, err)
 		}
@@ -216,14 +216,14 @@ func (c *ClusterTx) GetStoragePoolVolumes(poolID int64, memberSpecific bool, fil
 }
 
 // GetStoragePoolVolume returns the storage volume attached to a given storage pool.
-func (c *ClusterTx) GetStoragePoolVolume(poolID int64, projectName string, volumeType int, volumeName string, memberSpecific bool) (*StorageVolume, error) {
+func (c *ClusterTx) GetStoragePoolVolume(ctx context.Context, poolID int64, projectName string, volumeType int, volumeName string, memberSpecific bool) (*StorageVolume, error) {
 	filters := []StorageVolumeFilter{{
 		Project: &projectName,
 		Type:    &volumeType,
 		Name:    &volumeName,
 	}}
 
-	volumes, err := c.GetStoragePoolVolumes(poolID, memberSpecific, filters...)
+	volumes, err := c.GetStoragePoolVolumes(ctx, poolID, memberSpecific, filters...)
 	volumesLen := len(volumes)
 	if (err == nil && volumesLen <= 0) || errors.Is(err, sql.ErrNoRows) {
 		return nil, api.StatusErrorf(http.StatusNotFound, "Storage volume not found")
@@ -236,78 +236,6 @@ func (c *ClusterTx) GetStoragePoolVolume(poolID int64, projectName string, volum
 	return volumes[0], nil
 }
 
-// GetLocalStoragePoolVolumes returns all storage volumes attached to a given
-// storage pool on the current node. If there are no volumes, it returns an
-// empty list as well as a api.StatusError with code set to http.StatusNotFound.
-func (c *Cluster) GetLocalStoragePoolVolumes(project string, poolID int64, volumeTypes []int) ([]*api.StorageVolume, error) {
-	return c.storagePoolVolumesGet(project, poolID, c.nodeID, volumeTypes)
-}
-
-// Returns all storage volumes attached to a given storage pool on the given
-// node. If there are no volumes, it returns an empty list as well as a
-// api.StatusError with code set to http.StatusNotFound.
-func (c *Cluster) storagePoolVolumesGet(project string, poolID, nodeID int64, volumeTypes []int) ([]*api.StorageVolume, error) {
-	// Get all storage volumes of all types attached to a given storage pool.
-	result := []*api.StorageVolume{}
-	for _, volumeType := range volumeTypes {
-		volumeNames, err := c.storagePoolVolumesGetType(project, volumeType, poolID, nodeID)
-		if err != nil && err != sql.ErrNoRows {
-			return nil, fmt.Errorf("Failed to fetch volume types: %w", err)
-		}
-
-		for _, volumeName := range volumeNames {
-			_, volume, err := c.storagePoolVolumeGetType(project, volumeName, volumeType, poolID, nodeID)
-			if err != nil {
-				return nil, fmt.Errorf("Failed to fetch volume type: %w", err)
-			}
-
-			result = append(result, volume)
-		}
-	}
-
-	if len(result) == 0 {
-		return result, api.StatusErrorf(http.StatusNotFound, "Storage pool volume(s) not found")
-	}
-
-	return result, nil
-}
-
-// Get all storage volumes attached to a given storage pool of a given volume
-// type, on the given node.
-func (c *Cluster) storagePoolVolumesGetType(project string, volumeType int, poolID, nodeID int64) ([]string, error) {
-	var poolName string
-
-	remoteDrivers := StorageRemoteDriverNames()
-
-	query := fmt.Sprintf(`
-SELECT storage_volumes_all.name
-  FROM storage_volumes_all
-  JOIN projects ON projects.id=storage_volumes_all.project_id
-  join storage_pools ON storage_pools.id=storage_volumes_all.storage_pool_id
- WHERE projects.name=?
-   AND storage_volumes_all.storage_pool_id=?
-   AND storage_volumes_all.type=?
-   AND (storage_volumes_all.node_id=? OR storage_volumes_all.node_id IS NULL AND storage_pools.driver IN %s)`, query.Params(len(remoteDrivers)))
-	inargs := []any{project, poolID, volumeType, nodeID}
-	outargs := []any{poolName}
-
-	for _, driver := range remoteDrivers {
-		inargs = append(inargs, driver)
-	}
-
-	result, err := queryScan(c, query, inargs, outargs)
-	if err != nil {
-		return []string{}, err
-	}
-
-	response := []string{}
-	for _, r := range result {
-		response = append(response, r[0].(string))
-	}
-
-	return response, nil
-}
-
 // GetLocalStoragePoolVolumeSnapshotsWithType get all snapshots of a storage volume
 // attached to a given storage pool of a given volume type, on the local member.
 // Returns snapshots slice ordered by when they were created, oldest first.
@@ -318,7 +246,7 @@ func (c *Cluster) GetLocalStoragePoolVolumeSnapshotsWithType(projectName string,
 	// will be returned in the order that the snapshots were created. This is specifically used
 	// during migration to ensure that the storage engines can re-create snapshots using the
 	// correct deltas.
-	query := fmt.Sprintf(`
+	queryStr := fmt.Sprintf(`
   SELECT
     storage_volumes_snapshots.id, storage_volumes_snapshots.name, storage_volumes_snapshots.description, storage_volumes_snapshots.expiry_date,
     storage_volumes.content_type
@@ -342,7 +270,7 @@ func (c *Cluster) GetLocalStoragePoolVolumeSnapshotsWithType(projectName string,
 	var snapshots []StorageVolumeArgs
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		err = tx.QueryScan(query, func(scan func(dest ...any) error) error {
+		err = query.Scan(ctx, tx.Tx(), queryStr, func(scan func(dest ...any) error) error {
 			var s StorageVolumeArgs
 			var snapName string
 			var expiryDate sql.NullTime
@@ -374,7 +302,7 @@ func (c *Cluster) GetLocalStoragePoolVolumeSnapshotsWithType(projectName string,
 
 		// Populate config.
 		for i := range snapshots {
-			err := storageVolumeSnapshotConfig(tx, snapshots[i].ID, &snapshots[i])
+			err := storageVolumeSnapshotConfig(ctx, tx, snapshots[i].ID, &snapshots[i])
 			if err != nil {
 				return err
 			}
@@ -390,11 +318,11 @@ func (c *Cluster) GetLocalStoragePoolVolumeSnapshotsWithType(projectName string,
 }
 
 // storageVolumeSnapshotConfig populates the config map of the Storage Volume Snapshot with the given ID.
-func storageVolumeSnapshotConfig(tx *ClusterTx, volumeSnapshotID int64, volume *StorageVolumeArgs) error {
+func storageVolumeSnapshotConfig(ctx context.Context, tx *ClusterTx, volumeSnapshotID int64, volume *StorageVolumeArgs) error {
 	q := "SELECT key, value FROM storage_volumes_snapshots_config WHERE storage_volume_snapshot_id = ?"
 
 	volume.Config = make(map[string]string)
-	return tx.QueryScan(q, func(scan func(dest ...any) error) error {
+	return query.Scan(ctx, tx.Tx(), q, func(scan func(dest ...any) error) error {
 		var key, value string
 
 		err := scan(&key, &value)
@@ -413,103 +341,29 @@ func storageVolumeSnapshotConfig(tx *ClusterTx, volumeSnapshotID int64, volume *
 	}, volumeSnapshotID)
 }
 
-// GetLocalStoragePoolVolumesWithType returns all storage volumes attached to a
-// given storage pool of a given volume type, on the current node.
-func (c *Cluster) GetLocalStoragePoolVolumesWithType(projectName string, volumeType int, poolID int64) ([]string, error) {
-	return c.storagePoolVolumesGetType(projectName, volumeType, poolID, c.nodeID)
-}
-
-// Return a single storage volume attached to a given storage pool of a given
-// type, on the node with the given ID.
-func (c *Cluster) storagePoolVolumeGetType(project string, volumeName string, volumeType int, poolID, nodeID int64) (int64, *api.StorageVolume, error) {
-	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
-
-	volumeID, err := c.storagePoolVolumeGetTypeID(project, volumeName, volumeType, poolID, nodeID)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	isRemoteStorage, err := c.IsRemoteStorage(poolID)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	volumeNode := ""
-
-	if !isRemoteStorage {
-		volumeNode, err = c.storageVolumeNodeGet(volumeID)
-		if err != nil {
-			return -1, nil, err
-		}
-	}
-
-	volumeConfig, err := c.storageVolumeConfigGet(volumeID, isSnapshot)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	volumeDescription, err := c.GetStorageVolumeDescription(volumeID)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	volumeContentType, err := c.getStorageVolumeContentType(volumeID)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	volumeTypeName, err := storagePoolVolumeTypeToName(volumeType)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	volumeContentTypeName, err := storagePoolVolumeContentTypeToName(volumeContentType)
-	if err != nil {
-		return -1, nil, err
-	}
-
-	storageVolume := api.StorageVolume{
-		Type: volumeTypeName,
-	}
-
-	storageVolume.Name = volumeName
-	storageVolume.Description = volumeDescription
-	storageVolume.Config = volumeConfig
-	storageVolume.Location = volumeNode
-	storageVolume.ContentType = volumeContentTypeName
-
-	return volumeID, &storageVolume, nil
-}
-
-// GetLocalStoragePoolVolume gets a single storage volume attached to a
-// given storage pool of a given type, on the current node in the given project.
-func (c *Cluster) GetLocalStoragePoolVolume(project, volumeName string, volumeType int, poolID int64) (int64, *api.StorageVolume, error) {
-	return c.storagePoolVolumeGetType(project, volumeName, volumeType, poolID, c.nodeID)
-}
-
 // UpdateStoragePoolVolume updates the storage volume attached to a given storage pool.
-func (c *Cluster) UpdateStoragePoolVolume(project, volumeName string, volumeType int, poolID int64, volumeDescription string, volumeConfig map[string]string) error {
-	volumeID, _, err := c.GetLocalStoragePoolVolume(project, volumeName, volumeType, poolID)
-	if err != nil {
-		return err
-	}
+func (c *Cluster) UpdateStoragePoolVolume(projectName string, volumeName string, volumeType int, poolID int64, volumeDescription string, volumeConfig map[string]string) error {
+	var err error
 
 	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		err = storagePoolVolumeReplicateIfCeph(tx.tx, volumeID, project, volumeName, volumeType, poolID, func(volumeID int64) error {
-			err = storageVolumeConfigClear(tx.tx, volumeID, isSnapshot)
-			if err != nil {
-				return err
-			}
+		volume, err := tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
+		if err != nil {
+			return err
+		}
 
-			err = storageVolumeConfigAdd(tx.tx, volumeID, volumeConfig, isSnapshot)
-			if err != nil {
-				return err
-			}
+		err = storageVolumeConfigClear(tx.tx, volume.ID, isSnapshot)
+		if err != nil {
+			return err
+		}
 
-			return storageVolumeDescriptionUpdate(tx.tx, volumeID, volumeDescription, isSnapshot)
-		})
+		err = storageVolumeConfigAdd(tx.tx, volume.ID, volumeConfig, isSnapshot)
+		if err != nil {
+			return err
+		}
+
+		err = storageVolumeDescriptionUpdate(tx.tx, volume.ID, volumeDescription, isSnapshot)
 		if err != nil {
 			return err
 		}
@@ -522,11 +376,8 @@ func (c *Cluster) UpdateStoragePoolVolume(project, volumeName string, volumeType
 
 // RemoveStoragePoolVolume deletes the storage volume attached to a given storage
 // pool.
-func (c *Cluster) RemoveStoragePoolVolume(project, volumeName string, volumeType int, poolID int64) error {
-	volumeID, _, err := c.GetLocalStoragePoolVolume(project, volumeName, volumeType, poolID)
-	if err != nil {
-		return err
-	}
+func (c *Cluster) RemoveStoragePoolVolume(projectName string, volumeName string, volumeType int, poolID int64) error {
+	var err error
 
 	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
 	var stmt string
@@ -537,22 +388,25 @@ func (c *Cluster) RemoveStoragePoolVolume(project, volumeName string, volumeType
 	}
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		err := storagePoolVolumeReplicateIfCeph(tx.tx, volumeID, project, volumeName, volumeType, poolID, func(volumeID int64) error {
-			_, err := tx.tx.Exec(stmt, volumeID)
+		volume, err := tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
+		if err != nil {
 			return err
-		})
-		return err
+		}
+
+		_, err = tx.tx.Exec(stmt, volume.ID)
+		if err != nil {
+			return err
+		}
+
+		return nil
 	})
 
 	return err
 }
 
 // RenameStoragePoolVolume renames the storage volume attached to a given storage pool.
-func (c *Cluster) RenameStoragePoolVolume(project, oldVolumeName string, newVolumeName string, volumeType int, poolID int64) error {
-	volumeID, _, err := c.GetLocalStoragePoolVolume(project, oldVolumeName, volumeType, poolID)
-	if err != nil {
-		return err
-	}
+func (c *Cluster) RenameStoragePoolVolume(projectName string, oldVolumeName string, newVolumeName string, volumeType int, poolID int64) error {
+	var err error
 
 	isSnapshot := strings.Contains(oldVolumeName, shared.SnapshotDelimiter)
 	var stmt string
@@ -565,50 +419,24 @@ func (c *Cluster) RenameStoragePoolVolume(project, oldVolumeName string, newVolu
 	}
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		err := storagePoolVolumeReplicateIfCeph(tx.tx, volumeID, project, oldVolumeName, volumeType, poolID, func(volumeID int64) error {
-			_, err := tx.tx.Exec(stmt, newVolumeName, volumeID)
+		volume, err := tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, oldVolumeName, true)
+		if err != nil {
 			return err
-		})
-		return err
+		}
+
+		_, err = tx.tx.Exec(stmt, newVolumeName, volume.ID)
+		if err != nil {
+			return err
+		}
+
+		return nil
 	})
 
 	return err
 }
 
-// This a convenience to replicate a certain volume change to all nodes if the
-// underlying driver is ceph.
-func storagePoolVolumeReplicateIfCeph(tx *sql.Tx, volumeID int64, project, volumeName string, volumeType int, poolID int64, f func(int64) error) error {
-	driver, err := storagePoolDriverGet(tx, poolID)
-	if err != nil {
-		return err
-	}
-
-	volumeIDs := []int64{volumeID}
-
-	remoteDrivers := StorageRemoteDriverNames()
-
-	// If this is a ceph volume, we want to duplicate the change across the
-	// the rows for all other nodes.
-	if shared.StringInSlice(driver, remoteDrivers) {
-		volumeIDs, err = storageVolumeIDsGet(tx, project, volumeName, volumeType, poolID)
-		if err != nil {
-			return err
-		}
-	}
-
-	for _, volumeID := range volumeIDs {
-		err := f(volumeID)
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
-// CreateStoragePoolVolume creates a new storage volume attached to a given
-// storage pool.
-func (c *Cluster) CreateStoragePoolVolume(project, volumeName, volumeDescription string, volumeType int, poolID int64, volumeConfig map[string]string, contentType int) (int64, error) {
+// CreateStoragePoolVolume creates a new storage volume attached to a given storage pool.
+func (c *Cluster) CreateStoragePoolVolume(projectName string, volumeName string, volumeDescription string, volumeType int, poolID int64, volumeConfig map[string]string, contentType int) (int64, error) {
 	var volumeID int64
 
 	if shared.IsSnapshot(volumeName) {
@@ -618,7 +446,7 @@ func (c *Cluster) CreateStoragePoolVolume(project, volumeName, volumeDescription
 	remoteDrivers := StorageRemoteDriverNames()
 
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		driver, err := tx.GetStoragePoolDriver(poolID)
+		driver, err := tx.GetStoragePoolDriver(ctx, poolID)
 		if err != nil {
 			return err
 		}
@@ -630,13 +458,13 @@ func (c *Cluster) CreateStoragePoolVolume(project, volumeName, volumeDescription
 INSERT INTO storage_volumes (storage_pool_id, type, name, description, project_id, content_type)
  VALUES (?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?)
 `,
-				poolID, volumeType, volumeName, volumeDescription, project, contentType)
+				poolID, volumeType, volumeName, volumeDescription, projectName, contentType)
 		} else {
 			result, err = tx.tx.Exec(`
 INSERT INTO storage_volumes (storage_pool_id, node_id, type, name, description, project_id, content_type)
  VALUES (?, ?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?)
 `,
-				poolID, c.nodeID, volumeType, volumeName, volumeDescription, project, contentType)
+				poolID, c.nodeID, volumeType, volumeName, volumeDescription, projectName, contentType)
 		}
 
 		if err != nil {
@@ -650,7 +478,7 @@ INSERT INTO storage_volumes (storage_pool_id, node_id, type, name, description,
 
 		err = storageVolumeConfigAdd(tx.tx, volumeID, volumeConfig, false)
 		if err != nil {
-			return fmt.Errorf("Insert storage volume configuration: %w", err)
+			return fmt.Errorf("Failed inserting storage volume record configuration: %w", err)
 		}
 
 		return nil
@@ -668,7 +496,7 @@ func (c *Cluster) storagePoolVolumeGetTypeID(project string, volumeName string,
 	var id int64
 	err := c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		var err error
-		id, err = tx.storagePoolVolumeGetTypeID(project, volumeName, volumeType, poolID, nodeID)
+		id, err = tx.storagePoolVolumeGetTypeID(ctx, project, volumeName, volumeType, poolID, nodeID)
 		return err
 	})
 	if err != nil {
@@ -678,7 +506,7 @@ func (c *Cluster) storagePoolVolumeGetTypeID(project string, volumeName string,
 	return id, nil
 }
 
-func (c *ClusterTx) storagePoolVolumeGetTypeID(project string, volumeName string, volumeType int, poolID, nodeID int64) (int64, error) {
+func (c *ClusterTx) storagePoolVolumeGetTypeID(ctx context.Context, project string, volumeName string, volumeType int, poolID, nodeID int64) (int64, error) {
 	remoteDrivers := StorageRemoteDriverNames()
 
 	s := fmt.Sprintf(`
@@ -698,7 +526,7 @@ SELECT storage_volumes_all.id
 		args = append(args, driver)
 	}
 
-	result, err := query.SelectIntegers(c.tx, s, args...)
+	result, err := query.SelectIntegers(ctx, c.tx, s, args...)
 	if err != nil {
 		return -1, err
 	}
@@ -789,14 +617,9 @@ type StorageVolumeArgs struct {
 // The volume name can be either a regular name or a volume snapshot name.
 // If the volume is defined, but without a specific node, then the ErrNoClusterMember error is returned.
 // If the volume is not found then an api.StatusError with code set to http.StatusNotFound is returned.
-func (c *ClusterTx) GetStorageVolumeNodes(poolID int64, projectName string, volumeName string, volumeType int) ([]NodeInfo, error) {
+func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, projectName string, volumeName string, volumeType int) ([]NodeInfo, error) {
 	nodes := []NodeInfo{}
 
-	dest := func(i int) []any {
-		nodes = append(nodes, NodeInfo{})
-		return []any{&nodes[i].ID, &nodes[i].Address, &nodes[i].Name}
-	}
-
 	sql := `
 	SELECT coalesce(nodes.id,0) AS nodeID, coalesce(nodes.address,"") AS nodeAddress, coalesce(nodes.name,"") AS nodeName
 	FROM storage_volumes_all
@@ -807,13 +630,18 @@ func (c *ClusterTx) GetStorageVolumeNodes(poolID int64, projectName string, volu
 		AND storage_volumes_all.name=?
 		AND storage_volumes_all.type=?
 `
-	stmt, err := c.tx.Prepare(sql)
-	if err != nil {
-		return nil, err
-	}
 
-	defer func() { _ = stmt.Close() }()
-	err = query.SelectObjects(stmt, dest, poolID, projectName, volumeName, volumeType)
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		node := NodeInfo{}
+		err := scan(&node.ID, &node.Address, &node.Name)
+		if err != nil {
+			return err
+		}
+
+		nodes = append(nodes, node)
+
+		return nil
+	}, poolID, projectName, volumeName, volumeType)
 	if err != nil {
 		return nil, err
 	}
@@ -829,7 +657,7 @@ func (c *ClusterTx) GetStorageVolumeNodes(poolID int64, projectName string, volu
 	if nodeCount == 0 {
 		return nil, api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
 	} else if nodeCount > 1 {
-		driver, err := c.GetStoragePoolDriver(poolID)
+		driver, err := c.GetStoragePoolDriver(ctx, poolID)
 		if err != nil {
 			return nil, err
 		}
@@ -852,36 +680,13 @@ func (c *ClusterTx) GetStorageVolumeNodes(poolID int64, projectName string, volu
 	return nodes, nil
 }
 
-// Return the name of the node a storage volume is on.
-func (c *Cluster) storageVolumeNodeGet(volumeID int64) (string, error) {
-	name := ""
-	query := `
-SELECT nodes.name FROM storage_volumes_all
-  JOIN nodes ON nodes.id=storage_volumes_all.node_id
-   WHERE storage_volumes_all.id=?
-`
-	inargs := []any{volumeID}
-	outargs := []any{&name}
-
-	err := dbQueryRowScan(c, query, inargs, outargs)
-	if err != nil {
-		if err == sql.ErrNoRows {
-			return "", api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
-		}
-
-		return "", err
-	}
-
-	return name, nil
-}
-
 // Get the config of a storage volume.
 func (c *Cluster) storageVolumeConfigGet(volumeID int64, isSnapshot bool) (map[string]string, error) {
 	var err error
 	var result map[string]string
 
 	err = c.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
-		result, err = tx.storageVolumeConfigGet(volumeID, isSnapshot)
+		result, err = tx.storageVolumeConfigGet(ctx, volumeID, isSnapshot)
 		return err
 	})
 	if err != nil {
@@ -892,16 +697,16 @@ func (c *Cluster) storageVolumeConfigGet(volumeID int64, isSnapshot bool) (map[s
 }
 
 // Get the config of a storage volume.
-func (c *ClusterTx) storageVolumeConfigGet(volumeID int64, isSnapshot bool) (map[string]string, error) {
-	var query string
+func (c *ClusterTx) storageVolumeConfigGet(ctx context.Context, volumeID int64, isSnapshot bool) (map[string]string, error) {
+	var queryStr string
 	if isSnapshot {
-		query = "SELECT key, value FROM storage_volumes_snapshots_config WHERE storage_volume_snapshot_id=?"
+		queryStr = "SELECT key, value FROM storage_volumes_snapshots_config WHERE storage_volume_snapshot_id=?"
 	} else {
-		query = "SELECT key, value FROM storage_volumes_config WHERE storage_volume_id=?"
+		queryStr = "SELECT key, value FROM storage_volumes_config WHERE storage_volume_id=?"
 	}
 
 	config := map[string]string{}
-	err := c.QueryScan(query, func(scan func(dest ...any) error) error {
+	err := query.Scan(ctx, c.Tx(), queryStr, func(scan func(dest ...any) error) error {
 		var key string
 		var value string
 
@@ -921,44 +726,6 @@ func (c *ClusterTx) storageVolumeConfigGet(volumeID int64, isSnapshot bool) (map
 	return config, nil
 }
 
-// GetStorageVolumeDescription gets the description of a storage volume.
-func (c *Cluster) GetStorageVolumeDescription(volumeID int64) (string, error) {
-	var description string
-	query := "SELECT description FROM storage_volumes_all WHERE id=?"
-	inargs := []any{volumeID}
-	outargs := []any{&description}
-
-	err := dbQueryRowScan(c, query, inargs, outargs)
-	if err != nil {
-		if err == sql.ErrNoRows {
-			return "", api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
-		}
-
-		return "", err
-	}
-
-	return description, nil
-}
-
-// getStorageVolumeContentType gets the content type of a storage volume.
-func (c *Cluster) getStorageVolumeContentType(volumeID int64) (int, error) {
-	var contentType int
-	query := "SELECT content_type FROM storage_volumes_all WHERE id=?"
-	inargs := []any{volumeID}
-	outargs := []any{&contentType}
-
-	err := dbQueryRowScan(c, query, inargs, outargs)
-	if err != nil {
-		if err == sql.ErrNoRows {
-			return -1, api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
-		}
-
-		return -1, err
-	}
-
-	return contentType, nil
-}
-
 // GetNextStorageVolumeSnapshotIndex returns the index of the next snapshot of the storage
 // volume with the given name should have.
 //
@@ -1070,30 +837,6 @@ func storageVolumeConfigClear(tx *sql.Tx, volumeID int64, isSnapshot bool) error
 	return nil
 }
 
-// Get the IDs of all volumes with the given name and type associated with the
-// given pool, regardless of their node_id column.
-func storageVolumeIDsGet(tx *sql.Tx, project, volumeName string, volumeType int, poolID int64) ([]int64, error) {
-	ids, err := query.SelectIntegers(tx, `
-SELECT storage_volumes_all.id
-  FROM storage_volumes_all
-  JOIN projects ON projects.id = storage_volumes_all.project_id
- WHERE projects.name=?
-   AND storage_volumes_all.name=?
-   AND storage_volumes_all.type=?
-   AND storage_volumes_all.storage_pool_id=?
-`, project, volumeName, volumeType, poolID)
-	if err != nil {
-		return nil, err
-	}
-
-	ids64 := make([]int64, len(ids))
-	for i, id := range ids {
-		ids64[i] = int64(id)
-	}
-
-	return ids64, nil
-}
-
 // RemoveStorageVolumeImages removes the volumes associated with the images
 // with the given fingerprints.
 func (c *Cluster) RemoveStorageVolumeImages(fingerprints []string) error {
@@ -1138,7 +881,7 @@ func storagePoolVolumeContentTypeToName(contentType int) (string, error) {
 }
 
 // GetCustomVolumesInProject returns all custom volumes in the given project.
-func (c *ClusterTx) GetCustomVolumesInProject(project string) ([]StorageVolumeArgs, error) {
+func (c *ClusterTx) GetCustomVolumesInProject(ctx context.Context, project string) ([]StorageVolumeArgs, error) {
 	sql := `
 SELECT storage_volumes.id, storage_volumes.name, storage_pools.name, IFNULL(storage_volumes.node_id, -1)
 FROM storage_volumes
@@ -1146,31 +889,25 @@ JOIN storage_pools ON storage_pools.id = storage_volumes.storage_pool_id
 JOIN projects ON projects.id = storage_volumes.project_id
 WHERE storage_volumes.type = ? AND projects.name = ?
 `
-	stmt, err := c.tx.Prepare(sql)
-	if err != nil {
-		return nil, err
-	}
-
-	defer func() { _ = stmt.Close() }()
 
 	volumes := []StorageVolumeArgs{}
-	dest := func(i int) []any {
-		volumes = append(volumes, StorageVolumeArgs{})
-		return []any{
-			&volumes[i].ID,
-			&volumes[i].Name,
-			&volumes[i].PoolName,
-			&volumes[i].NodeID,
+	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
+		volume := StorageVolumeArgs{}
+		err := scan(&volume.ID, &volume.Name, &volume.PoolName, &volume.NodeID)
+		if err != nil {
+			return err
 		}
-	}
 
-	err = query.SelectObjects(stmt, dest, StoragePoolVolumeTypeCustom, project)
+		volumes = append(volumes, volume)
+
+		return nil
+	}, StoragePoolVolumeTypeCustom, project)
 	if err != nil {
 		return nil, fmt.Errorf("Fetch custom volumes: %w", err)
 	}
 
 	for i, volume := range volumes {
-		config, err := query.SelectConfig(c.tx, "storage_volumes_config", "storage_volume_id=?", volume.ID)
+		config, err := query.SelectConfig(ctx, c.tx, "storage_volumes_config", "storage_volume_id=?", volume.ID)
 		if err != nil {
 			return nil, fmt.Errorf("Fetch custom volume config: %w", err)
 		}
@@ -1183,8 +920,8 @@ WHERE storage_volumes.type = ? AND projects.name = ?
 
 // GetStorageVolumeURIs returns the URIs of the storage volumes, specifying
 // target node if applicable.
-func (c *ClusterTx) GetStorageVolumeURIs(project string) ([]string, error) {
-	volInfo, err := c.GetCustomVolumesInProject(project)
+func (c *ClusterTx) GetStorageVolumeURIs(ctx context.Context, project string) ([]string, error) {
+	volInfo, err := c.GetCustomVolumesInProject(ctx, project)
 	if err != nil {
 		return nil, err
 	}
@@ -1195,7 +932,7 @@ func (c *ClusterTx) GetStorageVolumeURIs(project string) ([]string, error) {
 
 		// Skip checking nodes if node_id is NULL.
 		if info.NodeID != -1 {
-			nodeInfo, err := c.GetNodes()
+			nodeInfo, err := c.GetNodes(ctx)
 			if err != nil {
 				return nil, err
 			}
