@@ -5,7 +5,6 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"os/exec"
@@ -53,6 +52,9 @@ const DiskFileDescriptorMountPrefix = "fd"
 // DiskDirectIO is used to indicate disk should use direct I/O.
 const DiskDirectIO = "directio"
 
+// DiskIOUring is used to indicate disk should use io_uring if the system supports it.
+const DiskIOUring = "io_uring"
+
 // DiskLoopBacked is used to indicate disk is backed onto a loop device.
 const DiskLoopBacked = "loop"
 
@@ -269,19 +271,23 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 			// Custom volume validation.
 			if d.config["source"] != "" && d.config["path"] != "/" {
 				// Derive the effective storage project name from the instance config's project.
-				storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, instConf.Project(), db.StoragePoolVolumeTypeCustom)
+				storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, instConf.Project().Name, db.StoragePoolVolumeTypeCustom)
 				if err != nil {
 					return err
 				}
 
-				// GetLocalStoragePoolVolume returns a volume with an empty Location field for remote drivers.
-				_, vol, err := d.state.DB.Cluster.GetLocalStoragePoolVolume(storageProjectName, d.config["source"], db.StoragePoolVolumeTypeCustom, d.pool.ID())
+				// GetStoragePoolVolume returns a volume with an empty Location field for remote drivers.
+				var dbVolume *db.StorageVolume
+				err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+					dbVolume, err = tx.GetStoragePoolVolume(ctx, d.pool.ID(), storageProjectName, db.StoragePoolVolumeTypeCustom, d.config["source"], true)
+					return err
+				})
 				if err != nil {
 					return fmt.Errorf("Failed loading custom volume: %w", err)
 				}
 
 				// Check storage volume is available to mount on this cluster member.
-				remoteInstance, err := storagePools.VolumeUsedByExclusiveRemoteInstancesWithProfiles(d.state, d.config["pool"], storageProjectName, vol)
+				remoteInstance, err := storagePools.VolumeUsedByExclusiveRemoteInstancesWithProfiles(d.state, d.config["pool"], storageProjectName, &dbVolume.StorageVolume)
 				if err != nil {
 					return fmt.Errorf("Failed checking if custom volume is exclusively attached to another instance: %w", err)
 				}
@@ -291,7 +297,7 @@ func (d *disk) validateConfig(instConf instance.ConfigReader) error {
 				}
 
 				// Check that block volumes are *only* attached to VM instances.
-				contentType, err := storagePools.VolumeContentTypeNameToContentType(vol.ContentType)
+				contentType, err := storagePools.VolumeContentTypeNameToContentType(dbVolume.ContentType)
 				if err != nil {
 					return err
 				}
@@ -343,27 +349,12 @@ func (d *disk) validateEnvironmentSourcePath() error {
 
 	// If project not default then check if using restricted disk paths.
 	// Default project cannot be restricted, so don't bother loading the project config in that case.
-	projectName := d.inst.Project()
-	if projectName != project.Default {
-		var p *api.Project
-		err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			project, err := cluster.GetProject(ctx, tx.Tx(), projectName)
-			if err != nil {
-				return err
-			}
-
-			p, err = project.ToAPI(ctx, tx.Tx())
-
-			return err
-		})
-		if err != nil {
-			return fmt.Errorf("Failed loading project %q: %w", projectName, err)
-		}
-
+	instProject := d.inst.Project()
+	if instProject.Name != project.Default {
 		// If restricted disk paths are in force, then check the disk's source is allowed, and record the
 		// allowed parent path for later user during device start up sequence.
-		if shared.IsTrue(p.Config["restricted"]) && p.Config["restricted.devices.disk.paths"] != "" {
-			allowed, restrictedParentSourcePath := project.CheckRestrictedDevicesDiskPaths(p.Config, d.config["source"])
+		if shared.IsTrue(instProject.Config["restricted"]) && instProject.Config["restricted.devices.disk.paths"] != "" {
+			allowed, restrictedParentSourcePath := project.CheckRestrictedDevicesDiskPaths(instProject.Config, d.config["source"])
 			if !allowed {
 				return fmt.Errorf("Disk source path %q not allowed by project for disk %q", d.config["source"], d.name)
 			}
@@ -421,7 +412,7 @@ func (d *disk) Register() error {
 			return err
 		}
 	} else if d.config["path"] != "/" && d.config["source"] != "" && d.config["pool"] != "" {
-		storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project(), db.StoragePoolVolumeTypeCustom)
+		storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project().Name, db.StoragePoolVolumeTypeCustom)
 		if err != nil {
 			return err
 		}
@@ -547,17 +538,21 @@ func (d *disk) startContainer() (*deviceConfig.RunConfig, error) {
 		// has owner shifting enabled, and if so enable shifting on this device too.
 		if ownerShift == deviceConfig.MountOwnerShiftNone && d.config["pool"] != "" {
 			// Only custom volumes can be attached currently.
-			storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project(), db.StoragePoolVolumeTypeCustom)
+			storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project().Name, db.StoragePoolVolumeTypeCustom)
 			if err != nil {
 				return nil, err
 			}
 
-			_, volume, err := d.state.DB.Cluster.GetLocalStoragePoolVolume(storageProjectName, d.config["source"], db.StoragePoolVolumeTypeCustom, d.pool.ID())
+			var dbVolume *db.StorageVolume
+			err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+				dbVolume, err = tx.GetStoragePoolVolume(ctx, d.pool.ID(), storageProjectName, db.StoragePoolVolumeTypeCustom, d.config["source"], true)
+				return err
+			})
 			if err != nil {
 				return nil, err
 			}
 
-			if shared.IsTrue(volume.Config["security.shifted"]) {
+			if shared.IsTrue(dbVolume.Config["security.shifted"]) {
 				ownerShift = "dynamic"
 			}
 		}
@@ -653,6 +648,10 @@ func (d *disk) detectVMPoolMountOpts() []string {
 		opts = append(opts, DiskDirectIO)
 	}
 
+	if d.pool.Driver().Info().IOUring {
+		opts = append(opts, DiskIOUring)
+	}
+
 	return opts
 }
 
@@ -736,7 +735,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 
 				// If the pool is ceph backed, don't mount it, instead pass config to QEMU instance
 				// to use the built in RBD support.
-				if d.pool.Driver().Info().Remote {
+				if d.pool.Driver().Info().Name == "ceph" {
 					config := d.pool.ToAPI().Config
 					poolName := config["ceph.osd.pool_name"]
 
@@ -802,7 +801,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 				// raw ID maps have been supplied, then we will be running the disk proxy processes
 				// inside a user namespace as the root userns user. Therefore we need to ensure
 				// that there is a root UID and GID mapping in the raw ID maps, and if not then add
-				// one mapping the root userns user to the the nouser/nogroup host ID.
+				// one mapping the root userns user to the nouser/nogroup host ID.
 				if d.restrictedParentSourcePath != "" || len(rawIDMaps) > 0 {
 					rawIDMaps = diskAddRootUserNSEntry(rawIDMaps, 65534)
 				}
@@ -821,10 +820,10 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 							d.logger.Warn("Unable to use virtio-fs for device, using 9p as a fallback", logger.Ctx{"err": errUnsupported})
 
 							if errUnsupported == ErrMissingVirtiofsd {
-								_ = d.state.DB.Cluster.UpsertWarningLocalNode(d.inst.Project(), cluster.TypeInstance, d.inst.ID(), warningtype.MissingVirtiofsd, "Using 9p as a fallback")
+								_ = d.state.DB.Cluster.UpsertWarningLocalNode(d.inst.Project().Name, cluster.TypeInstance, d.inst.ID(), warningtype.MissingVirtiofsd, "Using 9p as a fallback")
 							} else {
 								// Resolve previous warning.
-								_ = warnings.ResolveWarningsByLocalNodeAndProjectAndType(d.state.DB.Cluster, d.inst.Project(), warningtype.MissingVirtiofsd)
+								_ = warnings.ResolveWarningsByLocalNodeAndProjectAndType(d.state.DB.Cluster, d.inst.Project().Name, warningtype.MissingVirtiofsd)
 							}
 
 							return nil
@@ -839,7 +838,7 @@ func (d *disk) startVM() (*deviceConfig.RunConfig, error) {
 					runConf.PostHooks = append(runConf.PostHooks, unixListener.Close)
 
 					// Resolve previous warning
-					_ = warnings.ResolveWarningsByLocalNodeAndProjectAndType(d.state.DB.Cluster, d.inst.Project(), warningtype.MissingVirtiofsd)
+					_ = warnings.ResolveWarningsByLocalNodeAndProjectAndType(d.state.DB.Cluster, d.inst.Project().Name, warningtype.MissingVirtiofsd)
 
 					// Add the socket path to the mount options to indicate to the qemu driver
 					// that this share is available.
@@ -1186,7 +1185,7 @@ func (d *disk) mountPoolVolume() (func(), string, error) {
 	}
 
 	// Only custom volumes can be attached currently.
-	storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project(), db.StoragePoolVolumeTypeCustom)
+	storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project().Name, db.StoragePoolVolumeTypeCustom)
 	if err != nil {
 		return nil, "", err
 	}
@@ -1201,13 +1200,17 @@ func (d *disk) mountPoolVolume() (func(), string, error) {
 
 	revert.Add(func() { _, _ = d.pool.UnmountCustomVolume(storageProjectName, volumeName, nil) })
 
-	_, vol, err := d.state.DB.Cluster.GetLocalStoragePoolVolume(storageProjectName, volumeName, db.StoragePoolVolumeTypeCustom, d.pool.ID())
+	var dbVolume *db.StorageVolume
+	err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, d.pool.ID(), storageProjectName, db.StoragePoolVolumeTypeCustom, volumeName, true)
+		return err
+	})
 	if err != nil {
 		return nil, "", fmt.Errorf("Failed to fetch local storage volume record: %w", err)
 	}
 
 	if d.inst.Type() == instancetype.Container {
-		if vol.ContentType == db.StoragePoolVolumeContentTypeNameFS {
+		if dbVolume.ContentType == db.StoragePoolVolumeContentTypeNameFS {
 			err = d.storagePoolVolumeAttachShift(storageProjectName, d.pool.Name(), volumeName, db.StoragePoolVolumeTypeCustom, srcPath)
 			if err != nil {
 				return nil, "", fmt.Errorf("Failed shifting storage volume %q of type %q on storage pool %q: %w", volumeName, volumeTypeName, d.pool.Name(), err)
@@ -1217,7 +1220,7 @@ func (d *disk) mountPoolVolume() (func(), string, error) {
 		}
 	}
 
-	if vol.ContentType == db.StoragePoolVolumeContentTypeNameBlock {
+	if dbVolume.ContentType == db.StoragePoolVolumeContentTypeNameBlock {
 		srcPath, err = d.pool.GetCustomVolumeDisk(storageProjectName, volumeName)
 		if err != nil {
 			return nil, "", fmt.Errorf("Failed to get disk path: %w", err)
@@ -1415,18 +1418,17 @@ func (d *disk) localSourceOpen(srcPath string) (*os.File, error) {
 }
 
 func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName string, volumeType int, remapPath string) error {
-	// Load the DB records.
-	poolID, pool, _, err := d.state.DB.Cluster.GetStoragePool(poolName)
+	var err error
+	var dbVolume *db.StorageVolume
+	err = d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, d.pool.ID(), projectName, volumeType, volumeName, true)
+		return err
+	})
 	if err != nil {
 		return err
 	}
 
-	_, volume, err := d.state.DB.Cluster.GetLocalStoragePoolVolume(projectName, volumeName, volumeType, poolID)
-	if err != nil {
-		return err
-	}
-
-	poolVolumePut := volume.Writable()
+	poolVolumePut := dbVolume.StorageVolume.Writable()
 
 	// Check if unmapped.
 	if shared.IsTrue(poolVolumePut.Config["security.unmapped"]) {
@@ -1474,8 +1476,8 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 
 		if shared.IsFalseOrEmpty(poolVolumePut.Config["security.shifted"]) {
 			volumeUsedBy := []instance.Instance{}
-			err = storagePools.VolumeUsedByInstanceDevices(d.state, poolName, projectName, volume, true, func(dbInst db.InstanceArgs, project api.Project, usedByDevices []string) error {
-				inst, err := instance.Load(d.state, dbInst, nil)
+			err = storagePools.VolumeUsedByInstanceDevices(d.state, poolName, projectName, &dbVolume.StorageVolume, true, func(dbInst db.InstanceArgs, project api.Project, usedByDevices []string) error {
+				inst, err := instance.Load(d.state, dbInst, project)
 				if err != nil {
 					return err
 				}
@@ -1525,7 +1527,7 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 		if lastIdmap != nil {
 			var err error
 
-			if pool.Driver == "zfs" {
+			if d.pool.Driver().Info().Name == "zfs" {
 				err = lastIdmap.UnshiftRootfs(remapPath, storageDrivers.ShiftZFSSkipper)
 			} else {
 				err = lastIdmap.UnshiftRootfs(remapPath, nil)
@@ -1543,7 +1545,7 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 		if nextIdmap != nil {
 			var err error
 
-			if pool.Driver == "zfs" {
+			if d.pool.Driver().Info().Name == "zfs" {
 				err = nextIdmap.ShiftRootfs(remapPath, storageDrivers.ShiftZFSSkipper)
 			} else {
 				err = nextIdmap.ShiftRootfs(remapPath, nil)
@@ -1573,7 +1575,7 @@ func (d *disk) storagePoolVolumeAttachShift(projectName, poolName, volumeName st
 	// Update last idmap.
 	poolVolumePut.Config["volatile.idmap.last"] = jsonIdmap
 
-	err = d.state.DB.Cluster.UpdateStoragePoolVolume(projectName, volumeName, volumeType, poolID, poolVolumePut.Description, poolVolumePut.Config)
+	err = d.state.DB.Cluster.UpdateStoragePoolVolume(projectName, volumeName, volumeType, d.pool.ID(), poolVolumePut.Description, poolVolumePut.Config)
 	if err != nil {
 		return err
 	}
@@ -1639,7 +1641,7 @@ func (d *disk) postStop() error {
 	// Check if pool-specific action should be taken to unmount custom volume disks.
 	if d.config["pool"] != "" && d.config["path"] != "/" {
 		// Only custom volumes can be attached currently.
-		storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project(), db.StoragePoolVolumeTypeCustom)
+		storageProjectName, err := project.StorageVolumeProject(d.state.DB.Cluster, d.inst.Project().Name, db.StoragePoolVolumeTypeCustom)
 		if err != nil {
 			return err
 		}
@@ -1668,7 +1670,7 @@ func (d *disk) getDiskLimits() (map[string]diskBlockLimit, error) {
 	// Build a list of all valid block devices
 	validBlocks := []string{}
 
-	dents, err := ioutil.ReadDir("/sys/class/block/")
+	dents, err := os.ReadDir("/sys/class/block/")
 	if err != nil {
 		return nil, err
 	}
@@ -1683,7 +1685,7 @@ func (d *disk) getDiskLimits() (map[string]diskBlockLimit, error) {
 			continue
 		}
 
-		block, err := ioutil.ReadFile(fmt.Sprintf("%s/dev", fPath))
+		block, err := os.ReadFile(fmt.Sprintf("%s/dev", fPath))
 		if err != nil {
 			return nil, err
 		}
@@ -2032,7 +2034,7 @@ func (d *disk) generateVMConfigDrive() (string, error) {
 		}
 	}
 
-	err = ioutil.WriteFile(filepath.Join(scratchDir, "vendor-data"), []byte(vendorData), 0400)
+	err = os.WriteFile(filepath.Join(scratchDir, "vendor-data"), []byte(vendorData), 0400)
 	if err != nil {
 		return "", err
 	}
@@ -2046,7 +2048,7 @@ func (d *disk) generateVMConfigDrive() (string, error) {
 		}
 	}
 
-	err = ioutil.WriteFile(filepath.Join(scratchDir, "user-data"), []byte(userData), 0400)
+	err = os.WriteFile(filepath.Join(scratchDir, "user-data"), []byte(userData), 0400)
 	if err != nil {
 		return "", err
 	}
@@ -2058,7 +2060,7 @@ func (d *disk) generateVMConfigDrive() (string, error) {
 	}
 
 	if networkConfig != "" {
-		err = ioutil.WriteFile(filepath.Join(scratchDir, "network-config"), []byte(networkConfig), 0400)
+		err = os.WriteFile(filepath.Join(scratchDir, "network-config"), []byte(networkConfig), 0400)
 		if err != nil {
 			return "", err
 		}
@@ -2070,7 +2072,7 @@ local-hostname: %s
 %s
 `, d.inst.Name(), d.inst.Name(), instanceConfig["user.meta-data"])
 
-	err = ioutil.WriteFile(filepath.Join(scratchDir, "meta-data"), []byte(metaData), 0400)
+	err = os.WriteFile(filepath.Join(scratchDir, "meta-data"), []byte(metaData), 0400)
 	if err != nil {
 		return "", err
 	}
