@@ -359,9 +359,6 @@ func (d *nicBridged) validateConfig(instConf instance.ConfigReader) error {
 // Returns api.StatusError with status code set to http.StatusConflict if conflicting address found.
 func (d *nicBridged) checkAddressConflict() error {
 	node := d.inst.Location()
-	filter := cluster.InstanceFilter{
-		Node: &node, // Managed bridge networks have a per-server DHCP daemon.
-	}
 
 	ourNICIPs := make(map[string]net.IP, 2)
 	ourNICIPs["ipv4.address"] = net.ParseIP(d.config["ipv4.address"])
@@ -372,81 +369,64 @@ func (d *nicBridged) checkAddressConflict() error {
 		ourNICMAC, _ = net.ParseMAC(d.volatileGet()["hwaddr"])
 	}
 
-	return d.state.DB.Cluster.InstanceList(&filter, func(inst db.InstanceArgs, p api.Project) error {
-		// Get the instance's effective network project name.
-		instNetworkProject := project.NetworkProjectFromRecord(&p)
-		if instNetworkProject != project.Default {
-			return nil // Managed bridge networks can only exist in default project.
+	// Check if any instance devices use this network.
+	// Managed bridge networks have a per-server DHCP daemon so perform a node level search.
+	filter := cluster.InstanceFilter{Node: &node}
+
+	// Set network name for comparison (needs to support connecting to unmanaged networks).
+	networkName := d.config["parent"]
+	if d.network != nil {
+		networkName = d.network.Name()
+	}
+
+	// Bridge networks are always in the default project.
+	return network.UsedByInstanceDevices(d.state, project.Default, networkName, "bridge", func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
+		// Skip our own device. This avoids triggering duplicate device errors during
+		// updates or when making temporary copies of our instance during migrations.
+		if instance.IsSameLogicalInstance(d.inst, &inst) && d.Name() == nicName {
+			return nil
 		}
 
-		// Iterate through each of the instance's devices, looking for NICs that are linked to
-		// the same network, on the same cluster member as this NIC and have matching static IPs.
-		for devName, devConfig := range db.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles) {
-			if devConfig["type"] != "nic" {
-				continue
+		// Skip NICs connected to other VLANs (not perfect though as one NIC could
+		// explicitly specify the default untagged VLAN and these would be connected to
+		// same L2 even though the values are different, and there is a different default
+		// value for native and openvswith parent bridges).
+		if d.config["vlan"] != nicConfig["vlan"] {
+			return nil
+		}
+
+		// Check there isn't another instance with the same DNS name connected to a managed network
+		// that has DNS enabled and is connected to the same untagged VLAN.
+		if d.network != nil && d.network.Config()["dns.mode"] != "none" && nicCheckDNSNameConflict(d.inst.Name(), inst.Name) {
+			return api.StatusErrorf(http.StatusConflict, "Instance DNS name %q already used on network", strings.ToLower(inst.Name))
+		}
+
+		// Check NIC's MAC address doesn't match this NIC's MAC address.
+		devNICMAC, _ := net.ParseMAC(nicConfig["hwaddr"])
+		if devNICMAC == nil {
+			devNICMAC, _ = net.ParseMAC(inst.Config[fmt.Sprintf("volatile.%s.hwaddr", nicName)])
+		}
+
+		if ourNICMAC != nil && devNICMAC != nil && bytes.Equal(ourNICMAC, devNICMAC) {
+			return api.StatusErrorf(http.StatusConflict, "MAC address %q already defined on another NIC", devNICMAC.String())
+		}
+
+		// Check NIC's static IPs don't match this NIC's static IPs.
+		for _, key := range []string{"ipv4.address", "ipv6.address"} {
+			if d.config[key] == "" {
+				continue // No static IP specified on this NIC.
 			}
 
-			// Skip NICs that specify a NIC type that is not the same as our own.
-			if !shared.StringInSlice(devConfig["nictype"], []string{"", "bridged"}) {
-				continue
-			}
+			// Parse IPs to avoid being tripped up by presentation differences.
+			devNICIP := net.ParseIP(nicConfig[key])
 
-			// Skip our own device. This avoids triggering duplicate device errors during
-			// updates or when making temporary copies of our instance during migrations.
-			if instance.IsSameLogicalInstance(d.inst, &inst) && d.Name() == devName {
-				continue
-			}
-
-			// Skip NICs not connected to our NIC's managed network.
-			// If our NIC is connected to a managed network (either via network or parent keys)
-			// but the other NIC doesn't reference the same network name via either its network
-			// or parent keys then we can say it is connected to a different network, so the
-			// duplicate checks can be skipped.
-			if d.network != nil && !network.NICUsesNetwork(devConfig, &api.Network{Name: d.network.Name()}) {
-				continue
-			}
-
-			// Skip NICs that are connected to a managed network or different unmanaged parent
-			// when we are not connected to a managed network.
-			if d.network == nil && (devConfig["network"] != "" || d.config["parent"] != devConfig["parent"]) {
-				continue
-			}
-
-			// Skip NICs connected to other VLANs (not perfect though as one NIC could
-			// explicitly specify the default untagged VLAN and these would be connected to
-			// same L2 even though the values are different, and there is a different default
-			// value for native and openvswith parent bridges).
-			if d.config["vlan"] != devConfig["vlan"] {
-				continue
-			}
-
-			// Check NIC's MAC address doesn't match this NIC's MAC address.
-			devNICMAC, _ := net.ParseMAC(devConfig["hwaddr"])
-			if devNICMAC == nil {
-				devNICMAC, _ = net.ParseMAC(inst.Config[fmt.Sprintf("volatile.%s.hwaddr", devName)])
-			}
-
-			if ourNICMAC != nil && devNICMAC != nil && bytes.Equal(ourNICMAC, devNICMAC) {
-				return api.StatusErrorf(http.StatusConflict, "MAC address %q already defined on another NIC", devNICMAC.String())
-			}
-
-			// Check NIC's static IPs don't match this NIC's static IPs.
-			for _, key := range []string{"ipv4.address", "ipv6.address"} {
-				if d.config[key] == "" {
-					continue // No static IP specified on this NIC.
-				}
-
-				// Parse IPs to avoid being tripped up by presentation differences.
-				devNICIP := net.ParseIP(devConfig[key])
-
-				if ourNICIPs[key] != nil && devNICIP != nil && ourNICIPs[key].Equal(devNICIP) {
-					return api.StatusErrorf(http.StatusConflict, "IP address %q already defined on another NIC", devNICIP.String())
-				}
+			if ourNICIPs[key] != nil && devNICIP != nil && ourNICIPs[key].Equal(devNICIP) {
+				return api.StatusErrorf(http.StatusConflict, "IP address %q already defined on another NIC", devNICIP.String())
 			}
 		}
 
 		return nil
-	})
+	}, filter)
 }
 
 // validateEnvironment checks the runtime environment for correctness.
@@ -494,7 +474,7 @@ func (d *nicBridged) PreStartCheck() error {
 	}
 
 	// If managed network is not available, don't try and start instance.
-	if d.network.LocalStatus() == api.StoragePoolStatusUnvailable {
+	if d.network.LocalStatus() == api.NetworkStatusUnavailable {
 		return api.StatusErrorf(http.StatusServiceUnavailable, "Network %q unavailable on this server", d.network.Name())
 	}
 
@@ -546,7 +526,7 @@ func (d *nicBridged) Start() (*deviceConfig.RunConfig, error) {
 	// missing.
 	bridgeNet, ok := d.network.(bridgeNetwork)
 	if ok && d.network.IsManaged() && bridgeNet.UsesDNSMasq() {
-		deviceStaticFileName := dnsmasq.DHCPStaticAllocationPath(d.network.Name(), dnsmasq.StaticAllocationFileName(d.inst.Project(), d.inst.Name(), d.Name()))
+		deviceStaticFileName := dnsmasq.DHCPStaticAllocationPath(d.network.Name(), dnsmasq.StaticAllocationFileName(d.inst.Project().Name, d.inst.Name(), d.Name()))
 		if !shared.PathExists(deviceStaticFileName) {
 			err = d.rebuildDnsmasqEntry()
 			if err != nil {
@@ -669,13 +649,13 @@ func (d *nicBridged) Start() (*deviceConfig.RunConfig, error) {
 		{Key: "name", Value: d.config["name"]},
 		{Key: "flags", Value: "up"},
 		{Key: "link", Value: peerName},
+		{Key: "hwaddr", Value: d.config["hwaddr"]},
 	}
 
 	if d.inst.Type() == instancetype.VM {
 		runConf.NetworkInterface = append(runConf.NetworkInterface,
 			[]deviceConfig.RunConfigItem{
 				{Key: "devName", Value: d.name},
-				{Key: "hwaddr", Value: d.config["hwaddr"]},
 				{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
 			}...)
 	}
@@ -873,7 +853,7 @@ func (d *nicBridged) Remove() error {
 		}
 
 		// Remove dnsmasq config if it exists (doesn't return error if file is missing).
-		err := dnsmasq.RemoveStaticEntry(d.config["parent"], d.inst.Project(), d.inst.Name(), d.Name())
+		err := dnsmasq.RemoveStaticEntry(d.config["parent"], d.inst.Project().Name, d.inst.Name(), d.Name())
 		if err != nil {
 			return err
 		}
@@ -914,7 +894,7 @@ func (d *nicBridged) rebuildDnsmasqEntry() error {
 	// If IP filtering is enabled, and no static IP in config, check if there is already a
 	// dynamically assigned static IP in dnsmasq config and write that back out in new config.
 	if (shared.IsTrue(d.config["security.ipv4_filtering"]) && ipv4Address == "") || (shared.IsTrue(d.config["security.ipv6_filtering"]) && ipv6Address == "") {
-		deviceStaticFileName := dnsmasq.StaticAllocationFileName(d.inst.Project(), d.inst.Name(), d.Name())
+		deviceStaticFileName := dnsmasq.StaticAllocationFileName(d.inst.Project().Name, d.inst.Name(), d.Name())
 		_, curIPv4, curIPv6, err := dnsmasq.DHCPStaticAllocation(d.config["parent"], deviceStaticFileName)
 		if err != nil && !os.IsNotExist(err) {
 			return err
@@ -929,7 +909,7 @@ func (d *nicBridged) rebuildDnsmasqEntry() error {
 		}
 	}
 
-	err := dnsmasq.UpdateStaticEntry(d.config["parent"], d.inst.Project(), d.inst.Name(), d.Name(), d.network.Config(), d.config["hwaddr"], ipv4Address, ipv6Address)
+	err := dnsmasq.UpdateStaticEntry(d.config["parent"], d.inst.Project().Name, d.inst.Name(), d.Name(), d.network.Config(), d.config["hwaddr"], ipv4Address, ipv6Address)
 	if err != nil {
 		return err
 	}
@@ -999,7 +979,7 @@ func (d *nicBridged) removeFilters(m deviceConfig.Device) {
 	// Remove filters for static MAC and IPs (if specified above).
 	// This covers the case when filtering is used with an unmanaged bridge.
 	d.logger.Debug("Clearing instance firewall static filters", logger.Ctx{"parent": m["parent"], "host_name": m["host_name"], "hwaddr": m["hwaddr"], "IPv4Nets": IPv4Nets, "IPv6Nets": IPv6Nets})
-	err = d.state.Firewall.InstanceClearBridgeFilter(d.inst.Project(), d.inst.Name(), d.name, m["parent"], m["host_name"], m["hwaddr"], IPv4Nets, IPv6Nets)
+	err = d.state.Firewall.InstanceClearBridgeFilter(d.inst.Project().Name, d.inst.Name(), d.name, m["parent"], m["host_name"], m["hwaddr"], IPv4Nets, IPv6Nets)
 	if err != nil {
 		d.logger.Error("Failed to remove static IP network filters", logger.Ctx{"err": err})
 	}
@@ -1007,14 +987,14 @@ func (d *nicBridged) removeFilters(m deviceConfig.Device) {
 	// If allowedIPNets returned nil for IPv4 or IPv6, it is possible that total protocol blocking was set up
 	// because the device has a managed parent network with DHCP disabled. Pass in empty slices to catch this case.
 	d.logger.Debug("Clearing instance total protocol filters", logger.Ctx{"parent": m["parent"], "host_name": m["host_name"], "hwaddr": m["hwaddr"], "IPv4Nets": IPv4Nets, "IPv6Nets": IPv6Nets})
-	err = d.state.Firewall.InstanceClearBridgeFilter(d.inst.Project(), d.inst.Name(), d.name, m["parent"], m["host_name"], m["hwaddr"], make([]*net.IPNet, 0), make([]*net.IPNet, 0))
+	err = d.state.Firewall.InstanceClearBridgeFilter(d.inst.Project().Name, d.inst.Name(), d.name, m["parent"], m["host_name"], m["hwaddr"], make([]*net.IPNet, 0), make([]*net.IPNet, 0))
 	if err != nil {
 		d.logger.Error("Failed to remove total protocol network filters", logger.Ctx{"err": err})
 	}
 
 	// Read current static DHCP IP allocation configured from dnsmasq host config (if exists).
 	// This covers the case when IPs are not defined in config, but have been assigned in managed DHCP.
-	deviceStaticFileName := dnsmasq.StaticAllocationFileName(d.inst.Project(), d.inst.Name(), d.Name())
+	deviceStaticFileName := dnsmasq.StaticAllocationFileName(d.inst.Project().Name, d.inst.Name(), d.Name())
 	_, IPv4Alloc, IPv6Alloc, err := dnsmasq.DHCPStaticAllocation(m["parent"], deviceStaticFileName)
 	if err != nil {
 		if os.IsNotExist(err) {
@@ -1047,7 +1027,7 @@ func (d *nicBridged) removeFilters(m deviceConfig.Device) {
 	}
 
 	d.logger.Debug("Clearing instance firewall dynamic filters", logger.Ctx{"parent": m["parent"], "host_name": m["host_name"], "hwaddr": m["hwaddr"], "ipv4": IPv4Alloc.IP, "ipv6": IPv6Alloc.IP})
-	err = d.state.Firewall.InstanceClearBridgeFilter(d.inst.Project(), d.inst.Name(), d.name, m["parent"], m["host_name"], m["hwaddr"], IPv4AllocNets, IPv6AllocNets)
+	err = d.state.Firewall.InstanceClearBridgeFilter(d.inst.Project().Name, d.inst.Name(), d.name, m["parent"], m["host_name"], m["hwaddr"], IPv4AllocNets, IPv6AllocNets)
 	if err != nil {
 		logger.Errorf("Failed to remove DHCP network assigned filters  for %q: %v", d.name, err)
 	}
@@ -1095,7 +1075,7 @@ func (d *nicBridged) setFilters() (err error) {
 	// If parent bridge is managed, allocate the static IPs (if needed).
 	if d.network != nil && (IPv4 == nil || IPv6 == nil) {
 		opts := &dhcpalloc.Options{
-			ProjectName: d.inst.Project(),
+			ProjectName: d.inst.Project().Name,
 			HostName:    d.inst.Name(),
 			DeviceName:  d.Name(),
 			HostMAC:     mac,
@@ -1144,7 +1124,7 @@ func (d *nicBridged) setFilters() (err error) {
 		return err
 	}
 
-	err = d.state.Firewall.InstanceSetupBridgeFilter(d.inst.Project(), d.inst.Name(), d.name, d.config["parent"], d.config["host_name"], d.config["hwaddr"], IPv4Nets, IPv6Nets, d.network != nil)
+	err = d.state.Firewall.InstanceSetupBridgeFilter(d.inst.Project().Name, d.inst.Name(), d.name, d.config["parent"], d.config["host_name"], d.config["hwaddr"], IPv4Nets, IPv6Nets, d.network != nil)
 	if err != nil {
 		return err
 	}
@@ -1502,18 +1482,18 @@ func (d *nicBridged) setupNativeBridgePortVLANs(hostName string) error {
 			return err
 		}
 
-		// If the default is different to the specified untagged VLAN or if tagged VLAN is set to "none"
-		// then remove the default untagged membership.
-		if defaultPVID != d.config["vlan"] || d.config["vlan"] == "none" {
-			err = link.BridgeVLANDelete(defaultPVID, false, false)
+		// If the bridge has a default PVID and it is different to the specified untagged VLAN or if tagged
+		// VLAN is set to "none" then remove the default untagged membership.
+		if defaultPVID != "0" && (defaultPVID != d.config["vlan"] || d.config["vlan"] == "none") {
+			err = link.BridgeVLANDelete(defaultPVID, false)
 			if err != nil {
-				return err
+				return fmt.Errorf("Failed removing default PVID membership: %w", err)
 			}
 		}
 
 		// Configure the untagged membership settings of the port if VLAN ID specified.
 		if d.config["vlan"] != "none" {
-			err = link.BridgeVLANAdd(d.config["vlan"], true, true, false, true)
+			err = link.BridgeVLANAdd(d.config["vlan"], true, true, false)
 			if err != nil {
 				return err
 			}
@@ -1533,7 +1513,7 @@ func (d *nicBridged) setupNativeBridgePortVLANs(hostName string) error {
 				return fmt.Errorf("VLAN tagged ID 0 is not allowed for native Linux bridges")
 			}
 
-			err := link.BridgeVLANAdd(fmt.Sprintf("%d", vlanID), false, false, false, false)
+			err := link.BridgeVLANAdd(fmt.Sprintf("%d", vlanID), false, false, false)
 			if err != nil {
 				return err
 			}
@@ -1647,7 +1627,7 @@ func (d *nicBridged) State() (*api.InstanceStateNetwork, error) {
 			}
 
 			if shared.IsFalseOrEmpty(d.network.Config()["ipv6.dhcp.stateful"]) && v6subnet != nil {
-				// If stateful DHCPv6 is disabled, and IPv6 is enabled on the bridge, the the NIC
+				// If stateful DHCPv6 is disabled, and IPv6 is enabled on the bridge, the NIC
 				// is likely to use its MAC and SLAAC to configure its address.
 				if hwAddr != nil {
 					ip, err := eui64.ParseMAC(v6subnet.IP, hwAddr)
