@@ -1,13 +1,16 @@
 package device
 
 import (
+	"bytes"
 	"fmt"
 	"net"
 	"net/http"
 	"os"
+	"strings"
 
 	"github.com/mdlayher/netx/eui64"
 
+	"github.com/lxc/lxd/lxd/db"
 	deviceConfig "github.com/lxc/lxd/lxd/device/config"
 	pcidev "github.com/lxc/lxd/lxd/device/pci"
 	"github.com/lxc/lxd/lxd/dnsmasq/dhcpalloc"
@@ -31,8 +34,10 @@ type ovnNet interface {
 	network.Network
 
 	InstanceDevicePortValidateExternalRoutes(deviceInstance instance.Instance, deviceName string, externalRoutes []*net.IPNet) error
-	InstanceDevicePortSetup(opts *network.OVNInstanceNICSetupOpts, securityACLsRemove []string) (openvswitch.OVNSwitchPort, error)
-	InstanceDevicePortDelete(ovsExternalOVNPort openvswitch.OVNSwitchPort, opts *network.OVNInstanceNICStopOpts) error
+	InstanceDevicePortAdd(instanceUUID string, deviceName string, deviceConfig deviceConfig.Device) error
+	InstanceDevicePortStart(opts *network.OVNInstanceNICSetupOpts, securityACLsRemove []string) (openvswitch.OVNSwitchPort, error)
+	InstanceDevicePortStop(ovsExternalOVNPort openvswitch.OVNSwitchPort, opts *network.OVNInstanceNICStopOpts) error
+	InstanceDevicePortRemove(instanceUUID string, deviceName string, deviceConfig deviceConfig.Device) error
 	InstanceDevicePortDynamicIPs(instanceUUID string, deviceName string) ([]net.IP, error)
 }
 
@@ -89,7 +94,7 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 	}
 
 	// The NIC's network may be a non-default project, so lookup project and get network's project name.
-	networkProjectName, _, err := project.NetworkProject(d.state.DB.Cluster, instConf.Project())
+	networkProjectName, _, err := project.NetworkProject(d.state.DB.Cluster, instConf.Project().Name)
 	if err != nil {
 		return fmt.Errorf("Failed loading network project name: %w", err)
 	}
@@ -172,6 +177,15 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 	// Apply network level config options to device config before validation.
 	d.config["mtu"] = netConfig["bridge.mtu"]
 
+	// Check there isn't another NIC with any of the same addresses specified on the same network.
+	// Can only validate this when the instance is supplied (and not doing profile validation).
+	if d.inst != nil {
+		err := d.checkAddressConflict()
+		if err != nil {
+			return err
+		}
+	}
+
 	rules := nicValidationRules(requiredFields, optionalFields, instConf)
 
 	// Now run normal validation.
@@ -211,6 +225,65 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 	return nil
 }
 
+// checkAddressConflict checks for conflicting IP/MAC addresses on another NIC connected to same network.
+// Can only validate this when the instance is supplied (and not doing profile validation).
+// Returns api.StatusError with status code set to http.StatusConflict if conflicting address found.
+func (d *nicOVN) checkAddressConflict() error {
+	ourNICIPs := make(map[string]net.IP, 2)
+	ourNICIPs["ipv4.address"] = net.ParseIP(d.config["ipv4.address"])
+	ourNICIPs["ipv6.address"] = net.ParseIP(d.config["ipv6.address"])
+
+	ourNICMAC, _ := net.ParseMAC(d.config["hwaddr"])
+	if ourNICMAC == nil {
+		ourNICMAC, _ = net.ParseMAC(d.volatileGet()["hwaddr"])
+	}
+
+	// Check if any instance devices use this network.
+	return network.UsedByInstanceDevices(d.state, d.network.Project(), d.network.Name(), d.network.Type(), func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
+		// Skip our own device. This avoids triggering duplicate device errors during
+		// updates or when making temporary copies of our instance during migrations.
+		if instance.IsSameLogicalInstance(d.inst, &inst) && d.Name() == nicName {
+			return nil
+		}
+
+		// Check there isn't another instance with the same DNS name connected to managed network.
+		if d.network != nil && nicCheckDNSNameConflict(d.inst.Name(), inst.Name) {
+			return api.StatusErrorf(http.StatusConflict, "Instance DNS name %q already used on network", strings.ToLower(inst.Name))
+		}
+
+		// Check NIC's MAC address doesn't match this NIC's MAC address.
+		devNICMAC, _ := net.ParseMAC(nicConfig["hwaddr"])
+		if devNICMAC == nil {
+			devNICMAC, _ = net.ParseMAC(inst.Config[fmt.Sprintf("volatile.%s.hwaddr", nicName)])
+		}
+
+		if ourNICMAC != nil && devNICMAC != nil && bytes.Equal(ourNICMAC, devNICMAC) {
+			return api.StatusErrorf(http.StatusConflict, "MAC address %q already defined on another NIC", devNICMAC.String())
+		}
+
+		// Check NIC's static IPs don't match this NIC's static IPs.
+		for _, key := range []string{"ipv4.address", "ipv6.address"} {
+			if d.config[key] == "" {
+				continue // No static IP specified on this NIC.
+			}
+
+			// Parse IPs to avoid being tripped up by presentation differences.
+			devNICIP := net.ParseIP(nicConfig[key])
+
+			if ourNICIPs[key] != nil && devNICIP != nil && ourNICIPs[key].Equal(devNICIP) {
+				return api.StatusErrorf(http.StatusConflict, "IP address %q already defined on another NIC", devNICIP.String())
+			}
+		}
+
+		return nil
+	})
+}
+
+// Add is run when a device is added to a non-snapshot instance whether or not the instance is running.
+func (d *nicOVN) Add() error {
+	return d.network.InstanceDevicePortAdd(d.inst.LocalConfig()["volatile.uuid"], d.name, d.config)
+}
+
 // PreStartCheck checks the managed parent network is available (if relevant).
 func (d *nicOVN) PreStartCheck() error {
 	// Non-managed network NICs are not relevant for checking managed network availability.
@@ -219,7 +292,7 @@ func (d *nicOVN) PreStartCheck() error {
 	}
 
 	// If managed network is not available, don't try and start instance.
-	if d.network.LocalStatus() == api.StoragePoolStatusUnvailable {
+	if d.network.LocalStatus() == api.NetworkStatusUnavailable {
 		return api.StatusErrorf(http.StatusServiceUnavailable, "Network %q unavailable on this server", d.network.Name())
 	}
 
@@ -360,7 +433,8 @@ func (d *nicOVN) Start() (*deviceConfig.RunConfig, error) {
 		{Key: "link", Value: peerName},
 	}
 
-	if d.inst.Type() == instancetype.VM {
+	instType := d.inst.Type()
+	if instType == instancetype.VM {
 		if d.config["acceleration"] == "sriov" {
 			runConf.NetworkInterface = append(runConf.NetworkInterface,
 				[]deviceConfig.RunConfigItem{
@@ -377,6 +451,10 @@ func (d *nicOVN) Start() (*deviceConfig.RunConfig, error) {
 					{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
 				}...)
 		}
+	} else if instType == instancetype.Container {
+		runConf.NetworkInterface = append(runConf.NetworkInterface,
+			deviceConfig.RunConfigItem{Key: "hwaddr", Value: d.config["hwaddr"]},
+		)
 	}
 
 	revert.Success()
@@ -438,7 +516,7 @@ func (d *nicOVN) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 			}
 
 			// Update OVN logical switch port for instance.
-			_, err = d.network.InstanceDevicePortSetup(&network.OVNInstanceNICSetupOpts{
+			_, err = d.network.InstanceDevicePortStart(&network.OVNInstanceNICSetupOpts{
 				InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
 				DNSName:      d.inst.Name(),
 				DeviceName:   d.name,
@@ -484,7 +562,7 @@ func (d *nicOVN) Stop() (*deviceConfig.RunConfig, error) {
 	}
 
 	// Try and retrieve the last associated OVN switch port for the instance interface in the local OVS DB.
-	// If we cannot get this, don't fail, as InstanceDevicePortDelete will then try and generate the likely
+	// If we cannot get this, don't fail, as InstanceDevicePortStop will then try and generate the likely
 	// port name using the same regime it does for new ports. This part is only here in order to allow
 	// instance ports generated under an older regime to be cleaned up properly.
 	networkVethFillFromVolatile(d.config, d.volatileGet())
@@ -510,7 +588,7 @@ func (d *nicOVN) Stop() (*deviceConfig.RunConfig, error) {
 	}
 
 	instanceUUID := d.inst.LocalConfig()["volatile.uuid"]
-	err = d.network.InstanceDevicePortDelete(ovsExternalOVNPort, &network.OVNInstanceNICStopOpts{
+	err = d.network.InstanceDevicePortStop(ovsExternalOVNPort, &network.OVNInstanceNICStopOpts{
 		InstanceUUID: instanceUUID,
 		DeviceName:   d.name,
 		DeviceConfig: d.config,
@@ -595,7 +673,7 @@ func (d *nicOVN) Remove() error {
 		}
 	}
 
-	return nil
+	return d.network.InstanceDevicePortRemove(d.inst.LocalConfig()["volatile.uuid"], d.name, d.config)
 }
 
 // State gets the state of an OVN NIC by querying the OVN Northbound logical switch port record.
@@ -667,7 +745,7 @@ func (d *nicOVN) State() (*api.InstanceStateNetwork, error) {
 			})
 		} else if shared.IsFalseOrEmpty(netConfig["ipv6.dhcp.stateful"]) && d.config["hwaddr"] != "" && v6subnet != nil {
 			// If no static DHCPv6 allocation and stateful DHCPv6 is disabled, and IPv6 is enabled on
-			// the bridge, the the NIC is likely to use its MAC and SLAAC to configure its address.
+			// the bridge, the NIC is likely to use its MAC and SLAAC to configure its address.
 			hwAddr, err := net.ParseMAC(d.config["hwaddr"])
 			if err == nil {
 				ip, err := eui64.ParseMAC(v6subnet.IP, hwAddr)
@@ -747,7 +825,7 @@ func (d *nicOVN) setupHostNIC(hostName string, uplink *api.Network) (revert.Hook
 	}
 
 	// Add new OVN logical switch port for instance.
-	logicalPortName, err := d.network.InstanceDevicePortSetup(&network.OVNInstanceNICSetupOpts{
+	logicalPortName, err := d.network.InstanceDevicePortStart(&network.OVNInstanceNICSetupOpts{
 		InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
 		DNSName:      d.inst.Name(),
 		DeviceName:   d.name,
@@ -759,7 +837,7 @@ func (d *nicOVN) setupHostNIC(hostName string, uplink *api.Network) (revert.Hook
 	}
 
 	revert.Add(func() {
-		_ = d.network.InstanceDevicePortDelete("", &network.OVNInstanceNICStopOpts{
+		_ = d.network.InstanceDevicePortStop("", &network.OVNInstanceNICStopOpts{
 			InstanceUUID: d.inst.LocalConfig()["volatile.uuid"],
 			DeviceName:   d.name,
 			DeviceConfig: d.config,
