@@ -1,8 +1,9 @@
 package main
 
 import (
+	"context"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net"
 	"net/http"
 	"os"
@@ -10,7 +11,10 @@ import (
 	"strings"
 	"testing"
 
+	"golang.org/x/sys/unix"
+
 	"github.com/lxc/lxd/lxd/sys"
+	"github.com/lxc/lxd/lxd/ucred"
 )
 
 var testDir string
@@ -19,7 +23,7 @@ type DevLxdDialer struct {
 	Path string
 }
 
-func (d DevLxdDialer) DevLxdDial(network, path string) (net.Conn, error) {
+func (d DevLxdDialer) DevLxdDial(ctx context.Context, network, path string) (net.Conn, error) {
 	addr, err := net.ResolveUnixAddr("unix", d.Path)
 	if err != nil {
 		return nil, err
@@ -36,10 +40,11 @@ func (d DevLxdDialer) DevLxdDial(network, path string) (net.Conn, error) {
 func setupDir() error {
 	var err error
 
-	testDir, err = ioutil.TempDir("", "lxd_test_devlxd_")
+	testDir, err = os.MkdirTemp("", "lxd_test_devlxd_")
 	if err != nil {
 		return err
 	}
+
 	err = sys.SetupTestCerts(testDir)
 	if err != nil {
 		return err
@@ -50,13 +55,13 @@ func setupDir() error {
 		return err
 	}
 
-	os.MkdirAll(fmt.Sprintf("%s/devlxd", testDir), 0755)
+	_ = os.MkdirAll(fmt.Sprintf("%s/devlxd", testDir), 0755)
 
 	return os.Setenv("LXD_DIR", testDir)
 }
 
 func setupSocket() (*net.UnixListener, error) {
-	setupDir()
+	_ = setupDir()
 	path := filepath.Join(testDir, "test-devlxd-sock")
 	addr, err := net.ResolveUnixAddr("unix", path)
 	if err != nil {
@@ -67,6 +72,7 @@ func setupSocket() (*net.UnixListener, error) {
 	if err != nil {
 		return nil, err
 	}
+
 	return listener, nil
 }
 
@@ -91,8 +97,9 @@ func TestCredsSendRecv(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	defer listener.Close()
-	defer os.RemoveAll(testDir)
+
+	defer func() { _ = listener.Close() }()
+	defer func() { _ = os.RemoveAll(testDir) }()
 
 	go func() {
 		conn, err := listener.AcceptUnix()
@@ -101,22 +108,25 @@ func TestCredsSendRecv(t *testing.T) {
 			result <- -1
 			return
 		}
-		defer conn.Close()
 
-		cred, err := getCred(conn)
+		defer func() { _ = conn.Close() }()
+
+		cred, err := ucred.GetCred(conn)
 		if err != nil {
 			t.Log(err)
 			result <- -1
 			return
 		}
-		result <- cred.pid
+
+		result <- cred.Pid
 	}()
 
 	conn, err := connect(fmt.Sprintf("%s/test-devlxd-sock", testDir))
 	if err != nil {
 		t.Fatal(err)
 	}
-	defer conn.Close()
+
+	defer func() { _ = conn.Close() }()
 
 	pid := <-result
 	if pid != int32(os.Getpid()) {
@@ -131,18 +141,19 @@ func TestCredsSendRecv(t *testing.T) {
  * point where it realizes the pid isn't in a container without crashing).
  */
 func TestHttpRequest(t *testing.T) {
-	setupDir()
-	defer os.RemoveAll(testDir)
+	_ = setupDir()
+	defer func() { _ = os.RemoveAll(testDir) }()
 
-	d := DefaultDaemon()
+	d := defaultDaemon()
 	d.os.MockMode = true
 	err := d.Init()
 	if err != nil {
 		t.Fatal(err)
 	}
-	defer d.Stop()
 
-	c := http.Client{Transport: &http.Transport{Dial: DevLxdDialer{Path: fmt.Sprintf("%s/devlxd/sock", testDir)}.DevLxdDial}}
+	defer func() { _ = d.Stop(context.Background(), unix.SIGQUIT) }()
+
+	c := http.Client{Transport: &http.Transport{DialContext: DevLxdDialer{Path: fmt.Sprintf("%s/devlxd/sock", testDir)}.DevLxdDial}}
 
 	raw, err := c.Get("http://1.0")
 	if err != nil {
@@ -153,7 +164,7 @@ func TestHttpRequest(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	resp, err := ioutil.ReadAll(raw.Body)
+	resp, err := io.ReadAll(raw.Body)
 	if err != nil {
 		t.Fatal(err)
 	}
