@@ -1,15 +1,15 @@
 package endpoints
 
 import (
-	"crypto/tls"
 	"fmt"
+	"log"
 	"net"
-	"sync"
+	"strings"
 	"time"
 
+	"github.com/lxc/lxd/lxd/endpoints/listeners"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
-	log "github.com/lxc/lxd/shared/log15"
 	"github.com/lxc/lxd/shared/logger"
 )
 
@@ -40,7 +40,7 @@ func (e *Endpoints) NetworkCert() *shared.CertInfo {
 }
 
 // NetworkAddress returns the network addresss of the network endpoint, or an
-// empty string if there's no network endpoint
+// empty string if there's no network endpoint.
 func (e *Endpoints) NetworkAddress() string {
 	e.mu.RLock()
 	defer e.mu.RUnlock()
@@ -49,6 +49,7 @@ func (e *Endpoints) NetworkAddress() string {
 	if listener == nil {
 		return ""
 	}
+
 	return listener.Addr().String()
 }
 
@@ -56,7 +57,7 @@ func (e *Endpoints) NetworkAddress() string {
 // it down and restarting it.
 func (e *Endpoints) NetworkUpdateAddress(address string) error {
 	if address != "" {
-		address = util.CanonicalNetworkAddress(address)
+		address = util.CanonicalNetworkAddress(address, shared.HTTPSDefaultPort)
 	}
 
 	oldAddress := e.NetworkAddress()
@@ -64,19 +65,27 @@ func (e *Endpoints) NetworkUpdateAddress(address string) error {
 		return nil
 	}
 
+	clusterAddress := e.clusterAddress()
+
 	logger.Infof("Update network address")
 
 	e.mu.Lock()
 	defer e.mu.Unlock()
 
 	// Close the previous socket
-	e.closeListener(network)
+	_ = e.closeListener(network)
 
-	// If turning off listening, we're done
+	// If turning off listening, we're done.
 	if address == "" {
 		return nil
 	}
 
+	// If the new address covers the cluster one, turn off the cluster
+	// listener.
+	if clusterAddress != "" && util.IsAddressCovered(clusterAddress, address) {
+		_ = e.closeListener(cluster)
+	}
+
 	// Attempt to setup the new listening socket
 	getListener := func(address string) (*net.Listener, error) {
 		var err error
@@ -92,7 +101,7 @@ func (e *Endpoints) NetworkUpdateAddress(address string) error {
 		}
 
 		if err != nil {
-			return nil, fmt.Errorf("cannot listen on https socket: %v", err)
+			return nil, fmt.Errorf("Cannot listen on network HTTPS socket %q: %w", address, err)
 		}
 
 		return &listener, nil
@@ -105,15 +114,15 @@ func (e *Endpoints) NetworkUpdateAddress(address string) error {
 			// Attempt to revert to the previous address
 			listener, err1 := getListener(oldAddress)
 			if err1 == nil {
-				e.listeners[network] = networkTLSListener(*listener, e.cert)
-				e.serveHTTP(network)
+				e.listeners[network] = listeners.NewFancyTLSListener(*listener, e.cert)
+				e.serve(network)
 			}
 
 			return err
 		}
 
-		e.listeners[network] = networkTLSListener(*listener, e.cert)
-		e.serveHTTP(network)
+		e.listeners[network] = listeners.NewFancyTLSListener(*listener, e.cert)
+		e.serve(network)
 	}
 
 	return nil
@@ -129,58 +138,66 @@ func (e *Endpoints) NetworkUpdateCert(cert *shared.CertInfo) {
 	defer e.mu.Unlock()
 	e.cert = cert
 	listener, ok := e.listeners[network]
-	if !ok {
-		return
+	if ok {
+		listener.(*listeners.FancyTLSListener).Config(cert)
+	}
+
+	// Update the cluster listener too, if enabled.
+	listener, ok = e.listeners[cluster]
+	if ok {
+		listener.(*listeners.FancyTLSListener).Config(cert)
+	}
+}
+
+// NetworkUpdateTrustedProxy updates the https trusted proxy used by the network
+// endpoint.
+func (e *Endpoints) NetworkUpdateTrustedProxy(trustedProxy string) {
+	var proxies []net.IP
+	for _, p := range strings.Split(trustedProxy, ",") {
+		p = strings.ToLower(strings.TrimSpace(p))
+		if len(p) == 0 {
+			continue
+		}
+
+		proxyIP := net.ParseIP(p)
+		proxies = append(proxies, proxyIP)
+	}
+
+	e.mu.Lock()
+	defer e.mu.Unlock()
+	listener, ok := e.listeners[network]
+	if ok && listener != nil {
+		listener.(*listeners.FancyTLSListener).TrustedProxy(proxies)
+	}
+
+	// Update the cluster listener too, if enabled.
+	listener, ok = e.listeners[cluster]
+	if ok && listener != nil {
+		listener.(*listeners.FancyTLSListener).TrustedProxy(proxies)
+	}
+
+	server, ok := e.servers[network]
+	if ok && server != nil {
+		server.ErrorLog = log.New(networkServerErrorLogWriter{proxies: proxies}, "", 0)
 	}
-	listener.(*networkListener).Config(cert)
 }
 
 // Create a new net.Listener bound to the tcp socket of the network endpoint.
-func networkCreateListener(address string, cert *shared.CertInfo) net.Listener {
-	listener, err := net.Listen("tcp", util.CanonicalNetworkAddress(address))
+func networkCreateListener(address string, cert *shared.CertInfo) (net.Listener, error) {
+	// Listening on `tcp` network with address 0.0.0.0 will end up with listening
+	// on both IPv4 and IPv6 interfaces. Pass `tcp4` to make it
+	// work only on 0.0.0.0. https://go-review.googlesource.com/c/go/+/45771/
+	listenAddress := util.CanonicalNetworkAddress(address, shared.HTTPSDefaultPort)
+	protocol := "tcp"
+
+	if strings.HasPrefix(listenAddress, "0.0.0.0") {
+		protocol = "tcp4"
+	}
+
+	listener, err := net.Listen(protocol, listenAddress)
 	if err != nil {
-		logger.Error("Cannot listen on https socket, skipping...", log.Ctx{"err": err})
-		return nil
+		return nil, fmt.Errorf("Bind network address: %w", err)
 	}
-	return networkTLSListener(listener, cert)
-}
 
-// A variation of the standard tls.Listener that supports atomically swapping
-// the underlying TLS configuration. Requests served before the swap will
-// continue using the old configuration.
-type networkListener struct {
-	net.Listener
-	mu     sync.RWMutex
-	config *tls.Config
-}
-
-func networkTLSListener(inner net.Listener, cert *shared.CertInfo) *networkListener {
-	listener := &networkListener{
-		Listener: inner,
-	}
-	listener.Config(cert)
-	return listener
-}
-
-// Accept waits for and returns the next incoming TLS connection then use the
-// current TLS configuration to handle it.
-func (l *networkListener) Accept() (net.Conn, error) {
-	c, err := l.Listener.Accept()
-	if err != nil {
-		return nil, err
-	}
-	l.mu.RLock()
-	defer l.mu.RUnlock()
-	config := l.config
-	return tls.Server(c, config), nil
-}
-
-// Config safely swaps the underlying TLS configuration.
-func (l *networkListener) Config(cert *shared.CertInfo) {
-	config := util.ServerTLSConfig(cert)
-
-	l.mu.Lock()
-	defer l.mu.Unlock()
-
-	l.config = config
+	return listeners.NewFancyTLSListener(listener, cert), nil
 }
