@@ -1,9 +1,12 @@
+//go:build linux && cgo
+
 package endpoints
 
 import (
 	"fmt"
 	"net"
 	"os"
+	"os/user"
 	"strconv"
 
 	"github.com/lxc/lxd/client"
@@ -12,19 +15,18 @@ import (
 )
 
 // Bind to the given unix socket path.
-func socketUnixListen(path string) (net.Listener, error) {
+func socketUnixListen(path string) (*net.UnixListener, error) {
 	addr, err := net.ResolveUnixAddr("unix", path)
 	if err != nil {
-		return nil, fmt.Errorf("cannot resolve socket address: %v", err)
+		return nil, fmt.Errorf("cannot resolve socket address: %w", err)
 	}
 
 	listener, err := net.ListenUnix("unix", addr)
 	if err != nil {
-		return nil, fmt.Errorf("cannot bind socket: %v", err)
+		return nil, fmt.Errorf("cannot bind socket: %w", err)
 	}
 
 	return listener, err
-
 }
 
 // CheckAlreadyRunning checks if the socket at the given path is already
@@ -68,30 +70,36 @@ func socketUnixRemoveStale(path string) error {
 	logger.Debugf("Detected stale unix socket, deleting")
 	err := os.Remove(path)
 	if err != nil {
-		return fmt.Errorf("could not delete stale local socket: %v", err)
+		return fmt.Errorf("could not delete stale local socket: %w", err)
 	}
 
 	return nil
 }
 
-// Change the file mode of the given unix socket file,
+// Change the file mode of the given unix socket file,.
 func socketUnixSetPermissions(path string, mode os.FileMode) error {
 	err := os.Chmod(path, mode)
 	if err != nil {
-		return fmt.Errorf("cannot set permissions on local socket: %v", err)
+		return fmt.Errorf("cannot set permissions on local socket: %w", err)
 	}
+
 	return nil
 }
 
-// Change the ownership of the given unix socket file,
-func socketUnixSetOwnership(path string, group string) error {
+// Change the ownership of the given unix socket file,.
+func socketUnixSetOwnership(path string, groupName string) error {
 	var gid int
 	var err error
 
-	if group != "" {
-		gid, err = shared.GroupId(group)
+	if groupName != "" {
+		g, err := user.LookupGroup(groupName)
 		if err != nil {
-			return fmt.Errorf("cannot get group ID of '%s': %v", group, err)
+			return fmt.Errorf("cannot get group ID of '%s': %w", groupName, err)
+		}
+
+		gid, err = strconv.Atoi(g.Gid)
+		if err != nil {
+			return err
 		}
 	} else {
 		gid = os.Getgid()
@@ -99,8 +107,7 @@ func socketUnixSetOwnership(path string, group string) error {
 
 	err = os.Chown(path, os.Getuid(), gid)
 	if err != nil {
-		return fmt.Errorf("cannot change ownership on local socket: %v", err)
-
+		return fmt.Errorf("cannot change ownership on local socket: %w", err)
 	}
 
 	return nil
