@@ -84,45 +84,12 @@ func instanceImageTransfer(d *Daemon, r *http.Request, projectName string, hash
 }
 
 // instanceCreateFromImage creates an instance from a rootfs image.
-func instanceCreateFromImage(d *Daemon, r *http.Request, args db.InstanceArgs, hash string, op *operations.Operation) (instance.Instance, error) {
+func instanceCreateFromImage(d *Daemon, r *http.Request, img *api.Image, args db.InstanceArgs, op *operations.Operation) (instance.Instance, error) {
 	revert := revert.New()
 	defer revert.Fail()
 
 	s := d.State()
 
-	// Get the image properties.
-	var img *api.Image
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		var err error
-		_, img, err = tx.GetImageByFingerprintPrefix(ctx, hash, dbCluster.ImageFilter{Project: &args.Project})
-		if err != nil {
-			return fmt.Errorf("Fetch image %s from database: %w", hash, err)
-		}
-
-		// Set the default profiles if necessary.
-		if args.Profiles == nil {
-			args.Profiles = make([]api.Profile, 0, len(img.Profiles))
-			profiles, err := dbCluster.GetProfilesIfEnabled(ctx, tx.Tx(), args.Project, img.Profiles)
-			if err != nil {
-				return err
-			}
-
-			for _, profile := range profiles {
-				apiProfile, err := profile.ToAPI(ctx, tx.Tx())
-				if err != nil {
-					return err
-				}
-
-				args.Profiles = append(args.Profiles, *apiProfile)
-			}
-		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
-	}
-
 	// Validate the type of the image matches the type of the instance.
 	imgType, err := instancetype.New(img.Type)
 	if err != nil {
@@ -130,7 +97,7 @@ func instanceCreateFromImage(d *Daemon, r *http.Request, args db.InstanceArgs, h
 	}
 
 	if imgType != args.Type {
-		return nil, fmt.Errorf("Requested image's type '%s' doesn't match instance type '%s'", imgType, args.Type)
+		return nil, fmt.Errorf("Requested image's type %q doesn't match instance type %q", imgType, args.Type)
 	}
 
 	// Check if the image is available locally or it's on another member.
@@ -140,10 +107,10 @@ func instanceCreateFromImage(d *Daemon, r *http.Request, args db.InstanceArgs, h
 	// duplicate record errors.
 	unlock := d.imageOperationLock(img.Fingerprint)
 
-	nodeAddress, err := s.DB.Cluster.LocateImage(hash)
+	nodeAddress, err := s.DB.Cluster.LocateImage(img.Fingerprint)
 	if err != nil {
 		unlock()
-		return nil, fmt.Errorf("Locate image %q in the cluster: %w", hash, err)
+		return nil, fmt.Errorf("Locate image %q in the cluster: %w", img.Fingerprint, err)
 	}
 
 	if nodeAddress != "" {
@@ -172,7 +139,7 @@ func instanceCreateFromImage(d *Daemon, r *http.Request, args db.InstanceArgs, h
 	}
 
 	// Set the BaseImage field (regardless of previous value).
-	args.BaseImage = hash
+	args.BaseImage = img.Fingerprint
 
 	// Create the instance.
 	inst, instOp, cleanup, err := instance.CreateInternal(s, args, true)
@@ -183,7 +150,7 @@ func instanceCreateFromImage(d *Daemon, r *http.Request, args db.InstanceArgs, h
 	revert.Add(cleanup)
 	defer instOp.Done(nil)
 
-	err = s.DB.Cluster.UpdateImageLastUseDate(hash, time.Now().UTC())
+	err = s.DB.Cluster.UpdateImageLastUseDate(args.Project, img.Fingerprint, time.Now().UTC())
 	if err != nil {
 		return nil, fmt.Errorf("Error updating image last use date: %s", err)
 	}
@@ -193,7 +160,7 @@ func instanceCreateFromImage(d *Daemon, r *http.Request, args db.InstanceArgs, h
 		return nil, fmt.Errorf("Failed loading instance storage pool: %w", err)
 	}
 
-	err = pool.CreateInstanceFromImage(inst, hash, op)
+	err = pool.CreateInstanceFromImage(inst, img.Fingerprint, op)
 	if err != nil {
 		return nil, fmt.Errorf("Failed creating instance from image: %w", err)
 	}
@@ -267,21 +234,51 @@ func instanceCreateAsCopy(s *state.State, opts instanceCreateAsCopyOpts, op *ope
 	if !opts.instanceOnly {
 		if opts.refresh {
 			// Compare snapshots.
-			syncSnapshots, deleteSnapshots, err := instance.CompareSnapshots(opts.sourceInstance, inst)
+			sourceSnaps, err := opts.sourceInstance.Snapshots()
 			if err != nil {
 				return nil, err
 			}
 
+			sourceSnapshotComparable := make([]storagePools.ComparableSnapshot, 0, len(sourceSnaps))
+			for _, sourceSnap := range sourceSnaps {
+				_, sourceSnapName, _ := api.GetParentAndSnapshotName(sourceSnap.Name())
+
+				sourceSnapshotComparable = append(sourceSnapshotComparable, storagePools.ComparableSnapshot{
+					Name:         sourceSnapName,
+					CreationDate: sourceSnap.CreationDate(),
+				})
+			}
+
+			targetSnaps, err := inst.Snapshots()
+			if err != nil {
+				return nil, err
+			}
+
+			targetSnapshotsComparable := make([]storagePools.ComparableSnapshot, 0, len(targetSnaps))
+			for _, targetSnap := range targetSnaps {
+				_, targetSnapName, _ := api.GetParentAndSnapshotName(targetSnap.Name())
+
+				targetSnapshotsComparable = append(targetSnapshotsComparable, storagePools.ComparableSnapshot{
+					Name:         targetSnapName,
+					CreationDate: targetSnap.CreationDate(),
+				})
+			}
+
+			syncSourceSnapshotIndexes, deleteTargetSnapshotIndexes := storagePools.CompareSnapshots(sourceSnapshotComparable, targetSnapshotsComparable)
+
 			// Delete extra snapshots first.
-			for _, snap := range deleteSnapshots {
-				err := snap.Delete(true)
+			for _, deleteTargetSnapIndex := range deleteTargetSnapshotIndexes {
+				err := targetSnaps[deleteTargetSnapIndex].Delete(true)
 				if err != nil {
 					return nil, err
 				}
 			}
 
-			// Only care about the snapshots that need updating.
-			snapshots = syncSnapshots
+			// Only send the snapshots that need updating.
+			snapshots = make([]instance.Instance, 0, len(syncSourceSnapshotIndexes))
+			for _, syncSourceSnapIndex := range syncSourceSnapshotIndexes {
+				snapshots = append(snapshots, sourceSnaps[syncSourceSnapIndex])
+			}
 		} else {
 			// Get snapshots of source instance.
 			snapshots, err = opts.sourceInstance.Snapshots()
@@ -397,17 +394,13 @@ func instanceLoadNodeProjectAll(s *state.State, project string, instanceType ins
 	var err error
 	var instances []instance.Instance
 
-	filter := dbCluster.InstanceFilter{
-		Type:    &instanceType,
-		Project: &project,
-	}
-
+	filter := dbCluster.InstanceFilter{Type: instanceType.Filter(), Project: &project}
 	if s.ServerName != "" {
 		filter.Node = &s.ServerName
 	}
 
-	err = s.DB.Cluster.InstanceList(&filter, func(dbInst db.InstanceArgs, p api.Project) error {
-		inst, err := instance.Load(s, dbInst, nil)
+	err = s.DB.Cluster.InstanceList(context.TODO(), func(dbInst db.InstanceArgs, p api.Project) error {
+		inst, err := instance.Load(s, dbInst, p)
 		if err != nil {
 			return fmt.Errorf("Failed loading instance %q in project %q: %w", dbInst.Name, dbInst.Project, err)
 		}
@@ -415,7 +408,7 @@ func instanceLoadNodeProjectAll(s *state.State, project string, instanceType ins
 		instances = append(instances, inst)
 
 		return nil
-	})
+	}, filter)
 	if err != nil {
 		return nil, err
 	}
@@ -428,11 +421,12 @@ func autoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 		s := d.State()
 
 		var instanceArgs map[int]db.InstanceArgs
+		projects := make(map[string]*api.Project)
 
 		// Get eligible instances.
 		err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 			// Get all projects.
-			allProjects, err := dbCluster.GetProjects(context.Background(), tx.Tx(), dbCluster.ProjectFilter{})
+			allProjects, err := dbCluster.GetProjects(context.Background(), tx.Tx())
 			if err != nil {
 				return fmt.Errorf("Failed loading projects: %w", err)
 			}
@@ -440,16 +434,21 @@ func autoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 			dbInstances := []dbCluster.Instance{}
 
 			// Filter projects that aren't allowed to have snapshots.
-			for _, p := range allProjects {
-				err = project.AllowSnapshotCreation(tx, &p)
+			for _, dbProject := range allProjects {
+				p, err := dbProject.ToAPI(context.Background(), tx.Tx())
+				if err != nil {
+					return err
+				}
+
+				err = project.AllowSnapshotCreation(p)
 				if err != nil {
 					continue
 				}
 
-				// Get instances.
-				filter := db.InstanceTypeFilter(instancetype.Any)
-				filter.Project = &p.Name
+				projects[p.Name] = p
 
+				// Get instances.
+				filter := dbCluster.InstanceFilter{Project: &p.Name}
 				entries, err := tx.GetLocalInstancesInProject(ctx, filter)
 				if err != nil {
 					return err
@@ -472,9 +471,9 @@ func autoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 		// Figure out which need snapshotting (if any).
 		instances := make([]instance.Instance, 0)
 		for _, instArg := range instanceArgs {
-			inst, err := instance.Load(s, instArg, nil)
+			inst, err := instance.Load(s, instArg, *projects[instArg.Project])
 			if err != nil {
-				logger.Error("Failed loading instance for snapshot task", logger.Ctx{"project": inst.Project(), "instance": inst.Name()})
+				logger.Error("Failed loading instance for snapshot task", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
 				continue
 			}
 
@@ -541,7 +540,7 @@ func autoCreateInstanceSnapshots(ctx context.Context, d *Daemon, instances []ins
 	for _, inst := range instances {
 		ch := make(chan error)
 		go func(inst instance.Instance) {
-			l := logger.AddContext(logger.Log, logger.Ctx{"project": inst.Project(), "instance": inst.Name()})
+			l := logger.AddContext(logger.Log, logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
 
 			snapshotName, err := instance.NextSnapshotName(d.State(), inst, "snap%d")
 			if err != nil {
@@ -550,16 +549,16 @@ func autoCreateInstanceSnapshots(ctx context.Context, d *Daemon, instances []ins
 				return
 			}
 
-			expiry, err := shared.GetSnapshotExpiry(time.Now(), inst.ExpandedConfig()["snapshots.expiry"])
+			expiry, err := shared.GetExpiry(time.Now(), inst.ExpandedConfig()["snapshots.expiry"])
 			if err != nil {
-				logger.Error("Error getting expiry date")
+				l.Error("Error getting snapshots.expiry date")
 				ch <- nil
 				return
 			}
 
 			err = inst.Snapshot(snapshotName, expiry, false)
 			if err != nil {
-				logger.Error("Error creating snapshots", logger.Ctx{"err": err})
+				l.Error("Error creating snapshot", logger.Ctx{"snapshot": snapshotName, "err": err})
 			}
 
 			ch <- nil
@@ -614,11 +613,27 @@ func pruneExpiredInstanceSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 				return fmt.Errorf("Failed loading expired instance snapshots: %w", err)
 			}
 
+			projects := make(map[string]*api.Project)
+
 			expiredSnapshotInstances = make([]instance.Instance, 0)
 			for _, snapshotArg := range snapshotArgs {
-				inst, err := instance.Load(s, snapshotArg, nil)
+				// Load project if not already loaded.
+				p, found := projects[snapshotArg.Project]
+				if !found {
+					dbProject, err := dbCluster.GetProject(context.Background(), tx.Tx(), snapshotArg.Project)
+					if err != nil {
+						return err
+					}
+
+					p, err = dbProject.ToAPI(ctx, tx.Tx())
+					if err != nil {
+						return err
+					}
+				}
+
+				inst, err := instance.Load(s, snapshotArg, *p)
 				if err != nil {
-					logger.Error("Failed loading instance for snapshot prune task", logger.Ctx{"project": inst.Project(), "instance": inst.Name()})
+					logger.Error("Failed loading instance for snapshot prune task", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
 					continue
 				}
 
@@ -686,10 +701,10 @@ func pruneExpiredInstanceSnapshots(ctx context.Context, d *Daemon, snapshots []i
 		err := snapshot.Delete(true)
 		instSnapshotsPruneRunning.Delete(snapshot.ID())
 		if err != nil {
-			return fmt.Errorf("Failed to delete expired instance snapshot %q in project %q: %w", snapshot.Name(), snapshot.Project(), err)
+			return fmt.Errorf("Failed to delete expired instance snapshot %q in project %q: %w", snapshot.Name(), snapshot.Project().Name, err)
 		}
 
-		logger.Debug("Deleted instance snapshot", logger.Ctx{"project": snapshot.Project(), "snapshot": snapshot.Name()})
+		logger.Debug("Deleted instance snapshot", logger.Ctx{"project": snapshot.Project().Name, "snapshot": snapshot.Name()})
 	}
 
 	return nil
