@@ -35,6 +35,12 @@ import (
 	"github.com/lxc/lxd/shared/logger"
 )
 
+// ErrExecCommandNotFound indicates the command is not found.
+var ErrExecCommandNotFound = fmt.Errorf("Command not found")
+
+// ErrExecCommandNotExecutable indicates the command is not executable.
+var ErrExecCommandNotExecutable = fmt.Errorf("Command not executable")
+
 // ErrInstanceIsStopped indicates that the instance is stopped.
 var ErrInstanceIsStopped error = fmt.Errorf("The instance is already stopped")
 
@@ -67,7 +73,7 @@ type common struct {
 	name            string
 	node            string
 	profiles        []api.Profile
-	project         string
+	project         api.Project
 	snapshot        bool
 	stateful        bool
 
@@ -172,7 +178,7 @@ func (d *common) Profiles() []api.Profile {
 }
 
 // Project returns instance's project.
-func (d *common) Project() string {
+func (d *common) Project() api.Project {
 	return d.project
 }
 
@@ -198,7 +204,7 @@ func (d *common) Operation() *operations.Operation {
 // Backups returns a list of backups.
 func (d *common) Backups() ([]backup.InstanceBackup, error) {
 	// Get all the backups
-	backupNames, err := d.state.DB.Cluster.GetInstanceBackups(d.project, d.name)
+	backupNames, err := d.state.DB.Cluster.GetInstanceBackups(d.project.Name, d.name)
 	if err != nil {
 		return nil, err
 	}
@@ -206,7 +212,7 @@ func (d *common) Backups() ([]backup.InstanceBackup, error) {
 	// Build the backup list
 	backups := []backup.InstanceBackup{}
 	for _, backupName := range backupNames {
-		backup, err := instance.BackupLoadByName(d.state, d.project, backupName)
+		backup, err := instance.BackupLoadByName(d.state, d.project.Name, backupName)
 		if err != nil {
 			return nil, err
 		}
@@ -248,7 +254,7 @@ func (d *common) Snapshots() ([]instance.Instance, error) {
 	// Get all the snapshots for instance.
 	err := d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		filter := dbCluster.InstanceSnapshotFilter{
-			Project:  &d.project,
+			Project:  &d.project.Name,
 			Instance: &d.name,
 		}
 
@@ -278,7 +284,7 @@ func (d *common) Snapshots() ([]instance.Instance, error) {
 		// Populate profile info that was already loaded.
 		snapshotArg.Profiles = d.profiles
 
-		snapInst, err := instance.Load(d.state, snapshotArg, nil)
+		snapInst, err := instance.Load(d.state, snapshotArg, d.project)
 		if err != nil {
 			return nil, err
 		}
@@ -287,7 +293,20 @@ func (d *common) Snapshots() ([]instance.Instance, error) {
 	}
 
 	sort.SliceStable(snapshots, func(i, j int) bool {
-		return snapshots[i].CreationDate().Before(snapshots[j].CreationDate())
+		iCreation := snapshots[i].CreationDate()
+		jCreation := snapshots[j].CreationDate()
+
+		// Prefer sorting by creation date.
+		if iCreation.Before(jCreation) {
+			return true
+		}
+
+		// But if creation date is the same, then sort by ID.
+		if iCreation.Equal(jCreation) && snapshots[i].ID() < snapshots[j].ID() {
+			return true
+		}
+
+		return false
 	})
 
 	return snapshots, nil
@@ -344,19 +363,19 @@ func (d *common) ConsoleBufferLogPath() string {
 
 // DevicesPath returns the instance's devices path.
 func (d *common) DevicesPath() string {
-	name := project.Instance(d.project, d.name)
+	name := project.Instance(d.project.Name, d.name)
 	return shared.VarPath("devices", name)
 }
 
 // LogPath returns the instance's log path.
 func (d *common) LogPath() string {
-	name := project.Instance(d.project, d.name)
+	name := project.Instance(d.project.Name, d.name)
 	return shared.LogPath(name)
 }
 
 // Path returns the instance's path.
 func (d *common) Path() string {
-	return storagePools.InstancePath(d.dbType, d.project, d.name, d.snapshot)
+	return storagePools.InstancePath(d.dbType, d.project.Name, d.name, d.snapshot)
 }
 
 // RootfsPath returns the instance's rootfs path.
@@ -366,7 +385,7 @@ func (d *common) RootfsPath() string {
 
 // ShmountsPath returns the instance's shared mounts path.
 func (d *common) ShmountsPath() string {
-	name := project.Instance(d.project, d.name)
+	name := project.Instance(d.project.Name, d.name)
 	return shared.VarPath("shmounts", name)
 }
 
@@ -400,12 +419,12 @@ func (d *common) deviceVolatileReset(devName string, oldConfig, newConfig device
 	volatileClear := make(map[string]string)
 	devicePrefix := fmt.Sprintf("volatile.%s.", devName)
 
-	newNICType, err := nictype.NICType(d.state, d.project, newConfig)
+	newNICType, err := nictype.NICType(d.state, d.project.Name, newConfig)
 	if err != nil {
 		return err
 	}
 
-	oldNICType, err := nictype.NICType(d.state, d.project, oldConfig)
+	oldNICType, err := nictype.NICType(d.state, d.project.Name, oldConfig)
 	if err != nil {
 		return err
 	}
@@ -472,13 +491,9 @@ func (d *common) deviceVolatileSetFunc(devName string) func(save map[string]stri
 }
 
 // expandConfig applies the config of each profile in order, followed by the local config.
-func (d *common) expandConfig(profiles []api.Profile) error {
-	if profiles == nil && len(d.profiles) > 0 {
-		profiles = d.profiles
-	}
-
-	d.expandedConfig = db.ExpandInstanceConfig(d.localConfig, profiles)
-	d.expandedDevices = db.ExpandInstanceDevices(d.localDevices, profiles)
+func (d *common) expandConfig() error {
+	d.expandedConfig = db.ExpandInstanceConfig(d.localConfig, d.profiles)
+	d.expandedDevices = db.ExpandInstanceDevices(d.localDevices, d.profiles)
 
 	return nil
 }
@@ -486,7 +501,7 @@ func (d *common) expandConfig(profiles []api.Profile) error {
 // restartCommon handles the common part of instance restarts.
 func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) error {
 	// Setup a new operation for the stop/shutdown phase.
-	op, err := operationlock.Create(d.Project(), d.Name(), operationlock.ActionRestart, true, true)
+	op, err := operationlock.Create(d.Project().Name, d.Name(), operationlock.ActionRestart, true, true)
 	if err != nil {
 		return fmt.Errorf("Create restart operation: %w", err)
 	}
@@ -502,7 +517,7 @@ func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) er
 			Devices:      inst.LocalDevices(),
 			Ephemeral:    false,
 			Profiles:     inst.Profiles(),
-			Project:      inst.Project(),
+			Project:      inst.Project().Name,
 			Type:         inst.Type(),
 			Snapshot:     inst.IsSnapshot(),
 		}
@@ -540,7 +555,7 @@ func (d *common) restartCommon(inst instance.Instance, timeout time.Duration) er
 	}
 
 	// Setup a new operation for the start phase.
-	op, err = operationlock.Create(d.Project(), d.Name(), operationlock.ActionRestart, true, true)
+	op, err = operationlock.Create(d.Project().Name, d.Name(), operationlock.ActionRestart, true, true)
 	if err != nil {
 		return fmt.Errorf("Create restart (for start) operation: %w", err)
 	}
@@ -574,7 +589,7 @@ func (d *common) snapshotCommon(inst instance.Instance, name string, expiry time
 
 	// Setup the arguments.
 	args := db.InstanceArgs{
-		Project:      inst.Project(),
+		Project:      inst.Project().Name,
 		Architecture: inst.Architecture(),
 		Config:       inst.LocalConfig(),
 		Type:         inst.Type(),
@@ -703,7 +718,7 @@ func (d *common) startupSnapshot(inst instance.Instance) error {
 		return nil
 	}
 
-	expiry, err := shared.GetSnapshotExpiry(time.Now(), d.expandedConfig["snapshots.expiry"])
+	expiry, err := shared.GetExpiry(time.Now(), d.expandedConfig["snapshots.expiry"])
 	if err != nil {
 		return err
 	}
@@ -881,38 +896,37 @@ func (d *common) maasDelete(inst instance.Instance) error {
 // onStopOperationSetup creates or picks up the relevant operation. This is used in the stopns and stop hooks to
 // ensure that a lock on their activities is held before the instance process is stopped. This prevents a start
 // request run at the same time from overlapping with the stop process.
-// Returns the operation along with a boolean indicating if the operation was created or not.
-func (d *common) onStopOperationSetup(target string) (*operationlock.InstanceOperation, bool, error) {
+// Returns the operation (with the instance initiated marker set if the operation was created).
+func (d *common) onStopOperationSetup(target string) (*operationlock.InstanceOperation, error) {
 	var err error
 
 	// Pick up the existing stop operation lock created in Stop() function.
 	// If there is another ongoing operation (such as start), wait until that has finished before proceeding
 	// to run the hook (this should be quick as it will fail showing instance is already running).
-	op := operationlock.Get(d.Project(), d.Name())
+	op := operationlock.Get(d.Project().Name, d.Name())
 	if op != nil && !op.ActionMatch(operationlock.ActionStop, operationlock.ActionRestart, operationlock.ActionRestore) {
 		d.logger.Debug("Waiting for existing operation lock to finish before running hook", logger.Ctx{"action": op.Action()})
 		_ = op.Wait()
 		op = nil
 	}
 
-	instanceInitiated := false
-
 	if op == nil {
 		d.logger.Debug("Instance initiated stop", logger.Ctx{"action": target})
-		instanceInitiated = true
 
 		action := operationlock.ActionStop
 		if target == "reboot" {
 			action = operationlock.ActionRestart
 		}
 
-		op, err = operationlock.Create(d.Project(), d.Name(), action, false, false)
+		op, err = operationlock.Create(d.Project().Name, d.Name(), action, false, false)
 		if err != nil {
-			return nil, false, fmt.Errorf("Failed creating %q operation: %w", action, err)
+			return nil, fmt.Errorf("Failed creating %q operation: %w", action, err)
 		}
+
+		op.SetInstanceInitiated(true)
 	}
 
-	return op, instanceInitiated, nil
+	return op, nil
 }
 
 // warningsDelete deletes any persistent warnings for the instance.
@@ -1025,7 +1039,7 @@ func (d *common) getRootDiskDevice() (string, map[string]string, error) {
 		parentName, _, _ := api.GetParentAndSnapshotName(d.name)
 
 		// Load the parent.
-		storageInstance, err := instance.LoadByProjectAndName(d.state, d.project, parentName)
+		storageInstance, err := instance.LoadByProjectAndName(d.state, d.project.Name, parentName)
 		if err != nil {
 			return "", nil, err
 		}
@@ -1119,7 +1133,7 @@ func (d *common) getStoragePool() (storagePools.Pool, error) {
 		return d.storagePool, nil
 	}
 
-	poolName, err := d.state.DB.Cluster.GetInstancePool(d.Project(), d.Name())
+	poolName, err := d.state.DB.Cluster.GetInstancePool(d.Project().Name, d.Name())
 	if err != nil {
 		return nil, err
 	}
@@ -1193,12 +1207,16 @@ func (d *common) devicesAdd(inst instance.Instance, instanceRunning bool) (rever
 	for _, entry := range d.expandedDevices.Sorted() {
 		dev, err := d.deviceLoad(inst, entry.Name, entry.Config)
 		if err != nil {
+			if errors.Is(err, device.ErrUnsupportedDevType) {
+				continue // Skip unsupported device (allows for mixed instance type profiles).
+			}
+
 			// If device conflicts with another device then do not call the deviceAdd function below
 			// as this could cause the original device to be disrupted (such as allowing conflicting
 			// static NIC DHCP leases to be created). Instead just log an error.
 			// This will allow instances to be created with conflicting devices (such as when copying
 			// or restoring a backup) and allows the user to manually fix the conflicts in order to
-			// allow the the instance to start.
+			// allow the instance to start.
 			if api.StatusErrorCheck(err, http.StatusConflict) {
 				d.logger.Error("Failed add validation for device, skipping add action", logger.Ctx{"device": entry.Name, "err": err})
 
@@ -1234,6 +1252,10 @@ func (d *common) devicesRegister(inst instance.Instance) {
 	for _, entry := range d.ExpandedDevices().Sorted() {
 		dev, err := d.deviceLoad(inst, entry.Name, entry.Config)
 		if err != nil {
+			if errors.Is(err, device.ErrUnsupportedDevType) {
+				continue // Skip unsupported device (allows for mixed instance type profiles).
+			}
+
 			d.logger.Error("Failed register validation for device", logger.Ctx{"err": err, "device": entry.Name})
 			continue
 		}
@@ -1262,6 +1284,10 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 		l := d.logger.AddContext(logger.Ctx{"device": entry.Name, "userRequested": userRequested})
 		dev, err := d.deviceLoad(inst, entry.Name, entry.Config)
 		if err != nil {
+			if errors.Is(err, device.ErrUnsupportedDevType) {
+				continue // Skip unsupported device (allows for mixed instance type profiles).
+			}
+
 			// Just log an error, but still allow the device to be removed if usable device returned.
 			l.Error("Failed remove validation for device", logger.Ctx{"err": err})
 		}
@@ -1295,6 +1321,10 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 		l := d.logger.AddContext(logger.Ctx{"device": entry.Name, "userRequested": userRequested})
 		dev, err := d.deviceLoad(inst, entry.Name, entry.Config)
 		if err != nil {
+			if errors.Is(err, device.ErrUnsupportedDevType) {
+				continue // Skip unsupported device (allows for mixed instance type profiles).
+			}
+
 			if userRequested {
 				return fmt.Errorf("Failed add validation for device %q: %w", entry.Name, err)
 			}
@@ -1338,6 +1368,10 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 		l := d.logger.AddContext(logger.Ctx{"device": entry.Name, "userRequested": userRequested})
 		dev, err := d.deviceLoad(inst, entry.Name, entry.Config)
 		if err != nil {
+			if errors.Is(err, device.ErrUnsupportedDevType) {
+				continue // Skip unsupported device (allows for mixed instance type profiles).
+			}
+
 			if userRequested {
 				return fmt.Errorf("Failed update validation for device %q: %w", entry.Name, err)
 			}
@@ -1385,6 +1419,10 @@ func (d *common) devicesRemove(inst instance.Instance) {
 	for _, entry := range d.expandedDevices.Reversed() {
 		dev, err := d.deviceLoad(inst, entry.Name, entry.Config)
 		if err != nil {
+			if errors.Is(err, device.ErrUnsupportedDevType) {
+				continue // Skip unsupported device (allows for mixed instance type profiles).
+			}
+
 			// Just log an error, but still allow the device to be removed if usable device returned.
 			d.logger.Error("Failed remove validation for device", logger.Ctx{"device": entry.Name, "err": err})
 		}
