@@ -5,7 +5,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"net/url"
 
@@ -72,6 +72,9 @@ var internalClusterInstanceMovedCmd = APIEndpoint{
 //   "500":
 //     $ref: "#/responses/InternalServerError"
 func instancePost(d *Daemon, r *http.Request) response.Response {
+	// Don't mess with instance while in setup mode.
+	<-d.waitReady.Done()
+
 	s := d.State()
 
 	instanceType, err := urlInstanceTypeDetect(r)
@@ -127,13 +130,13 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			// Check if user is allowed to use cluster member targeting
-			err = project.CheckClusterTargetRestriction(tx, r, apiProject, targetNode)
+			err = project.CheckClusterTargetRestriction(r, apiProject, targetNode)
 			if err != nil {
 				return err
 			}
 
 			// Load target node.
-			node, err := tx.GetNodeByName(targetNode)
+			node, err := tx.GetNodeByName(ctx, targetNode)
 			if err != nil {
 				return fmt.Errorf("Failed to get target node: %w", err)
 			}
@@ -141,7 +144,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 			targetNodeOffline = node.IsOffline(s.GlobalConfig.OfflineThreshold())
 
 			// Load source node.
-			address, err := tx.GetNodeAddressOfInstance(projectName, name, db.InstanceTypeFilter(instanceType))
+			address, err := tx.GetNodeAddressOfInstance(ctx, projectName, name, instanceType)
 			if err != nil {
 				return fmt.Errorf("Failed to get address of instance's member: %w", err)
 			}
@@ -152,7 +155,7 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 				return nil
 			}
 
-			node, err = tx.GetNodeByAddress(address)
+			node, err = tx.GetNodeByAddress(ctx, address)
 			if err != nil {
 				return fmt.Errorf("Failed to get source member for %s: %w", address, err)
 			}
@@ -203,13 +206,13 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	body, err := ioutil.ReadAll(r.Body)
+	body, err := io.ReadAll(r.Body)
 	if err != nil {
 		return response.InternalError(err)
 	}
 
-	rdr1 := ioutil.NopCloser(bytes.NewBuffer(body))
-	rdr2 := ioutil.NopCloser(bytes.NewBuffer(body))
+	rdr1 := io.NopCloser(bytes.NewBuffer(body))
+	rdr2 := io.NopCloser(bytes.NewBuffer(body))
 
 	reqRaw := shared.Jmap{}
 	err = json.NewDecoder(rdr1).Decode(&reqRaw)
@@ -234,6 +237,17 @@ func instancePost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	// If new instance name not supplied, assume it will be keeping its current name.
+	if req.Name == "" {
+		req.Name = inst.Name()
+	}
+
+	// Check the new instance name is valid.
+	err = instance.ValidName(req.Name, false)
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
 	if req.Migration {
 		// Server-side pool migration.
 		if req.Pool != "" {
@@ -418,7 +432,7 @@ func instancePostPoolMigration(d *Daemon, inst instance.Instance, newName string
 		BaseImage:    localConfig["volatile.base_image"],
 		Config:       localConfig,
 		Devices:      localDevices,
-		Project:      inst.Project(),
+		Project:      inst.Project().Name,
 		Type:         inst.Type(),
 		Architecture: inst.Architecture(),
 		Description:  inst.Description(),
@@ -556,12 +570,12 @@ func instancePostClusteringMigrate(d *Daemon, r *http.Request, inst instance.Ins
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		sourceAddress, err = tx.GetLocalNodeAddress()
+		sourceAddress, err = tx.GetLocalNodeAddress(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed to get local member address: %w", err)
 		}
 
-		node, err := tx.GetNodeByName(newNode)
+		node, err := tx.GetNodeByName(ctx, newNode)
 		if err != nil {
 			return fmt.Errorf("Failed to get new member address: %w", err)
 		}
@@ -581,7 +595,7 @@ func instancePostClusteringMigrate(d *Daemon, r *http.Request, inst instance.Ins
 			return fmt.Errorf("Failed to connect to source server %q: %w", sourceAddress, err)
 		}
 
-		source = source.UseProject(inst.Project())
+		source = source.UseProject(inst.Project().Name)
 
 		// Connect to the destination host, i.e. the node to migrate the container to.
 		dest, err := cluster.Connect(targetAddress, d.endpoints.NetworkCert(), d.serverCert(), r, false)
@@ -589,7 +603,7 @@ func instancePostClusteringMigrate(d *Daemon, r *http.Request, inst instance.Ins
 			return fmt.Errorf("Failed to connect to destination server %q: %w", targetAddress, err)
 		}
 
-		dest = dest.UseTarget(newNode).UseProject(inst.Project())
+		dest = dest.UseTarget(newNode).UseProject(inst.Project().Name)
 
 		destName := newName
 		isSameName := false
@@ -692,7 +706,7 @@ func instancePostClusteringMigrate(d *Daemon, r *http.Request, inst instance.Ins
 		}
 
 		// Restore the original value of "volatile.apply_template"
-		project := inst.Project()
+		project := inst.Project().Name
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			id, err := dbCluster.GetInstanceID(ctx, tx.Tx(), project, destName)
 			if err != nil {
@@ -757,7 +771,7 @@ func instancePostClusteringMigrateWithCeph(d *Daemon, r *http.Request, inst inst
 		// If the source member is online then get its address so we can connect to it and see if the
 		// instance is running later.
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			sourceMember, err = tx.GetNodeByName(inst.Location())
+			sourceMember, err = tx.GetNodeByName(ctx, inst.Location())
 			if err != nil {
 				return fmt.Errorf("Failed getting cluster member of instance %q", inst.Name())
 			}
@@ -779,7 +793,7 @@ func instancePostClusteringMigrateWithCeph(d *Daemon, r *http.Request, inst inst
 				return fmt.Errorf("Failed to connect to source server %q: %w", sourceMember.Address, err)
 			}
 
-			source = source.UseProject(inst.Project())
+			source = source.UseProject(inst.Project().Name)
 
 			// Get instance state on source member.
 			entry, _, err := source.GetInstance(inst.Name())
@@ -826,7 +840,7 @@ func instancePostClusteringMigrateWithCeph(d *Daemon, r *http.Request, inst inst
 
 		// Trigger a rename in the Ceph driver.
 		args := migration.VolumeSourceArgs{
-			Data: project.Instance(inst.Project(), newName), // Indicate new storage volume name.
+			Data: project.Instance(inst.Project().Name, newName), // Indicate new storage volume name.
 			Info: &migration.Info{Config: srcConfig},
 		}
 
@@ -837,7 +851,7 @@ func instancePostClusteringMigrateWithCeph(d *Daemon, r *http.Request, inst inst
 
 		// Re-link the database entries against the new node name.
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			err := tx.UpdateInstanceNode(ctx, inst.Project(), inst.Name(), newName, newNode, volDBType)
+			err := tx.UpdateInstanceNode(ctx, inst.Project().Name, inst.Name(), newName, newNode, volDBType)
 			if err != nil {
 				return fmt.Errorf("Failed updating cluster member to %q for instance %q: %w", newName, inst.Name(), err)
 			}
@@ -849,13 +863,13 @@ func instancePostClusteringMigrateWithCeph(d *Daemon, r *http.Request, inst inst
 		}
 
 		// Reload instance from database with new state now its been updated.
-		inst, err := instance.LoadByProjectAndName(d.State(), inst.Project(), newName)
+		inst, err := instance.LoadByProjectAndName(d.State(), inst.Project().Name, newName)
 		if err != nil {
 			return fmt.Errorf("Failed loading instance %q: %w", inst.Name(), err)
 		}
 
 		// Create the instance mount point on the target node.
-		target, err := cluster.ConnectIfInstanceIsRemote(d.db.Cluster, inst.Project(), newName, d.endpoints.NetworkCert(), d.serverCert(), r, inst.Type())
+		target, err := cluster.ConnectIfInstanceIsRemote(d.db.Cluster, inst.Project().Name, newName, d.endpoints.NetworkCert(), d.serverCert(), r, inst.Type())
 		if err != nil {
 			return fmt.Errorf("Failed to connect to target node: %w", err)
 		}
@@ -876,7 +890,7 @@ func instancePostClusteringMigrateWithCeph(d *Daemon, r *http.Request, inst inst
 			}
 		} else {
 			// Create the instance mount point.
-			url := api.NewURL().Project(inst.Project()).Path("internal", "cluster", "instance-moved", newName)
+			url := api.NewURL().Project(inst.Project().Name).Path("internal", "cluster", "instance-moved", newName)
 			resp, _, err := target.RawQuery("POST", url.String(), nil, "")
 			if err != nil {
 				return fmt.Errorf("Failed creating mount point on target member: %w", err)
