@@ -2,10 +2,10 @@ package main
 
 import (
 	"fmt"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"path/filepath"
+	"runtime"
 	"sort"
 	"strconv"
 	"sync"
@@ -235,7 +235,7 @@ func instancesStart(s *state.State, instances []instance.Instance) {
 
 	maxAttempts := 3
 
-	// Restart the instances
+	// Start the instances
 	for _, inst := range instances {
 		// Get the instance config.
 		config := inst.ExpandedConfig()
@@ -254,7 +254,7 @@ func instancesStart(s *state.State, instances []instance.Instance) {
 			continue
 		}
 
-		instLogger := logger.AddContext(logger.Log, logger.Ctx{"project": inst.Project(), "instance": inst.Name()})
+		instLogger := logger.AddContext(logger.Log, logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
 
 		// Try to start the instance.
 		var attempt = 0
@@ -270,7 +270,7 @@ func instancesStart(s *state.State, instances []instance.Instance) {
 
 				if attempt >= maxAttempts {
 					// If unable to start after 3 tries, record a warning.
-					warnErr := s.DB.Cluster.UpsertWarningLocalNode(inst.Project(), cluster.TypeInstance, inst.ID(), warningtype.InstanceAutostartFailure, fmt.Sprintf("%v", err))
+					warnErr := s.DB.Cluster.UpsertWarningLocalNode(inst.Project().Name, cluster.TypeInstance, inst.ID(), warningtype.InstanceAutostartFailure, fmt.Sprintf("%v", err))
 					if warnErr != nil {
 						instLogger.Warn("Failed to create instance autostart failure warning", logger.Ctx{"err": warnErr})
 					}
@@ -286,7 +286,7 @@ func instancesStart(s *state.State, instances []instance.Instance) {
 			}
 
 			// Resolve any previous warning.
-			warnErr := warnings.ResolveWarningsByLocalNodeAndProjectAndTypeAndEntity(s.DB.Cluster, inst.Project(), warningtype.InstanceAutostartFailure, cluster.TypeInstance, inst.ID())
+			warnErr := warnings.ResolveWarningsByLocalNodeAndProjectAndTypeAndEntity(s.DB.Cluster, inst.Project().Name, warningtype.InstanceAutostartFailure, cluster.TypeInstance, inst.ID())
 			if warnErr != nil {
 				instLogger.Warn("Failed to resolve instance autostart failure warning", logger.Ctx{"err": warnErr})
 			}
@@ -336,14 +336,14 @@ func instancesOnDisk(s *state.State) ([]instance.Instance, error) {
 		instancetype.VM:        shared.VarPath("virtual-machines"),
 	}
 
-	instanceTypeNames := make(map[instancetype.Type][]os.FileInfo, 2)
+	instanceTypeNames := make(map[instancetype.Type][]os.DirEntry, 2)
 
-	instanceTypeNames[instancetype.Container], err = ioutil.ReadDir(instancePaths[instancetype.Container])
+	instanceTypeNames[instancetype.Container], err = os.ReadDir(instancePaths[instancetype.Container])
 	if err != nil && !os.IsNotExist(err) {
 		return nil, err
 	}
 
-	instanceTypeNames[instancetype.VM], err = ioutil.ReadDir(instancePaths[instancetype.VM])
+	instanceTypeNames[instancetype.VM], err = os.ReadDir(instancePaths[instancetype.VM])
 	if err != nil && !os.IsNotExist(err) {
 		return nil, err
 	}
@@ -377,7 +377,11 @@ func instancesOnDisk(s *state.State) ([]instance.Instance, error) {
 					Config:  make(map[string]string),
 				}
 
-				inst, err = instance.Load(s, *instDBArgs, nil)
+				emptyProject := api.Project{
+					Name: projectName,
+				}
+
+				inst, err = instance.Load(s, *instDBArgs, emptyProject)
 				if err != nil {
 					logger.Warn("Failed loading instance", logger.Ctx{"project": projectName, "instance": instanceName, "err": err})
 					continue
@@ -392,31 +396,20 @@ func instancesOnDisk(s *state.State) ([]instance.Instance, error) {
 }
 
 func instancesShutdown(s *state.State, instances []instance.Instance) {
-	var wg sync.WaitGroup
-
 	sort.Sort(instanceStopList(instances))
 
-	var lastPriority int
-
-	if len(instances) != 0 {
-		lastPriority, _ = strconv.Atoi(instances[0].ExpandedConfig()["boot.stop.priority"])
+	// Limit shutdown concurrency to number of instances or number of CPU cores (which ever is less).
+	var wg sync.WaitGroup
+	instShutdownCh := make(chan instance.Instance)
+	maxConcurrent := runtime.NumCPU()
+	instCount := len(instances)
+	if instCount < maxConcurrent {
+		maxConcurrent = instCount
 	}
 
-	for _, inst := range instances {
-		priority, _ := strconv.Atoi(inst.ExpandedConfig()["boot.stop.priority"])
-
-		// Enforce shutdown priority
-		if priority != lastPriority {
-			lastPriority = priority
-
-			// Wait for instances with higher priority to finish
-			wg.Wait()
-		}
-
-		// Stop the instance if running.
-		if inst.IsRunning() {
-			wg.Add(1)
-			go func(inst instance.Instance) {
+	for i := 0; i < maxConcurrent; i++ {
+		go func(instShutdownCh <-chan instance.Instance) {
+			for inst := range instShutdownCh {
 				// Determine how long to wait for the instance to shutdown cleanly.
 				timeoutSeconds := 30
 				value, ok := inst.ExpandedConfig()["boot.host_shutdown_timeout"]
@@ -426,10 +419,10 @@ func instancesShutdown(s *state.State, instances []instance.Instance) {
 
 				err := inst.Shutdown(time.Second * time.Duration(timeoutSeconds))
 				if err != nil {
-					logger.Warn("Failed shutting down instance, forcefully stopping", logger.Ctx{"project": inst.Project(), "instance": inst.Name(), "err": err})
+					logger.Warn("Failed shutting down instance, forcefully stopping", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "err": err})
 					err = inst.Stop(false)
 					if err != nil {
-						logger.Warn("Failed forcefully stopping instance", logger.Ctx{"project": inst.Project(), "instance": inst.Name(), "err": err})
+						logger.Warn("Failed forcefully stopping instance", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "err": err})
 					}
 				}
 
@@ -441,8 +434,32 @@ func instancesShutdown(s *state.State, instances []instance.Instance) {
 				}
 
 				wg.Done()
-			}(inst)
-		}
+			}
+		}(instShutdownCh)
 	}
+
+	var currentBatchPriority int
+	for i, inst := range instances {
+		// Skip stopped instances.
+		if !inst.IsRunning() {
+			continue
+		}
+
+		priority, _ := strconv.Atoi(inst.ExpandedConfig()["boot.stop.priority"])
+
+		// Shutdown instances in priority batches, logging at the start of each batch.
+		if i == 0 || priority != currentBatchPriority {
+			currentBatchPriority = priority
+
+			// Wait for instances with higher priority to finish before starting next batch.
+			wg.Wait()
+			logger.Info("Stopping instances", logger.Ctx{"stopPriority": currentBatchPriority})
+		}
+
+		wg.Add(1)
+		instShutdownCh <- inst
+	}
+
 	wg.Wait()
+	close(instShutdownCh)
 }
