@@ -3,6 +3,7 @@ package main
 import (
 	"context"
 	"fmt"
+	"net"
 	"net/http"
 	"sort"
 	"strconv"
@@ -284,7 +285,7 @@ func doInstancesGet(d *Daemon, r *http.Request) (any, error) {
 		var err error
 
 		if allProjects {
-			projects, err := dbCluster.GetProjects(context.Background(), tx.Tx(), dbCluster.ProjectFilter{})
+			projects, err := dbCluster.GetProjects(context.Background(), tx.Tx())
 			if err != nil {
 				return err
 			}
@@ -300,12 +301,12 @@ func doInstancesGet(d *Daemon, r *http.Request) (any, error) {
 			filteredProjects = []string{projectName}
 		}
 
-		nodesProjectsInstances, err = tx.GetProjectAndInstanceNamesByNodeAddress(filteredProjects, db.InstanceTypeFilter(instanceType))
+		nodesProjectsInstances, err = tx.GetProjectAndInstanceNamesByNodeAddress(ctx, filteredProjects, instanceType)
 		if err != nil {
 			return err
 		}
 
-		projectInstanceToNodeName, err = tx.GetProjectInstanceToNodeMap(filteredProjects, db.InstanceTypeFilter(instanceType))
+		projectInstanceToNodeName, err = tx.GetProjectInstanceToNodeMap(ctx, filteredProjects, instanceType)
 		if err != nil {
 			return err
 		}
@@ -327,7 +328,7 @@ func doInstancesGet(d *Daemon, r *http.Request) (any, error) {
 			}
 
 			for _, inst := range insts {
-				nodeInstances[[2]string{inst.Project(), inst.Name()}] = inst
+				nodeInstances[[2]string{inst.Project().Name, inst.Name()}] = inst
 			}
 		}
 	}
@@ -447,6 +448,7 @@ func doInstancesGet(d *Daemon, r *http.Request) (any, error) {
 				threads = len(projectsInstances)
 			}
 
+			hostInterfaces, _ := net.Interfaces()
 			queue := make(chan [2]string, threads)
 
 			for i := 0; i < threads; i++ {
@@ -475,9 +477,9 @@ func doInstancesGet(d *Daemon, r *http.Request) (any, error) {
 							continue
 						}
 
-						c, _, err := inst.RenderFull()
+						c, _, err := inst.RenderFull(hostInterfaces)
 						if err != nil {
-							logger.Error("Unable to list instance", logger.Ctx{"project": inst.Project(), "instance": inst.Name(), "err": err})
+							logger.Error("Unable to list instance", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "err": err})
 							resultFullListAppend(projectInstance, api.InstanceFull{}, err)
 						} else {
 							resultFullListAppend(projectInstance, *c, err)
