@@ -5,18 +5,19 @@ import (
 	"os"
 	"time"
 
+	"github.com/canonical/go-dqlite"
 	"github.com/spf13/cobra"
 
+	"github.com/lxc/lxd/lxd/daemon"
+	"github.com/lxc/lxd/lxd/events"
+	"github.com/lxc/lxd/lxd/operations"
+	"github.com/lxc/lxd/lxd/response"
+	"github.com/lxc/lxd/lxd/rsync"
 	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/logging"
 	"github.com/lxc/lxd/shared/version"
 )
 
-// Global variables
-var debug bool
-var verbose bool
-
-// Initialize the random number generator
+// Initialize the random number generator.
 func init() {
 	rand.Seed(time.Now().UTC().UnixNano())
 }
@@ -35,9 +36,26 @@ type cmdGlobal struct {
 }
 
 func (c *cmdGlobal) Run(cmd *cobra.Command, args []string) error {
+	// Configure dqlite to *not* disable internal SQLite locking, since we
+	// use SQLite both through dqlite and through go-dqlite, potentially
+	// from different threads at the same time. We need to call this
+	// function as early as possible since this is a global setting in
+	// SQLite, which can't be changed afterwise.
+	err := dqlite.ConfigMultiThread()
+	if err != nil {
+		return err
+	}
+
 	// Set logging global variables
-	debug = c.flagLogVerbose
-	verbose = c.flagLogDebug
+	daemon.Debug = c.flagLogDebug
+	rsync.Debug = c.flagLogDebug
+	daemon.Verbose = c.flagLogVerbose
+
+	// Set debug for the operations package
+	operations.Init(daemon.Debug)
+
+	// Set debug for the response package
+	response.Init(daemon.Debug)
 
 	// Setup logger
 	syslog := ""
@@ -45,21 +63,31 @@ func (c *cmdGlobal) Run(cmd *cobra.Command, args []string) error {
 		syslog = "lxd"
 	}
 
-	handler := eventsHandler{}
-	log, err := logging.GetLogger(syslog, c.flagLogFile, c.flagLogVerbose, c.flagLogDebug, handler)
+	err = logger.InitLogger(c.flagLogFile, syslog, c.flagLogVerbose, c.flagLogDebug, events.NewEventHandler())
 	if err != nil {
 		return err
 	}
-	logger.Log = log
 
 	return nil
 }
 
+// rawArgs returns the raw unprocessed arguments from os.Args after the command name arg is found.
+func (c *cmdGlobal) rawArgs(cmd *cobra.Command) []string {
+	for i, arg := range os.Args {
+		if arg == cmd.Name() && len(os.Args)-1 > i {
+			return os.Args[i+1:]
+		}
+	}
+
+	return []string{}
+}
+
 func main() {
 	// daemon command (main)
 	daemonCmd := cmdDaemon{}
 	app := daemonCmd.Command()
 	app.SilenceUsage = true
+	app.CompletionOptions = cobra.CompletionOptions{DisableDefaultCmd: true}
 
 	// Workaround for main command
 	app.Args = cobra.ArbitraryArgs
@@ -92,6 +120,10 @@ func main() {
 	forkconsoleCmd := cmdForkconsole{global: &globalCmd}
 	app.AddCommand(forkconsoleCmd.Command())
 
+	// forkdns sub-command
+	forkDNSCmd := cmdForkDNS{global: &globalCmd}
+	app.AddCommand(forkDNSCmd.Command())
+
 	// forkexec sub-command
 	forkexecCmd := cmdForkexec{global: &globalCmd}
 	app.AddCommand(forkexecCmd.Command())
@@ -100,10 +132,22 @@ func main() {
 	forkfileCmd := cmdForkfile{global: &globalCmd}
 	app.AddCommand(forkfileCmd.Command())
 
+	// forklimits sub-command
+	forklimitsCmd := cmdForklimits{global: &globalCmd}
+	app.AddCommand(forklimitsCmd.Command())
+
 	// forkmigrate sub-command
 	forkmigrateCmd := cmdForkmigrate{global: &globalCmd}
 	app.AddCommand(forkmigrateCmd.Command())
 
+	// forksyscall sub-command
+	forksyscallCmd := cmdForksyscall{global: &globalCmd}
+	app.AddCommand(forksyscallCmd.Command())
+
+	// forkcoresched sub-command
+	forkcoreschedCmd := cmdForkcoresched{global: &globalCmd}
+	app.AddCommand(forkcoreschedCmd.Command())
+
 	// forkmount sub-command
 	forkmountCmd := cmdForkmount{global: &globalCmd}
 	app.AddCommand(forkmountCmd.Command())
@@ -120,6 +164,14 @@ func main() {
 	forkstartCmd := cmdForkstart{global: &globalCmd}
 	app.AddCommand(forkstartCmd.Command())
 
+	// forkuevent sub-command
+	forkueventCmd := cmdForkuevent{global: &globalCmd}
+	app.AddCommand(forkueventCmd.Command())
+
+	// forkzfs sub-command
+	forkzfsCmd := cmdForkZFS{global: &globalCmd}
+	app.AddCommand(forkzfsCmd.Command())
+
 	// import sub-command
 	importCmd := cmdImport{global: &globalCmd}
 	app.AddCommand(importCmd.Command())
@@ -140,6 +192,10 @@ func main() {
 	netcatCmd := cmdNetcat{global: &globalCmd}
 	app.AddCommand(netcatCmd.Command())
 
+	// recover sub-command
+	recoverCmd := cmdRecover{global: &globalCmd}
+	app.AddCommand(recoverCmd.Command())
+
 	// shutdown sub-command
 	shutdownCmd := cmdShutdown{global: &globalCmd}
 	app.AddCommand(shutdownCmd.Command())
@@ -156,6 +212,10 @@ func main() {
 	waitreadyCmd := cmdWaitready{global: &globalCmd}
 	app.AddCommand(waitreadyCmd.Command())
 
+	// cluster sub-command
+	clusterCmd := cmdCluster{global: &globalCmd}
+	app.AddCommand(clusterCmd.Command())
+
 	// Run the main command and handle errors
 	err := app.Execute()
 	if err != nil {
