@@ -3,21 +3,23 @@ package main
 import (
 	"fmt"
 
-	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 
 	"github.com/lxc/lxd/client"
+	"github.com/lxc/lxd/lxd/project"
+	storageDrivers "github.com/lxc/lxd/lxd/storage/drivers"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 )
 
-func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServer) (*cmdInitData, error) {
-	// Sanity checks
-	if c.flagStorageBackend != "" && !shared.StringInSlice(c.flagStorageBackend, supportedStoragePoolDrivers) {
+func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.InstanceServer, server *api.Server) (*api.InitPreseed, error) {
+	// Quick checks.
+	if c.flagStorageBackend != "" && !shared.StringInSlice(c.flagStorageBackend, storageDrivers.AllDriverNames()) {
 		return nil, fmt.Errorf("The requested backend '%s' isn't supported by lxd init", c.flagStorageBackend)
 	}
 
-	if c.flagStorageBackend != "" && !shared.StringInSlice(c.flagStorageBackend, c.availableStorageDrivers("all")) {
+	if c.flagStorageBackend != "" && !shared.StringInSlice(c.flagStorageBackend, util.AvailableStorageDrivers(server.Environment.StorageSupportedDrivers, util.PoolTypeAny)) {
 		return nil, fmt.Errorf("The requested backend '%s' isn't available on your system (missing tools)", c.flagStorageBackend)
 	}
 
@@ -43,7 +45,7 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 
 	storagePools, err := d.GetStoragePoolNames()
 	if err != nil {
-		return nil, errors.Wrap(err, "Failed to retrieve list of storage pools")
+		return nil, fmt.Errorf("Failed to retrieve list of storage pools: %w", err)
 	}
 
 	if len(storagePools) > 0 && (c.flagStorageBackend != "" || c.flagStorageDevice != "" || c.flagStorageLoopSize != -1 || c.flagStoragePool != "") {
@@ -56,16 +58,16 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 	}
 
 	if c.flagNetworkPort == -1 {
-		c.flagNetworkPort = 8443
+		c.flagNetworkPort = shared.HTTPSDefaultPort
 	}
 
 	// Fill in the node configuration
-	config := initDataNode{}
-	config.Config = map[string]interface{}{}
+	config := api.InitLocalPreseed{}
+	config.Config = map[string]any{}
 
 	// Network listening
 	if c.flagNetworkAddress != "" {
-		config.Config["core.https_address"] = fmt.Sprintf("%s:%d", c.flagNetworkAddress, c.flagNetworkPort)
+		config.Config["core.https_address"] = util.CanonicalNetworkAddressFromAddressAndPort(c.flagNetworkAddress, c.flagNetworkPort, shared.HTTPSDefaultPort)
 
 		if c.flagTrustPassword != "" {
 			config.Config["core.trust_password"] = c.flagTrustPassword
@@ -79,6 +81,7 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 			Name:   "default",
 			Driver: c.flagStorageBackend,
 		}
+
 		pool.Config = map[string]string{}
 
 		if c.flagStorageDevice != "" {
@@ -114,7 +117,7 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 	// Network configuration
 	networks, err := d.GetNetworks()
 	if err != nil {
-		return nil, errors.Wrap(err, "Failed to retrieve list of networks")
+		return nil, fmt.Errorf("Failed to retrieve list of networks: %w", err)
 	}
 
 	// Extract managed networks
@@ -151,8 +154,9 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 		}
 
 		// Define the new network
-		network := api.NetworksPost{}
+		network := api.InitNetworksProjectPost{}
 		network.Name = fmt.Sprintf("lxdbr%d", idx)
+		network.Project = project.Default
 		config.Networks = append(config.Networks, network)
 
 		// Add it to the profile
@@ -163,8 +167,7 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 					Devices: map[string]map[string]string{
 						"eth0": {
 							"type":    "nic",
-							"nictype": "bridged",
-							"parent":  network.Name,
+							"network": network.Name,
 							"name":    "eth0",
 						},
 					},
@@ -173,12 +176,11 @@ func (c *cmdInit) RunAuto(cmd *cobra.Command, args []string, d lxd.ContainerServ
 		} else {
 			config.Profiles[0].Devices["eth0"] = map[string]string{
 				"type":    "nic",
-				"nictype": "bridged",
-				"parent":  network.Name,
+				"network": network.Name,
 				"name":    "eth0",
 			}
 		}
 	}
 
-	return &cmdInitData{Node: config}, nil
+	return &api.InitPreseed{Node: config}, nil
 }
