@@ -3,11 +3,10 @@ package main
 import (
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 
 	"github.com/olekukonko/tablewriter"
-	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 
 	"github.com/lxc/lxd/client"
@@ -38,10 +37,10 @@ func (c *cmdSql) Command() *cobra.Command {
   If <query> is the special value "-", then the query is read from
   standard input.
 
-  If <query> is the special value ".dump", the the command returns a SQL text
+  If <query> is the special value ".dump", the command returns a SQL text
   dump of the given database.
 
-  If <query> is the special value ".schema", the the command returns the SQL
+  If <query> is the special value ".schema", the command returns the SQL
   text schema of the given database.
 
   This internal command is mostly useful for debugging and disaster
@@ -59,7 +58,7 @@ func (c *cmdSql) Command() *cobra.Command {
 
 func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 	if len(args) != 2 {
-		cmd.Help()
+		_ = cmd.Help()
 
 		if len(args) == 0 {
 			return nil
@@ -72,22 +71,27 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 	query := args[1]
 
 	if !shared.StringInSlice(database, []string{"local", "global"}) {
-		cmd.Help()
+		_ = cmd.Help()
 
 		return fmt.Errorf("Invalid database type")
 	}
 
 	if query == "-" {
 		// Read from stdin
-		bytes, err := ioutil.ReadAll(os.Stdin)
+		bytes, err := io.ReadAll(os.Stdin)
 		if err != nil {
-			return errors.Wrap(err, "Failed to read from stdin")
+			return fmt.Errorf("Failed to read from stdin: %w", err)
 		}
+
 		query = string(bytes)
 	}
 
 	// Connect to LXD
-	d, err := lxd.ConnectLXDUnix("", nil)
+	lxdArgs := lxd.ConnectionArgs{
+		SkipGetServer: true,
+	}
+
+	d, err := lxd.ConnectLXDUnix("", &lxdArgs)
 	if err != nil {
 		return err
 	}
@@ -97,16 +101,19 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 		if query == ".schema" {
 			url += "&schema=1"
 		}
+
 		response, _, err := d.RawQuery("GET", url, nil, "")
 		if err != nil {
-			return errors.Wrap(err, "failed to request dump")
+			return fmt.Errorf("failed to request dump: %w", err)
 		}
+
 		dump := internalSQLDump{}
 		err = json.Unmarshal(response.Metadata, &dump)
 		if err != nil {
-			return errors.Wrap(err, "failed to parse dump response")
+			return fmt.Errorf("failed to parse dump response: %w", err)
 		}
-		fmt.Printf(dump.Text)
+
+		fmt.Print(dump.Text)
 		return nil
 	}
 
@@ -114,6 +121,7 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 		Database: database,
 		Query:    query,
 	}
+
 	response, _, err := d.RawQuery("POST", "/internal/sql", data, "")
 	if err != nil {
 		return err
@@ -124,15 +132,18 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		return err
 	}
+
 	for i, result := range batch.Results {
 		if len(batch.Results) > 1 {
 			fmt.Printf("=> Query %d:\n\n", i)
 		}
+
 		if result.Type == "select" {
 			sqlPrintSelectResult(result)
 		} else {
 			fmt.Printf("Rows affected: %d\n", result.RowsAffected)
 		}
+
 		if len(batch.Results) > 1 {
 			fmt.Printf("\n")
 		}
@@ -151,7 +162,9 @@ func sqlPrintSelectResult(result internalSQLResult) {
 		for _, col := range row {
 			data = append(data, fmt.Sprintf("%v", col))
 		}
+
 		table.Append(data)
 	}
+
 	table.Render()
 }
