@@ -1,21 +1,23 @@
 package main
 
 import (
+	"context"
 	"fmt"
-	"io/ioutil"
 	"os"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/sys"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
+	"golang.org/x/sys/unix"
 
+	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/sys"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/idmap"
 )
 
 func mockStartDaemon() (*Daemon, error) {
-	d := DefaultDaemon()
+	d := defaultDaemon()
 	d.os.MockMode = true
 
 	// Setup test certificates. We re-use the ones already on disk under
@@ -26,7 +28,8 @@ func mockStartDaemon() (*Daemon, error) {
 		return nil, err
 	}
 
-	if err := d.Init(); err != nil {
+	err = d.Init()
+	if err != nil {
 		return nil, err
 	}
 
@@ -48,70 +51,66 @@ type lxdTestSuite struct {
 const lxdTestSuiteDefaultStoragePool string = "lxdTestrunPool"
 
 func (suite *lxdTestSuite) SetupTest() {
-	tmpdir, err := ioutil.TempDir("", "lxd_testrun_")
+	tmpdir, err := os.MkdirTemp("", "lxd_testrun_")
 	if err != nil {
-		suite.T().Fatalf("failed to create temp dir: %v", err)
+		suite.T().Errorf("failed to create temp dir: %v", err)
 	}
+
 	suite.tmpdir = tmpdir
 
-	if err := os.Setenv("LXD_DIR", suite.tmpdir); err != nil {
-		suite.T().Fatalf("failed to set LXD_DIR: %v", err)
+	err = os.Setenv("LXD_DIR", suite.tmpdir)
+	if err != nil {
+		suite.T().Errorf("failed to set LXD_DIR: %v", err)
 	}
 
 	suite.d, err = mockStartDaemon()
 	if err != nil {
-		suite.T().Fatalf("failed to start daemon: %v", err)
+		suite.T().Errorf("failed to start daemon: %v", err)
 	}
 
 	// Create default storage pool. Make sure that we don't pass a nil to
 	// the next function.
 	poolConfig := map[string]string{}
 
-	mockStorage, _ := storageTypeToString(storageTypeMock)
 	// Create the database entry for the storage pool.
 	poolDescription := fmt.Sprintf("%s storage pool", lxdTestSuiteDefaultStoragePool)
-	_, err = dbStoragePoolCreateAndUpdateCache(suite.d.cluster, lxdTestSuiteDefaultStoragePool, poolDescription, mockStorage, poolConfig)
+	_, err = dbStoragePoolCreateAndUpdateCache(suite.d.State(), lxdTestSuiteDefaultStoragePool, poolDescription, "mock", poolConfig)
 	if err != nil {
-		suite.T().Fatalf("failed to create default storage pool: %v", err)
+		suite.T().Errorf("failed to create default storage pool: %v", err)
 	}
 
 	rootDev := map[string]string{}
-	rootDev["type"] = "disk"
 	rootDev["path"] = "/"
 	rootDev["pool"] = lxdTestSuiteDefaultStoragePool
-	devicesMap := map[string]map[string]string{}
-	devicesMap["root"] = rootDev
-
-	defaultID, _, err := suite.d.cluster.ProfileGet("default")
-	if err != nil {
-		suite.T().Fatalf("failed to get default profile: %v", err)
+	device := cluster.Device{
+		Name:   "root",
+		Type:   cluster.TypeDisk,
+		Config: rootDev,
 	}
 
-	tx, err := suite.d.cluster.Begin()
+	err = suite.d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		profile, err := cluster.GetProfile(ctx, tx.Tx(), "default", "default")
+		if err != nil {
+			return err
+		}
+
+		return cluster.UpdateProfileDevices(ctx, tx.Tx(), int64(profile.ID), map[string]cluster.Device{"root": device})
+	})
 	if err != nil {
-		suite.T().Fatalf("failed to begin transaction: %v", err)
+		suite.T().Errorf("failed to update default profile: %v", err)
 	}
 
-	err = db.DevicesAdd(tx, "profile", defaultID, devicesMap)
-	if err != nil {
-		tx.Rollback()
-		suite.T().Fatalf("failed to rollback transaction: %v", err)
-	}
-
-	err = tx.Commit()
-	if err != nil {
-		suite.T().Fatalf("failed to commit transaction: %v", err)
-	}
 	suite.Req = require.New(suite.T())
 }
 
 func (suite *lxdTestSuite) TearDownTest() {
-	err := suite.d.Stop()
+	err := suite.d.Stop(context.Background(), unix.SIGQUIT)
 	if err != nil {
-		suite.T().Fatalf("failed to stop daemon: %v", err)
+		suite.T().Errorf("failed to stop daemon: %v", err)
 	}
+
 	err = os.RemoveAll(suite.tmpdir)
 	if err != nil {
-		suite.T().Fatalf("failed to remove temp dir: %v", err)
+		suite.T().Errorf("failed to remove temp dir: %v", err)
 	}
 }
