@@ -14,15 +14,27 @@ import (
 	"os"
 	"strings"
 	"sync"
+	"time"
 
-	"github.com/golang/protobuf/proto"
 	"github.com/gorilla/websocket"
+	"google.golang.org/protobuf/proto"
 
+	"github.com/lxc/lxd/lxd/instance"
+	"github.com/lxc/lxd/lxd/instance/instancetype"
 	"github.com/lxc/lxd/lxd/migration"
+	"github.com/lxc/lxd/lxd/operations"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/tcp"
 )
 
+// migrationControlResponse encapsulates migration.MigrationControl with a receive error.
+type migrationControlResponse struct {
+	migration.MigrationControl
+	err error
+}
+
 type migrationFields struct {
 	controlSecret string
 	controlConn   *websocket.Conn
@@ -35,12 +47,13 @@ type migrationFields struct {
 	fsConn   *websocket.Conn
 
 	// container specific fields
-	live          bool
-	containerOnly bool
-	container     container
+	live         bool
+	instanceOnly bool
+	instance     instance.Instance
 
 	// storage specific fields
-	storage storage
+	volumeOnly        bool
+	allowInconsistent bool
 }
 
 func (c *migrationFields) send(m proto.Message) error {
@@ -75,9 +88,11 @@ func (c *migrationFields) disconnect() {
 
 	c.controlLock.Lock()
 	if c.controlConn != nil {
-		c.controlConn.WriteMessage(websocket.CloseMessage, closeMsg)
+		_ = c.controlConn.WriteMessage(websocket.CloseMessage, closeMsg)
+		_ = c.controlConn.Close()
 		c.controlConn = nil /* don't close twice */
 	}
+
 	c.controlLock.Unlock()
 
 	/* Below we just Close(), which doesn't actually write to the
@@ -89,36 +104,40 @@ func (c *migrationFields) disconnect() {
 	 * anyway.
 	 */
 	if c.fsConn != nil {
-		c.fsConn.Close()
+		_ = c.fsConn.Close()
 	}
 
 	if c.criuConn != nil {
-		c.criuConn.Close()
+		_ = c.criuConn.Close()
 	}
 }
 
 func (c *migrationFields) sendControl(err error) {
 	c.controlLock.Lock()
-	defer c.controlLock.Unlock()
+	if c.controlConn != nil {
+		migration.ProtoSendControl(c.controlConn, err)
+	}
 
-	migration.ProtoSendControl(c.controlConn, err)
+	c.controlLock.Unlock()
 
 	if err != nil {
 		c.disconnect()
 	}
 }
 
-func (c *migrationFields) controlChannel() <-chan migration.MigrationControl {
-	ch := make(chan migration.MigrationControl)
+func (c *migrationFields) controlChannel() <-chan *migrationControlResponse {
+	ch := make(chan *migrationControlResponse)
 	go func() {
-		msg := migration.MigrationControl{}
-		err := c.recv(&msg)
+		resp := migrationControlResponse{}
+		err := c.recv(&resp.MigrationControl)
 		if err != nil {
-			logger.Debugf("Got error reading migration control socket %s", err)
-			close(ch)
+			resp.err = err
+			ch <- &resp
+
 			return
 		}
-		ch <- msg
+
+		ch <- &resp
 	}()
 
 	return ch
@@ -127,10 +146,10 @@ func (c *migrationFields) controlChannel() <-chan migration.MigrationControl {
 type migrationSourceWs struct {
 	migrationFields
 
-	allConnected chan bool
+	allConnected chan struct{}
 }
 
-func (s *migrationSourceWs) Metadata() interface{} {
+func (s *migrationSourceWs) Metadata() any {
 	secrets := shared.Jmap{
 		"control": s.controlSecret,
 		"fs":      s.fsSecret,
@@ -143,7 +162,7 @@ func (s *migrationSourceWs) Metadata() interface{} {
 	return secrets
 }
 
-func (s *migrationSourceWs) Connect(op *operation, r *http.Request, w http.ResponseWriter) error {
+func (s *migrationSourceWs) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
 		return fmt.Errorf("missing secret")
@@ -170,12 +189,32 @@ func (s *migrationSourceWs) Connect(op *operation, r *http.Request, w http.Respo
 		return err
 	}
 
+	// Set TCP timeout options.
+	remoteTCP, _ := tcp.ExtractConn(c.UnderlyingConn())
+	if remoteTCP != nil {
+		err = tcp.SetTimeouts(remoteTCP, 0)
+		if err != nil {
+			logger.Error("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
+		}
+	}
+
 	*conn = c
 
-	if s.controlConn != nil && (!s.live || s.criuConn != nil) && s.fsConn != nil {
-		s.allConnected <- true
+	// Check criteria for considering all channels to be connected.
+	if s.instance != nil && s.instance.Type() == instancetype.Container && s.live && s.criuConn == nil {
+		return nil
 	}
 
+	if s.controlConn == nil {
+		return nil
+	}
+
+	if s.fsConn == nil {
+		return nil
+	}
+
+	close(s.allConnected)
+
 	return nil
 }
 
@@ -201,8 +240,9 @@ func (s *migrationSourceWs) ConnectTarget(certificate string, operation string,
 	}
 
 	dialer := websocket.Dialer{
-		TLSClientConfig: config,
-		NetDial:         shared.RFC3493Dialer,
+		TLSClientConfig:  config,
+		NetDialContext:   shared.RFC3493Dialer,
+		HandshakeTimeout: time.Second * 5,
 	}
 
 	for name, secret := range websockets {
@@ -222,9 +262,9 @@ func (s *migrationSourceWs) ConnectTarget(certificate string, operation string,
 		query := url.Values{"secret": []string{secret}}
 
 		// The URL is a https URL to the operation, mangle to be a wss URL to the secret
-		wsUrl := fmt.Sprintf("wss://%s/websocket?%s", strings.TrimPrefix(operation, "https://"), query.Encode())
+		wsURL := fmt.Sprintf("wss://%s/websocket?%s", strings.TrimPrefix(operation, "https://"), query.Encode())
 
-		wsConn, _, err := dialer.Dial(wsUrl, http.Header{})
+		wsConn, _, err := dialer.Dial(wsURL, http.Header{})
 		if err != nil {
 			return err
 		}
@@ -232,7 +272,7 @@ func (s *migrationSourceWs) ConnectTarget(certificate string, operation string,
 		*conn = wsConn
 	}
 
-	s.allConnected <- true
+	close(s.allConnected)
 
 	return nil
 }
@@ -247,36 +287,42 @@ type migrationSink struct {
 
 	url          string
 	dialer       websocket.Dialer
-	allConnected chan bool
+	allConnected chan struct{}
 	push         bool
-	rsyncArgs    []string
+	refresh      bool
 }
 
+// MigrationSinkArgs arguments to configure migration sink.
 type MigrationSinkArgs struct {
-	Url     string
+	// General migration fields
 	Dialer  websocket.Dialer
-	Secrets map[string]string
 	Push    bool
+	Secrets map[string]string
+	URL     string
 
-	// container specific fields
-	Live          bool
-	Container     container
-	ContainerOnly bool
+	// Instance specific fields
+	Instance     instance.Instance
+	InstanceOnly bool
+	Idmap        *idmap.IdmapSet
+	Live         bool
+	Refresh      bool
+	Snapshots    []*migration.Snapshot
 
-	// storage specific fields
-	Storage storage
+	// Storage specific fields
+	VolumeOnly bool
+	VolumeSize int64
 
-	// transport specific fields
-	RsyncArgs []string
+	// Transport specific fields
+	RsyncFeatures []string
 }
 
-func (c *migrationSink) connectWithSecret(secret string) (*websocket.Conn, error) {
+func (s *migrationSink) connectWithSecret(secret string) (*websocket.Conn, error) {
 	query := url.Values{"secret": []string{secret}}
 
 	// The URL is a https URL to the operation, mangle to be a wss URL to the secret
-	wsUrl := fmt.Sprintf("wss://%s/websocket?%s", strings.TrimPrefix(c.url, "https://"), query.Encode())
+	wsURL := fmt.Sprintf("wss://%s/websocket?%s", strings.TrimPrefix(s.url, "https://"), query.Encode())
 
-	conn, _, err := c.dialer.Dial(wsUrl, http.Header{})
+	conn, _, err := s.dialer.Dial(wsURL, http.Header{})
 	if err != nil {
 		return nil, err
 	}
@@ -284,7 +330,8 @@ func (c *migrationSink) connectWithSecret(secret string) (*websocket.Conn, error
 	return conn, err
 }
 
-func (s *migrationSink) Metadata() interface{} {
+// Metadata returns metadata for the migration sink.
+func (s *migrationSink) Metadata() any {
 	secrets := shared.Jmap{
 		"control": s.dest.controlSecret,
 		"fs":      s.dest.fsSecret,
@@ -297,7 +344,8 @@ func (s *migrationSink) Metadata() interface{} {
 	return secrets
 }
 
-func (s *migrationSink) Connect(op *operation, r *http.Request, w http.ResponseWriter) error {
+// Connect connects to the migration source.
+func (s *migrationSink) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
 		return fmt.Errorf("missing secret")
@@ -325,9 +373,20 @@ func (s *migrationSink) Connect(op *operation, r *http.Request, w http.ResponseW
 
 	*conn = c
 
-	if s.dest.controlConn != nil && (!s.dest.live || s.dest.criuConn != nil) && s.dest.fsConn != nil {
-		s.allConnected <- true
+	// Check criteria for considering all channels to be connected.
+	if s.src.instance != nil && s.src.instance.Type() == instancetype.Container && s.dest.live && s.dest.criuConn == nil {
+		return nil
 	}
 
+	if s.dest.controlConn == nil {
+		return nil
+	}
+
+	if s.dest.fsConn == nil {
+		return nil
+	}
+
+	close(s.allConnected)
+
 	return nil
 }
