@@ -1,9 +1,7 @@
 package main
 
 import (
-	"encoding/binary"
 	"fmt"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"os/exec"
@@ -11,6 +9,7 @@ import (
 	"strconv"
 	"time"
 
+	"github.com/checkpoint-restore/go-criu/v6/crit"
 	"github.com/gorilla/websocket"
 	liblxc "github.com/lxc/go-lxc"
 	"google.golang.org/protobuf/proto"
@@ -209,39 +208,14 @@ func (s *migrationSourceWs) checkForPreDumpSupport() (bool, int) {
 // The function readCriuStatsDump() reads the CRIU 'stats-dump' file
 // in path and returns the pages_written, pages_skipped_parent, error.
 func readCriuStatsDump(path string) (uint64, uint64, error) {
-	statsDump := shared.AddSlash(path) + "stats-dump"
-	in, err := ioutil.ReadFile(statsDump)
+	// Get dump statistics with crit
+	dumpStats, err := crit.GetDumpStats(path)
 	if err != nil {
-		logger.Errorf("Error reading CRIU's 'stats-dump' file: %s", err.Error())
-		return 0, 0, err
-	}
-
-	// According to the CRIU file image format it starts with two magic values.
-	// First magic IMG_SERVICE: 1427134784
-	if binary.LittleEndian.Uint32(in[0:4]) != 1427134784 {
-		msg := "IMG_SERVICE(1427134784) criu magic not found"
-		logger.Errorf(msg)
-		return 0, 0, fmt.Errorf(msg)
-	}
-	// Second magic STATS: 1460220678
-	if binary.LittleEndian.Uint32(in[4:8]) != 1460220678 {
-		msg := "STATS(1460220678) criu magic not found"
-		logger.Errorf(msg)
-		return 0, 0, fmt.Errorf(msg)
-	}
-
-	// Next, read the size of the image payload
-	size := binary.LittleEndian.Uint32(in[8:12])
-
-	statsEntry := &migration.StatsEntry{}
-	if err = proto.Unmarshal(in[12:12+size], statsEntry); err != nil {
 		logger.Errorf("Failed to parse CRIU's 'stats-dump' file: %s", err.Error())
 		return 0, 0, err
 	}
 
-	written := statsEntry.GetDump().GetPagesWritten()
-	skipped := statsEntry.GetDump().GetPagesSkippedParent()
-	return written, skipped, nil
+	return dumpStats.GetPagesWritten(), dumpStats.GetPagesSkippedParent(), nil
 }
 
 type preDumpLoopArgs struct {
@@ -349,7 +323,7 @@ func (s *migrationSourceWs) preDumpLoop(state *state.State, args *preDumpLoopArg
 }
 
 func (s *migrationSourceWs) Do(state *state.State, migrateOp *operations.Operation) error {
-	l := logger.AddContext(logger.Log, logger.Ctx{"project": s.instance.Project(), "instance": s.instance.Name()})
+	l := logger.AddContext(logger.Log, logger.Ctx{"project": s.instance.Project().Name, "instance": s.instance.Name()})
 
 	l.Info("Waiting for migration channel connections on source")
 
@@ -529,11 +503,14 @@ func (s *migrationSourceWs) Do(state *state.State, migrateOp *operations.Operati
 		}
 	}
 
-	// If s.live is true or Criu is set to CRIUTYPE_NONE rather than nil, it indicates that the
-	// source instance is running and that we should do a two stage transfer to minimize downtime.
-	// Indicate this info to the storage driver so that it can alter its behaviour if needed.
-	if s.instance.Type() == instancetype.Container {
-		volSourceArgs.MultiSync = s.live || (respHeader.Criu != nil && *respHeader.Criu == migration.CRIUType_NONE)
+	// If s.live is true or Criu is set to CRIUTYPE_NONE rather than nil, it indicates that the source instance
+	// is running, and if we are doing a non-optimized transfer (i.e using rsync or raw block transfer) then we
+	// should do a two stage transfer to minimize downtime.
+	instanceRunning := s.live || (respHeader.Criu != nil && *respHeader.Criu == migration.CRIUType_NONE)
+	nonOptimizedMigration := volSourceArgs.MigrationType.FSType == migration.MigrationFSType_RSYNC || volSourceArgs.MigrationType.FSType == migration.MigrationFSType_BLOCK_AND_RSYNC
+	if s.instance.Type() == instancetype.Container && instanceRunning && nonOptimizedMigration {
+		// Indicate this info to the storage driver so that it can alter its behaviour if needed.
+		volSourceArgs.MultiSync = true
 	}
 
 	if s.instance.Type() == instancetype.VM && s.live {
@@ -558,7 +535,7 @@ func (s *migrationSourceWs) Do(state *state.State, migrateOp *operations.Operati
 			return abort(fmt.Errorf("Formats other than criu rsync not understood"))
 		}
 
-		checkpointDir, err := ioutil.TempDir("", "lxd_checkpoint_")
+		checkpointDir, err := os.MkdirTemp("", "lxd_checkpoint_")
 		if err != nil {
 			return abort(err)
 		}
@@ -583,7 +560,7 @@ func (s *migrationSourceWs) Do(state *state.State, migrateOp *operations.Operati
 
 			actionScriptOp, err := operations.OperationCreate(
 				state,
-				s.instance.Project(),
+				s.instance.Project().Name,
 				operations.OperationClassWebsocket,
 				operationtype.InstanceLiveMigrate,
 				nil,
@@ -828,7 +805,7 @@ func newMigrationSink(args *MigrationSinkArgs) (*migrationSink, error) {
 }
 
 func (c *migrationSink) Do(state *state.State, revert *revert.Reverter, migrateOp *operations.Operation) error {
-	l := logger.AddContext(logger.Log, logger.Ctx{"push": c.push, "project": c.src.instance.Project(), "instance": c.src.instance.Name()})
+	l := logger.AddContext(logger.Log, logger.Ctx{"push": c.push, "project": c.src.instance.Project().Name, "instance": c.src.instance.Name()})
 
 	var err error
 
@@ -1026,37 +1003,57 @@ func (c *migrationSink) Do(state *state.State, revert *revert.Reverter, migrateO
 	respHeader.Criu = criuType
 
 	if c.refresh {
-		// Get our existing snapshots.
+		// Get the remote snapshots on the source.
+		sourceSnapshots := offerHeader.GetSnapshots()
+		sourceSnapshotComparable := make([]storagePools.ComparableSnapshot, 0, len(sourceSnapshots))
+		for _, sourceSnap := range sourceSnapshots {
+			sourceSnapshotComparable = append(sourceSnapshotComparable, storagePools.ComparableSnapshot{
+				Name:         sourceSnap.GetName(),
+				CreationDate: time.Unix(sourceSnap.GetCreationDate(), 0),
+			})
+		}
+
+		// Get existing snapshots on the local target.
 		targetSnapshots, err := c.src.instance.Snapshots()
 		if err != nil {
 			controller(err)
 			return err
 		}
 
-		// Get the remote snapshots.
-		sourceSnapshots := offerHeader.GetSnapshots()
+		targetSnapshotsComparable := make([]storagePools.ComparableSnapshot, 0, len(targetSnapshots))
+		for _, targetSnap := range targetSnapshots {
+			_, targetSnapName, _ := api.GetParentAndSnapshotName(targetSnap.Name())
+
+			targetSnapshotsComparable = append(targetSnapshotsComparable, storagePools.ComparableSnapshot{
+				Name:         targetSnapName,
+				CreationDate: targetSnap.CreationDate(),
+			})
+		}
 
 		// Compare the two sets.
-		syncSnapshots, deleteSnapshots := migrationCompareSnapshots(sourceSnapshots, targetSnapshots)
+		syncSourceSnapshotIndexes, deleteTargetSnapshotIndexes := storagePools.CompareSnapshots(sourceSnapshotComparable, targetSnapshotsComparable)
 
-		// Delete the extra local ones.
-		for _, snap := range deleteSnapshots {
-			err := snap.Delete(true)
+		// Delete the extra local snapshots first.
+		for _, deleteTargetSnapshotIndex := range deleteTargetSnapshotIndexes {
+			err := targetSnapshots[deleteTargetSnapshotIndex].Delete(true)
 			if err != nil {
 				controller(err)
 				return err
 			}
 		}
 
-		snapshotNames := []string{}
-		for _, snap := range syncSnapshots {
-			snapshotNames = append(snapshotNames, snap.GetName())
+		// Only request to send the snapshots that need updating.
+		syncSnapshotNames := make([]string, 0, len(syncSourceSnapshotIndexes))
+		syncSnapshots := make([]*migration.Snapshot, 0, len(syncSourceSnapshotIndexes))
+		for _, syncSourceSnapshotIndex := range syncSourceSnapshotIndexes {
+			syncSnapshotNames = append(syncSnapshotNames, sourceSnapshots[syncSourceSnapshotIndex].GetName())
+			syncSnapshots = append(syncSnapshots, sourceSnapshots[syncSourceSnapshotIndex])
 		}
 
 		respHeader.Snapshots = syncSnapshots
-		respHeader.SnapshotNames = snapshotNames
+		respHeader.SnapshotNames = syncSnapshotNames
 		offerHeader.Snapshots = syncSnapshots
-		offerHeader.SnapshotNames = snapshotNames
+		offerHeader.SnapshotNames = syncSnapshotNames
 	}
 
 	if offerHeader.GetPredump() {
@@ -1190,7 +1187,7 @@ func (c *migrationSink) Do(state *state.State, revert *revert.Reverter, migrateO
 
 		if live && c.src.instance.Type() == instancetype.Container {
 			var err error
-			imagesDir, err = ioutil.TempDir("", "lxd_restore_")
+			imagesDir, err = os.MkdirTemp("", "lxd_restore_")
 			if err != nil {
 				restore <- err
 				return
@@ -1330,41 +1327,3 @@ func (c *migrationSink) Do(state *state.State, revert *revert.Reverter, migrateO
 func (s *migrationSourceWs) ConnectContainerTarget(target api.InstancePostTarget) error {
 	return s.ConnectTarget(target.Certificate, target.Operation, target.Websockets)
 }
-
-func migrationCompareSnapshots(sourceSnapshots []*migration.Snapshot, targetSnapshots []instance.Instance) ([]*migration.Snapshot, []instance.Instance) {
-	// Compare source and target
-	sourceSnapshotsTime := map[string]int64{}
-	targetSnapshotsTime := map[string]int64{}
-
-	toDelete := []instance.Instance{}
-	toSync := []*migration.Snapshot{}
-
-	for _, snap := range sourceSnapshots {
-		snapName := snap.GetName()
-
-		sourceSnapshotsTime[snapName] = snap.GetCreationDate()
-	}
-
-	for _, snap := range targetSnapshots {
-		_, snapName, _ := api.GetParentAndSnapshotName(snap.Name())
-
-		targetSnapshotsTime[snapName] = snap.CreationDate().Unix()
-		existDate, exists := sourceSnapshotsTime[snapName]
-		if !exists {
-			toDelete = append(toDelete, snap)
-		} else if existDate != snap.CreationDate().Unix() {
-			toDelete = append(toDelete, snap)
-		}
-	}
-
-	for _, snap := range sourceSnapshots {
-		snapName := snap.GetName()
-
-		existDate, exists := targetSnapshotsTime[snapName]
-		if !exists || existDate != snap.GetCreationDate() {
-			toSync = append(toSync, snap)
-		}
-	}
-
-	return toSync, toDelete
-}
