@@ -5,7 +5,6 @@ import (
 	"encoding/binary"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"os"
@@ -26,14 +25,12 @@ import (
 	"github.com/lxc/lxd/lxd/db"
 	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/db/warningtype"
-	"github.com/lxc/lxd/lxd/device/nictype"
 	"github.com/lxc/lxd/lxd/dnsmasq"
 	"github.com/lxc/lxd/lxd/dnsmasq/dhcpalloc"
 	firewallDrivers "github.com/lxc/lxd/lxd/firewall/drivers"
 	"github.com/lxc/lxd/lxd/ip"
 	"github.com/lxc/lxd/lxd/network/acl"
 	"github.com/lxc/lxd/lxd/network/openvswitch"
-	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/project"
 	"github.com/lxc/lxd/lxd/revert"
 	"github.com/lxc/lxd/lxd/util"
@@ -59,11 +56,6 @@ type bridge struct {
 	common
 }
 
-// Type returns the network type.
-func (n *bridge) Type() string {
-	return "bridge"
-}
-
 // DBType returns the network type DB ID.
 func (n *bridge) DBType() db.NetworkType {
 	return db.NetworkTypeBridge
@@ -81,7 +73,7 @@ func (n *bridge) Info() Info {
 // cluster nodes. It is not suitable to use a static MAC address when "bridge.external_interfaces" is non-empty and
 // the bridge interface has no IPv4 or IPv6 address set. This is because in a clustered environment the same bridge
 // config is applied to all nodes, and if the bridge is being used to connect multiple nodes to the same network
-// segment it would cause MAC conflicts to use the the same MAC on all nodes. If an IP address is specified then
+// segment it would cause MAC conflicts to use the same MAC on all nodes. If an IP address is specified then
 // connecting multiple nodes to the same network segment would also cause IP conflicts, so if an IP is defined
 // then we assume this is not being done. However if IP addresses are explicitly set to "none" and
 // "bridge.external_interfaces" is set then it may not be safe to use a the same MAC address on all nodes.
@@ -122,7 +114,7 @@ func (n *bridge) FillConfig(config map[string]string) error {
 		}
 
 		if config["ipv6.address"] == "" {
-			content, err := ioutil.ReadFile("/proc/sys/net/ipv6/conf/default/disable_ipv6")
+			content, err := os.ReadFile("/proc/sys/net/ipv6/conf/default/disable_ipv6")
 			if err == nil && string(content) == "0\n" {
 				config["ipv6.address"] = "auto"
 			}
@@ -243,10 +235,10 @@ func (n *bridge) Validate(config map[string]string) error {
 		"ipv4.dhcp":         validate.Optional(validate.IsBool),
 		"ipv4.dhcp.gateway": validate.Optional(validate.IsNetworkAddressV4),
 		"ipv4.dhcp.expiry":  validate.IsAny,
-		"ipv4.dhcp.ranges":  validate.Optional(validate.IsNetworkRangeV4List),
-		"ipv4.routes":       validate.Optional(validate.IsNetworkV4List),
+		"ipv4.dhcp.ranges":  validate.Optional(validate.IsListOf(validate.IsNetworkRangeV4)),
+		"ipv4.routes":       validate.Optional(validate.IsListOf(validate.IsNetworkV4)),
 		"ipv4.routing":      validate.Optional(validate.IsBool),
-		"ipv4.ovn.ranges":   validate.Optional(validate.IsNetworkRangeV4List),
+		"ipv4.ovn.ranges":   validate.Optional(validate.IsListOf(validate.IsNetworkRangeV4)),
 
 		"ipv6.address": validate.Optional(func(value string) error {
 			if validate.IsOneOf("none", "auto")(value) == nil {
@@ -262,10 +254,10 @@ func (n *bridge) Validate(config map[string]string) error {
 		"ipv6.dhcp":                            validate.Optional(validate.IsBool),
 		"ipv6.dhcp.expiry":                     validate.IsAny,
 		"ipv6.dhcp.stateful":                   validate.Optional(validate.IsBool),
-		"ipv6.dhcp.ranges":                     validate.Optional(validate.IsNetworkRangeV6List),
-		"ipv6.routes":                          validate.Optional(validate.IsNetworkV6List),
+		"ipv6.dhcp.ranges":                     validate.Optional(validate.IsListOf(validate.IsNetworkRangeV6)),
+		"ipv6.routes":                          validate.Optional(validate.IsListOf(validate.IsNetworkV6)),
 		"ipv6.routing":                         validate.Optional(validate.IsBool),
-		"ipv6.ovn.ranges":                      validate.Optional(validate.IsNetworkRangeV6List),
+		"ipv6.ovn.ranges":                      validate.Optional(validate.IsListOf(validate.IsNetworkRangeV6)),
 		"dns.domain":                           validate.IsAny,
 		"dns.mode":                             validate.Optional(validate.IsOneOf("dynamic", "managed", "none")),
 		"dns.search":                           validate.IsAny,
@@ -765,15 +757,10 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 
 	// Enable VLAN filtering for Linux bridges.
 	if n.config["bridge.driver"] != "openvswitch" {
+		// Enable filtering.
 		err = BridgeVLANFilterSetStatus(n.name, "1")
 		if err != nil {
-			n.logger.Warn(fmt.Sprintf("%v", err))
-		}
-
-		// Set the default PVID for new ports to 1.
-		err = BridgeVLANSetDefaultPVID(n.name, "1")
-		if err != nil {
-			n.logger.Warn(fmt.Sprintf("%v", err))
+			n.logger.Warn(fmt.Sprintf("Failed enabling VLAN filtering: %v", err))
 		}
 	}
 
@@ -1118,14 +1105,14 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 		// Allow forwarding.
 		if n.config["ipv6.routing"] == "" || shared.IsTrue(n.config["ipv6.routing"]) {
 			// Get a list of proc entries.
-			entries, err := ioutil.ReadDir("/proc/sys/net/ipv6/conf/")
+			entries, err := os.ReadDir("/proc/sys/net/ipv6/conf/")
 			if err != nil {
 				return err
 			}
 
 			// First set accept_ra to 2 for everything.
 			for _, entry := range entries {
-				content, err := ioutil.ReadFile(fmt.Sprintf("/proc/sys/net/ipv6/conf/%s/accept_ra", entry.Name()))
+				content, err := os.ReadFile(fmt.Sprintf("/proc/sys/net/ipv6/conf/%s/accept_ra", entry.Name()))
 				if err == nil && string(content) != "1\n" {
 					continue
 				}
@@ -1293,6 +1280,7 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 		dnsmasqCmd = append(dnsmasqCmd, []string{
 			fmt.Sprintf("--listen-address=%s", addr[0]),
 			"--dhcp-no-override", "--dhcp-authoritative",
+			fmt.Sprintf("--dhcp-option-force=26,%d", fanMtuInt),
 			fmt.Sprintf("--dhcp-leasefile=%s", shared.VarPath("networks", n.name, "dnsmasq.leases")),
 			fmt.Sprintf("--dhcp-hostsfile=%s", shared.VarPath("networks", n.name, "dnsmasq.hosts")),
 			"--dhcp-range", fmt.Sprintf("%s,%s,%s", dhcpalloc.GetIP(hostSubnet, 2).String(), dhcpalloc.GetIP(hostSubnet, -2).String(), expiry)}...)
@@ -1379,16 +1367,13 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 		}
 
 		// Setup clustered DNS.
-		clusterAddress, err := node.ClusterAddress(n.state.DB.Node)
-		if err != nil {
-			return err
-		}
+		localClusterAddress := n.state.LocalConfig.ClusterAddress()
 
 		// If clusterAddress is non-empty, this indicates the intention for this node to be
 		// part of a cluster and so we should ensure that dnsmasq and forkdns are started
 		// in cluster mode. Note: During LXD initialisation the cluster may not actually be
 		// setup yet, but we want the DNS processes to be ready for when it is.
-		if clusterAddress != "" {
+		if localClusterAddress != "" {
 			dnsClustered = true
 		}
 
@@ -1546,7 +1531,7 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 		}
 
 		// Create a config file to contain additional config (and to prevent dnsmasq from reading /etc/dnsmasq.conf)
-		err = ioutil.WriteFile(shared.VarPath("networks", n.name, "dnsmasq.raw"), []byte(fmt.Sprintf("%s\n", n.config["raw.dnsmasq"])), 0644)
+		err = os.WriteFile(shared.VarPath("networks", n.name, "dnsmasq.raw"), []byte(fmt.Sprintf("%s\n", n.config["raw.dnsmasq"])), 0644)
 		if err != nil {
 			return err
 		}
@@ -1607,7 +1592,7 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 		}
 
 		// Start dnsmasq.
-		err = p.Start()
+		err = p.Start(context.Background())
 		if err != nil {
 			return fmt.Errorf("Failed to run: %s %s: %w", command, strings.Join(dnsmasqCmd, " "), err)
 		}
@@ -1616,11 +1601,10 @@ func (n *bridge) setup(oldConfig map[string]string) error {
 		ctx, cancel := context.WithDeadline(context.Background(), time.Now().Add(time.Millisecond*time.Duration(500)))
 		_, err = p.Wait(ctx)
 		if !errors.Is(err, context.DeadlineExceeded) {
-			stderr, _ := ioutil.ReadFile(dnsmasqLogPath)
+			stderr, _ := os.ReadFile(dnsmasqLogPath)
+			cancel()
 
-			// Just log an error if dnsmasq has exited, and still proceed with normal setup so we
-			// don't leave the firewall in an inconsistent state.
-			n.logger.Error("The dnsmasq process exited prematurely", logger.Ctx{"err": err, "stderr": strings.TrimSpace(string(stderr))})
+			return fmt.Errorf("The DNS and DHCP service exited prematurely: %w (%q)", err, strings.TrimSpace(string(stderr)))
 		}
 
 		cancel()
@@ -1919,7 +1903,7 @@ func (n *bridge) spawnForkDNS(listenAddress string) error {
 	// Apply AppArmor profile.
 	p.SetApparmor(apparmor.ForkdnsProfileName(n))
 
-	err = p.Start()
+	err = p.Start(context.Background())
 	if err != nil {
 		return fmt.Errorf("Failed to run: %s %s: %w", command, strings.Join(forkdnsargs, " "), err)
 	}
@@ -1947,16 +1931,13 @@ func (n *bridge) HandleHeartbeat(heartbeatData *cluster.APIHeartbeat) error {
 	}
 
 	addresses := []string{}
-	localAddress, err := node.HTTPSAddress(n.state.DB.Node)
-	if err != nil {
-		return err
-	}
+	localClusterAddress := n.state.LocalConfig.ClusterAddress()
 
 	n.logger.Info("Refreshing forkdns peers")
 
 	networkCert := n.state.Endpoints.NetworkCert()
 	for _, node := range heartbeatData.Members {
-		if node.Address == localAddress {
+		if node.Address == localClusterAddress {
 			// No need to query ourselves.
 			continue
 		}
@@ -2452,7 +2433,7 @@ func (n *bridge) bridgeNetworkExternalSubnets(bridgeProjectNetworks map[string][
 func (n *bridge) bridgedNICExternalRoutes(bridgeProjectNetworks map[string][]*api.Network) ([]externalSubnetUsage, error) {
 	externalRoutes := make([]externalSubnetUsage, 0)
 
-	err := n.state.DB.Cluster.InstanceList(nil, func(inst db.InstanceArgs, p api.Project) error {
+	err := n.state.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
 		// Get the instance's effective network project name.
 		instNetworkProject := project.NetworkProjectFromRecord(&p)
 
@@ -2515,13 +2496,13 @@ func (n *bridge) getExternalSubnetInUse() ([]externalSubnetUsage, error) {
 
 	err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get all managed networks across all projects.
-		projectNetworks, err = tx.GetCreatedNetworks()
+		projectNetworks, err = tx.GetCreatedNetworks(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed to load all networks: %w", err)
 		}
 
 		// Get all network forward listen addresses for forwards assigned to this specific cluster member.
-		projectNetworksForwardsOnUplink, err = tx.GetProjectNetworkForwardListenAddressesOnMember()
+		projectNetworksForwardsOnUplink, err = tx.GetProjectNetworkForwardListenAddressesOnMember(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed loading network forward listen addresses: %w", err)
 		}
@@ -2663,11 +2644,8 @@ func (n *bridge) ForwardCreate(forward api.NetworkForwardsPost, clientType reque
 
 			// If we are the first forward on this bridge, enable hairpin mode on active NIC ports.
 			if len(listenAddresses) <= 1 {
-				filter := dbCluster.InstanceFilter{
-					Node: &n.state.ServerName,
-				}
-
-				err = n.state.DB.Cluster.InstanceList(&filter, func(inst db.InstanceArgs, p api.Project) error {
+				filter := dbCluster.InstanceFilter{Node: &n.state.ServerName}
+				err = n.state.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
 					// Get the instance's effective network project name.
 					instNetworkProject := project.NetworkProjectFromRecord(&p)
 
@@ -2702,7 +2680,7 @@ func (n *bridge) ForwardCreate(forward api.NetworkForwardsPost, clientType reque
 					}
 
 					return nil
-				})
+				}, filter)
 				if err != nil {
 					return err
 				}
@@ -2885,19 +2863,30 @@ func (n *bridge) forwardSetupFirewall() error {
 // Leases returns a list of leases for the bridged network. It will reach out to other cluster members as needed.
 // The projectName passed here refers to the initial project from the API request which may differ from the network's project.
 func (n *bridge) Leases(projectName string, clientType request.ClientType) ([]api.NetworkLease, error) {
+	var err error
+	var projectMacs []string
 	leases := []api.NetworkLease{}
-	projectMacs := []string{}
 
 	// Get all static leases.
 	if clientType == request.ClientTypeNormal {
-		// Get the downstream networks.
-		if n.project == project.Default {
-			var err error
+		// If requested project matches network's project then include gateway and downstream uplink IPs.
+		if projectName == n.project {
+			// Add our own gateway IPs.
+			for _, addr := range []string{n.config["ipv4.address"], n.config["ipv6.address"]} {
+				ip, _, _ := net.ParseCIDR(addr)
+				if ip != nil {
+					leases = append(leases, api.NetworkLease{
+						Hostname: fmt.Sprintf("%s.gw", n.Name()),
+						Address:  ip.String(),
+						Type:     "gateway",
+					})
+				}
+			}
 
-			// Load all the networks.
+			// Include downstream OVN routers using the network as an uplink.
 			var projectNetworks map[string]map[int64]api.Network
 			err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-				projectNetworks, err = tx.GetCreatedNetworks()
+				projectNetworks, err = tx.GetCreatedNetworks(ctx)
 				return err
 			})
 			if err != nil {
@@ -2926,111 +2915,72 @@ func (n *bridge) Leases(projectName string, clientType request.ClientType) ([]ap
 			}
 		}
 
-		// Get all the instances in project.
-		filter := dbCluster.InstanceFilter{
-			Project: &projectName,
-		}
+		// Get all the instances in the requested project that are connected to this network.
+		filter := dbCluster.InstanceFilter{Project: &projectName}
+		err = UsedByInstanceDevices(n.state, n.Project(), n.Name(), n.Type(), func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
+			// Fill in the hwaddr from volatile.
+			if nicConfig["hwaddr"] == "" {
+				nicConfig["hwaddr"] = inst.Config[fmt.Sprintf("volatile.%s.hwaddr", nicName)]
+			}
 
-		err := n.state.DB.Cluster.InstanceList(&filter, func(inst db.InstanceArgs, p api.Project) error {
-			devices := db.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles)
+			// Record the MAC.
+			hwAddr, _ := net.ParseMAC(nicConfig["hwaddr"])
+			if hwAddr != nil {
+				projectMacs = append(projectMacs, hwAddr.String())
+			}
 
-			// Go through all its devices (including profiles).
-			for k, dev := range devices {
-				// Skip uninteresting entries.
-				if dev["type"] != "nic" {
-					continue
-				}
+			// Add the lease.
+			nicIP4 := net.ParseIP(nicConfig["ipv4.address"])
+			if nicIP4 != nil {
+				leases = append(leases, api.NetworkLease{
+					Hostname: inst.Name,
+					Address:  nicIP4.String(),
+					Hwaddr:   hwAddr.String(),
+					Type:     "static",
+					Location: inst.Node,
+				})
+			}
 
-				nicType, err := nictype.NICType(n.state, inst.Project, dev)
-				if err != nil || nicType != "bridged" {
-					continue
-				}
+			nicIP6 := net.ParseIP(nicConfig["ipv6.address"])
+			if nicIP6 != nil {
+				leases = append(leases, api.NetworkLease{
+					Hostname: inst.Name,
+					Address:  nicIP6.String(),
+					Hwaddr:   hwAddr.String(),
+					Type:     "static",
+					Location: inst.Node,
+				})
+			}
 
-				// Temporarily populate parent from network setting if used.
-				if dev["network"] != "" {
-					dev["parent"] = dev["network"]
-				}
-
-				if dev["parent"] != n.name {
-					continue
-				}
-
-				// Fill in the hwaddr from volatile.
-				if dev["hwaddr"] == "" {
-					dev["hwaddr"] = inst.Config[fmt.Sprintf("volatile.%s.hwaddr", k)]
-				}
-
-				// Record the MAC.
-				if dev["hwaddr"] != "" {
-					projectMacs = append(projectMacs, dev["hwaddr"])
-				}
-
-				// Add the lease.
-				if dev["ipv4.address"] != "" {
+			// Add EUI64 records.
+			_, netIP6, _ := net.ParseCIDR(n.config["ipv6.address"])
+			if netIP6 != nil && hwAddr != nil && shared.IsFalseOrEmpty(n.config["ipv6.dhcp.stateful"]) {
+				eui64IP6, err := eui64.ParseMAC(netIP6.IP, hwAddr)
+				if err == nil {
 					leases = append(leases, api.NetworkLease{
 						Hostname: inst.Name,
-						Address:  dev["ipv4.address"],
-						Hwaddr:   dev["hwaddr"],
-						Type:     "static",
+						Address:  eui64IP6.String(),
+						Hwaddr:   hwAddr.String(),
+						Type:     "dynamic",
 						Location: inst.Node,
 					})
 				}
-
-				if dev["ipv6.address"] != "" {
-					leases = append(leases, api.NetworkLease{
-						Hostname: inst.Name,
-						Address:  dev["ipv6.address"],
-						Hwaddr:   dev["hwaddr"],
-						Type:     "static",
-						Location: inst.Node,
-					})
-				}
-
-				// Add EUI64 records.
-				ipv6Address := n.config["ipv6.address"]
-				if ipv6Address != "" && ipv6Address != "none" && shared.IsFalseOrEmpty(n.config["ipv6.dhcp.stateful"]) {
-					_, netAddress, _ := net.ParseCIDR(ipv6Address)
-					hwAddr, _ := net.ParseMAC(dev["hwaddr"])
-					if netAddress != nil && hwAddr != nil {
-						ipv6, err := eui64.ParseMAC(netAddress.IP, hwAddr)
-						if err == nil {
-							leases = append(leases, api.NetworkLease{
-								Hostname: inst.Name,
-								Address:  ipv6.String(),
-								Hwaddr:   dev["hwaddr"],
-								Type:     "dynamic",
-								Location: inst.Node,
-							})
-						}
-					}
-				}
 			}
 
 			return nil
-		})
+		}, filter)
 		if err != nil {
 			return nil, err
 		}
 	}
 
-	// Local server name.
-	var err error
-	var serverName string
-	err = n.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		serverName, err = tx.GetLocalNodeName()
-		return err
-	})
-	if err != nil {
-		return nil, err
-	}
-
 	// Get dynamic leases.
 	leaseFile := shared.VarPath("networks", n.name, "dnsmasq.leases")
 	if !shared.PathExists(leaseFile) {
 		return leases, nil
 	}
 
-	content, err := ioutil.ReadFile(leaseFile)
+	content, err := os.ReadFile(leaseFile)
 	if err != nil {
 		return nil, err
 	}
@@ -3078,7 +3028,7 @@ func (n *bridge) Leases(projectName string, clientType request.ClientType) ([]ap
 				Address:  fields[2],
 				Hwaddr:   macStr,
 				Type:     "dynamic",
-				Location: serverName,
+				Location: n.state.ServerName,
 			})
 		}
 	}
