@@ -6,7 +6,6 @@ import (
 	"context"
 	"encoding/hex"
 	"fmt"
-	"io/ioutil"
 	"math/big"
 	"math/rand"
 	"net"
@@ -66,9 +65,10 @@ func RandomDevName(prefix string) string {
 	return iface
 }
 
-// usedByInstanceDevices looks for instance NIC devices using the network and runs the supplied usageFunc for each.
-func usedByInstanceDevices(s *state.State, networkProjectName string, networkName string, usageFunc func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error) error {
-	return s.DB.Cluster.InstanceList(nil, func(inst db.InstanceArgs, p api.Project) error {
+// UsedByInstanceDevices looks for instance NIC devices using the network and runs the supplied usageFunc for each.
+// Accepts optional filter arguments to specify a subset of instances.
+func UsedByInstanceDevices(s *state.State, networkProjectName string, networkName string, networkType string, usageFunc func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error, filters ...cluster.InstanceFilter) error {
+	return s.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
 		// Get the instance's effective network project name.
 		instNetworkProject := project.NetworkProjectFromRecord(&p)
 
@@ -80,7 +80,7 @@ func usedByInstanceDevices(s *state.State, networkProjectName string, networkNam
 		// Look for NIC devices using this network.
 		devices := db.ExpandInstanceDevices(inst.Devices.Clone(), inst.Profiles)
 		for devName, devConfig := range devices {
-			if isInUseByDevice(networkName, devConfig) {
+			if isInUseByDevice(networkName, networkType, devConfig) {
 				err := usageFunc(inst, devName, devConfig)
 				if err != nil {
 					return err
@@ -89,12 +89,12 @@ func usedByInstanceDevices(s *state.State, networkProjectName string, networkNam
 		}
 
 		return nil
-	})
+	}, filters...)
 }
 
 // UsedBy returns list of API resources using network. Accepts firstOnly argument to indicate that only the first
 // resource using network should be returned. This can help to quickly check if the network is in use.
-func UsedBy(s *state.State, networkProjectName string, networkID int64, networkName string, firstOnly bool) ([]string, error) {
+func UsedBy(s *state.State, networkProjectName string, networkID int64, networkName string, networkType string, firstOnly bool) ([]string, error) {
 	var err error
 	var usedBy []string
 
@@ -123,7 +123,7 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 		var projectNetworks map[string]map[int64]api.Network
 
 		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			projectNetworks, err = tx.GetCreatedNetworks()
+			projectNetworks, err = tx.GetCreatedNetworks(ctx)
 			return err
 		})
 		if err != nil {
@@ -151,7 +151,7 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 
 	// Look for profiles. Next cheapest to do.
 	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		profiles, err := cluster.GetProfiles(ctx, tx.Tx(), cluster.ProfileFilter{})
+		profiles, err := cluster.GetProfiles(ctx, tx.Tx())
 		if err != nil {
 			return err
 		}
@@ -172,7 +172,7 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 				return err
 			}
 
-			inUse, err := usedByProfileDevices(s, profileDevices, apiProfileProject, networkProjectName, networkName)
+			inUse, err := usedByProfileDevices(s, profileDevices, apiProfileProject, networkProjectName, networkName, networkType)
 			if err != nil {
 				return err
 			}
@@ -193,7 +193,7 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 	}
 
 	// Check if any instance devices use this network.
-	err = usedByInstanceDevices(s, networkProjectName, networkName, func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
+	err = UsedByInstanceDevices(s, networkProjectName, networkName, networkType, func(inst db.InstanceArgs, nicName string, nicConfig map[string]string) error {
 		usedBy = append(usedBy, api.NewURL().Path(version.APIVersion, "instances", inst.Name).Project(inst.Project).String())
 
 		if firstOnly {
@@ -216,7 +216,7 @@ func UsedBy(s *state.State, networkProjectName string, networkID int64, networkN
 
 // usedByProfileDevices indicates if network is referenced by a profile's NIC devices.
 // Checks if the device's parent or network properties match the network name.
-func usedByProfileDevices(s *state.State, profileDevices map[string]cluster.Device, profileProject *api.Project, networkProjectName string, networkName string) (bool, error) {
+func usedByProfileDevices(s *state.State, profileDevices map[string]cluster.Device, profileProject *api.Project, networkProjectName string, networkName string, networkType string) (bool, error) {
 	// Get the translated network project name from the profiles's project.
 
 	// Skip profiles who's translated network project doesn't match the requested network's project.
@@ -227,7 +227,7 @@ func usedByProfileDevices(s *state.State, profileDevices map[string]cluster.Devi
 	}
 
 	for _, d := range deviceConfig.NewDevices(cluster.DevicesToAPI(profileDevices)) {
-		if isInUseByDevice(networkName, d) {
+		if isInUseByDevice(networkName, networkType, d) {
 			return true, nil
 		}
 	}
@@ -236,7 +236,7 @@ func usedByProfileDevices(s *state.State, profileDevices map[string]cluster.Devi
 }
 
 // isInUseByDevices inspects a device's config to find references for a network being used.
-func isInUseByDevice(networkName string, d deviceConfig.Device) bool {
+func isInUseByDevice(networkName string, networkType string, d deviceConfig.Device) bool {
 	if d["type"] != "nic" {
 		return false
 	}
@@ -245,6 +245,11 @@ func isInUseByDevice(networkName string, d deviceConfig.Device) bool {
 		return true
 	}
 
+	// OVN networks can only use managed networks.
+	if networkType == "ovn" {
+		return false
+	}
+
 	if d["parent"] != "" && GetHostDevice(d["parent"], d["vlan"]) == networkName {
 		return true
 	}
@@ -254,7 +259,7 @@ func isInUseByDevice(networkName string, d deviceConfig.Device) bool {
 
 // GetDevMTU retrieves the current MTU setting for a named network device.
 func GetDevMTU(devName string) (uint32, error) {
-	content, err := ioutil.ReadFile(fmt.Sprintf("/sys/class/net/%s/mtu", devName))
+	content, err := os.ReadFile(fmt.Sprintf("/sys/class/net/%s/mtu", devName))
 	if err != nil {
 		return 0, err
 	}
@@ -268,6 +273,22 @@ func GetDevMTU(devName string) (uint32, error) {
 	return uint32(mtu), nil
 }
 
+// GetTXQueueLength retrieves the current txqlen setting for a named network device.
+func GetTXQueueLength(devName string) (uint32, error) {
+	content, err := os.ReadFile(fmt.Sprintf("/sys/class/net/%s/tx_queue_len", devName))
+	if err != nil {
+		return 0, err
+	}
+
+	// Parse value
+	txqlen, err := strconv.ParseUint(strings.TrimSpace(string(content)), 10, 32)
+	if err != nil {
+		return 0, err
+	}
+
+	return uint32(txqlen), nil
+}
+
 // DefaultGatewaySubnetV4 returns subnet of default gateway interface.
 func DefaultGatewaySubnetV4() (*net.IPNet, string, error) {
 	file, err := os.Open("/proc/net/route")
@@ -370,7 +391,7 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 				continue
 			}
 
-			nicType, err := nictype.NICType(s, inst.Project(), d)
+			nicType, err := nictype.NICType(s, inst.Project().Name, d)
 			if err != nil || nicType != "bridged" {
 				continue
 			}
@@ -398,7 +419,7 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 			}
 
 			if (shared.IsTrue(d["security.ipv4_filtering"]) && d["ipv4.address"] == "") || (shared.IsTrue(d["security.ipv6_filtering"]) && d["ipv6.address"] == "") {
-				deviceStaticFileName := dnsmasq.StaticAllocationFileName(inst.Project(), inst.Name(), deviceName)
+				deviceStaticFileName := dnsmasq.StaticAllocationFileName(inst.Project().Name, inst.Name(), deviceName)
 				_, curIPv4, curIPv6, err := dnsmasq.DHCPStaticAllocation(d["parent"], deviceStaticFileName)
 				if err != nil && !os.IsNotExist(err) {
 					return err
@@ -413,7 +434,7 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 				}
 			}
 
-			entries[d["parent"]] = append(entries[d["parent"]], []string{d["hwaddr"], inst.Project(), inst.Name(), d["ipv4.address"], d["ipv6.address"], deviceName})
+			entries[d["parent"]] = append(entries[d["parent"]], []string{d["hwaddr"], inst.Project().Name, inst.Name(), d["ipv4.address"], d["ipv6.address"], deviceName})
 		}
 	}
 
@@ -435,7 +456,7 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 		config := n.Config()
 
 		// Wipe everything clean.
-		files, err := ioutil.ReadDir(shared.VarPath("networks", network, "dnsmasq.hosts"))
+		files, err := os.ReadDir(shared.VarPath("networks", network, "dnsmasq.hosts"))
 		if err != nil {
 			return err
 		}
@@ -791,7 +812,7 @@ func GetLeaseAddresses(networkName string, hwaddr string) ([]net.IP, error) {
 		return nil, fmt.Errorf("Leases file not found for network %q", networkName)
 	}
 
-	content, err := ioutil.ReadFile(leaseFile)
+	content, err := os.ReadFile(leaseFile)
 	if err != nil {
 		return nil, err
 	}
