@@ -72,9 +72,6 @@ const ovnExtIDLXDSwitchPort = "lxd_switch_port"
 const ovnExtIDLXDProjectID = "lxd_project_id"
 const ovnExtIDLXDPortGroup = "lxd_port_group"
 
-// ErrOVNNoPortIPs used when no IPs are found for a logical port.
-var ErrOVNNoPortIPs = fmt.Errorf("No port IPs")
-
 // OVNIPv6RAOpts IPv6 router advertisements options that can be applied to a router.
 type OVNIPv6RAOpts struct {
 	SendPeriodic       bool
@@ -648,6 +645,26 @@ func (o *OVN) logicalSwitchFindAssociatedPortGroups(switchName OVNSwitch) ([]OVN
 	return portGroups, nil
 }
 
+// logicalSwitchParseExcludeIPs parses the ips into OVN exclude_ips format.
+func (o *OVN) logicalSwitchParseExcludeIPs(ips []shared.IPRange) ([]string, error) {
+	excludeIPs := make([]string, 0, len(ips))
+	for _, v := range ips {
+		if v.Start == nil || v.Start.To4() == nil {
+			return nil, fmt.Errorf("Invalid exclude IPv4 range start address")
+		} else if v.End == nil {
+			excludeIPs = append(excludeIPs, v.Start.String())
+		} else {
+			if v.End != nil && v.End.To4() == nil {
+				return nil, fmt.Errorf("Invalid exclude IPv4 range end address")
+			}
+
+			excludeIPs = append(excludeIPs, fmt.Sprintf("%s..%s", v.Start.String(), v.End.String()))
+		}
+	}
+
+	return excludeIPs, nil
+}
+
 // LogicalSwitchSetIPAllocation sets the IP allocation config on the logical switch.
 func (o *OVN) LogicalSwitchSetIPAllocation(switchName OVNSwitch, opts *OVNIPAllocationOpts) error {
 	var removeOtherConfigKeys []string
@@ -666,15 +683,9 @@ func (o *OVN) LogicalSwitchSetIPAllocation(switchName OVNSwitch, opts *OVNIPAllo
 	}
 
 	if len(opts.ExcludeIPv4) > 0 {
-		excludeIPs := make([]string, 0, len(opts.ExcludeIPv4))
-		for _, v := range opts.ExcludeIPv4 {
-			if v.Start == nil {
-				return fmt.Errorf("Invalid exclude IPv4 range start address")
-			} else if v.End == nil {
-				excludeIPs = append(excludeIPs, v.Start.String())
-			} else {
-				excludeIPs = append(excludeIPs, fmt.Sprintf("%s..%s", v.Start.String(), v.End.String()))
-			}
+		excludeIPs, err := o.logicalSwitchParseExcludeIPs(opts.ExcludeIPv4)
+		if err != nil {
+			return err
 		}
 
 		args = append(args, fmt.Sprintf("other_config:exclude_ips=%s", strings.Join(excludeIPs, " ")))
@@ -702,6 +713,91 @@ func (o *OVN) LogicalSwitchSetIPAllocation(switchName OVNSwitch, opts *OVNIPAllo
 	return nil
 }
 
+// LogicalSwitchDHCPv4RevervationsSet sets the DHCPv4 IP reservations.
+func (o *OVN) LogicalSwitchDHCPv4RevervationsSet(switchName OVNSwitch, reservedIPs []shared.IPRange) error {
+	var removeOtherConfigKeys []string
+	args := []string{"set", "logical_switch", string(switchName)}
+
+	if len(reservedIPs) > 0 {
+		excludeIPs, err := o.logicalSwitchParseExcludeIPs(reservedIPs)
+		if err != nil {
+			return err
+		}
+
+		args = append(args, fmt.Sprintf("other_config:exclude_ips=%s", strings.Join(excludeIPs, " ")))
+	} else {
+		removeOtherConfigKeys = append(removeOtherConfigKeys, "exclude_ips")
+	}
+
+	// Clear any unused keys first.
+	if len(removeOtherConfigKeys) > 0 {
+		removeArgs := append([]string{"remove", "logical_switch", string(switchName), "other_config"}, removeOtherConfigKeys...)
+		_, err := o.nbctl(removeArgs...)
+		if err != nil {
+			return err
+		}
+	}
+
+	// Only run command if at least one setting is specified.
+	if len(args) > 3 {
+		_, err := o.nbctl(args...)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+// LogicalSwitchDHCPv4RevervationsGet gets the DHCPv4 IP reservations.
+func (o *OVN) LogicalSwitchDHCPv4RevervationsGet(switchName OVNSwitch) ([]shared.IPRange, error) {
+	excludeIPsRaw, err := o.nbctl("get", "logical_switch", string(switchName), "other_config:exclude_ips")
+	if err != nil {
+		return nil, err
+	}
+
+	excludeIPsRaw = strings.TrimSpace(excludeIPsRaw)
+
+	// Check if no dynamic IPs set.
+	if excludeIPsRaw == "[]" {
+		return []shared.IPRange{}, nil
+	}
+
+	excludeIPsRaw, err = unquote(excludeIPsRaw)
+	if err != nil {
+		return nil, fmt.Errorf("Failed unquoting exclude_ips: %w", err)
+	}
+
+	excludeIPsParts := shared.SplitNTrimSpace(strings.TrimSpace(excludeIPsRaw), " ", -1, true)
+	excludeIPs := make([]shared.IPRange, 0, len(excludeIPsParts))
+
+	for _, excludeIPsPart := range excludeIPsParts {
+		ip := net.ParseIP(excludeIPsPart) // Check if single IP part.
+		if ip == nil {
+			// Check if IP range part.
+			start, end, found := strings.Cut(excludeIPsPart, "..")
+			if found {
+				startIP := net.ParseIP(start)
+				endIP := net.ParseIP(end)
+
+				if startIP == nil || endIP == nil {
+					return nil, fmt.Errorf("Invalid exclude_ips range: %q", excludeIPsPart)
+				}
+
+				// Add range IP part to list.
+				excludeIPs = append(excludeIPs, shared.IPRange{Start: startIP, End: endIP})
+			} else {
+				return nil, fmt.Errorf("Unrecognised exclude_ips part: %q", excludeIPsPart)
+			}
+		} else {
+			// Add single IP part to list.
+			excludeIPs = append(excludeIPs, shared.IPRange{Start: ip})
+		}
+	}
+
+	return excludeIPs, nil
+}
+
 // LogicalSwitchDHCPv4OptionsSet creates or updates a DHCPv4 option set associated with the specified switchName
 // and subnet. If uuid is non-empty then the record that exists with that ID is updated, otherwise a new record
 // is created.
@@ -1071,69 +1167,9 @@ func (o *OVN) LogicalSwitchPortDynamicIPs(portName OVNSwitchPort) ([]net.IP, err
 	return dynamicIPs, nil
 }
 
-// LogicalSwitchPortSetDNS sets up the switch DNS records for the DNS name resolving to the IPs of the switch port.
-// Attempts to find at most one IP for each IP protocol, preferring static addresses over dynamic.
+// LogicalSwitchPortSetDNS sets up the switch port DNS records for the DNS name.
 // Returns the DNS record UUID, IPv4 and IPv6 addresses used for DNS records.
-func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPort, dnsName string) (OVNDNSUUID, net.IP, net.IP, error) {
-	var dnsIPv4, dnsIPv6 net.IP
-
-	// checkAndStoreIP checks if the supplied IP is valid and can be used for a missing DNS IP variable.
-	// If the found IP is needed, stores into the relevant dnsIPvP{X} variable and returns true.
-	checkAndStoreIP := func(ip net.IP) bool {
-		if ip != nil {
-			isV4 := ip.To4() != nil
-			if dnsIPv4 == nil && isV4 {
-				dnsIPv4 = ip
-				return true
-			} else if dnsIPv6 == nil && !isV4 {
-				dnsIPv6 = ip
-				return true
-			}
-		}
-
-		return false
-	}
-
-	// Get static and dynamic IPs for switch port.
-	staticAddressesRaw, err := o.nbctl("lsp-get-addresses", string(portName))
-	if err != nil {
-		return "", nil, nil, err
-	}
-
-	staticAddresses := strings.Split(strings.TrimSpace(staticAddressesRaw), " ")
-	hasDynamic := false
-	for _, staticAddress := range staticAddresses {
-		// Record that there should be at least one dynamic address (may be a MAC address though).
-		if staticAddress == "dynamic" {
-			hasDynamic = true
-			continue
-		}
-
-		// Try and find the first IPv4 and IPv6 addresses from the static address list.
-		if checkAndStoreIP(net.ParseIP(staticAddress)) {
-			if dnsIPv4 != nil && dnsIPv6 != nil {
-				break // We've found all we wanted.
-			}
-		}
-	}
-
-	// Get dynamic IPs for switch port if indicated and needed.
-	if hasDynamic && (dnsIPv4 == nil || dnsIPv6 == nil) {
-		dynamicIPs, err := o.LogicalSwitchPortDynamicIPs(portName)
-		if err != nil {
-			return "", nil, nil, err
-		}
-
-		for _, dynamicIP := range dynamicIPs {
-			// Try and find the first IPv4 and IPv6 addresses from the dynamic address list.
-			if checkAndStoreIP(dynamicIP) {
-				if dnsIPv4 != nil && dnsIPv6 != nil {
-					break // We've found all we wanted.
-				}
-			}
-		}
-	}
-
+func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPort, dnsName string, dnsIPv4 net.IP, dnsIPv6 net.IP) (OVNDNSUUID, error) {
 	// Create a list of IPs for the DNS record.
 	dnsIPs := make([]string, 0, 2)
 	if dnsIPv4 != nil {
@@ -1144,36 +1180,41 @@ func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPo
 		dnsIPs = append(dnsIPs, dnsIPv6.String())
 	}
 
-	if len(dnsIPs) <= 0 {
-		return "", nil, nil, ErrOVNNoPortIPs
-	}
-
 	// Check if existing DNS record exists for switch port.
 	dnsUUID, err := o.nbctl("--format=csv", "--no-headings", "--data=bare", "--colum=_uuid", "find", "dns",
 		fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDSwitchPort, portName),
 	)
 	if err != nil {
-		return "", nil, nil, err
+		return "", err
 	}
 
 	cmdArgs := []string{
-		fmt.Sprintf(`records={"%s"="%s"}`, strings.ToLower(dnsName), strings.Join(dnsIPs, " ")),
 		fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDSwitch, switchName),
 		fmt.Sprintf("external_ids:%s=%s", ovnExtIDLXDSwitchPort, portName),
 	}
 
+	// Only include DNS name record if IPs supplied.
+	if len(dnsIPs) > 0 {
+		cmdArgs = append(cmdArgs, fmt.Sprintf(`records={"%s"="%s"}`, strings.ToLower(dnsName), strings.Join(dnsIPs, " ")))
+	}
+
 	dnsUUID = strings.TrimSpace(dnsUUID)
 	if dnsUUID != "" {
+		// Clear any existing DNS name if no IPs supplied.
+		if len(dnsIPs) < 1 {
+			cmdArgs = append(cmdArgs, "--", "clear", "dns", string(dnsUUID), "records")
+		}
+
 		// Update existing record if exists.
 		_, err = o.nbctl(append([]string{"set", "dns", dnsUUID}, cmdArgs...)...)
 		if err != nil {
-			return "", nil, nil, err
+			return "", err
 		}
 	} else {
 		// Create new record if needed.
 		dnsUUID, err = o.nbctl(append([]string{"create", "dns"}, cmdArgs...)...)
 		if err != nil {
-			return "", nil, nil, err
+			return "", err
 		}
 
 		dnsUUID = strings.TrimSpace(dnsUUID)
@@ -1182,10 +1223,10 @@ func (o *OVN) LogicalSwitchPortSetDNS(switchName OVNSwitch, portName OVNSwitchPo
 	// Add DNS record to switch DNS records.
 	_, err = o.nbctl("add", "logical_switch", string(switchName), "dns_records", dnsUUID)
 	if err != nil {
-		return "", nil, nil, err
+		return "", err
 	}
 
-	return OVNDNSUUID(dnsUUID), dnsIPv4, dnsIPv6, nil
+	return OVNDNSUUID(dnsUUID), nil
 }
 
 // LogicalSwitchPortGetDNS returns the logical switch port DNS info (UUID, name and IPs).
@@ -1222,25 +1263,30 @@ func (o *OVN) LogicalSwitchPortGetDNS(portName OVNSwitchPort) (OVNDNSUUID, strin
 	return OVNDNSUUID(dnsUUID), dnsName, ips, nil
 }
 
-// logicalSwitchPortDeleteDNSAppendArgs adds the command arguments to remove DNS records for a switch port.
+// logicalSwitchPortDeleteDNSAppendArgs adds the command arguments to remove DNS records from a switch port.
+// If destroyEntry the DNS entry record itself is also removed, otherwise it is just cleared but left in place.
 // Returns args with the commands added to it.
-func (o *OVN) logicalSwitchPortDeleteDNSAppendArgs(args []string, switchName OVNSwitch, dnsUUID OVNDNSUUID) []string {
+func (o *OVN) logicalSwitchPortDeleteDNSAppendArgs(args []string, switchName OVNSwitch, dnsUUID OVNDNSUUID, destroyEntry bool) []string {
 	if len(args) > 0 {
 		args = append(args, "--")
 	}
 
-	args = append(args,
-		"remove", "logical_switch", string(switchName), "dns_records", string(dnsUUID), "--",
-		"destroy", "dns", string(dnsUUID),
-	)
+	args = append(args, "remove", "logical_switch", string(switchName), "dns_records", string(dnsUUID), "--")
+
+	if destroyEntry {
+		args = append(args, "destroy", "dns", string(dnsUUID))
+	} else {
+		args = append(args, "clear", "dns", string(dnsUUID), "records")
+	}
 
 	return args
 }
 
-// LogicalSwitchPortDeleteDNS removes DNS records for a switch port.
-func (o *OVN) LogicalSwitchPortDeleteDNS(switchName OVNSwitch, dnsUUID OVNDNSUUID) error {
+// LogicalSwitchPortDeleteDNS removes DNS records from a switch port.
+// If destroyEntry the DNS entry record itself is also removed, otherwise it is just cleared but left in place.
+func (o *OVN) LogicalSwitchPortDeleteDNS(switchName OVNSwitch, dnsUUID OVNDNSUUID, destroyEntry bool) error {
 	// Remove DNS record association from switch, and remove DNS record entry itself.
-	_, err := o.nbctl(o.logicalSwitchPortDeleteDNSAppendArgs(nil, switchName, dnsUUID)...)
+	_, err := o.nbctl(o.logicalSwitchPortDeleteDNSAppendArgs(nil, switchName, dnsUUID, destroyEntry)...)
 	if err != nil {
 		return err
 	}
@@ -1284,7 +1330,9 @@ func (o *OVN) LogicalSwitchPortCleanup(portName OVNSwitchPort, switchName OVNSwi
 	args = o.logicalSwitchPortDeleteAppendArgs(args, portName)
 
 	// Remove DNS records.
-	args = o.logicalSwitchPortDeleteDNSAppendArgs(args, switchName, dnsUUID)
+	if dnsUUID != "" {
+		args = o.logicalSwitchPortDeleteDNSAppendArgs(args, switchName, dnsUUID, false)
+	}
 
 	_, err = o.nbctl(args...)
 	if err != nil {
@@ -1375,30 +1423,26 @@ func (o *OVN) ChassisGroupChassisAdd(haChassisGroupName OVNChassisGroup, chassis
 // ChassisGroupChassisDelete deletes a chassis ID from an HA chassis group.
 func (o *OVN) ChassisGroupChassisDelete(haChassisGroupName OVNChassisGroup, chassisID string) error {
 	// Check if chassis group exists. ovn-nbctl doesn't provide an "--if-exists" option for this.
-	existingChassisGroup, err := o.nbctl("--no-headings", "--data=bare", "--colum=name", "find", "ha_chassis_group", fmt.Sprintf("name=%s", string(haChassisGroupName)))
+	output, err := o.nbctl("--no-headings", "--data=bare", "--colum=name,ha_chassis", "find", "ha_chassis_group", fmt.Sprintf("name=%s", string(haChassisGroupName)))
 	if err != nil {
 		return err
 	}
 
-	// Nothing to do if chassis group doesn't exist.
-	if strings.TrimSpace(existingChassisGroup) == "" {
-		return nil
-	}
+	lines := shared.SplitNTrimSpace(output, "\n", -1, true)
+	if len(lines) > 1 {
+		existingChassisGroup := lines[0]
+		members := shared.SplitNTrimSpace(lines[1], " ", -1, true)
 
-	// Check if chassis exists. ovn-nbctl doesn't provide an "--if-exists" option for this.
-	existingChassis, err := o.nbctl("--no-headings", "--data=bare", "--colum=chassis_name", "find", "ha_chassis", fmt.Sprintf("chassis_name=%s", chassisID))
-	if err != nil {
-		return err
-	}
-
-	// Remove chassis from group if exists.
-	if strings.TrimSpace(existingChassis) != "" {
-		_, err := o.nbctl("ha-chassis-group-remove-chassis", string(haChassisGroupName), chassisID)
-		if err != nil {
-			return err
+		// Remove chassis from group if exists.
+		if existingChassisGroup == string(haChassisGroupName) && shared.StringInSlice(chassisID, members) {
+			_, err := o.nbctl("ha-chassis-group-remove-chassis", string(haChassisGroupName), chassisID)
+			if err != nil {
+				return err
+			}
 		}
 	}
 
+	// Nothing to do if chassis group doesn't exist.
 	return nil
 }
 
