@@ -52,7 +52,7 @@ var networksCmd = APIEndpoint{
 }
 
 var networkCmd = APIEndpoint{
-	Path: "networks/{name}",
+	Path: "networks/{networkName}",
 
 	Delete: APIEndpointAction{Handler: networkDelete, AccessHandler: allowProjectPermission("networks", "manage-networks")},
 	Get:    APIEndpointAction{Handler: networkGet, AccessHandler: allowProjectPermission("networks", "view")},
@@ -62,13 +62,13 @@ var networkCmd = APIEndpoint{
 }
 
 var networkLeasesCmd = APIEndpoint{
-	Path: "networks/{name}/leases",
+	Path: "networks/{networkName}/leases",
 
 	Get: APIEndpointAction{Handler: networkLeasesGet, AccessHandler: allowProjectPermission("networks", "view")},
 }
 
 var networkStateCmd = APIEndpoint{
-	Path: "networks/{name}/state",
+	Path: "networks/{networkName}/state",
 
 	Get: APIEndpointAction{Handler: networkStateGet, AccessHandler: allowProjectPermission("networks", "view")},
 }
@@ -168,7 +168,7 @@ var networkStateCmd = APIEndpoint{
 //   "500":
 //     $ref: "#/responses/InternalServerError"
 func networksGet(d *Daemon, r *http.Request) response.Response {
-	projectName, _, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -212,7 +212,7 @@ func networksGet(d *Daemon, r *http.Request) response.Response {
 		if !recursion {
 			resultString = append(resultString, fmt.Sprintf("/%s/networks/%s", version.APIVersion, networkName))
 		} else {
-			net, err := doNetworkGet(d, r, clustered, projectName, networkName)
+			net, err := doNetworkGet(d, r, clustered, projectName, reqProject.Config, networkName)
 			if err != nil {
 				continue
 			}
@@ -267,7 +267,7 @@ func networksGet(d *Daemon, r *http.Request) response.Response {
 //   "500":
 //     $ref: "#/responses/InternalServerError"
 func networksPost(d *Daemon, r *http.Request) response.Response {
-	projectName, projectConfig, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -288,6 +288,11 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("No name provided"))
 	}
 
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProject.Config, req.Name, true) {
+		return response.SmartError(api.StatusErrorf(http.StatusForbidden, "Network not allowed in project"))
+	}
+
 	if req.Type == "" {
 		if projectName != project.Default {
 			req.Type = "ovn" // Only OVN networks are allowed inside network enabled projects.
@@ -316,8 +321,8 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Check if project has limits.network and if so check we are allowed to create another network.
-	if projectName != project.Default && projectConfig != nil && projectConfig["limits.networks"] != "" {
-		networksLimit, err := strconv.Atoi(projectConfig["limits.networks"])
+	if projectName != project.Default && reqProject.Config != nil && reqProject.Config["limits.networks"] != "" {
+		networksLimit, err := strconv.Atoi(reqProject.Config["limits.networks"])
 		if err != nil {
 			return response.InternalError(fmt.Errorf("Invalid project limits.network value: %w", err))
 		}
@@ -344,7 +349,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 	if isClusterNotification(r) {
 		n, err := network.LoadByName(d.State(), projectName, req.Name)
 		if err != nil {
-			return response.SmartError(err)
+			return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
 		}
 
 		// This is an internal request which triggers the actual creation of the network across all nodes
@@ -372,7 +377,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			return tx.CreatePendingNetwork(targetNode, projectName, req.Name, netType.DBType(), req.Config)
+			return tx.CreatePendingNetwork(ctx, targetNode, projectName, req.Name, netType.DBType(), req.Config)
 		})
 		if err != nil {
 			if err == db.ErrAlreadyDefined {
@@ -404,17 +409,17 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		if !netTypeInfo.NodeSpecificConfig && clientType != clusterRequest.ClientTypeJoiner {
 			// Create pending entry for each node.
 			err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-				nodes, err := tx.GetNodes()
+				members, err := tx.GetNodes(ctx)
 				if err != nil {
-					return err
+					return fmt.Errorf("Failed getting cluster members: %w", err)
 				}
 
-				for _, node := range nodes {
+				for _, member := range members {
 					// Don't pass in any config, as these nodes don't have any node-specific
 					// config and we don't want to create duplicate global config.
-					err = tx.CreatePendingNetwork(node.Name, projectName, req.Name, netType.DBType(), nil)
+					err = tx.CreatePendingNetwork(ctx, member.Name, projectName, req.Name, netType.DBType(), nil)
 					if err != nil && !errors.Is(err, db.ErrAlreadyDefined) {
-						return fmt.Errorf("Failed creating pending network for node %q: %w", node.Name, err)
+						return fmt.Errorf("Failed creating pending network for member %q: %w", member.Name, err)
 					}
 				}
 
@@ -457,7 +462,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 
 	n, err := network.LoadByName(d.State(), projectName, req.Name)
 	if err != nil {
-		return response.SmartError(err)
+		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
 	}
 
 	err = doNetworksCreate(d, n, clientType)
@@ -530,13 +535,13 @@ func networksPostCluster(d *Daemon, projectName string, netInfo *api.Network, re
 		}
 
 		// Fetch the network ID.
-		networkID, err := tx.GetNetworkID(projectName, req.Name)
+		networkID, err := tx.GetNetworkID(ctx, projectName, req.Name)
 		if err != nil {
 			return err
 		}
 
 		// Fetch the node-specific configs and check the network is defined for all nodes.
-		nodeConfigs, err = tx.NetworkNodeConfigs(networkID)
+		nodeConfigs, err = tx.NetworkNodeConfigs(ctx, networkID)
 		if err != nil {
 			return err
 		}
@@ -573,7 +578,7 @@ func networksPostCluster(d *Daemon, projectName string, netInfo *api.Network, re
 	// Load the network from the database for the local member.
 	n, err := network.LoadByName(d.State(), projectName, req.Name)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed loading network: %w", err)
 	}
 
 	netConfig := n.Config()
@@ -751,7 +756,12 @@ func networkGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	projectName, _, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -766,12 +776,7 @@ func networkGet(d *Daemon, r *http.Request) response.Response {
 		allNodes = true
 	}
 
-	name, err := url.PathUnescape(mux.Vars(r)["name"])
-	if err != nil {
-		return response.SmartError(err)
-	}
-
-	n, err := doNetworkGet(d, r, allNodes, projectName, name)
+	n, err := doNetworkGet(d, r, allNodes, projectName, reqProject.Config, networkName)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -784,20 +789,28 @@ func networkGet(d *Daemon, r *http.Request) response.Response {
 // doNetworkGet returns information about the specified network.
 // If the network being requested is a managed network and allNodes is true then node specific config is removed.
 // Otherwise if allNodes is false then the network's local status is returned.
-func doNetworkGet(d *Daemon, r *http.Request, allNodes bool, projectName string, networkName string) (api.Network, error) {
+func doNetworkGet(d *Daemon, r *http.Request, allNodes bool, projectName string, reqProjectConfig map[string]string, networkName string) (api.Network, error) {
 	// Ignore veth pairs (for performance reasons).
 	if strings.HasPrefix(networkName, "veth") {
 		return api.Network{}, api.StatusErrorf(http.StatusNotFound, "Network not found")
 	}
 
 	// Get some information.
-	n, _ := network.LoadByName(d.State(), projectName, networkName)
+	n, err := network.LoadByName(d.State(), projectName, networkName)
+	if err != nil && !api.StatusErrorCheck(err, http.StatusNotFound) {
+		return api.Network{}, fmt.Errorf("Failed loading network: %w", err)
+	}
 
 	// Don't allow retrieving info about the local server interfaces when not using default project.
 	if projectName != project.Default && n == nil {
 		return api.Network{}, api.StatusErrorf(http.StatusNotFound, "Network not found")
 	}
 
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProjectConfig, networkName, n != nil && n.IsManaged()) {
+		return api.Network{}, api.StatusErrorf(http.StatusNotFound, "Network not found")
+	}
+
 	osInfo, _ := net.InterfaceByName(networkName)
 
 	// Quick check.
@@ -855,7 +868,7 @@ func doNetworkGet(d *Daemon, r *http.Request, allNodes bool, projectName string,
 			networkID = n.ID()
 		}
 
-		usedBy, err := network.UsedBy(d.State(), projectName, networkID, apiNet.Name, false)
+		usedBy, err := network.UsedBy(d.State(), projectName, networkID, apiNet.Name, apiNet.Type, false)
 		if err != nil {
 			return api.Network{}, err
 		}
@@ -901,12 +914,12 @@ func doNetworkGet(d *Daemon, r *http.Request, allNodes bool, projectName string,
 //   "500":
 //     $ref: "#/responses/InternalServerError"
 func networkDelete(d *Daemon, r *http.Request) response.Response {
-	projectName, _, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	name, err := url.PathUnescape(mux.Vars(r)["name"])
+	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -914,9 +927,14 @@ func networkDelete(d *Daemon, r *http.Request) response.Response {
 	state := d.State()
 
 	// Get the existing network.
-	n, err := network.LoadByName(state, projectName, name)
+	n, err := network.LoadByName(state, projectName, networkName)
 	if err != nil {
-		return response.SmartError(err)
+		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
+	}
+
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProject.Config, networkName, n.IsManaged()) {
+		return response.SmartError(api.StatusErrorf(http.StatusNotFound, "Network not found"))
 	}
 
 	clientType := clusterRequest.UserAgentClientType(r.Header.Get("User-Agent"))
@@ -1028,7 +1046,12 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Renaming clustered network not supported"))
 	}
 
-	name, err := url.PathUnescape(mux.Vars(r)["name"])
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1042,15 +1065,15 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	projectName, _, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	// Get the existing network.
+	n, err := network.LoadByName(state, projectName, networkName)
 	if err != nil {
-		return response.SmartError(err)
+		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
 	}
 
-	// Get the existing network.
-	n, err := network.LoadByName(state, projectName, name)
-	if err != nil {
-		return response.SmartError(err)
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProject.Config, networkName, n.IsManaged()) {
+		return response.SmartError(api.StatusErrorf(http.StatusNotFound, "Network not found"))
 	}
 
 	if n.Status() != api.NetworkStatusCreated {
@@ -1094,7 +1117,7 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	requestor := request.CreateRequestor(r)
-	lc := lifecycle.NetworkRenamed.Event(n, requestor, map[string]any{"old_name": name})
+	lc := lifecycle.NetworkRenamed.Event(n, requestor, map[string]any{"old_name": networkName})
 	d.State().Events.SendLifecycle(projectName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
@@ -1146,20 +1169,25 @@ func networkPut(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	projectName, _, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	name, err := url.PathUnescape(mux.Vars(r)["name"])
+	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Get the existing network.
-	n, err := network.LoadByName(d.State(), projectName, name)
+	n, err := network.LoadByName(d.State(), projectName, networkName)
 	if err != nil {
-		return response.SmartError(err)
+		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
+	}
+
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProject.Config, networkName, n.IsManaged()) {
+		return response.SmartError(api.StatusErrorf(http.StatusNotFound, "Network not found"))
 	}
 
 	targetNode := queryParam(r, "target")
@@ -1366,26 +1394,29 @@ func doNetworkUpdate(d *Daemon, projectName string, n network.Network, req api.N
 //   "500":
 //     $ref: "#/responses/InternalServerError"
 func networkLeasesGet(d *Daemon, r *http.Request) response.Response {
-	projectName := projectParam(r)
-	name, err := url.PathUnescape(mux.Vars(r)["name"])
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	// The project we should use to load the network.
-	networkProjectName, _, err := project.NetworkProject(d.State().DB.Cluster, projectName)
+	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Attempt to load the network.
-	n, err := network.LoadByName(d.State(), networkProjectName, name)
+	n, err := network.LoadByName(d.State(), projectName, networkName)
 	if err != nil {
-		return response.SmartError(err)
+		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
+	}
+
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProject.Config, networkName, n.IsManaged()) {
+		return response.SmartError(api.StatusErrorf(http.StatusNotFound, "Network not found"))
 	}
 
 	clientType := clusterRequest.UserAgentClientType(r.Header.Get("User-Agent"))
-	leases, err := n.Leases(projectName, clientType)
+	leases, err := n.Leases(reqProject.Name, clientType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1407,11 +1438,16 @@ func networkStartup(s *state.State) error {
 		return fmt.Errorf("Failed to load projects: %w", err)
 	}
 
-	// List of networks that need to be started after non-dependent networks.
-	deferredNetworks := make([]network.Network, 0)
-
 	// Build a list of networks to initialise, keyed by project and network name.
-	initNetworks := make(map[network.ProjectNetwork]struct{}, 0)
+	const networkPriorityStandalone = 0 // Start networks not dependent on any other network first.
+	const networkPriorityPhysical = 1   // Start networks dependent on physical interfaces second.
+	const networkPriorityLogical = 2    // Start networks dependent logical networks third.
+	initNetworks := []map[network.ProjectNetwork]struct{}{
+		networkPriorityStandalone: make(map[network.ProjectNetwork]struct{}),
+		networkPriorityPhysical:   make(map[network.ProjectNetwork]struct{}),
+		networkPriorityLogical:    make(map[network.ProjectNetwork]struct{}),
+	}
+
 	for _, projectName := range projectNames {
 		networkNames, err := s.DB.Cluster.GetCreatedNetworks(projectName)
 		if err != nil {
@@ -1424,11 +1460,14 @@ func networkStartup(s *state.State) error {
 				NetworkName: networkName,
 			}
 
-			initNetworks[pn] = struct{}{}
+			// Asssume all networks are networkPriorityStandalone initially.
+			initNetworks[networkPriorityStandalone][pn] = struct{}{}
 		}
 	}
 
-	initNetwork := func(n network.Network) error {
+	loadedNetworks := make(map[network.ProjectNetwork]network.Network)
+
+	initNetwork := func(n network.Network, priority int) error {
 		err = n.Start()
 		if err != nil {
 			err = fmt.Errorf("Failed starting: %w", err)
@@ -1445,31 +1484,33 @@ func networkStartup(s *state.State) error {
 			NetworkName: n.Name(),
 		}
 
-		delete(initNetworks, pn)
+		delete(initNetworks[priority], pn)
 
 		_ = warnings.ResolveWarningsByLocalNodeAndProjectAndTypeAndEntity(s.DB.Cluster, n.Project(), warningtype.NetworkUnvailable, dbCluster.TypeNetwork, int(n.ID()))
 
 		return nil
 	}
 
-	errDeferredStartup := fmt.Errorf("Deferred start")
+	loadAndInitNetwork := func(pn network.ProjectNetwork, priority int, firstPass bool) error {
+		var err error
+		var n network.Network
 
-	loadAndInitNetwork := func(projectName, networkName string, firstPass bool) error {
-		n, err := network.LoadByName(s, projectName, networkName)
-		if err != nil {
-			if api.StatusErrorCheck(err, http.StatusNotFound) {
-				// Network has been deleted since we started trying to start it so delete entry.
-				pn := network.ProjectNetwork{
-					ProjectName: n.Project(),
-					NetworkName: n.Name(),
+		if firstPass && loadedNetworks[pn] != nil {
+			// Check if network already loaded from during first pass phase.
+			n = loadedNetworks[pn]
+		} else {
+			n, err = network.LoadByName(s, pn.ProjectName, pn.NetworkName)
+			if err != nil {
+				if api.StatusErrorCheck(err, http.StatusNotFound) {
+					// Network has been deleted since we began trying to start it so delete
+					// entry.
+					delete(initNetworks[priority], pn)
+
+					return nil
 				}
 
-				delete(initNetworks, pn) // Can't start a network that no longer exists.
-
-				return nil
+				return fmt.Errorf("Failed loading: %w", err)
 			}
-
-			return fmt.Errorf("Failed loading: %w", err)
 		}
 
 		netConfig := n.Config()
@@ -1478,14 +1519,24 @@ func networkStartup(s *state.State) error {
 			return fmt.Errorf("Failed validating: %w", err)
 		}
 
-		// Defer network start until after non-dependent networks on first pass.
-		if firstPass && netConfig["network"] != "" {
-			deferredNetworks = append(deferredNetworks, n)
+		// Update network start priority based on dependencies.
+		if netConfig["parent"] != "" && priority != networkPriorityPhysical {
+			// Start networks that depend on physical interfaces existing after
+			// non-dependent networks.
+			delete(initNetworks[priority], pn)
+			initNetworks[networkPriorityPhysical][pn] = struct{}{}
 
-			return errDeferredStartup
+			return nil
+		} else if netConfig["network"] != "" && priority != networkPriorityLogical {
+			// Start networks that depend on other logical networks after networks after
+			// non-dependent networks and networks that depend on physical interfaces.
+			delete(initNetworks[priority], pn)
+			initNetworks[networkPriorityLogical][pn] = struct{}{}
+
+			return nil
 		}
 
-		err = initNetwork(n)
+		err = initNetwork(n, priority)
 		if err != nil {
 			return err
 		}
@@ -1493,35 +1544,28 @@ func networkStartup(s *state.State) error {
 		return nil
 	}
 
-	// Try initializing networks in a random order.
-	for pn := range initNetworks {
-		err := loadAndInitNetwork(pn.ProjectName, pn.NetworkName, true)
-		if err != nil {
-			if errors.Is(err, errDeferredStartup) {
+	// Try initializing networks in priority order.
+	for priority := range initNetworks {
+		for pn := range initNetworks[priority] {
+			err := loadAndInitNetwork(pn, priority, true)
+			if err != nil {
+				logger.Error("Failed initializing network", logger.Ctx{"project": pn.ProjectName, "network": pn.NetworkName, "err": err})
+
 				continue
 			}
-
-			logger.Error("Failed initializing network", logger.Ctx{"project": pn.ProjectName, "network": pn.NetworkName, "err": err})
-
-			continue
 		}
 	}
 
-	// Bring up deferred networks after non-dependent networks have been started.
-	for _, n := range deferredNetworks {
-		err = initNetwork(n)
-		if err != nil {
-			logger.Error("Failed initializing network", logger.Ctx{"project": n.Project(), "network": n.Name(), "err": err})
+	loadedNetworks = nil // Don't store loaded networks after first pass.
 
-			continue
-		}
+	remainingNetworks := 0
+	for _, networks := range initNetworks {
+		remainingNetworks += len(networks)
 	}
 
-	deferredNetworks = nil // Don't keep references to the deferred networks around from here.
-
 	// For any remaining networks that were not successfully initialised, we now start a go routine to
 	// periodically try to initialize them again in the background.
-	if len(initNetworks) > 0 {
+	if remainingNetworks > 0 {
 		go func() {
 			for {
 				t := time.NewTimer(time.Duration(time.Minute))
@@ -1533,20 +1577,28 @@ func networkStartup(s *state.State) error {
 				case <-t.C:
 					t.Stop()
 
-					// Try initializing remaining networks in random order.
 					tryInstancesStart := false
-					for pn := range initNetworks {
-						err := loadAndInitNetwork(pn.ProjectName, pn.NetworkName, false)
-						if err != nil {
-							logger.Error("Failed initializing network", logger.Ctx{"project": pn.ProjectName, "network": pn.NetworkName, "err": err})
 
-							continue
+					// Try initializing networks in priority order.
+					for priority := range initNetworks {
+						for pn := range initNetworks[priority] {
+							err := loadAndInitNetwork(pn, priority, false)
+							if err != nil {
+								logger.Error("Failed initializing network", logger.Ctx{"project": pn.ProjectName, "network": pn.NetworkName, "err": err})
+
+								continue
+							}
+
+							tryInstancesStart = true // We initialized at least one network.
 						}
-
-						tryInstancesStart = true // We initialized at least one network.
 					}
 
-					if len(initNetworks) <= 0 {
+					remainingNetworks := 0
+					for _, networks := range initNetworks {
+						remainingNetworks += len(networks)
+					}
+
+					if remainingNetworks <= 0 {
 						logger.Info("All networks initialized")
 					}
 
@@ -1561,15 +1613,16 @@ func networkStartup(s *state.State) error {
 						}
 					}
 
-					if len(initNetworks) <= 0 {
+					if remainingNetworks <= 0 {
 						return // Our job here is done.
 					}
 				}
 			}
 		}()
+	} else {
+		logger.Info("All networks initialized")
 	}
 
-	logger.Info("All networks initialized")
 	return nil
 }
 
@@ -1709,16 +1762,28 @@ func networkStateGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	networkName, err := url.PathUnescape(mux.Vars(r)["name"])
+	projectName, reqProject, err := project.NetworkProject(d.State().DB.Cluster, projectParam(r))
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	projectName := projectParam(r)
+	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	n, err := network.LoadByName(d.State(), projectName, networkName)
+	if err != nil && !api.StatusErrorCheck(err, http.StatusNotFound) {
+		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
+	}
+
+	// Check if project allows access to network.
+	if !project.NetworkAllowed(reqProject.Config, networkName, n != nil && n.IsManaged()) {
+		return response.SmartError(api.StatusErrorf(http.StatusNotFound, "Network not found"))
+	}
 
 	var state *api.NetworkState
-	n, networkLoadError := network.LoadByName(d.State(), projectName, networkName)
-	if networkLoadError == nil {
+	if n != nil {
 		state, err = n.State()
 		if err != nil {
 			return response.SmartError(err)
