@@ -16,7 +16,6 @@ import (
 	dbCluster "github.com/lxc/lxd/lxd/db/cluster"
 	"github.com/lxc/lxd/lxd/db/operationtype"
 	"github.com/lxc/lxd/lxd/lifecycle"
-	"github.com/lxc/lxd/lxd/node"
 	"github.com/lxc/lxd/lxd/operations"
 	"github.com/lxc/lxd/lxd/project"
 	"github.com/lxc/lxd/lxd/rbac"
@@ -59,9 +58,6 @@ var operationWebsocket = APIEndpoint{
 func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdownTimeout time.Duration) {
 	timeout := time.After(consoleShutdownTimeout)
 
-	tick := time.Tick(time.Second)
-	logTick := time.Tick(time.Minute)
-
 	defer func() {
 		_ = cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			err := dbCluster.DeleteOperations(ctx, tx.Tx(), cluster.GetNodeID())
@@ -73,35 +69,23 @@ func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdown
 		})
 	}()
 
-	for {
-		<-tick
+	// Check operation status every second.
+	tick := time.NewTicker(time.Second)
+	defer tick.Stop()
 
+	var i int
+	for {
 		// Get all the operations
 		ops := operations.Clone()
 
-		runningOps := 0
-
+		var runningOps, execConsoleOps int
 		for _, op := range ops {
-			if op.Status() != api.Running {
-				continue
-			}
-
-			if op.Class() == operations.OperationClassToken {
+			if op.Status() != api.Running || op.Class() == operations.OperationClassToken {
 				continue
 			}
 
 			runningOps++
-		}
 
-		// No more running operations left. Exit function.
-		if runningOps == 0 {
-			logger.Info("All running operations finished, shutting down")
-			return
-		}
-
-		execConsoleOps := 0
-
-		for _, op := range ops {
 			opType := op.Type()
 			if opType == operationtype.CommandExec || opType == operationtype.ConsoleShow {
 				execConsoleOps++
@@ -115,24 +99,34 @@ func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdown
 			}
 		}
 
+		// No more running operations left. Exit function.
+		if runningOps == 0 {
+			logger.Info("All running operations finished, shutting down")
+			return
+		}
+
+		// Print log message every minute.
+		if i%60 == 0 {
+			logger.Infof("Waiting for %d operation(s) to finish", runningOps)
+		}
+
+		i++
+
 		select {
 		case <-timeout:
-			// We wait up to 5 minutes for exec/console operations to finish.
-			// If there are still running operations, we shut down the instances
-			// which will terminate the operations.
+			// We wait up to core.shutdown_timeout minutes for exec/console operations to finish.
+			// If there are still running operations, we continue shutdown which will stop any running
+			// instances and terminate the operations.
 			if execConsoleOps > 0 {
-				logger.Info("Timeout reached, continuing with shutdown")
+				logger.Info("Shutdown timeout reached, continuing with shutdown")
 			}
 
 			return
-		case <-logTick:
-			// Print log message every minute.
-			logger.Infof("Waiting for %d operation(s) to finish", runningOps)
 		case <-ctx.Done():
 			// Return here, and ignore any running operations.
 			logger.Info("Forcing shutdown, ignoring running operations")
 			return
-		default:
+		case <-tick.C:
 		}
 	}
 }
@@ -557,23 +551,23 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 	// Get all nodes with running operations in this project.
 	var membersWithOps []string
 	var offlineThreshold time.Duration
-	var nodes []db.NodeInfo
+	var members []db.NodeInfo
 	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		membersWithOps, err = tx.GetNodesWithOperations(projectName)
+		membersWithOps, err = tx.GetNodesWithOperations(ctx, projectName)
 		if err != nil {
 			return fmt.Errorf("Failed getting members with operations: %w", err)
 		}
 
-		offlineThreshold, err = tx.GetNodeOfflineThreshold()
+		offlineThreshold, err = tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed getting member offline threshold value: %w", err)
 		}
 
-		nodes, err = tx.GetNodes()
+		members, err = tx.GetNodes(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed getting members: %w", err)
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
 		return nil
@@ -583,16 +577,13 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Get local address.
-	localClusterAddress, err := node.ClusterAddress(d.db.Node)
-	if err != nil {
-		return response.InternalError(fmt.Errorf("Failed getting member local cluster address: %w", err))
-	}
+	localClusterAddress := d.State().LocalConfig.ClusterAddress()
 
 	memberOnline := func(memberAddress string) bool {
-		for _, node := range nodes {
-			if node.Address == memberAddress {
-				if node.IsOffline(offlineThreshold) {
-					logger.Warn("Excluding offline member from operations list", logger.Ctx{"member": node.Name, "address": node.Address, "ID": node.ID, "lastHeartbeat": node.Heartbeat})
+		for _, member := range members {
+			if member.Address == memberAddress {
+				if member.IsOffline(offlineThreshold) {
+					logger.Warn("Excluding offline member from operations list", logger.Ctx{"member": member.Name, "address": member.Address, "ID": member.ID, "lastHeartbeat": member.Heartbeat})
 					return false
 				}
 
@@ -682,20 +673,20 @@ func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType
 
 	// Get all operations of the specified type in project.
 	var offlineThreshold time.Duration
-	var nodes []db.NodeInfo
+	var members []db.NodeInfo
 	memberOps := make(map[string]map[string]dbCluster.Operation)
 	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		offlineThreshold, err = tx.GetNodeOfflineThreshold()
+		offlineThreshold, err = tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
 			return fmt.Errorf("Failed getting member offline threshold value: %w", err)
 		}
 
-		nodes, err = tx.GetNodes()
+		members, err = tx.GetNodes(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed getting members: %w", err)
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
-		ops, err := tx.GetOperationsOfType(projectName, opType)
+		ops, err := tx.GetOperationsOfType(ctx, projectName, opType)
 		if err != nil {
 			return fmt.Errorf("Failed getting operations for project %q and type %d: %w", projectName, opType, err)
 		}
@@ -716,16 +707,13 @@ func operationsGetByType(d *Daemon, r *http.Request, projectName string, opType
 	}
 
 	// Get local address.
-	localClusterAddress, err := node.ClusterAddress(d.db.Node)
-	if err != nil {
-		return nil, fmt.Errorf("Failed getting member local cluster address: %w", err)
-	}
+	localClusterAddress := d.State().LocalConfig.ClusterAddress()
 
 	memberOnline := func(memberAddress string) bool {
-		for _, node := range nodes {
-			if node.Address == memberAddress {
-				if node.IsOffline(offlineThreshold) {
-					logger.Warn("Excluding offline member from operations by type list", logger.Ctx{"member": node.Name, "address": node.Address, "ID": node.ID, "lastHeartbeat": node.Heartbeat, "opType": opType})
+		for _, member := range members {
+			if member.Address == memberAddress {
+				if member.IsOffline(offlineThreshold) {
+					logger.Warn("Excluding offline member from operations by type list", logger.Ctx{"member": member.Name, "address": member.Address, "ID": member.ID, "lastHeartbeat": member.Heartbeat, "opType": opType})
 					return false
 				}
 
@@ -1083,11 +1071,7 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 
 func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 	f := func(ctx context.Context) {
-		localAddress, err := node.ClusterAddress(d.db.Node)
-		if err != nil {
-			logger.Error("Failed to get current cluster member address", logger.Ctx{"err": err})
-			return
-		}
+		localClusterAddress := d.State().LocalConfig.ClusterAddress()
 
 		leader, err := d.gateway.LeaderAddress()
 		if err != nil {
@@ -1099,7 +1083,7 @@ func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 			return
 		}
 
-		if localAddress != leader {
+		if localClusterAddress != leader {
 			logger.Debug("Skipping remove orphaned operations task since we're not leader")
 			return
 		}
@@ -1135,23 +1119,23 @@ func autoRemoveOrphanedOperations(ctx context.Context, d *Daemon) error {
 
 	err := d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get offline threshold.
-		offlineThreshold, err := tx.GetNodeOfflineThreshold()
+		offlineThreshold, err := tx.GetNodeOfflineThreshold(ctx)
 		if err != nil {
 			return fmt.Errorf("Load offline threshold config: %w", err)
 		}
 
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed to get nodes: %w", err)
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
-		for _, node := range nodes {
+		for _, member := range members {
 			// Skip online nodes
-			if !node.IsOffline(offlineThreshold) {
+			if !member.IsOffline(offlineThreshold) {
 				continue
 			}
 
-			err = dbCluster.DeleteOperations(ctx, tx.Tx(), node.ID)
+			err = dbCluster.DeleteOperations(ctx, tx.Tx(), member.ID)
 			if err != nil {
 				return fmt.Errorf("Failed to delete operations: %w", err)
 			}
