@@ -4,7 +4,6 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -145,7 +144,7 @@ func patchDnsmasqEntriesIncludeDeviceName(name string, d *Daemon) error {
 
 func patchRemoveWarningsWithEmptyNode(name string, d *Daemon) error {
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		warnings, err := dbCluster.GetWarnings(ctx, tx.Tx(), dbCluster.WarningFilter{})
+		warnings, err := dbCluster.GetWarnings(ctx, tx.Tx())
 		if err != nil {
 			return err
 		}
@@ -180,7 +179,7 @@ func patchClusteringServerCertTrust(name string, d *Daemon) error {
 
 	var serverName string
 	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		serverName, err = tx.GetLocalNodeName()
+		serverName, err = tx.GetLocalNodeName(ctx)
 		return err
 	})
 	if err != nil {
@@ -208,7 +207,7 @@ func patchClusteringServerCertTrust(name string, d *Daemon) error {
 		var err error
 		var dbCerts []dbCluster.Certificate
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			dbCerts, err = dbCluster.GetCertificates(ctx, tx.Tx(), dbCluster.CertificateFilter{})
+			dbCerts, err = dbCluster.GetCertificates(ctx, tx.Tx())
 			return err
 		})
 		if err != nil {
@@ -223,11 +222,11 @@ func patchClusteringServerCertTrust(name string, d *Daemon) error {
 			}
 		}
 
-		var nodes []db.NodeInfo
+		var members []db.NodeInfo
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			nodes, err = tx.GetNodes()
+			members, err = tx.GetNodes(ctx)
 			if err != nil {
-				return err
+				return fmt.Errorf("Failed getting cluster members: %w", err)
 			}
 
 			return nil
@@ -237,10 +236,10 @@ func patchClusteringServerCertTrust(name string, d *Daemon) error {
 		}
 
 		missingCerts := false
-		for _, n := range nodes {
-			_, found := trustedServerCerts[n.Name]
+		for _, member := range members {
+			_, found := trustedServerCerts[member.Name]
 			if !found {
-				logger.Warnf("Missing trusted server certificate for cluster member %q", n.Name)
+				logger.Warnf("Missing trusted server certificate for cluster member %q", member.Name)
 				missingCerts = true
 				break
 			}
@@ -341,7 +340,11 @@ func patchDBNodesAutoInc(name string, d *Daemon) error {
 		}
 
 		// Only apply patch on leader, otherwise wait for it to be applied.
-		clusterAddress, err := node.ClusterAddress(d.db.Node)
+		var localConfig *node.Config
+		err = d.db.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+			localConfig, err = node.ConfigLoad(ctx, tx)
+			return err
+		})
 		if err != nil {
 			return err
 		}
@@ -355,7 +358,7 @@ func patchDBNodesAutoInc(name string, d *Daemon) error {
 			return err
 		}
 
-		if clusterAddress == leaderAddress {
+		if localConfig.ClusterAddress() == leaderAddress {
 			break // Apply change on leader node.
 		}
 
@@ -401,7 +404,7 @@ func patchVMRenameUUIDKey(name string, d *Daemon) error {
 	oldUUIDKey := "volatile.vm.uuid"
 	newUUIDKey := "volatile.uuid"
 
-	return d.State().DB.Cluster.InstanceList(nil, func(inst db.InstanceArgs, p api.Project) error {
+	return d.State().DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
 		if inst.Type != instancetype.VM {
 			return nil
 		}
@@ -466,21 +469,20 @@ func patchThinpoolTypoFix(name string, d *Daemon) error {
 	revert.Add(func() { _ = tx.Rollback() })
 
 	// Fetch the IDs of all existing nodes.
-	nodeIDs, err := query.SelectIntegers(tx, "SELECT id FROM nodes")
+	nodeIDs, err := query.SelectIntegers(context.TODO(), tx, "SELECT id FROM nodes")
 	if err != nil {
 		return fmt.Errorf("Failed to get IDs of current nodes: %w", err)
 	}
 
 	// Fetch the IDs of all existing lvm pools.
-	poolIDs, err := query.SelectIntegers(tx, "SELECT id FROM storage_pools WHERE driver='lvm'")
+	poolIDs, err := query.SelectIntegers(context.TODO(), tx, "SELECT id FROM storage_pools WHERE driver='lvm'")
 	if err != nil {
 		return fmt.Errorf("Failed to get IDs of current lvm pools: %w", err)
 	}
 
 	for _, poolID := range poolIDs {
 		// Fetch the config for this lvm pool and check if it has the lvm.thinpool_name.
-		config, err := query.SelectConfig(
-			tx, "storage_pools_config", "storage_pool_id=? AND node_id IS NULL", poolID)
+		config, err := query.SelectConfig(context.TODO(), tx, "storage_pools_config", "storage_pool_id=? AND node_id IS NULL", poolID)
 		if err != nil {
 			return fmt.Errorf("Failed to fetch of lvm pool config: %w", err)
 		}
@@ -524,7 +526,7 @@ INSERT INTO storage_pools_config(storage_pool_id, node_id, key, value)
 // having "ipv4.nat" set is to disable NAT (bringing in line with the non-fan bridge behavior and docs).
 func patchNetworkFANEnableNAT(name string, d *Daemon) error {
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		projectNetworks, err := tx.GetCreatedNetworks()
+		projectNetworks, err := tx.GetCreatedNetworks(ctx)
 		if err != nil {
 			return err
 		}
@@ -572,7 +574,7 @@ func patchNetworkFANEnableNAT(name string, d *Daemon) error {
 // networks. It was decided that the OVN NIC level equivalent settings were sufficient.
 func patchNetworkOVNRemoveRoutes(name string, d *Daemon) error {
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		projectNetworks, err := tx.GetCreatedNetworks()
+		projectNetworks, err := tx.GetCreatedNetworks(ctx)
 		if err != nil {
 			return err
 		}
@@ -624,7 +626,7 @@ func patchNetworkOVNRemoveRoutes(name string, d *Daemon) error {
 // patchNetworkCearBridgeVolatileHwaddr removes the unsupported `volatile.bridge.hwaddr` config key from networks.
 func patchNetworkOVNEnableNAT(name string, d *Daemon) error {
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		projectNetworks, err := tx.GetCreatedNetworks()
+		projectNetworks, err := tx.GetCreatedNetworks(ctx)
 		if err != nil {
 			return err
 		}
@@ -681,7 +683,7 @@ func patchMoveBackupsInstances(name string, d *Daemon) error {
 		return fmt.Errorf("Failed creating instances backup directory %q: %w", backupsPath, err)
 	}
 
-	backups, err := ioutil.ReadDir(shared.VarPath("backups"))
+	backups, err := os.ReadDir(shared.VarPath("backups"))
 	if err != nil {
 		return fmt.Errorf("Failed listing existing backup directory %q: %w", shared.VarPath("backups"), err)
 	}
@@ -720,13 +722,13 @@ func patchGenericNetwork(f func(name string, d *Daemon) error) func(name string,
 
 func patchClusteringDropDatabaseRole(name string, d *Daemon) error {
 	return d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		nodes, err := tx.GetNodes()
+		members, err := tx.GetNodes(ctx)
 		if err != nil {
-			return err
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
-		for _, node := range nodes {
-			err := tx.UpdateNodeRoles(node.ID, nil)
+		for _, member := range members {
+			err := tx.UpdateNodeRoles(member.ID, nil)
 			if err != nil {
 				return err
 			}
