@@ -14,10 +14,10 @@ import (
 	"github.com/lxc/lxd/shared/api"
 )
 
-func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profile *api.Profile, req api.ProfilePut) error {
+func doProfileUpdate(d *Daemon, p api.Project, profileName string, id int64, profile *api.Profile, req api.ProfilePut) error {
 	// Check project limits.
 	err := d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		return project.AllowProfileUpdate(tx, projectName, name, req)
+		return project.AllowProfileUpdate(tx, p.Name, profileName, req)
 	})
 	if err != nil {
 		return err
@@ -31,18 +31,18 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 
 	// Profiles can be applied to any instance type, so just use instancetype.Any type for validation so that
 	// instance type specific validation checks are not performed.
-	err = instance.ValidDevices(d.State(), projectName, instancetype.Any, deviceConfig.NewDevices(req.Devices), false)
+	err = instance.ValidDevices(d.State(), p, instancetype.Any, deviceConfig.NewDevices(req.Devices), nil)
 	if err != nil {
 		return err
 	}
 
-	insts, err := getProfileInstancesInfo(d.db.Cluster, projectName, name)
+	insts, projects, err := getProfileInstancesInfo(d.db.Cluster, p.Name, profileName)
 	if err != nil {
-		return fmt.Errorf("Failed to query instances associated with profile %q: %w", name, err)
+		return fmt.Errorf("Failed to query instances associated with profile %q: %w", profileName, err)
 	}
 
-	// Check if the root disk device's pool is supposed to be changed or removed and prevent that if there are
-	// instances using that root disk device.
+	// Check if the root disk device's pool would be changed or removed and prevent that if there are instances
+	// using that root disk device.
 	oldProfileRootDiskDeviceKey, oldProfileRootDiskDevice, _ := shared.GetRootDiskDevice(profile.Devices)
 	_, newProfileRootDiskDevice, _ := shared.GetRootDiskDevice(req.Devices)
 	if len(insts) > 0 && oldProfileRootDiskDevice["pool"] != "" && newProfileRootDiskDevice["pool"] == "" || (oldProfileRootDiskDevice["pool"] != newProfileRootDiskDevice["pool"]) {
@@ -56,7 +56,7 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 
 			// Check what profile the device comes from by working backwards along the profiles list.
 			for i := len(inst.Profiles) - 1; i >= 0; i-- {
-				_, profile, err := d.db.Cluster.GetProfile(projectName, inst.Profiles[i].Name)
+				_, profile, err := d.db.Cluster.GetProfile(p.Name, inst.Profiles[i].Name)
 				if err != nil {
 					return err
 				}
@@ -65,7 +65,7 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 				_, ok := profile.Devices[oldProfileRootDiskDeviceKey]
 				if ok {
 					// Found the profile.
-					if inst.Profiles[i].Name == name {
+					if inst.Profiles[i].Name == profileName {
 						// If it's the current profile, then we can't modify that root device.
 						return fmt.Errorf("At least one instance relies on this profile's root disk device")
 					}
@@ -84,16 +84,16 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 			return err
 		}
 
-		err = cluster.UpdateProfile(ctx, tx.Tx(), projectName, name, cluster.Profile{
-			Project:     projectName,
-			Name:        name,
+		err = cluster.UpdateProfile(ctx, tx.Tx(), p.Name, profileName, cluster.Profile{
+			Project:     p.Name,
+			Name:        profileName,
 			Description: req.Description,
 		})
 		if err != nil {
 			return err
 		}
 
-		id, err := cluster.GetProfileID(ctx, tx.Tx(), projectName, name)
+		id, err := cluster.GetProfileID(ctx, tx.Tx(), p.Name, profileName)
 		if err != nil {
 			return err
 		}
@@ -105,31 +105,16 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 
 		err = cluster.UpdateProfileDevices(ctx, tx.Tx(), id, devices)
 		if err != nil {
-			return nil
+			return err
 		}
 
-		newProfiles, err := cluster.GetProfilesIfEnabled(ctx, tx.Tx(), projectName, []string{name})
+		newProfiles, err := cluster.GetProfilesIfEnabled(ctx, tx.Tx(), p.Name, []string{profileName})
 		if err != nil {
 			return err
 		}
 
 		if len(newProfiles) != 1 {
-			return fmt.Errorf("Failed to find profile %q in project %q", name, projectName)
-		}
-
-		apiProfile, err := newProfiles[0].ToAPI(ctx, tx.Tx())
-		if err != nil {
-			return err
-		}
-
-		// Update the profile on our current list of instances.
-		for i := range insts {
-			for j, profile := range insts[i].Profiles {
-				if profile.Name == apiProfile.Name {
-					insts[i].Profiles[j] = *apiProfile
-					break
-				}
-			}
+			return fmt.Errorf("Failed to find profile %q in project %q", profileName, p.Name)
 		}
 
 		return nil
@@ -144,7 +129,12 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 	failures := map[*db.InstanceArgs]error{}
 	for _, it := range insts {
 		inst := it // Local var for instance pointer.
-		err := doProfileUpdateInstance(d, name, profile.ProfilePut, serverName, inst)
+
+		if inst.Node != "" && inst.Node != serverName {
+			continue // This instance does not belong to this member, skip.
+		}
+
+		err := doProfileUpdateInstance(d, inst, *projects[inst.Project])
 		if err != nil {
 			failures[&inst] = err
 		}
@@ -164,18 +154,34 @@ func doProfileUpdate(d *Daemon, projectName string, name string, id int64, profi
 
 // Like doProfileUpdate but does not update the database, since it was already
 // updated by doProfileUpdate itself, called on the notifying node.
-func doProfileUpdateCluster(d *Daemon, projectName string, name string, old api.ProfilePut) error {
+func doProfileUpdateCluster(d *Daemon, projectName string, profileName string, old api.ProfilePut) error {
 	serverName := d.State().ServerName
 
-	insts, err := getProfileInstancesInfo(d.db.Cluster, projectName, name)
+	insts, projects, err := getProfileInstancesInfo(d.db.Cluster, projectName, profileName)
 	if err != nil {
-		return fmt.Errorf("Failed to query instances associated with profile %q: %w", name, err)
+		return fmt.Errorf("Failed to query instances associated with profile %q: %w", profileName, err)
 	}
 
 	failures := map[*db.InstanceArgs]error{}
 	for _, it := range insts {
 		inst := it // Local var for instance pointer.
-		err := doProfileUpdateInstance(d, name, old, serverName, inst)
+
+		if inst.Node != "" && inst.Node != serverName {
+			continue // This instance does not belong to this member, skip.
+		}
+
+		for i, profile := range inst.Profiles {
+			if profile.Name == profileName {
+				// As profile has already been updated in the database by this point, overwrite the
+				// new config from the database with the old config and devices, so that
+				// doProfileUpdateInstance will detect the changes and apply them.
+				inst.Profiles[i].Config = old.Config
+				inst.Profiles[i].Devices = old.Devices
+				break
+			}
+		}
+
+		err := doProfileUpdateInstance(d, inst, *projects[inst.Project])
 		if err != nil {
 			failures[&inst] = err
 		}
@@ -194,12 +200,7 @@ func doProfileUpdateCluster(d *Daemon, projectName string, name string, old api.
 }
 
 // Profile update of a single instance.
-func doProfileUpdateInstance(d *Daemon, name string, old api.ProfilePut, nodeName string, args db.InstanceArgs) error {
-	if args.Node != "" && args.Node != nodeName {
-		// No-op, this instance does not belong to this node.
-		return nil
-	}
-
+func doProfileUpdateInstance(d *Daemon, args db.InstanceArgs, p api.Project) error {
 	profileNames := make([]string, 0, len(args.Profiles))
 	for _, profile := range args.Profiles {
 		profileNames = append(profileNames, profile.Name)
@@ -210,17 +211,8 @@ func doProfileUpdateInstance(d *Daemon, name string, old api.ProfilePut, nodeNam
 		return err
 	}
 
-	for i, profile := range args.Profiles {
-		if profile.Name == name {
-			// Overwrite the new config from the database with the old config and devices.
-			profiles[i].Config = old.Config
-			profiles[i].Devices = old.Devices
-			break
-		}
-	}
-
 	// Load the instance using the old profile config.
-	inst, err := instance.Load(d.State(), args, profiles)
+	inst, err := instance.Load(d.State(), args, p)
 	if err != nil {
 		return err
 	}
@@ -232,26 +224,42 @@ func doProfileUpdateInstance(d *Daemon, name string, old api.ProfilePut, nodeNam
 		Description:  inst.Description(),
 		Devices:      inst.LocalDevices(),
 		Ephemeral:    inst.IsEphemeral(),
-		Profiles:     inst.Profiles(),
-		Project:      inst.Project(),
+		Profiles:     profiles, // Supply with new profile config.
+		Project:      inst.Project().Name,
 		Type:         inst.Type(),
 		Snapshot:     inst.IsSnapshot(),
 	}, true)
 }
 
 // Query the db for information about instances associated with the given profile.
-func getProfileInstancesInfo(dbCluster *db.Cluster, projectName string, profileName string) (map[int]db.InstanceArgs, error) {
+func getProfileInstancesInfo(dbCluster *db.Cluster, projectName string, profileName string) (map[int]db.InstanceArgs, map[string]*api.Project, error) {
 	// Query the db for information about instances associated with the given profile.
 	projectInstNames, err := dbCluster.GetInstancesWithProfile(projectName, profileName)
 	if err != nil {
-		return nil, fmt.Errorf("Failed to query instances with profile %q: %w", profileName, err)
+		return nil, nil, fmt.Errorf("Failed to query instances with profile %q: %w", profileName, err)
 	}
 
 	var instances map[int]db.InstanceArgs
+	projects := make(map[string]*api.Project)
+
 	err = dbCluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var dbInstances []cluster.Instance
 
 		for instProject, instNames := range projectInstNames {
+			// Load project if not already loaded.
+			_, found := projects[instProject]
+			if !found {
+				dbProject, err := cluster.GetProject(context.Background(), tx.Tx(), instProject)
+				if err != nil {
+					return err
+				}
+
+				projects[instProject], err = dbProject.ToAPI(ctx, tx.Tx())
+				if err != nil {
+					return err
+				}
+			}
+
 			for _, instName := range instNames {
 				dbInst, err := cluster.GetInstance(ctx, tx.Tx(), instProject, instName)
 				if err != nil {
@@ -270,8 +278,8 @@ func getProfileInstancesInfo(dbCluster *db.Cluster, projectName string, profileN
 		return nil
 	})
 	if err != nil {
-		return nil, fmt.Errorf("Failed to fetch instances: %w", err)
+		return nil, nil, fmt.Errorf("Failed to fetch instances: %w", err)
 	}
 
-	return instances, nil
+	return instances, projects, nil
 }
