@@ -80,7 +80,7 @@ func AllowInstanceCreation(tx *db.ClusterTx, projectName string, req api.Instanc
 
 	err = checkRestrictionsAndAggregateLimits(tx, info)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed checking if instance creation allowed: %w", err)
 	}
 
 	return nil
@@ -213,15 +213,11 @@ func checkRestrictionsOnVolatileConfig(project api.Project, instanceType instanc
 
 		currentValue, ok := currentConfig[key]
 		if !ok {
-			return fmt.Errorf(
-				"Setting %q on %s %q in project %q is forbidden",
-				key, instanceType, instanceName, project.Name)
+			return fmt.Errorf("Setting %q on %s %q in project %q is forbidden", key, instanceType, instanceName, project.Name)
 		}
 
 		if currentValue != value {
-			return fmt.Errorf(
-				"Changing %q on %s %q in project %q is forbidden",
-				key, instanceType, instanceName, project.Name)
+			return fmt.Errorf("Changing %q on %s %q in project %q is forbidden", key, instanceType, instanceName, project.Name)
 		}
 	}
 
@@ -253,7 +249,7 @@ func AllowVolumeCreation(tx *db.ClusterTx, projectName string, req api.StorageVo
 
 	err = checkRestrictionsAndAggregateLimits(tx, info)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed checking if volume creation allowed: %w", err)
 	}
 
 	return nil
@@ -394,7 +390,7 @@ func checkAggregateLimits(info *projectInfo, aggregateKeys []string) error {
 
 	totals, err := getTotalsAcrossProjectEntities(info, aggregateKeys, false)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed getting usage of project entities: %w", err)
 	}
 
 	for _, key := range aggregateKeys {
@@ -405,9 +401,7 @@ func checkAggregateLimits(info *projectInfo, aggregateKeys []string) error {
 		}
 
 		if totals[key] > max {
-			return fmt.Errorf(
-				"Reached maximum aggregate value %s for %q in project %s",
-				info.Project.Config[key], key, info.Project.Name)
+			return fmt.Errorf("Reached maximum aggregate value %q for %q in project %q", info.Project.Config[key], key, info.Project.Name)
 		}
 	}
 	return nil
@@ -445,15 +439,17 @@ func checkRestrictions(project api.Project, instances []api.Instance, profiles [
 	allowVMLowLevel := false
 	var allowedIDMapHostUIDs, allowedIDMapHostGIDs []idmap.IdmapEntry
 
-	for key, defaultValue := range allRestrictions {
-		// Check if this particular restriction is defined explicitly
-		// in the project config. If not, use the default value.
-		restrictionValue, ok := project.Config[key]
+	for i := range allRestrictions {
+		// Check if this particular restriction is defined explicitly in the project config.
+		// If not, use the default value. Assign to local var so it doesn't change to the default value of
+		// another restriction by time check functions run.
+		restrictionKey := i
+		restrictionValue, ok := project.Config[restrictionKey]
 		if !ok {
-			restrictionValue = defaultValue
+			restrictionValue = allRestrictions[restrictionKey]
 		}
 
-		switch key {
+		switch restrictionKey {
 		case "restricted.containers.interception":
 			for _, key := range allowableIntercept {
 				containerConfigChecks[key] = func(instanceValue string) error {
@@ -576,6 +572,7 @@ func checkRestrictions(project api.Project, instances []api.Instance, profiles [
 
 		case "restricted.devices.nic":
 			devicesChecks["nic"] = func(device map[string]string) error {
+				// Check if the NICs are allowed at all.
 				switch restrictionValue {
 				case "block":
 					return fmt.Errorf("Network devices are forbidden")
@@ -584,6 +581,19 @@ func checkRestrictions(project api.Project, instances []api.Instance, profiles [
 						return fmt.Errorf("Only managed network devices are allowed")
 					}
 				}
+
+				// Check if the NIC's parent/network setting is allowed based on the
+				// restricted.devices.nic and restricted.networks.access settings.
+				if device["network"] != "" {
+					if !NetworkAllowed(project.Config, device["network"], true) {
+						return fmt.Errorf("Network not allowed in project")
+					}
+				} else if device["parent"] != "" {
+					if !NetworkAllowed(project.Config, device["parent"], false) {
+						return fmt.Errorf("Network not allowed in project")
+					}
+				}
+
 				return nil
 			}
 
@@ -796,6 +806,7 @@ var allRestrictions = map[string]string{
 	"restricted.devices.disk.paths":        "",
 	"restricted.idmap.uid":                 "",
 	"restricted.idmap.gid":                 "",
+	"restricted.networks.access":           "",
 	"restricted.snapshots":                 "block",
 }
 
@@ -882,7 +893,7 @@ func AllowInstanceUpdate(tx *db.ClusterTx, projectName, instanceName string, req
 
 	err = checkRestrictionsAndAggregateLimits(tx, info)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed checking if instance update allowed: %w", err)
 	}
 
 	return nil
@@ -916,7 +927,7 @@ func AllowVolumeUpdate(tx *db.ClusterTx, projectName, volumeName string, req api
 
 	err = checkRestrictionsAndAggregateLimits(tx, info)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed checking if volume update allowed: %w", err)
 	}
 
 	return nil
@@ -946,7 +957,7 @@ func AllowProfileUpdate(tx *db.ClusterTx, projectName, profileName string, req a
 
 	err = checkRestrictionsAndAggregateLimits(tx, info)
 	if err != nil {
-		return err
+		return fmt.Errorf("Failed checking if profile update allowed: %w", err)
 	}
 
 	return nil
@@ -1043,7 +1054,7 @@ func validateTotalInstanceCountLimit(instances []api.Instance, value, project st
 	count := len(instances)
 
 	if limit < count {
-		return fmt.Errorf("'limits.instances' is too low: there currently are %d total instances in project %s", count, project)
+		return fmt.Errorf(`"limits.instances" is too low: there currently are %d total instances in project %q`, count, project)
 	}
 
 	return nil
@@ -1075,9 +1086,7 @@ func validateInstanceCountLimit(instances []api.Instance, key, value, project st
 	}
 
 	if limit < count {
-		return fmt.Errorf(
-			"'%s' is too low: there currently are %d instances of type %s in project %s",
-			key, count, instanceType, project)
+		return fmt.Errorf(`%q is too low: there currently are %d instances of type %s in project %q`, key, count, instanceType, project)
 	}
 
 	return nil
@@ -1098,13 +1107,13 @@ func validateAggregateLimit(totals map[string]int64, key, value string) error {
 	parser := aggregateLimitConfigValueParsers[key]
 	limit, err := parser(value)
 	if err != nil {
-		return fmt.Errorf("Invalid value '%s' for limit %s: %w", value, key, err)
+		return fmt.Errorf("Invalid value %q for limit %q: %w", value, key, err)
 	}
 
 	total := totals[key]
 	if limit < total {
 		printer := aggregateLimitConfigValuePrinters[key]
-		return fmt.Errorf("'%s' is too low: current total is %s", key, printer(total))
+		return fmt.Errorf("%q is too low: current total is %q", key, printer(total))
 	}
 
 	return nil
@@ -1198,7 +1207,7 @@ func fetchProject(tx *db.ClusterTx, projectName string, skipIfNoLimits bool) (*p
 		instances = append(instances, *apiInstance)
 	}
 
-	volumes, err := tx.GetCustomVolumesInProject(projectName)
+	volumes, err := tx.GetCustomVolumesInProject(ctx, projectName)
 	if err != nil {
 		return nil, fmt.Errorf("Fetch project custom volumes from database: %w", err)
 	}
@@ -1255,16 +1264,12 @@ func getTotalsAcrossProjectEntities(info *projectInfo, keys []string, skipUnset
 						continue
 					}
 
-					return nil, fmt.Errorf(
-						"Custom volume %s in project %s has no 'size' config set",
-						volume.Name, info.Project.Name)
+					return nil, fmt.Errorf(`Custom volume %q in project %q has no "size" config set`, volume.Name, info.Project.Name)
 				}
 
 				limit, err := units.ParseByteSizeString(value)
 				if err != nil {
-					return nil, fmt.Errorf(
-						"Parse 'size' for custom volume %s in project %s: %w",
-						volume.Name, info.Project.Name, err)
+					return nil, fmt.Errorf(`Parse "size" for custom volume %q in project %q: %w`, volume.Name, info.Project.Name, err)
 				}
 
 				totals[key] += limit
@@ -1344,7 +1349,7 @@ func getInstanceLimits(instance api.Instance, keys []string, skipUnset bool) (ma
 					continue
 				}
 
-				return nil, fmt.Errorf("Instance %q in project %s has no %q config, either directly or via a profile", instance.Name, instance.Project, key)
+				return nil, fmt.Errorf("Instance %q in project %q has no %q config, either directly or via a profile", instance.Name, instance.Project, key)
 			}
 
 			limit, err = parser(value)
@@ -1465,7 +1470,7 @@ func projectHasRestriction(project *api.Project, restrictionKey string, blockVal
 }
 
 // CheckClusterTargetRestriction check if user is allowed to use cluster member targeting.
-func CheckClusterTargetRestriction(tx *db.ClusterTx, r *http.Request, project *api.Project, targetFlag string) error {
+func CheckClusterTargetRestriction(r *http.Request, project *api.Project, targetFlag string) error {
 	// Allow server administrators to move instances around even when restricted (node evacuation, ...)
 	if rbac.UserIsAdmin(r) {
 		return nil
@@ -1493,7 +1498,7 @@ func AllowBackupCreation(tx *db.ClusterTx, projectName string) error {
 	}
 
 	if projectHasRestriction(project, "restricted.backups", "block") {
-		return fmt.Errorf("Project %s doesn't allow for backup creation", projectName)
+		return fmt.Errorf("Project %q doesn't allow for backup creation", projectName)
 	}
 
 	return nil
@@ -1501,14 +1506,9 @@ func AllowBackupCreation(tx *db.ClusterTx, projectName string) error {
 
 // AllowSnapshotCreation returns an error if any project-specific restriction is violated
 // when creating a new snapshot in a project.
-func AllowSnapshotCreation(tx *db.ClusterTx, dbProject *cluster.Project) error {
-	project, err := dbProject.ToAPI(context.Background(), tx.Tx())
-	if err != nil {
-		return err
-	}
-
-	if projectHasRestriction(project, "restricted.snapshots", "block") {
-		return fmt.Errorf("Project %s doesn't allow for snapshot creation", project.Name)
+func AllowSnapshotCreation(p *api.Project) error {
+	if projectHasRestriction(p, "restricted.snapshots", "block") {
+		return fmt.Errorf("Project %q doesn't allow for snapshot creation", p.Name)
 	}
 
 	return nil
