@@ -6,7 +6,6 @@ import (
 	"context"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net"
 	"os"
 	"path"
@@ -26,6 +25,7 @@ import (
 	"github.com/lxc/lxd/lxd/ucred"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/netutils"
@@ -595,7 +595,7 @@ stub_x32_execveat errno 38
 // This is used rather than instance.Instance to avoid import loops.
 type Instance interface {
 	Name() string
-	Project() string
+	Project() api.Project
 	ExpandedConfig() map[string]string
 	IsPrivileged() bool
 	Architecture() int
@@ -610,7 +610,7 @@ var seccompPath = shared.VarPath("security", "seccomp")
 
 // ProfilePath returns the seccomp path for the instance.
 func ProfilePath(c Instance) string {
-	return path.Join(seccompPath, project.Instance(c.Project(), c.Name()))
+	return path.Join(seccompPath, project.Instance(c.Project().Name, c.Name()))
 }
 
 // InstanceNeedsPolicy returns whether the instance needs a policy or not.
@@ -847,7 +847,7 @@ func CreateProfile(s *state.State, c Instance) error {
 		return err
 	}
 
-	return ioutil.WriteFile(ProfilePath(c), []byte(profile), 0600)
+	return os.WriteFile(ProfilePath(c), []byte(profile), 0600)
 }
 
 // DeleteProfile removes a seccomp profile.
@@ -1093,7 +1093,7 @@ func NewSeccompServer(s *state.State, path string, findPID func(pid int32, state
 
 // TaskIDs returns the task IDs for a process.
 func TaskIDs(pid int) (int64, int64, int64, int64, error) {
-	status, err := ioutil.ReadFile(fmt.Sprintf("/proc/%d/status", pid))
+	status, err := os.ReadFile(fmt.Sprintf("/proc/%d/status", pid))
 	if err != nil {
 		return -1, -1, -1, -1, err
 	}
@@ -1176,7 +1176,7 @@ func FindTGID(procFd int) (int, error) {
 	}
 
 	statusFile = os.NewFile(uintptr(fd), "/proc/<pid>/status")
-	status, err := ioutil.ReadAll(statusFile)
+	status, err := io.ReadAll(statusFile)
 	_ = statusFile.Close()
 	if err != nil {
 		return -1, err
@@ -1281,7 +1281,7 @@ func (s *Server) doDeviceSyscall(c Instance, args *MknodArgs, siov *Iovec) int {
 // HandleMknodSyscall handles a mknod syscall.
 func (s *Server) HandleMknodSyscall(c Instance, siov *Iovec) int {
 	ctx := logger.Ctx{"container": c.Name(),
-		"project":               c.Project(),
+		"project":               c.Project().Name,
 		"syscall_number":        siov.req.data.nr,
 		"audit_architecture":    siov.req.data.arch,
 		"seccomp_notify_id":     siov.req.id,
@@ -1332,7 +1332,7 @@ func (s *Server) HandleMknodSyscall(c Instance, siov *Iovec) int {
 // HandleMknodatSyscall handles a mknodat syscall.
 func (s *Server) HandleMknodatSyscall(c Instance, siov *Iovec) int {
 	ctx := logger.Ctx{"container": c.Name(),
-		"project":               c.Project(),
+		"project":               c.Project().Name,
 		"syscall_number":        siov.req.data.nr,
 		"audit_architecture":    siov.req.data.arch,
 		"seccomp_notify_id":     siov.req.id,
@@ -1412,7 +1412,7 @@ type SetxattrArgs struct {
 // HandleSetxattrSyscall handles setxattr syscalls.
 func (s *Server) HandleSetxattrSyscall(c Instance, siov *Iovec) int {
 	ctx := logger.Ctx{"container": c.Name(),
-		"project":               c.Project(),
+		"project":               c.Project().Name,
 		"syscall_number":        siov.req.data.nr,
 		"audit_architecture":    siov.req.data.arch,
 		"seccomp_notify_id":     siov.req.id,
@@ -1564,7 +1564,7 @@ type SchedSetschedulerArgs struct {
 // HandleSchedSetschedulerSyscall handles sched_setscheduler syscalls.
 func (s *Server) HandleSchedSetschedulerSyscall(c Instance, siov *Iovec) int {
 	ctx := logger.Ctx{"container": c.Name(),
-		"project":               c.Project(),
+		"project":               c.Project().Name,
 		"syscall_number":        siov.req.data.nr,
 		"audit_architecture":    siov.req.data.arch,
 		"seccomp_notify_id":     siov.req.id,
@@ -1850,7 +1850,7 @@ func (s *Server) mountHandleHugetlbfsArgs(c Instance, args *MountArgs, nsuid int
 // HandleMountSyscall handles mount syscalls.
 func (s *Server) HandleMountSyscall(c Instance, siov *Iovec) int {
 	ctx := logger.Ctx{"container": c.Name(),
-		"project":               c.Project(),
+		"project":               c.Project().Name,
 		"syscall_number":        siov.req.data.nr,
 		"audit_architecture":    siov.req.data.arch,
 		"seccomp_notify_id":     siov.req.id,
@@ -2056,7 +2056,7 @@ func (s *Server) HandleMountSyscall(c Instance, siov *Iovec) int {
 // HandleBpfSyscall handles mount syscalls.
 func (s *Server) HandleBpfSyscall(c Instance, siov *Iovec) int {
 	ctx := logger.Ctx{"container": c.Name(),
-		"project":               c.Project(),
+		"project":               c.Project().Name,
 		"syscall_number":        siov.req.data.nr,
 		"audit_architecture":    siov.req.data.arch,
 		"seccomp_notify_id":     siov.req.id,
