@@ -1,9 +1,14 @@
+//go:build linux && cgo && !agent
+
 package state
 
 import (
+	"context"
 	"testing"
 
+	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/firewall"
 	"github.com/lxc/lxd/lxd/sys"
 )
 
@@ -23,7 +28,14 @@ func NewTestState(t *testing.T) (*State, func()) {
 		osCleanup()
 	}
 
-	state := NewState(node, cluster, nil, os, nil)
+	state := &State{
+		ShutdownCtx:            context.TODO(),
+		DB:                     &db.DB{Node: node, Cluster: cluster},
+		OS:                     os,
+		Firewall:               firewall.New(),
+		UpdateCertificateCache: func() {},
+		GlobalConfig:           &clusterConfig.Config{},
+	}
 
 	return state, cleanup
 }
