@@ -88,11 +88,13 @@ func (d *lvm) Info() Info {
 		OptimizedImages:   d.usesThinpool(), // Only thinpool pools support optimized images.
 		PreservesInodes:   false,
 		Remote:            d.isRemote(),
-		VolumeTypes:       []VolumeType{VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
+		VolumeTypes:       []VolumeType{VolumeTypeBucket, VolumeTypeCustom, VolumeTypeImage, VolumeTypeContainer, VolumeTypeVM},
 		BlockBacking:      true,
 		RunningCopyFreeze: true,
 		DirectIO:          true,
+		IOUring:           true,
 		MountedRoot:       false,
+		Buckets:           true,
 	}
 }
 
@@ -645,3 +647,22 @@ func (d *lvm) GetResources() (*api.ResourcesStoragePool, error) {
 
 	return &res, nil
 }
+
+// roundVolumeBlockSizeBytes returns size rounded to the nearest multiple of the volume group extent size that is
+// equal to or larger than sizeBytes.
+func (d *lvm) roundVolumeBlockSizeBytes(sizeBytes int64) int64 {
+	// Get the volume group's physical extent size, and use that as minimum size.
+	vgExtentSize, _ := d.volumeGroupExtentSize(d.config["lvm.vg_name"])
+	if sizeBytes < vgExtentSize {
+		sizeBytes = vgExtentSize
+	}
+
+	roundedSizeBytes := int64(sizeBytes/vgExtentSize) * vgExtentSize
+
+	// Ensure the rounded size is at least the size specified in sizeBytes.
+	if roundedSizeBytes < sizeBytes {
+		roundedSizeBytes += vgExtentSize
+	}
+
+	return roundedSizeBytes
+}
