@@ -1,6 +1,7 @@
 package drivers
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"os"
@@ -42,6 +43,9 @@ func (t VolumeType) IsInstance() bool {
 	return false
 }
 
+// VolumeTypeBucket represents a bucket storage volume.
+const VolumeTypeBucket = VolumeType("buckets")
+
 // VolumeTypeImage represents an image storage volume.
 const VolumeTypeImage = VolumeType("images")
 
@@ -69,6 +73,7 @@ type VolumePostHook func(vol Volume) error
 
 // BaseDirectories maps volume types to the expected directories.
 var BaseDirectories = map[VolumeType][]string{
+	VolumeTypeBucket:    {"buckets"},
 	VolumeTypeContainer: {"containers", "containers-snapshots"},
 	VolumeTypeCustom:    {"custom", "custom-snapshots"},
 	VolumeTypeImage:     {"images"},
@@ -162,7 +167,7 @@ func (v Volume) mountLockName() string {
 
 // MountLock attempts to lock the mount lock for the volume and returns the UnlockFunc.
 func (v Volume) MountLock() locking.UnlockFunc {
-	return locking.Lock(v.mountLockName())
+	return locking.Lock(context.TODO(), v.mountLockName())
 }
 
 // MountRefCountIncrement increments the mount ref counter for the volume and returns the new value.
@@ -207,9 +212,9 @@ func (v Volume) EnsureMountPath() error {
 		revert.Add(func() { _ = os.Remove(volPath) })
 	}
 
-	// Set very restrictive mode 0100 for non-custom and non-image volumes.
+	// Set very restrictive mode 0100 for non-custom, non-bucket and non-image volumes.
 	mode := os.FileMode(0711)
-	if v.volType != VolumeTypeCustom && v.volType != VolumeTypeImage {
+	if v.volType != VolumeTypeCustom && v.volType != VolumeTypeImage && v.volType != VolumeTypeBucket {
 		mode = os.FileMode(0100)
 	}
 
@@ -496,6 +501,13 @@ func (v Volume) ConfigSizeFromSource(srcVol Volume) (string, error) {
 			return volSize, err
 		}
 
+		// Round the vol size (for comparison only) because some storage drivers round volumes they create,
+		// and so the published images created from those volumes will also be rounded and will not be
+		// directly usable with the same size setting without also rounding for this check.
+		// Because we are not altering the actual size returned to use for the new volume, this will not
+		// affect storage drivers that do not use rounding.
+		volSizeBytes = v.driver.roundVolumeBlockSizeBytes(volSizeBytes)
+
 		// The volume/pool specified size is smaller than image minimum size. We must not continue as
 		// these specified sizes provide protection against unpacking a massive image and filling the pool.
 		if volSizeBytes < imgSizeBytes {
