@@ -4,7 +4,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
@@ -12,6 +11,7 @@ import (
 
 	"golang.org/x/sys/unix"
 
+	"github.com/lxc/lxd/lxd/apparmor"
 	"github.com/lxc/lxd/lxd/archive"
 	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/db/cluster"
@@ -99,7 +99,7 @@ func VolumeTypeToDBType(volType drivers.VolumeType) (int, error) {
 		return db.StoragePoolVolumeTypeCustom, nil
 	}
 
-	return -1, fmt.Errorf("Invalid storage volume type")
+	return -1, fmt.Errorf("Invalid storage volume type: %q", volType)
 }
 
 // VolumeDBTypeToType converts internal volume type DB code to storage driver volume type.
@@ -115,7 +115,7 @@ func VolumeDBTypeToType(volDBType int) (drivers.VolumeType, error) {
 		return drivers.VolumeTypeCustom, nil
 	}
 
-	return "", fmt.Errorf("Invalid storage volume type")
+	return "", fmt.Errorf("Invalid storage volume DB type: %d", volDBType)
 }
 
 // InstanceTypeToVolumeType converts instance type to storage driver volume type.
@@ -179,7 +179,7 @@ func VolumeContentTypeNameToContentType(contentTypeName string) (int, error) {
 }
 
 // VolumeDBGet loads a volume from the database.
-func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType drivers.VolumeType) (*api.StorageVolume, error) {
+func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType drivers.VolumeType) (*db.StorageVolume, error) {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
 		return nil, fmt.Errorf("Pool is not a lxdBackend")
@@ -190,17 +190,25 @@ func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType dr
 		return nil, err
 	}
 
-	// Get volume config.
-	_, vol, err := p.state.DB.Cluster.GetLocalStoragePoolVolume(projectName, volumeName, volDBType, pool.ID())
-	if err != nil {
-		if response.IsNotFoundError(err) {
-			return vol, fmt.Errorf("Storage volume %q in project %q of type %q does not exist on pool %q: %w", volumeName, projectName, volumeType, pool.Name(), err)
+	// Get the storage volume.
+	var dbVolume *db.StorageVolume
+	err = p.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, volDBType, volumeName, true)
+		if err != nil {
+			if response.IsNotFoundError(err) {
+				return fmt.Errorf("Storage volume %q in project %q of type %q does not exist on pool %q: %w", volumeName, projectName, volumeType, pool.Name(), err)
+			}
+
+			return err
 		}
 
+		return nil
+	})
+	if err != nil {
 		return nil, err
 	}
 
-	return vol, nil
+	return dbVolume, nil
 }
 
 // VolumeDBCreate creates a volume in the database.
@@ -212,6 +220,11 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 		return fmt.Errorf("Pool is not a lxdBackend")
 	}
 
+	// Prevent using this function to create storage volume bucket records.
+	if volumeType == drivers.VolumeTypeBucket {
+		return fmt.Errorf("Cannot store volume using bucket type")
+	}
+
 	// If the volumeType represents an instance type then check that the volumeConfig doesn't contain any of
 	// the instance disk effective override fields (which should not be stored in the database).
 	if volumeType.IsInstance() {
@@ -322,7 +335,7 @@ func poolAndVolumeCommonRules(vol *drivers.Volume) map[string]func(string) error
 		"size": validate.Optional(validate.IsSize),
 		"snapshots.expiry": func(value string) error {
 			// Validate expression
-			_, err := shared.GetSnapshotExpiry(time.Time{}, value)
+			_, err := shared.GetExpiry(time.Time{}, value)
 			return err
 		},
 		"snapshots.schedule": validate.Optional(validate.IsCron([]string{"@hourly", "@daily", "@midnight", "@weekly", "@monthly", "@annually", "@yearly"})),
@@ -456,9 +469,10 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 		// Use prlimit because qemu-img can consume considerable RAM & CPU time if fed a maliciously
 		// crafted disk image. Since cloud tenants are not to be trusted, ensure QEMU is limits to 1 GB
 		// address space and 2 seconds CPU time, which ought to be more than enough for real world images.
-		imgJSON, err := shared.RunCommand("prlimit", "--cpu=2", "--as=1000000000", "qemu-img", "info", "-f", "qcow2", "--output=json", imgPath)
+		cmd := []string{"prlimit", "--cpu=2", "--as=1000000000", "qemu-img", "info", "-f", "qcow2", "--output=json", imgPath}
+		imgJSON, err := apparmor.QemuImg(sysOS, cmd, imgPath, "")
 		if err != nil {
-			return -1, fmt.Errorf("Failed reading image info %q: %w", dstPath, err)
+			return -1, fmt.Errorf("Failed reading image info %q: %w", imgPath, err)
 		}
 
 		imgInfo := struct {
@@ -510,7 +524,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 		// Convert the qcow2 format to a raw block device.
 		l.Debug("Converting qcow2 image to raw disk", logger.Ctx{"imgPath": imgPath, "dstPath": dstPath})
 
-		cmd := []string{
+		cmd = []string{
 			"nice", "-n19", // Run with low priority to reduce CPU impact on other processes.
 			"qemu-img", "convert", "-f", "qcow2", "-O", "raw",
 		}
@@ -535,7 +549,8 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 
 		cmd = append(cmd, imgPath, dstPath)
 
-		_, err = shared.RunCommand(cmd[0], cmd[1:]...)
+		_, err = apparmor.QemuImg(sysOS, cmd, imgPath, dstPath)
+
 		if err != nil {
 			return -1, fmt.Errorf("Failed converting image to raw at %q: %w", dstPath, err)
 		}
@@ -559,7 +574,7 @@ func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, blo
 		}
 	} else {
 		// Dealing with unified tarballs require an initial unpack to a temporary directory.
-		tempDir, err := ioutil.TempDir(shared.VarPath("images"), "lxd_image_unpack_")
+		tempDir, err := os.MkdirTemp(shared.VarPath("images"), "lxd_image_unpack_")
 		if err != nil {
 			return -1, err
 		}
@@ -610,30 +625,33 @@ func InstanceContentType(inst instance.Instance) drivers.ContentType {
 // VolumeUsedByProfileDevices finds profiles using a volume and passes them to profileFunc for evaluation.
 // The profileFunc is provided with a profile config, project config and a list of device names that are using
 // the volume.
-func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName string, vol *api.StorageVolume, profileFunc func(profileID int64, profile api.Profile, project cluster.Project, usedByDevices []string) error) error {
+func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName string, vol *api.StorageVolume, profileFunc func(profileID int64, profile api.Profile, project api.Project, usedByDevices []string) error) error {
 	// Convert the volume type name to our internal integer representation.
 	volumeType, err := VolumeTypeNameToDBType(vol.Type)
 	if err != nil {
 		return err
 	}
 
-	projectMap := map[string]cluster.Project{}
 	var profiles []api.Profile
 	var profileIDs []int64
 	var profileProjects []*api.Project
 	// Retrieve required info from the database in single transaction for performance.
 	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		projects, err := cluster.GetProjects(ctx, tx.Tx(), cluster.ProjectFilter{})
+		projects, err := cluster.GetProjects(ctx, tx.Tx())
 		if err != nil {
 			return fmt.Errorf("Failed loading projects: %w", err)
 		}
 
 		// Index of all projects by name.
-		for i, project := range projects {
-			projectMap[project.Name] = projects[i]
+		projectMap := make(map[string]*api.Project, len(projects))
+		for _, project := range projects {
+			projectMap[project.Name], err = project.ToAPI(ctx, tx.Tx())
+			if err != nil {
+				return fmt.Errorf("Failed loading config for projec %q: %w", project.Name, err)
+			}
 		}
 
-		dbProfiles, err := cluster.GetProfiles(ctx, tx.Tx(), cluster.ProfileFilter{})
+		dbProfiles, err := cluster.GetProfiles(ctx, tx.Tx())
 		if err != nil {
 			return fmt.Errorf("Failed loading profiles: %w", err)
 		}
@@ -650,11 +668,7 @@ func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName str
 
 		profileProjects = make([]*api.Project, len(dbProfiles))
 		for i, p := range dbProfiles {
-			project := projectMap[p.Project]
-			profileProjects[i], err = project.ToAPI(ctx, tx.Tx())
-			if err != nil {
-				return err
-			}
+			profileProjects[i] = projectMap[p.Project]
 		}
 
 		return nil
@@ -697,7 +711,7 @@ func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName str
 		}
 
 		if len(usedByDevices) > 0 {
-			err = profileFunc(profileIDs[i], profile, projectMap[profileProjects[i].Name], usedByDevices)
+			err = profileFunc(profileIDs[i], profile, *profileProjects[i], usedByDevices)
 			if err != nil {
 				return err
 			}
@@ -719,7 +733,7 @@ func VolumeUsedByInstanceDevices(s *state.State, poolName string, projectName st
 		return err
 	}
 
-	return s.DB.Cluster.InstanceList(nil, func(inst db.InstanceArgs, p api.Project) error {
+	return s.DB.Cluster.InstanceList(context.TODO(), func(inst db.InstanceArgs, p api.Project) error {
 		// If the volume has a specific cluster member which is different than the instance then skip as
 		// instance cannot be using this volume.
 		if vol.Location != "" && inst.Node != vol.Location {
@@ -811,8 +825,8 @@ func VolumeUsedByExclusiveRemoteInstancesWithProfiles(s *state.State, poolName s
 func VolumeUsedByDaemon(s *state.State, poolName string, volumeName string) (bool, error) {
 	var storageBackups string
 	var storageImages string
-	err := s.DB.Node.Transaction(func(tx *db.NodeTx) error {
-		nodeConfig, err := node.ConfigLoad(tx)
+	err := s.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+		nodeConfig, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
 		}
@@ -919,3 +933,57 @@ func InstanceDiskBlockSize(pool Pool, inst instance.Instance, op *operations.Ope
 
 	return blockDiskSize, nil
 }
+
+// ComparableSnapshot is used when comparing snapshots on different pools to see whether they differ.
+type ComparableSnapshot struct {
+	// Name of the snapshot (without the parent name).
+	Name string
+
+	// Identifier of the snapshot (that remains the same when copied between pools).
+	ID string
+
+	// Creation date time of the snapshot.
+	CreationDate time.Time
+}
+
+// CompareSnapshots returns a list of snapshot indexes (from the associated input slices) to sync from the source
+// and to delete from the target respectively.
+// A snapshot will be added to "to sync from source" slice if it either doesn't exist in the target or its ID or
+// creation date is different to the source.
+// A snapshot will be added to the "to delete from target" slice if it doesn't exist in the source or its ID or
+// creation date is different to the source.
+func CompareSnapshots(sourceSnapshots []ComparableSnapshot, targetSnapshots []ComparableSnapshot) ([]int, []int) {
+	// Compare source and target.
+	sourceSnapshotsByName := make(map[string]*ComparableSnapshot, len(sourceSnapshots))
+	targetSnapshotsByName := make(map[string]*ComparableSnapshot, len(targetSnapshots))
+
+	var syncFromSource, deleteFromTarget []int
+
+	// Generate a list of source snapshots by name.
+	for sourceSnapIndex := range sourceSnapshots {
+		sourceSnapshotsByName[sourceSnapshots[sourceSnapIndex].Name] = &sourceSnapshots[sourceSnapIndex]
+	}
+
+	// If target snapshot doesn't exist in source, or its creation date or ID differ,
+	// then mark it for deletion on target.
+	for targetSnapIndex := range targetSnapshots {
+		// Generate a list of target snapshots by name for later comparison.
+		targetSnapshotsByName[targetSnapshots[targetSnapIndex].Name] = &targetSnapshots[targetSnapIndex]
+
+		sourceSnap, sourceSnapExists := sourceSnapshotsByName[targetSnapshots[targetSnapIndex].Name]
+		if !sourceSnapExists || !sourceSnap.CreationDate.Equal(targetSnapshots[targetSnapIndex].CreationDate) || sourceSnap.ID != targetSnapshots[targetSnapIndex].ID {
+			deleteFromTarget = append(deleteFromTarget, targetSnapIndex)
+		}
+	}
+
+	// If source snapshot doesn't exist in target, or its creation date or ID differ,
+	// then mark it for syncing to target.
+	for sourceSnapIndex := range sourceSnapshots {
+		targetSnap, targetSnapExists := targetSnapshotsByName[sourceSnapshots[sourceSnapIndex].Name]
+		if !targetSnapExists || !targetSnap.CreationDate.Equal(sourceSnapshots[sourceSnapIndex].CreationDate) || targetSnap.ID != sourceSnapshots[sourceSnapIndex].ID {
+			syncFromSource = append(syncFromSource, sourceSnapIndex)
+		}
+	}
+
+	return syncFromSource, deleteFromTarget
+}
