@@ -261,6 +261,10 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("No driver provided"))
 	}
 
+	if req.Config == nil {
+		req.Config = map[string]string{}
+	}
+
 	ctx := logger.Ctx{}
 
 	targetNode := queryParam(r, "target")
@@ -310,7 +314,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			return tx.CreatePendingStoragePool(targetNode, req.Name, req.Driver, req.Config)
+			return tx.CreatePendingStoragePool(ctx, targetNode, req.Name, req.Driver, req.Config)
 		})
 		if err != nil {
 			if err == db.ErrAlreadyDefined {
@@ -405,7 +409,7 @@ func storagePoolsPostCluster(d *Daemon, pool *api.StoragePool, req api.StoragePo
 		var err error
 
 		// Check that the pool was defined at all. Must come before partially created checks.
-		poolID, err = tx.GetStoragePoolID(req.Name)
+		poolID, err = tx.GetStoragePoolID(ctx, req.Name)
 		if err != nil {
 			return err
 		}
@@ -421,7 +425,7 @@ func storagePoolsPostCluster(d *Daemon, pool *api.StoragePool, req api.StoragePo
 		}
 
 		// Fetch the node-specific configs and check the pool is defined for all nodes.
-		configs, err = tx.GetStoragePoolNodeConfigs(poolID)
+		configs, err = tx.GetStoragePoolNodeConfigs(ctx, poolID)
 		if err != nil {
 			return err
 		}
@@ -929,7 +933,7 @@ func storagePoolDelete(d *Daemon, r *http.Request) response.Response {
 		err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 			// Get all the volumes using the storage pool on this server.
 			// Only image volumes should remain now.
-			volumes, err := tx.GetStoragePoolVolumes(pool.ID(), true)
+			volumes, err := tx.GetStoragePoolVolumes(ctx, pool.ID(), true)
 			if err != nil {
 				return fmt.Errorf("Failed loading storage volumes: %w", err)
 			}
