@@ -8,7 +8,6 @@ import (
 	"encoding/pem"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"net/url"
 	"os"
@@ -338,6 +337,7 @@ func storagePoolVolumesGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	var dbVolumes []*db.StorageVolume
+	var projectImages []string
 
 	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var customVolProjectName string
@@ -356,6 +356,11 @@ func storagePoolVolumesGet(d *Daemon, r *http.Request) response.Response {
 			// The project name used for custom volumes varies based on whether the
 			// project has the featues.storage.volumes feature enabled.
 			customVolProjectName = project.StorageVolumeProjectFromRecord(p, db.StoragePoolVolumeTypeCustom)
+
+			projectImages, err = tx.GetImagesFingerprints(ctx, requestProjectName, false)
+			if err != nil {
+				return err
+			}
 		}
 
 		filters := make([]db.StorageVolumeFilter, 0)
@@ -400,7 +405,7 @@ func storagePoolVolumesGet(d *Daemon, r *http.Request) response.Response {
 			}
 		}
 
-		dbVolumes, err = tx.GetStoragePoolVolumes(poolID, memberSpecific, filters...)
+		dbVolumes, err = tx.GetStoragePoolVolumes(ctx, poolID, memberSpecific, filters...)
 		if err != nil {
 			return fmt.Errorf("Failed loading storage volumes: %w", err)
 		}
@@ -411,14 +416,6 @@ func storagePoolVolumesGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	var projectImages []string
-	if !allProjects {
-		projectImages, err = d.db.Cluster.GetImagesFingerprints(requestProjectName, false)
-		if err != nil {
-			return response.SmartError(err)
-		}
-	}
-
 	dbVolumes = filterVolumes(dbVolumes, clauses, allProjects, projectImages)
 
 	// Sort by type then volume name.
@@ -494,6 +491,7 @@ func filterVolumes(volumes []*db.StorageVolume, clauses []filter.Clause, allProj
 // Add a storage volume
 //
 // Creates a new storage volume (type specific endpoint).
+// Will return an empty sync response on simple volume creation but an operation on copy or migration.
 //
 // ---
 // consumes:
@@ -518,6 +516,8 @@ func filterVolumes(volumes []*db.StorageVolume, clauses []filter.Clause, allProj
 //     schema:
 //       $ref: "#/definitions/StorageVolumesPost"
 // responses:
+//   "200":
+//     $ref: "#/responses/EmptySyncResponse"
 //   "202":
 //     $ref: "#/responses/Operation"
 //   "400":
@@ -591,29 +591,31 @@ func storagePoolVolumesTypePost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Check if destination volume exists.
-	_, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, req.Name, db.StoragePoolVolumeTypeCustom, poolID)
-	if !response.IsNotFoundError(err) {
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, poolID, projectName, db.StoragePoolVolumeTypeCustom, req.Name, true)
+		if err != nil && !response.IsNotFoundError(err) {
+			return err
+		}
+
+		err = project.AllowVolumeCreation(tx, projectName, req)
 		if err != nil {
-			return response.SmartError(err)
+			return err
 		}
 
-		if !req.Source.Refresh {
-			return response.Conflict(fmt.Errorf("Volume name %q already exists", req.Name))
-		}
-	}
-
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		return project.AllowVolumeCreation(tx, projectName, req)
+		return nil
 	})
 	if err != nil {
 		return response.SmartError(err)
+	} else if dbVolume != nil && !req.Source.Refresh {
+		return response.Conflict(fmt.Errorf("Volume by that name already exists"))
 	}
 
 	switch req.Source.Type {
 	case "":
 		return doVolumeCreateOrCopy(d, r, projectParam(r), projectName, poolName, &req)
 	case "copy":
-		if vol != nil {
+		if dbVolume != nil {
 			return doCustomVolumeRefresh(d, r, projectParam(r), projectName, poolName, &req)
 		}
 
@@ -711,6 +713,7 @@ func doVolumeCreateOrCopy(d *Daemon, r *http.Request, requestProjectName string,
 // Add a storage volume
 //
 // Creates a new storage volume.
+// Will return an empty sync response on simple volume creation but an operation on copy or migration.
 //
 // ---
 // consumes:
@@ -735,6 +738,8 @@ func doVolumeCreateOrCopy(d *Daemon, r *http.Request, requestProjectName string,
 //     schema:
 //       $ref: "#/definitions/StorageVolumesPost"
 // responses:
+//   "200":
+//     $ref: "#/responses/EmptySyncResponse"
 //   "202":
 //     $ref: "#/responses/Operation"
 //   "400":
@@ -799,13 +804,19 @@ func storagePoolVolumesPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Check if destination volume exists.
-	_, _, err = d.db.Cluster.GetLocalStoragePoolVolume(projectName, req.Name, db.StoragePoolVolumeTypeCustom, poolID)
-	if !response.IsNotFoundError(err) {
-		if err != nil {
-			return response.SmartError(err)
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, poolID, projectName, db.StoragePoolVolumeTypeCustom, req.Name, true)
+		if err != nil && !response.IsNotFoundError(err) {
+			return err
+		} else if dbVolume != nil {
+			return api.StatusErrorf(http.StatusConflict, "Volume by that name already exists")
 		}
 
-		return response.Conflict(fmt.Errorf("Volume by that name already exists"))
+		return nil
+	})
+	if err != nil {
+		return response.SmartError(err)
 	}
 
 	switch req.Source.Type {
@@ -1073,14 +1084,18 @@ func storagePoolVolumePost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	_, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, volumeName, volumeType, srcPoolID)
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, srcPoolID, projectName, volumeType, volumeName, true)
+		return err
+	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Check if a running instance is using it.
-	err = storagePools.VolumeUsedByInstanceDevices(d.State(), srcPoolName, projectName, vol, true, func(dbInst db.InstanceArgs, project api.Project, usedByDevices []string) error {
-		inst, err := instance.Load(d.State(), dbInst, nil)
+	err = storagePools.VolumeUsedByInstanceDevices(d.State(), srcPoolName, projectName, &dbVolume.StorageVolume, true, func(dbInst db.InstanceArgs, project api.Project, usedByDevices []string) error {
+		inst, err := instance.Load(d.State(), dbInst, project)
 		if err != nil {
 			return err
 		}
@@ -1097,11 +1112,11 @@ func storagePoolVolumePost(d *Daemon, r *http.Request) response.Response {
 
 	// Detect a rename request.
 	if (req.Pool == "" || req.Pool == srcPoolName) && (projectName == targetProjectName) {
-		return storagePoolVolumeTypePostRename(d, r, srcPoolName, projectName, vol, req)
+		return storagePoolVolumeTypePostRename(d, r, srcPoolName, projectName, &dbVolume.StorageVolume, req)
 	}
 
 	// Otherwise this is a move request.
-	return storagePoolVolumeTypePostMove(d, r, srcPoolName, projectName, targetProjectName, vol, req)
+	return storagePoolVolumeTypePostMove(d, r, srcPoolName, projectName, targetProjectName, &dbVolume.StorageVolume, req)
 }
 
 // storagePoolVolumeTypePostMigration handles volume migration type POST requests.
@@ -1349,7 +1364,7 @@ func storagePoolVolumeGet(d *Daemon, r *http.Request) response.Response {
 	// Get the storage volume.
 	var dbVolume *db.StorageVolume
 	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		dbVolume, err = tx.GetStoragePoolVolume(poolID, volumeProjectName, volumeType, volumeName, true)
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, poolID, volumeProjectName, volumeType, volumeName, true)
 		return err
 	})
 	if err != nil {
@@ -1458,13 +1473,17 @@ func storagePoolVolumePut(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Get the existing storage volume.
-	_, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, volumeName, volumeType, pool.ID())
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, volumeType, volumeName, true)
+		return err
+	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Validate the ETag
-	etag := []any{volumeName, vol.Type, vol.Config}
+	etag := []any{volumeName, dbVolume.Type, dbVolume.Config}
 
 	err = util.EtagCheck(r, etag)
 	if err != nil {
@@ -1482,19 +1501,11 @@ func storagePoolVolumePut(d *Daemon, r *http.Request) response.Response {
 	op.SetRequestor(r)
 
 	if volumeType == db.StoragePoolVolumeTypeCustom {
-		// Possibly check if project limits are honored.
-		err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			return project.AllowVolumeUpdate(tx, projectName, volumeName, req, vol.Config)
-		})
-		if err != nil {
-			return response.SmartError(err)
-		}
-
 		// Restore custom volume from snapshot if requested. This should occur first
 		// before applying config changes so that changes are applied to the
 		// restored volume.
 		if req.Restore != "" {
-			err = pool.RestoreCustomVolume(projectName, vol.Name, req.Restore, op)
+			err = pool.RestoreCustomVolume(projectName, dbVolume.Name, req.Restore, op)
 			if err != nil {
 				return response.SmartError(err)
 			}
@@ -1504,13 +1515,21 @@ func storagePoolVolumePut(d *Daemon, r *http.Request) response.Response {
 		// Only apply changes during a snapshot restore if a non-nil config is supplied to avoid clearing
 		// the volume's config if only restoring snapshot.
 		if req.Config != nil || req.Restore == "" {
-			err = pool.UpdateCustomVolume(projectName, vol.Name, req.Description, req.Config, op)
+			// Possibly check if project limits are honored.
+			err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+				return project.AllowVolumeUpdate(tx, projectName, volumeName, req, dbVolume.Config)
+			})
+			if err != nil {
+				return response.SmartError(err)
+			}
+
+			err = pool.UpdateCustomVolume(projectName, dbVolume.Name, req.Description, req.Config, op)
 			if err != nil {
 				return response.SmartError(err)
 			}
 		}
 	} else if volumeType == db.StoragePoolVolumeTypeContainer || volumeType == db.StoragePoolVolumeTypeVM {
-		inst, err := instance.LoadByProjectAndName(d.State(), projectName, vol.Name)
+		inst, err := instance.LoadByProjectAndName(d.State(), projectName, dbVolume.Name)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -1522,7 +1541,7 @@ func storagePoolVolumePut(d *Daemon, r *http.Request) response.Response {
 		}
 	} else if volumeType == db.StoragePoolVolumeTypeImage {
 		// Handle image update requests.
-		err = pool.UpdateImage(vol.Name, req.Description, req.Config, op)
+		err = pool.UpdateImage(dbVolume.Name, req.Description, req.Config, op)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -1626,13 +1645,17 @@ func storagePoolVolumePatch(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Get the existing storage volume.
-	_, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, volumeName, volumeType, pool.ID())
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, volumeType, volumeName, true)
+		return err
+	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Validate the ETag.
-	etag := []any{volumeName, vol.Type, vol.Config}
+	etag := []any{volumeName, dbVolume.Type, dbVolume.Config}
 
 	err = util.EtagCheck(r, etag)
 	if err != nil {
@@ -1650,7 +1673,7 @@ func storagePoolVolumePatch(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Merge current config with requested changes.
-	for k, v := range vol.Config {
+	for k, v := range dbVolume.Config {
 		_, ok := req.Config[k]
 		if !ok {
 			req.Config[k] = v
@@ -1661,7 +1684,7 @@ func storagePoolVolumePatch(d *Daemon, r *http.Request) response.Response {
 	op := &operations.Operation{}
 	op.SetRequestor(r)
 
-	err = pool.UpdateCustomVolume(projectName, vol.Name, req.Description, req.Config, op)
+	err = pool.UpdateCustomVolume(projectName, dbVolume.Name, req.Description, req.Config, op)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1760,7 +1783,7 @@ func storagePoolVolumeDelete(d *Daemon, r *http.Request) response.Response {
 	// Get the storage volume.
 	var dbVolume *db.StorageVolume
 	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		dbVolume, err = tx.GetStoragePoolVolume(pool.ID(), volumeProjectName, volumeType, volumeName, true)
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), volumeProjectName, volumeType, volumeName, true)
 		return err
 	})
 	if err != nil {
@@ -1814,7 +1837,7 @@ func createStoragePoolVolumeFromBackup(d *Daemon, r *http.Request, requestProjec
 	defer revert.Fail()
 
 	// Create temporary file to store uploaded backup data.
-	backupFile, err := ioutil.TempFile(shared.VarPath("backups"), fmt.Sprintf("%s_", backup.WorkingDirPrefix))
+	backupFile, err := os.CreateTemp(shared.VarPath("backups"), fmt.Sprintf("%s_", backup.WorkingDirPrefix))
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -1844,7 +1867,7 @@ func createStoragePoolVolumeFromBackup(d *Daemon, r *http.Request, requestProjec
 		decomArgs := append(decomArgs, backupFile.Name())
 
 		// Create temporary file to store the decompressed tarball in.
-		tarFile, err := ioutil.TempFile(shared.VarPath("backups"), fmt.Sprintf("%s_decompress_", backup.WorkingDirPrefix))
+		tarFile, err := os.CreateTemp(shared.VarPath("backups"), fmt.Sprintf("%s_decompress_", backup.WorkingDirPrefix))
 		if err != nil {
 			return response.InternalError(err)
 		}
