@@ -120,16 +120,23 @@ func storagePoolVolumeSnapshotsTypePost(d *Daemon, r *http.Request) response.Res
 		return response.SmartError(err)
 	}
 
-	var proj *dbCluster.Project
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		proj, err = dbCluster.GetProject(context.Background(), tx.Tx(), projectName)
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbProject, err := dbCluster.GetProject(context.Background(), tx.Tx(), projectName)
 		if err != nil {
 			return err
 		}
 
-		err = project.AllowSnapshotCreation(tx, proj)
+		p, err := dbProject.ToAPI(ctx, tx.Tx())
+		if err != nil {
+			return err
+		}
 
-		return err
+		err = project.AllowSnapshotCreation(p)
+		if err != nil {
+			return err
+		}
+
+		return nil
 	})
 	if err != nil {
 		return response.SmartError(err)
@@ -181,18 +188,24 @@ func storagePoolVolumeSnapshotsTypePost(d *Daemon, r *http.Request) response.Res
 		return response.BadRequest(err)
 	}
 
-	// Ensure that the snapshot doesn't already exist.
-	_, _, err = d.db.Cluster.GetLocalStoragePoolVolume(projectName, fmt.Sprintf("%s/%s", volumeName, req.Name), volumeType, pool.ID())
-	if !response.IsNotFoundError(err) {
-		if err != nil {
-			return response.SmartError(err)
+	var parentDBVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		// Ensure that the snapshot doesn't already exist.
+		snapDBVolume, err := tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, volumeType, fmt.Sprintf("%s/%s", volumeName, req.Name), true)
+		if err != nil && !response.IsNotFoundError(err) {
+			return err
+		} else if snapDBVolume != nil {
+			return api.StatusErrorf(http.StatusConflict, "Snapshot %q already in use", req.Name)
 		}
 
-		return response.Conflict(fmt.Errorf("Snapshot '%s' already in use", req.Name))
-	}
+		// Get the parent volume so we can get the config.
+		parentDBVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, volumeType, volumeName, true)
+		if err != nil {
+			return err
+		}
 
-	// Get the parent volume so we can get the config.
-	_, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, volumeName, volumeType, pool.ID())
+		return nil
+	})
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -202,7 +215,7 @@ func storagePoolVolumeSnapshotsTypePost(d *Daemon, r *http.Request) response.Res
 	if req.ExpiresAt != nil {
 		expiry = *req.ExpiresAt
 	} else {
-		expiry, err = shared.GetSnapshotExpiry(time.Now(), vol.Config["snapshots.expiry"])
+		expiry, err = shared.GetExpiry(time.Now(), parentDBVolume.Config["snapshots.expiry"])
 		if err != nil {
 			return response.BadRequest(err)
 		}
@@ -386,7 +399,7 @@ func storagePoolVolumeSnapshotsTypeGet(d *Daemon, r *http.Request) response.Resp
 		} else {
 			var vol *db.StorageVolume
 			err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-				vol, err = tx.GetStoragePoolVolume(poolID, projectName, volumeType, volume.Name, true)
+				vol, err = tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volume.Name, true)
 				return err
 			})
 			if err != nil {
@@ -650,22 +663,26 @@ func storagePoolVolumeSnapshotTypeGet(d *Daemon, r *http.Request) response.Respo
 		return response.SmartError(err)
 	}
 
-	volID, volume, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, fullSnapshotName, volumeType, poolID)
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, fullSnapshotName, true)
+		return err
+	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	expiry, err := d.db.Cluster.GetStorageVolumeSnapshotExpiry(volID)
+	expiry, err := d.db.Cluster.GetStorageVolumeSnapshotExpiry(dbVolume.ID)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	snapshot := api.StorageVolumeSnapshot{}
-	snapshot.Config = volume.Config
-	snapshot.Description = volume.Description
+	snapshot.Config = dbVolume.Config
+	snapshot.Description = dbVolume.Description
 	snapshot.Name = snapshotName
 	snapshot.ExpiresAt = &expiry
-	snapshot.ContentType = volume.ContentType
+	snapshot.ContentType = dbVolume.ContentType
 
 	etag := []any{snapshot.Description, expiry}
 	return response.SyncResponseETag(true, &snapshot, etag)
@@ -766,18 +783,22 @@ func storagePoolVolumeSnapshotTypePut(d *Daemon, r *http.Request) response.Respo
 		return response.SmartError(err)
 	}
 
-	volID, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, fullSnapshotName, volumeType, poolID)
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, fullSnapshotName, true)
+		return err
+	})
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	expiry, err := d.db.Cluster.GetStorageVolumeSnapshotExpiry(volID)
+	expiry, err := d.db.Cluster.GetStorageVolumeSnapshotExpiry(dbVolume.ID)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Validate the ETag
-	etag := []any{vol.Description, expiry}
+	etag := []any{dbVolume.Description, expiry}
 	err = util.EtagCheck(r, etag)
 	if err != nil {
 		return response.PreconditionFailed(err)
@@ -790,7 +811,7 @@ func storagePoolVolumeSnapshotTypePut(d *Daemon, r *http.Request) response.Respo
 		return response.BadRequest(err)
 	}
 
-	return doStoragePoolVolumeSnapshotUpdate(d, r, poolName, projectName, vol.Name, volumeType, req)
+	return doStoragePoolVolumeSnapshotUpdate(d, r, poolName, projectName, dbVolume.Name, volumeType, req)
 }
 
 // swagger:operation PATCH /1.0/storage-pools/{name}/volumes/{type}/{volume}/snapshots/{snapshot} storage storage_pool_volumes_type_snapshot_patch
@@ -888,25 +909,26 @@ func storagePoolVolumeSnapshotTypePatch(d *Daemon, r *http.Request) response.Res
 		return response.SmartError(err)
 	}
 
-	volID, vol, err := d.db.Cluster.GetLocalStoragePoolVolume(projectName, fullSnapshotName, volumeType, poolID)
-	if err != nil {
-		return response.SmartError(err)
-	}
+	var dbVolume *db.StorageVolume
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, fullSnapshotName, true)
+		return err
+	})
 
-	expiry, err := d.db.Cluster.GetStorageVolumeSnapshotExpiry(volID)
+	expiry, err := d.db.Cluster.GetStorageVolumeSnapshotExpiry(dbVolume.ID)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	// Validate the ETag
-	etag := []any{vol.Description, expiry}
+	etag := []any{dbVolume.Description, expiry}
 	err = util.EtagCheck(r, etag)
 	if err != nil {
 		return response.PreconditionFailed(err)
 	}
 
 	req := api.StorageVolumeSnapshotPut{
-		Description: vol.Description,
+		Description: dbVolume.Description,
 		ExpiresAt:   &expiry,
 	}
 
@@ -915,7 +937,7 @@ func storagePoolVolumeSnapshotTypePatch(d *Daemon, r *http.Request) response.Res
 		return response.BadRequest(err)
 	}
 
-	return doStoragePoolVolumeSnapshotUpdate(d, r, poolName, projectName, vol.Name, volumeType, req)
+	return doStoragePoolVolumeSnapshotUpdate(d, r, poolName, projectName, dbVolume.Name, volumeType, req)
 }
 
 func doStoragePoolVolumeSnapshotUpdate(d *Daemon, r *http.Request, poolName string, projectName string, volName string, volumeType int, req api.StorageVolumeSnapshotPut) response.Response {
@@ -1137,22 +1159,24 @@ func autoCreateCustomVolumeSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 		s := d.State()
 
 		// Get projects.
-		var projects map[string]*dbCluster.Project
 		var volumes, remoteVolumes []db.StorageVolumeArgs
 		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
-			projs, err := dbCluster.GetProjects(ctx, tx.Tx(), dbCluster.ProjectFilter{})
+			projs, err := dbCluster.GetProjects(ctx, tx.Tx())
 			if err != nil {
 				return fmt.Errorf("Failed loading projects: %w", err)
 			}
 
 			// Key by project name for lookup later.
-			projects = make(map[string]*dbCluster.Project, len(projs))
+			projects := make(map[string]*api.Project, len(projs))
 			for _, p := range projs {
-				projects[p.Name] = &p
+				projects[p.Name], err = p.ToAPI(ctx, tx.Tx())
+				if err != nil {
+					return fmt.Errorf("Failed loading config for project %q: %w", p.Name, err)
+				}
 			}
 
-			allVolumes, err := tx.GetStoragePoolVolumesWithType(db.StoragePoolVolumeTypeCustom)
+			allVolumes, err := tx.GetStoragePoolVolumesWithType(ctx, db.StoragePoolVolumeTypeCustom)
 			if err != nil {
 				return fmt.Errorf("Failed getting volumes for auto custom volume snapshot task: %w", err)
 			}
@@ -1169,7 +1193,7 @@ func autoCreateCustomVolumeSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 					continue
 				}
 
-				err = project.AllowSnapshotCreation(tx, projects[v.ProjectName])
+				err = project.AllowSnapshotCreation(projects[v.ProjectName])
 				if err != nil {
 					continue
 				}
@@ -1193,24 +1217,24 @@ func autoCreateCustomVolumeSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 
 		if len(remoteVolumes) > 0 {
 			// Get list of cluster members.
-			var nodeCount int
+			var memberCount int
 			var onlineNodeIDs []int64
 			err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 				// Get all the members.
-				nodes, err := tx.GetNodes()
+				members, err := tx.GetNodes(ctx)
 				if err != nil {
-					return err
+					return fmt.Errorf("Failed getting cluster members: %w", err)
 				}
 
-				nodeCount = len(nodes)
+				memberCount = len(members)
 
 				// Filter to online members.
-				for _, node := range nodes {
-					if node.IsOffline(s.GlobalConfig.OfflineThreshold()) {
+				for _, member := range members {
+					if member.IsOffline(s.GlobalConfig.OfflineThreshold()) {
 						continue
 					}
 
-					onlineNodeIDs = append(onlineNodeIDs, node.ID)
+					onlineNodeIDs = append(onlineNodeIDs, member.ID)
 				}
 
 				return nil
@@ -1223,7 +1247,7 @@ func autoCreateCustomVolumeSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 			// Skip snapshotting remote custom volumes if there are no online members, as we can't be
 			// sure that the cluster isn't partitioned and we may end up attempting the snapshot on
 			// multiple members.
-			if nodeCount > 1 && len(onlineNodeIDs) <= 0 {
+			if memberCount > 1 && len(onlineNodeIDs) <= 0 {
 				logger.Error("Skipping remote volumes for auto custom volume snapshot task due to no online members")
 			} else {
 				for _, v := range remoteVolumes {
@@ -1231,7 +1255,7 @@ func autoCreateCustomVolumeSnapshotsTask(d *Daemon) (task.Func, task.Schedule) {
 					// to perform the snapshot from. This avoids taking the snapshot on every
 					// member and spreads the load taking the snapshots across the online
 					// cluster members.
-					if nodeCount > 1 {
+					if memberCount > 1 {
 						selectedNodeID, err := util.GetStableRandomInt64FromList(int64(v.ID), onlineNodeIDs)
 						if err != nil {
 							logger.Error("Failed scheduling remote auto custom volume snapshot task", logger.Ctx{"volName": v.Name, "project": v.ProjectName, "pool": v.PoolName, "err": err})
@@ -1304,7 +1328,7 @@ func autoCreateCustomVolumeSnapshots(ctx context.Context, d *Daemon, volumes []d
 				return
 			}
 
-			expiry, err := shared.GetSnapshotExpiry(time.Now(), v.Config["snapshots.expiry"])
+			expiry, err := shared.GetExpiry(time.Now(), v.Config["snapshots.expiry"])
 			if err != nil {
 				logger.Error("Error getting expiry date", logger.Ctx{"err": err, "volume": v})
 				ch <- struct{}{}
