@@ -1,30 +1,46 @@
+//go:build linux && cgo && !agent
+
 package sys
 
 import (
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"strconv"
 	"strings"
 
+	"github.com/syndtr/gocapability/capability"
+
+	"github.com/lxc/lxd/lxd/db/cluster"
+	"github.com/lxc/lxd/lxd/db/warningtype"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
-	"github.com/syndtr/gocapability/capability"
-
-	log "github.com/lxc/lxd/shared/log15"
 )
 
 // Initialize AppArmor-specific attributes.
-func (s *OS) initAppArmor() {
+func (s *OS) initAppArmor() []cluster.Warning {
+	var dbWarnings []cluster.Warning
+
 	/* Detect AppArmor availability */
 	_, err := exec.LookPath("apparmor_parser")
 	if os.Getenv("LXD_SECURITY_APPARMOR") == "false" {
 		logger.Warnf("AppArmor support has been manually disabled")
+		dbWarnings = append(dbWarnings, cluster.Warning{
+			TypeCode:    warningtype.AppArmorNotAvailable,
+			LastMessage: "Manually disabled",
+		})
 	} else if !shared.IsDir("/sys/kernel/security/apparmor") {
 		logger.Warnf("AppArmor support has been disabled because of lack of kernel support")
+		dbWarnings = append(dbWarnings, cluster.Warning{
+			TypeCode:    warningtype.AppArmorNotAvailable,
+			LastMessage: "Disabled because of lack of kernel support",
+		})
 	} else if err != nil {
 		logger.Warnf("AppArmor support has been disabled because 'apparmor_parser' couldn't be found")
+		dbWarnings = append(dbWarnings, cluster.Warning{
+			TypeCode:    warningtype.AppArmorNotAvailable,
+			LastMessage: "Disabled because 'apparmor_parser' couldn't be found",
+		})
 	} else {
 		s.AppArmorAvailable = true
 	}
@@ -34,7 +50,7 @@ func (s *OS) initAppArmor() {
 
 	/* Detect existing AppArmor stack */
 	if shared.PathExists("/sys/kernel/security/apparmor/.ns_stacked") {
-		contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/.ns_stacked")
+		contentBytes, err := os.ReadFile("/sys/kernel/security/apparmor/.ns_stacked")
 		if err == nil && string(contentBytes) == "yes\n" {
 			s.AppArmorStacked = true
 		}
@@ -59,24 +75,34 @@ func (s *OS) initAppArmor() {
 		if s.AppArmorAvailable {
 			logger.Warnf("Per-container AppArmor profiles are disabled because LXD is already protected by AppArmor")
 		}
+
 		s.AppArmorConfined = true
 	}
+
+	return dbWarnings
 }
 
 func haveMacAdmin() bool {
-	c, err := capability.NewPid(0)
+	c, err := capability.NewPid2(0)
 	if err != nil {
 		return false
 	}
+
+	err = c.Load()
+	if err != nil {
+		return false
+	}
+
 	if c.Get(capability.EFFECTIVE, capability.CAP_MAC_ADMIN) {
 		return true
 	}
+
 	return false
 }
 
 // Returns true if AppArmor stacking support is available.
 func appArmorCanStack() bool {
-	contentBytes, err := ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
+	contentBytes, err := os.ReadFile("/sys/kernel/security/apparmor/features/domain/stack")
 	if err != nil {
 		return false
 	}
@@ -85,7 +111,7 @@ func appArmorCanStack() bool {
 		return false
 	}
 
-	contentBytes, err = ioutil.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
+	contentBytes, err = os.ReadFile("/sys/kernel/security/apparmor/features/domain/version")
 	if err != nil {
 		return false
 	}
@@ -95,13 +121,13 @@ func appArmorCanStack() bool {
 	parts := strings.Split(strings.TrimSpace(content), ".")
 
 	if len(parts) == 0 {
-		logger.Warn("Unknown apparmor domain version", log.Ctx{"version": content})
+		logger.Warn("Unknown apparmor domain version", logger.Ctx{"version": content})
 		return false
 	}
 
 	major, err := strconv.Atoi(parts[0])
 	if err != nil {
-		logger.Warn("Unknown apparmor domain version", log.Ctx{"version": content})
+		logger.Warn("Unknown apparmor domain version", logger.Ctx{"version": content})
 		return false
 	}
 
@@ -109,7 +135,7 @@ func appArmorCanStack() bool {
 	if len(parts) == 2 {
 		minor, err = strconv.Atoi(parts[1])
 		if err != nil {
-			logger.Warn("Unknown apparmor domain version", log.Ctx{"version": content})
+			logger.Warn("Unknown apparmor domain version", logger.Ctx{"version": content})
 			return false
 		}
 	}
