@@ -1,7 +1,8 @@
+//go:build linux && cgo && !agent
+
 package sys
 
 import (
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"runtime"
@@ -12,7 +13,7 @@ import (
 
 // NewTestOS returns a new OS instance initialized with test values.
 func NewTestOS(t *testing.T) (*OS, func()) {
-	dir, err := ioutil.TempDir("", "lxd-sys-os-test-")
+	dir, err := os.MkdirTemp("", "lxd-sys-os-test-")
 	require.NoError(t, err)
 	require.NoError(t, SetupTestCerts(dir))
 
@@ -30,7 +31,8 @@ func NewTestOS(t *testing.T) (*OS, func()) {
 		LogDir:   filepath.Join(dir, "log"),
 	}
 
-	require.NoError(t, os.Init())
+	_, err = os.Init()
+	require.NoError(t, err)
 
 	return os, cleanup
 }
