@@ -1,12 +1,13 @@
 package task_test
 
 import (
+	"context"
 	"testing"
 	"time"
 
-	"github.com/lxc/lxd/lxd/task"
 	"github.com/stretchr/testify/assert"
-	"golang.org/x/net/context"
+
+	"github.com/lxc/lxd/lxd/task"
 )
 
 func TestGroup_Add(t *testing.T) {
@@ -14,7 +15,7 @@ func TestGroup_Add(t *testing.T) {
 	ok := make(chan struct{})
 	f := func(context.Context) { close(ok) }
 	group.Add(f, task.Every(time.Second))
-	group.Start()
+	group.Start(context.Background())
 
 	assertRecv(t, ok)
 
@@ -24,7 +25,7 @@ func TestGroup_Add(t *testing.T) {
 func TestGroup_StopUngracefully(t *testing.T) {
 	group := &task.Group{}
 
-	// Create a task function that hangs.
+	// Create a task function that blocks.
 	ok := make(chan struct{})
 	defer close(ok)
 	f := func(context.Context) {
@@ -33,11 +34,11 @@ func TestGroup_StopUngracefully(t *testing.T) {
 	}
 
 	group.Add(f, task.Every(time.Second))
-	group.Start()
+	group.Start(context.Background())
 
 	assertRecv(t, ok)
 
-	assert.EqualError(t, group.Stop(time.Millisecond), "tasks 0 are still running")
+	assert.EqualError(t, group.Stop(time.Millisecond), "Task(s) still running: IDs [0]")
 }
 
 // Assert that the given channel receives an object within a second.
