@@ -19,6 +19,7 @@ func CompareConfigs(config1, config2 map[string]string, exclude []string) error
 		if shared.StringInSlice(key, exclude) {
 			continue
 		}
+
 		if config2[key] != value {
 			delta = append(delta, key)
 		}
@@ -27,19 +28,21 @@ func CompareConfigs(config1, config2 map[string]string, exclude []string) error
 		if shared.StringInSlice(key, exclude) {
 			continue
 		}
+
 		if config1[key] != value {
 			present := false
 			for i := range delta {
 				if delta[i] == key {
 					present = true
+					break
 				}
-				break
 			}
 			if !present {
 				delta = append(delta, key)
 			}
 		}
 	}
+
 	sort.Strings(delta)
 	if len(delta) > 0 {
 		return fmt.Errorf("different values for keys: %s", strings.Join(delta, ", "))
@@ -50,9 +53,10 @@ func CompareConfigs(config1, config2 map[string]string, exclude []string) error
 
 // CopyConfig creates a new map with a copy of the given config.
 func CopyConfig(config map[string]string) map[string]string {
-	copy := map[string]string{}
+	copy := make(map[string]string, len(config))
 	for key, value := range config {
 		copy[key] = value
 	}
+
 	return copy
 }
