@@ -3,9 +3,9 @@ package util_test
 import (
 	"testing"
 
-	"github.com/lxc/lxd/lxd/util"
-	"github.com/mpvl/subtest"
 	"github.com/stretchr/testify/assert"
+
+	"github.com/lxc/lxd/lxd/util"
 )
 
 func Test_CompareConfigsMismatch(t *testing.T) {
@@ -25,8 +25,9 @@ func Test_CompareConfigsMismatch(t *testing.T) {
 			"different values for keys: egg, foo",
 		},
 	}
+
 	for _, c := range cases {
-		subtest.Run(t, c.error, func(t *testing.T) {
+		t.Run(c.error, func(t *testing.T) {
 			err := util.CompareConfigs(c.config1, c.config2, nil)
 			assert.EqualError(t, err, c.error)
 		})
