@@ -1,17 +1,14 @@
 package shared
 
 import (
-	"bytes"
-	"encoding/json"
 	"fmt"
-
-	"github.com/lxc/lxd/shared/logger"
 )
 
-type Jmap map[string]interface{}
+type Jmap map[string]any
 
 func (m Jmap) GetString(key string) (string, error) {
-	if val, ok := m[key]; !ok {
+	val, ok := m[key]
+	if !ok {
 		return "", fmt.Errorf("Response was missing `%s`", key)
 	} else if val, ok := val.(string); !ok {
 		return "", fmt.Errorf("`%s` was not a string", key)
@@ -21,9 +18,10 @@ func (m Jmap) GetString(key string) (string, error) {
 }
 
 func (m Jmap) GetMap(key string) (Jmap, error) {
-	if val, ok := m[key]; !ok {
+	val, ok := m[key]
+	if !ok {
 		return nil, fmt.Errorf("Response was missing `%s`", key)
-	} else if val, ok := val.(map[string]interface{}); !ok {
+	} else if val, ok := val.(map[string]any); !ok {
 		return nil, fmt.Errorf("`%s` was not a map, got %T", key, m[key])
 	} else {
 		return val, nil
@@ -31,7 +29,8 @@ func (m Jmap) GetMap(key string) (Jmap, error) {
 }
 
 func (m Jmap) GetInt(key string) (int, error) {
-	if val, ok := m[key]; !ok {
+	val, ok := m[key]
+	if !ok {
 		return -1, fmt.Errorf("Response was missing `%s`", key)
 	} else if val, ok := val.(float64); !ok {
 		return -1, fmt.Errorf("`%s` was not an int", key)
@@ -41,7 +40,8 @@ func (m Jmap) GetInt(key string) (int, error) {
 }
 
 func (m Jmap) GetBool(key string) (bool, error) {
-	if val, ok := m[key]; !ok {
+	val, ok := m[key]
+	if !ok {
 		return false, fmt.Errorf("Response was missing `%s`", key)
 	} else if val, ok := val.(bool); !ok {
 		return false, fmt.Errorf("`%s` was not an int", key)
@@ -49,15 +49,3 @@ func (m Jmap) GetBool(key string) (bool, error) {
 		return val, nil
 	}
 }
-
-func DebugJson(r *bytes.Buffer) {
-	pretty := &bytes.Buffer{}
-	if err := json.Indent(pretty, r.Bytes(), "\t", "\t"); err != nil {
-		logger.Debugf("error indenting json: %s", err)
-		return
-	}
-
-	// Print the JSON without the last "\n"
-	str := pretty.String()
-	logger.Debugf("\n\t%s", str[0:len(str)-1])
-}
