@@ -2,6 +2,8 @@ package validate
 
 import (
 	"bytes"
+	"crypto/x509"
+	"encoding/pem"
 	"fmt"
 	"net"
 	"net/url"
@@ -14,6 +16,7 @@ import (
 	"github.com/kballard/go-shellquote"
 	"github.com/pborman/uuid"
 	"github.com/robfig/cron/v3"
+	"gopkg.in/yaml.v2"
 
 	"github.com/lxc/lxd/shared/osarch"
 	"github.com/lxc/lxd/shared/units"
@@ -268,19 +271,6 @@ func IsNetworkAddress(value string) error {
 	return nil
 }
 
-// IsNetworkAddressList validates a comma delimited list of IPv4 or IPv6 addresses.
-func IsNetworkAddressList(value string) error {
-	for _, v := range strings.Split(value, ",") {
-		v = strings.TrimSpace(v)
-		err := IsNetworkAddress(v)
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetwork validates an IP network CIDR string.
 func IsNetwork(value string) error {
 	ip, subnet, err := net.ParseCIDR(value)
@@ -295,18 +285,6 @@ func IsNetwork(value string) error {
 	return nil
 }
 
-// IsNetworkList validates a comma delimited list of IP network CIDR strings.
-func IsNetworkList(value string) error {
-	for _, network := range strings.Split(value, ",") {
-		err := IsNetwork(strings.TrimSpace(network))
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetworkAddressCIDR validates an IP addresss string in CIDR format.
 func IsNetworkAddressCIDR(value string) error {
 	_, _, err := net.ParseCIDR(value)
@@ -363,19 +341,6 @@ func IsNetworkV4(value string) error {
 	return nil
 }
 
-// IsNetworkV4List validates a comma delimited list of IPv4 CIDR strings.
-func IsNetworkV4List(value string) error {
-	for _, network := range strings.Split(value, ",") {
-		network = strings.TrimSpace(network)
-		err := IsNetworkV4(network)
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetworkAddressV4 validates an IPv4 addresss string.
 func IsNetworkAddressV4(value string) error {
 	ip := net.ParseIP(value)
@@ -386,19 +351,6 @@ func IsNetworkAddressV4(value string) error {
 	return nil
 }
 
-// IsNetworkAddressV4List validates a comma delimited list of IPv4 addresses.
-func IsNetworkAddressV4List(value string) error {
-	for _, v := range strings.Split(value, ",") {
-		v = strings.TrimSpace(v)
-		err := IsNetworkAddressV4(v)
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetworkAddressCIDRV4 validates an IPv4 addresss string in CIDR format.
 func IsNetworkAddressCIDRV4(value string) error {
 	ip, subnet, err := net.ParseCIDR(value)
@@ -434,18 +386,6 @@ func IsNetworkRangeV4(value string) error {
 	return nil
 }
 
-// IsNetworkRangeV4List validates a comma delimited list of IPv4 ranges.
-func IsNetworkRangeV4List(value string) error {
-	for _, ipRange := range strings.Split(value, ",") {
-		err := IsNetworkRangeV4(strings.TrimSpace(ipRange))
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetworkV6 validates an IPv6 CIDR string.
 func IsNetworkV6(value string) error {
 	ip, subnet, err := net.ParseCIDR(value)
@@ -464,19 +404,6 @@ func IsNetworkV6(value string) error {
 	return nil
 }
 
-// IsNetworkV6List validates a comma delimited list of IPv6 CIDR strings.
-func IsNetworkV6List(value string) error {
-	for _, network := range strings.Split(value, ",") {
-		network = strings.TrimSpace(network)
-		err := IsNetworkV6(network)
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetworkAddressV6 validates an IPv6 addresss string.
 func IsNetworkAddressV6(value string) error {
 	ip := net.ParseIP(value)
@@ -487,18 +414,6 @@ func IsNetworkAddressV6(value string) error {
 	return nil
 }
 
-// IsNetworkAddressV6List validates a comma delimited list of IPv6 addresses.
-func IsNetworkAddressV6List(value string) error {
-	for _, v := range strings.Split(value, ",") {
-		v = strings.TrimSpace(v)
-		err := IsNetworkAddressV6(v)
-		if err != nil {
-			return err
-		}
-	}
-	return nil
-}
-
 // IsNetworkAddressCIDRV6 validates an IPv6 addresss string in CIDR format.
 func IsNetworkAddressCIDRV6(value string) error {
 	ip, subnet, err := net.ParseCIDR(value)
@@ -534,18 +449,6 @@ func IsNetworkRangeV6(value string) error {
 	return nil
 }
 
-// IsNetworkRangeV6List validates a comma delimited list of IPv6 ranges.
-func IsNetworkRangeV6List(value string) error {
-	for _, ipRange := range strings.Split(value, ",") {
-		err := IsNetworkRangeV6(strings.TrimSpace(ipRange))
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // IsNetworkVLAN validates a VLAN ID.
 func IsNetworkVLAN(value string) error {
 	vlanID, err := strconv.Atoi(value)
@@ -755,6 +658,18 @@ func IsListenAddress(allowDNS bool, allowWildcard bool, requirePort bool) func(v
 	}
 }
 
+// IsX509Certificate checks if the value is a valid x509 PEM Certificate.
+func IsX509Certificate(value string) error {
+	certBlock, _ := pem.Decode([]byte(value))
+	if certBlock == nil {
+		return fmt.Errorf("Invalid certificate")
+	}
+
+	_, err := x509.ParseCertificate(certBlock.Bytes)
+
+	return err
+}
+
 // IsAbsFilePath checks if value is an absolute file path.
 func IsAbsFilePath(value string) error {
 	if !filepath.IsAbs(value) {
@@ -873,3 +788,85 @@ func IsRequestURL(value string) error {
 
 	return nil
 }
+
+// IsCloudInitUserData checks value is valid cloud-init user data.
+func IsCloudInitUserData(value string) error {
+	if value == "#cloud-config" || strings.HasPrefix(value, "#cloud-config\n") {
+		lines := strings.SplitN(value, "\n", 2)
+
+		// If value only contains the cloud-config header, it is valid.
+		if len(lines) == 1 {
+			return nil
+		}
+
+		return IsYAML(lines[1])
+	}
+
+	// Since there are various other user-data formats besides cloud-config, consider those valid.
+	return nil
+}
+
+// IsYAML checks value is valid YAML.
+func IsYAML(value string) error {
+	out := struct{}{}
+
+	err := yaml.Unmarshal([]byte(value), &out)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// IsValidCPUSet checks value is a valid CPU set.
+func IsValidCPUSet(value string) error {
+	// Validate the CPU set syntax.
+	match, _ := regexp.MatchString("^([0-9]+([,-][0-9]+)?)(,[0-9]+([,-][0-9]+)*)?$", value)
+	if !match {
+		return fmt.Errorf("Invalid CPU limit syntax")
+	}
+
+	cpus := make(map[int64]int)
+	chunks := strings.Split(value, ",")
+
+	for _, chunk := range chunks {
+		if strings.Contains(chunk, "-") {
+			// Range
+			fields := strings.SplitN(chunk, "-", 2)
+			if len(fields) != 2 {
+				return fmt.Errorf("Invalid cpuset value: %s", value)
+			}
+
+			low, err := strconv.ParseInt(fields[0], 10, 64)
+			if err != nil {
+				return fmt.Errorf("Invalid cpuset value: %s", value)
+			}
+
+			high, err := strconv.ParseInt(fields[1], 10, 64)
+			if err != nil {
+				return fmt.Errorf("Invalid cpuset value: %s", value)
+			}
+
+			for i := low; i <= high; i++ {
+				cpus[i]++
+			}
+		} else {
+			// Simple entry
+			nr, err := strconv.ParseInt(chunk, 10, 64)
+			if err != nil {
+				return fmt.Errorf("Invalid cpuset value: %s", value)
+			}
+
+			cpus[nr]++
+		}
+	}
+
+	for i := range cpus {
+		// The CPU was specified more than once, e.g. 1-3,3.
+		if cpus[i] > 1 {
+			return fmt.Errorf("Cannot define CPU multiple times")
+		}
+	}
+
+	return nil
+}
