@@ -334,6 +334,11 @@ var APIExtensions = []string{
 	"storage_cephfs_fscache",
 	"vsock_api",
 	"storage_volumes_all_projects",
+	"projects_networks_restricted_access",
+	"cluster_join_token_expiry",
+	"remote_token_expiry",
+	"init_preseed",
+	"cpu_hotplug",
 }
 
 // APIExtensionsCount returns the number of available API extensions.
