@@ -3,7 +3,7 @@
 package version
 
 import (
-	"io/ioutil"
+	"os"
 	"strings"
 
 	"github.com/lxc/lxd/shared"
@@ -34,7 +34,7 @@ func getPlatformVersionStrings() []string {
 
 	// Add chromebook info
 	if len(versions) == 1 && shared.PathExists("/run/cros_milestone") {
-		content, err := ioutil.ReadFile("/run/cros_milestone")
+		content, err := os.ReadFile("/run/cros_milestone")
 		if err == nil {
 			versions = append(versions, "Chrome OS")
 			versions = append(versions, strings.TrimSpace(string(content)))
