@@ -1,12 +1,12 @@
 # Miscellaneous test checks.
 
 check_dependencies() {
-    # shellcheck disable=SC2039
+    # shellcheck disable=SC2039,3043
     local dep missing
     missing=""
 
     for dep in "$@"; do
-        if ! which "$dep" >/dev/null 2>&1; then
+        if ! command -v "$dep" >/dev/null 2>&1; then
             [ "$missing" ] && missing="$missing $dep" || missing="$dep"
         fi
     done
