@@ -14,8 +14,8 @@ func newCredentialsChecker() credentialsChecker {
 	return credentialsChecker{creds: map[string]string{}}
 }
 
-func (c *credentialsChecker) Check(form interface{}) bool {
-	m := form.(map[string]interface{})
+func (c *credentialsChecker) Check(form any) bool {
+	m := form.(map[string]any)
 	username := m["username"].(string)
 	password := m["password"].(string)
 	pass, ok := c.creds[username]
@@ -44,8 +44,10 @@ func (c *credentialsChecker) LoadCreds(csvFile string) error {
 		if len(row) != 2 {
 			return fmt.Errorf("invalid length on row %d", i+1)
 		}
+
 		creds[row[0]] = row[1]
 	}
+
 	c.AddCreds(creds)
 	return nil
 }
