@@ -7,7 +7,7 @@ test_basic_usage() {
   ensure_has_localhost_remote "${LXD_ADDR}"
 
   # Test image export
-  sum=$(lxc image info testimage | grep ^Fingerprint | cut -d' ' -f2)
+  sum="$(lxc image info testimage | awk '/^Fingerprint/ {print $2}')"
   lxc image export testimage "${LXD_DIR}/"
   [ "${sum}" = "$(sha256sum "${LXD_DIR}/${sum}.tar.xz" | cut -d' ' -f1)" ]
 
@@ -68,7 +68,7 @@ test_basic_usage() {
   # Test image export with a split image.
   deps/import-busybox --split --alias splitimage
 
-  sum=$(lxc image info splitimage | grep ^Fingerprint | cut -d' ' -f2)
+  sum="$(lxc image info splitimage | awk '/^Fingerprint/ {print $2}')"
 
   lxc image export splitimage "${LXD_DIR}"
   [ "${sum}" = "$(cat "${LXD_DIR}/meta-${sum}.tar.xz" "${LXD_DIR}/${sum}.tar.xz" | sha256sum | cut -d' ' -f1)" ]
@@ -91,6 +91,11 @@ test_basic_usage() {
   rm "${LXD_DIR}/meta-${sum}.tar.xz"
   lxc image delete splitimage
 
+  # Test --no-profiles flag
+  poolName=$(lxc profile device get default root pool)
+  ! lxc init testimage foo --no-profiles || false
+  lxc init testimage foo --no-profiles -s "${poolName}"
+  lxc delete -f foo
 
   # Test container creation
   lxc init testimage foo
@@ -517,8 +522,7 @@ test_basic_usage() {
 
   REBOOTED="false"
 
-  # shellcheck disable=SC2034
-  for i in $(seq 60); do
+  for _ in $(seq 60); do
     NEW_INIT=$(lxc info foo | awk '/^PID:/ {print $2}' || true)
 
     # If init process is running, check if is old or new process.
@@ -548,6 +552,35 @@ test_basic_usage() {
   ! lxc profile delete default || false
 
   lxc init testimage c1
-  ! lxc config device override c1 root pool bla || false
+  result="$(! lxc config device override c1 root pool=bla 2>&1)"
+  if ! echo "${result}" | grep "Error: Cannot update root disk device pool name"; then
+    echo "Should fail device override because root disk device storage pool cannot be changed."
+    false
+  fi
+
   lxc rm -f c1
+
+  # Should fail to override root device storage pool when the new pool does not exist.
+  ! lxc init testimage c1 -d root,pool=bla || false
+
+  # Should succeed in overriding root device storage pool when the pool does exist and the override occurs at create time.
+  lxc storage create bla dir
+  lxc init testimage c1 -d root,pool=bla
+  lxc config show c1 --expanded | grep -Pz '  root:\n    path: /\n    pool: bla\n    type: disk\n'
+
+  lxc storage volume create bla vol1
+  lxc storage volume create bla vol2
+  lxc config device add c1 dev disk source=vol1 pool=bla path=/vol
+
+  # Should not be able to override a device that is not part of a profile (i.e. has been specifically added).
+  result="$(! lxc config device override c1 dev source=vol2 2>&1)"
+  if ! echo "${result}" | grep "Error: The device already exists"; then
+    echo "Should fail because device is defined against the instance not the profile."
+    false
+  fi
+
+  lxc rm -f c1
+  lxc storage volume delete bla vol1
+  lxc storage volume delete bla vol2
+  lxc storage delete bla
 }
