@@ -1,5 +1,5 @@
 test_console() {
-  lxc_version=$(lxc info | grep "driver_version: " | cut -d' ' -f4)
+  lxc_version=$(lxc info | awk '/driver_version:/ {print $NF}')
   lxc_major=$(echo "${lxc_version}" | cut -d. -f1)
 
   if [ "${lxc_major}" -lt 3 ]; then
@@ -15,20 +15,17 @@ test_console() {
 
   lxc start cons1
 
-  # Let the init system come up.
-  sleep 2
-
   # Make sure there's something in the console ringbuffer.
   echo 'some content' | lxc exec cons1 -- tee /dev/console
   echo 'some more content' | lxc exec cons1 -- tee /dev/console
 
   # Retrieve the ringbuffer contents.
-  lxc console cons1 --show-log
+  lxc console cons1 --show-log | grep 'some content'
 
   lxc stop --force cons1
 
   # Retrieve on-disk representation of the console ringbuffer.
-  lxc console cons1 --show-log
+  lxc console cons1 --show-log | grep 'some more content'
 
   lxc delete --force cons1
 }
