@@ -25,10 +25,11 @@ test_devlxd() {
   lxc config set devlxd security.nesting true
   ! lxc exec devlxd devlxd-client security.nesting | grep true || false
 
-  lxc exec devlxd devlxd-client monitor-websocket > "${TEST_DIR}/devlxd-websocket.log" &
+  cmd=$(unset -f lxc; command -v lxc)
+  ${cmd} exec devlxd devlxd-client monitor-websocket > "${TEST_DIR}/devlxd-websocket.log" &
   client_websocket=$!
 
-  lxc exec devlxd devlxd-client monitor-stream > "${TEST_DIR}/devlxd-stream.log" &
+  ${cmd} exec devlxd devlxd-client monitor-stream > "${TEST_DIR}/devlxd-stream.log" &
   client_stream=$!
 
   (
@@ -65,8 +66,7 @@ EOF
 
   MATCH=0
 
-  # shellcheck disable=SC2034
-  for i in $(seq 10); do
+  for _ in $(seq 10); do
     lxc config set devlxd user.foo bar
     lxc config set devlxd security.nesting true
 
@@ -89,6 +89,11 @@ EOF
 
   kill -9 "${client_websocket}"
   kill -9 "${client_stream}"
+
+  # Check device configs are available and that NIC hwaddr is available even if volatile.
+  hwaddr=$(lxc config get devlxd volatile.eth0.hwaddr)
+  lxc exec devlxd devlxd-client devices | jq -r .eth0.hwaddr | grep -Fx "${hwaddr}"
+
   lxc delete devlxd --force
 
   [ "${MATCH}" = "1" ] || false
