@@ -21,12 +21,19 @@ test_fdleak() {
       lxc delete "leaktest${i}"
     done
 
-    sleep 5
+    lxc list
+    lxc query /internal/gc
 
     exit 0
   )
-  afterfds=$(/bin/ls "/proc/${pid}/fd" | wc -l)
-  leakedfds=$((afterfds - beforefds))
+
+  for i in $(seq 20); do
+    afterfds=$(/bin/ls "/proc/${pid}/fd" | wc -l)
+    leakedfds=$((afterfds - beforefds))
+
+    [ "${leakedfds}" -gt 5 ] || break
+    sleep 0.5
+  done
 
   bad=0
   # shellcheck disable=SC2015
