@@ -15,7 +15,7 @@ test_snapshots() {
 }
 
 snapshots() {
-  # shellcheck disable=2039
+  # shellcheck disable=2039,3043
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
 
@@ -105,7 +105,7 @@ test_snap_restore() {
 }
 
 snap_restore() {
-  # shellcheck disable=2039
+  # shellcheck disable=2039,3043
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
 
@@ -127,6 +127,17 @@ snap_restore() {
   lxc exec bar -- ln -s file_only_in_snap0 /root/statelink
   lxc stop bar --force
 
+  # Get container's pool.
+  pool=$(lxc config profile device get default root pool)
+
+  lxc storage volume set "${pool}" container/bar user.foo=snap0
+
+  # Check parent volume.block.filesystem is copied to snapshot and not from pool.
+  if [ "$lxd_backend" = "lvm" ] || [ "$lxd_backend" = "ceph" ]; then
+    # Change pool volume.block.filesystem setting after creation of instance and before snapshot.
+    lxc storage set "${pool}" volume.block.filesystem=xfs
+  fi
+
   lxc snapshot bar snap0
 
   ## prepare snap1
@@ -141,6 +152,7 @@ snap_restore() {
   lxc exec bar -- ln -s file_only_in_snap1 /root/statelink
   lxc exec bar -- mkdir /root/dir_only_in_snap1
   lxc stop bar --force
+  lxc storage volume set "${pool}" container/bar user.foo=snap1
 
   # Delete the state file we created to prevent leaking.
   rm state
@@ -148,6 +160,22 @@ snap_restore() {
   lxc config set bar limits.cpu 1
 
   lxc snapshot bar snap1
+  lxc storage volume set "${pool}" container/bar user.foo=postsnaps
+
+  # Check volume.block.filesystem on storage volume in parent and snapshot match.
+  if [ "${lxd_backend}" = "lvm" ] || [ "${lxd_backend}" = "ceph" ]; then
+    # Change pool volume.block.filesystem setting after creation of instance and before snapshot.
+    pool=$(lxc config profile device get default root pool)
+    parentFS=$(lxc storage volume get "${pool}" container/bar block.filesystem)
+    snapFS=$(lxc storage volume get "${pool}" container/bar/snap0 block.filesystem)
+
+    if [ "${parentFS}" != "${snapFS}" ]; then
+      echo "block.filesystem settings do not match in parent and snapshot"
+      false
+    fi
+
+    lxc storage unset "${pool}" volume.block.filesystem
+  fi
 
   ##########################################################
 
@@ -162,6 +190,9 @@ snap_restore() {
       echo "==> config didn't match expected value after restore (${cpus})"
       false
     fi
+
+    # Check storage volume has been restored (user.foo=snap0)
+    lxc storage volume get "${pool}" container/bar user.foo | grep -Fx "snap0"
   fi
 
   ##########################################################
@@ -176,6 +207,9 @@ snap_restore() {
    false
   fi
 
+  # Check storage volume has been restored (user.foo=snap0)
+  lxc storage volume get "${pool}" container/bar user.foo | grep -Fx "snap1"
+
   ##########################################################
 
   # Start container and then restore snapshot to verify the running state after restore.
@@ -194,16 +228,12 @@ snap_restore() {
   lxc delete bar
 
   # Test if container's with hyphen's in their names are treated correctly.
-  if [ "$lxd_backend" = "lvm" ]; then
-    lxc launch testimage a-b
-    lxc snapshot a-b base
-    lxc restore a-b base
-
-    lxc snapshot a-b c-d
-    lxc restore a-b c-d
-
-    lxc delete -f a-b
-  fi
+  lxc launch testimage a-b
+  lxc snapshot a-b base
+  lxc restore a-b base
+  lxc snapshot a-b c-d
+  lxc restore a-b c-d
+  lxc delete -f a-b
 }
 
 restore_and_compare_fs() {
@@ -218,3 +248,82 @@ restore_and_compare_fs() {
     diff -r "${LXD_DIR}/containers/bar/rootfs" "${LXD_DIR}/snapshots/bar/${snap}/rootfs"
   fi
 }
+
+test_snap_expiry() {
+  # shellcheck disable=2039,3043
+  local lxd_backend
+  lxd_backend=$(storage_backend "$LXD_DIR")
+
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
+  lxc launch testimage c1
+  lxc snapshot c1
+  lxc config show c1/snap0 | grep -q 'expires_at: 0001-01-01T00:00:00Z'
+
+  lxc config set c1 snapshots.expiry '1d'
+  lxc snapshot c1
+  ! lxc config show c1/snap1 | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
+
+  lxc copy c1 c2
+  ! lxc config show c2/snap1 | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
+
+  lxc snapshot c1 --no-expiry
+  lxc config show c1/snap2 | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
+
+  lxc rm -f c1
+  lxc rm -f c2
+}
+
+test_snap_schedule() {
+  # shellcheck disable=2039,3043
+  local lxd_backend
+  lxd_backend=$(storage_backend "$LXD_DIR")
+
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
+  # Check we get a snapshot on first start
+  lxc launch testimage c1 -c snapshots.schedule='@startup'
+  lxc launch testimage c2 -c snapshots.schedule='@startup, @daily'
+  lxc launch testimage c3 -c snapshots.schedule='@startup, 10 5,6 * * *'
+  lxc launch testimage c4 -c snapshots.schedule='@startup, 10 5-8 * * *'
+  lxc launch testimage c5 -c snapshots.schedule='@startup, 10 2,5-8/2 * * *'
+  lxc info c1 | grep -q snap0
+  lxc info c2 | grep -q snap0
+  lxc info c3 | grep -q snap0
+  lxc info c4 | grep -q snap0
+  lxc info c5 | grep -q snap0
+
+  # Check we get a new snapshot on restart
+  lxc restart c1 -f
+  lxc info c1 | grep -q snap1
+
+  lxc rm -f c1 c2 c3 c4 c5
+}
+
+test_snap_volume_db_recovery() {
+  # shellcheck disable=2039,3043
+  local lxd_backend
+  lxd_backend=$(storage_backend "$LXD_DIR")
+
+  ensure_import_testimage
+  ensure_has_localhost_remote "${LXD_ADDR}"
+
+  poolName=$(lxc profile device get default root pool)
+
+  lxc init testimage c1
+  lxc snapshot c1
+  lxc snapshot c1
+  lxc start c1
+  lxc stop -f c1
+  lxd sql global 'DELETE FROM storage_volumes_snapshots' # Remove volume snapshot DB records.
+  lxd sql local 'DELETE FROM  patches WHERE name = "storage_missing_snapshot_records"' # Clear patch indicator.
+  ! lxc start c1 || false # Shouldn't be able to start as backup.yaml generation checks for DB consistency.
+  lxd shutdown
+  respawn_lxd "${LXD_DIR}" true
+  lxc storage volume show "${poolName}" container/c1/snap0 | grep "Auto repaired"
+  lxc storage volume show "${poolName}" container/c1/snap1 | grep "Auto repaired"
+  lxc start c1
+  lxc delete -f c1
+}
